@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -23,92 +23,116 @@
 * ************************************************************************ */
 
 #include "rocsparse_arguments_config.hpp"
+#include "rocsparse_clients_matrices_dir.hpp"
 #include "rocsparse_enum.hpp"
+#include "rocsparse_importer_format_t.hpp"
+
 rocsparse_arguments_config::rocsparse_arguments_config()
 {
     //
     // Arguments must be a C-compatible struct so cppcheck complains about non-initialized member variables.
     // Then we need to initialize.
     {
-        this->M              = static_cast<rocsparse_int>(0);
-        this->N              = static_cast<rocsparse_int>(0);
-        this->K              = static_cast<rocsparse_int>(0);
-        this->nnz            = static_cast<rocsparse_int>(0);
-        this->block_dim      = static_cast<rocsparse_int>(0);
-        this->row_block_dimA = static_cast<rocsparse_int>(0);
-        this->col_block_dimA = static_cast<rocsparse_int>(0);
-        this->row_block_dimB = static_cast<rocsparse_int>(0);
-        this->col_block_dimB = static_cast<rocsparse_int>(0);
-        this->dimx           = static_cast<rocsparse_int>(0);
-        this->dimy           = static_cast<rocsparse_int>(0);
-        this->dimz           = static_cast<rocsparse_int>(0);
-        this->ll             = static_cast<rocsparse_int>(0);
-        this->l              = static_cast<rocsparse_int>(0);
-        this->u              = static_cast<rocsparse_int>(0);
-        this->uu             = static_cast<rocsparse_int>(0);
-        this->index_type_I   = static_cast<rocsparse_indextype>(0);
-        this->index_type_J   = static_cast<rocsparse_indextype>(0);
-        this->compute_type   = static_cast<rocsparse_datatype>(0);
-        this->alpha          = static_cast<double>(0);
-        this->alphai         = static_cast<double>(0);
-        this->beta           = static_cast<double>(0);
-        this->betai          = static_cast<double>(0);
-        this->threshold      = static_cast<double>(0);
-        this->percentage     = static_cast<double>(0);
-        this->transA         = static_cast<rocsparse_operation>(0);
-        this->transB         = static_cast<rocsparse_operation>(0);
-        this->baseA          = static_cast<rocsparse_index_base>(0);
-        this->baseB          = static_cast<rocsparse_index_base>(0);
-        this->baseC          = static_cast<rocsparse_index_base>(0);
-        this->baseD          = static_cast<rocsparse_index_base>(0);
-        this->action         = static_cast<rocsparse_action>(0);
-        this->part           = static_cast<rocsparse_hyb_partition>(0);
-        this->matrix_type    = static_cast<rocsparse_matrix_type>(0);
-        this->diag           = static_cast<rocsparse_diag_type>(0);
-        this->uplo           = static_cast<rocsparse_fill_mode>(0);
-        this->storage        = static_cast<rocsparse_storage_mode>(0);
-        this->apol           = static_cast<rocsparse_analysis_policy>(0);
-        this->spol           = static_cast<rocsparse_solve_policy>(0);
-        this->direction      = static_cast<rocsparse_direction>(0);
-        this->order          = static_cast<rocsparse_order>(0);
-        this->format         = static_cast<rocsparse_format>(0);
-
-        this->itilu0_alg          = rocsparse_itilu0_alg_default;
-        this->sddmm_alg           = rocsparse_sddmm_alg_default;
-        this->spmv_alg            = rocsparse_spmv_alg_default;
-        this->spsv_alg            = rocsparse_spsv_alg_default;
-        this->spitsv_alg          = rocsparse_spitsv_alg_default;
-        this->spsm_alg            = rocsparse_spsm_alg_default;
-        this->spmm_alg            = rocsparse_spmm_alg_default;
-        this->spgemm_alg          = rocsparse_spgemm_alg_default;
-        this->sparse_to_dense_alg = rocsparse_sparse_to_dense_alg_default;
-        this->dense_to_sparse_alg = rocsparse_dense_to_sparse_alg_default;
+        this->M               = static_cast<rocsparse_int>(0);
+        this->N               = static_cast<rocsparse_int>(0);
+        this->K               = static_cast<rocsparse_int>(0);
+        this->nnz             = static_cast<rocsparse_int>(0);
+        this->block_dim       = static_cast<rocsparse_int>(0);
+        this->row_block_dimA  = static_cast<rocsparse_int>(0);
+        this->col_block_dimA  = static_cast<rocsparse_int>(0);
+        this->row_block_dimB  = static_cast<rocsparse_int>(0);
+        this->col_block_dimB  = static_cast<rocsparse_int>(0);
+        this->dimx            = static_cast<rocsparse_int>(0);
+        this->dimy            = static_cast<rocsparse_int>(0);
+        this->dimz            = static_cast<rocsparse_int>(0);
+        this->ll              = static_cast<rocsparse_int>(0);
+        this->l               = static_cast<rocsparse_int>(0);
+        this->u               = static_cast<rocsparse_int>(0);
+        this->uu              = static_cast<rocsparse_int>(0);
+        this->index_type_I    = static_cast<rocsparse_indextype>(0);
+        this->index_type_J    = static_cast<rocsparse_indextype>(0);
+        this->a_type          = static_cast<rocsparse_datatype>(0);
+        this->b_type          = static_cast<rocsparse_datatype>(0);
+        this->c_type          = static_cast<rocsparse_datatype>(0);
+        this->x_type          = static_cast<rocsparse_datatype>(0);
+        this->y_type          = static_cast<rocsparse_datatype>(0);
+        this->compute_type    = static_cast<rocsparse_datatype>(0);
+        this->A_row_indextype = static_cast<rocsparse_indextype>(0);
+        this->A_col_indextype = static_cast<rocsparse_indextype>(0);
+        this->B_row_indextype = static_cast<rocsparse_indextype>(0);
+        this->B_col_indextype = static_cast<rocsparse_indextype>(0);
+        this->C_row_indextype = static_cast<rocsparse_indextype>(0);
+        this->C_col_indextype = static_cast<rocsparse_indextype>(0);
+        this->alpha           = static_cast<double>(0);
+        this->alphai          = static_cast<double>(0);
+        this->beta            = static_cast<double>(0);
+        this->betai           = static_cast<double>(0);
+        this->threshold       = static_cast<double>(0);
+        this->percentage      = static_cast<double>(0);
+        this->transA          = static_cast<rocsparse_operation>(0);
+        this->transB          = static_cast<rocsparse_operation>(0);
+        this->baseA           = static_cast<rocsparse_index_base>(0);
+        this->baseB           = static_cast<rocsparse_index_base>(0);
+        this->baseC           = static_cast<rocsparse_index_base>(0);
+        this->baseD           = static_cast<rocsparse_index_base>(0);
+        this->action          = static_cast<rocsparse_action>(0);
+        this->part            = static_cast<rocsparse_hyb_partition>(0);
+        this->matrix_type     = static_cast<rocsparse_matrix_type>(0);
+        this->diag            = static_cast<rocsparse_diag_type>(0);
+        this->uplo            = static_cast<rocsparse_fill_mode>(0);
+        this->storage         = static_cast<rocsparse_storage_mode>(0);
+        this->apol            = static_cast<rocsparse_analysis_policy>(0);
+        this->spol            = static_cast<rocsparse_solve_policy>(0);
+        this->direction       = static_cast<rocsparse_direction>(0);
+        this->order           = static_cast<rocsparse_order>(0);
+        this->orderB          = static_cast<rocsparse_order>(0);
+        this->orderC          = static_cast<rocsparse_order>(0);
+        this->formatA         = static_cast<rocsparse_format>(0);
+        this->formatB         = static_cast<rocsparse_format>(0);
 
+        this->itilu0_alg           = rocsparse_itilu0_alg_default;
+        this->sddmm_alg            = rocsparse_sddmm_alg_default;
+        this->spmv_alg             = rocsparse_spmv_alg_default;
+        this->spsv_alg             = rocsparse_spsv_alg_default;
+        this->spitsv_alg           = rocsparse_spitsv_alg_default;
+        this->spsm_alg             = rocsparse_spsm_alg_default;
+        this->spmm_alg             = rocsparse_spmm_alg_default;
+        this->spgemm_alg           = rocsparse_spgemm_alg_default;
+        this->sparse_to_dense_alg  = rocsparse_sparse_to_dense_alg_default;
+        this->dense_to_sparse_alg  = rocsparse_dense_to_sparse_alg_default;
         this->gtsv_interleaved_alg = static_cast<rocsparse_gtsv_interleaved_alg>(0);
         this->gpsv_interleaved_alg = static_cast<rocsparse_gpsv_interleaved_alg>(0);
-        this->matrix               = static_cast<rocsparse_matrix_init>(0);
-        this->matrix_init_kind     = static_cast<rocsparse_matrix_init_kind>(0);
-        this->unit_check           = static_cast<rocsparse_int>(0);
-        this->timing               = static_cast<rocsparse_int>(1);
-        this->iters                = static_cast<rocsparse_int>(0);
-        this->denseld              = static_cast<rocsparse_int>(0);
-        this->batch_count          = static_cast<rocsparse_int>(0);
-        this->batch_count_A        = static_cast<rocsparse_int>(0);
-        this->batch_count_B        = static_cast<rocsparse_int>(0);
-        this->batch_count_C        = static_cast<rocsparse_int>(0);
-        this->batch_stride         = static_cast<rocsparse_int>(0);
-        this->algo                 = static_cast<uint32_t>(0);
-        this->numericboost         = static_cast<int>(0);
-        this->boosttol             = static_cast<double>(0);
-        this->boostval             = static_cast<double>(0);
-        this->boostvali            = static_cast<double>(0);
-        this->tolm                 = static_cast<double>(0);
-        this->graph_test           = static_cast<bool>(0);
-        this->filename[0]          = '\0';
-        this->function[0]          = '\0';
-        this->name[0]              = '\0';
-        this->category[0]          = '\0';
-        this->hardware[0]          = '\0';
+
+        this->matrix                      = static_cast<rocsparse_matrix_init>(0);
+        this->matrix_init_kind            = static_cast<rocsparse_matrix_init_kind>(0);
+        this->unit_check                  = static_cast<rocsparse_int>(0);
+        this->timing                      = static_cast<rocsparse_int>(1);
+        this->iters                       = static_cast<rocsparse_int>(0);
+        this->nfreeiter                   = static_cast<rocsparse_int>(0);
+        this->nmaxiter                    = static_cast<rocsparse_int>(0);
+        this->denseld                     = static_cast<int64_t>(0);
+        this->batch_count                 = static_cast<rocsparse_int>(0);
+        this->batch_count_A               = static_cast<rocsparse_int>(0);
+        this->batch_count_B               = static_cast<rocsparse_int>(0);
+        this->batch_count_C               = static_cast<rocsparse_int>(0);
+        this->batch_stride                = static_cast<rocsparse_int>(0);
+        this->ld_multiplier_B             = static_cast<rocsparse_int>(2);
+        this->ld_multiplier_C             = static_cast<rocsparse_int>(2);
+        this->algo                        = static_cast<uint32_t>(0);
+        this->numericboost                = static_cast<int>(0);
+        this->boosttol                    = static_cast<double>(0);
+        this->boostval                    = static_cast<double>(0);
+        this->boostvali                   = static_cast<double>(0);
+        this->tolm                        = static_cast<double>(0);
+        this->graph_test                  = static_cast<bool>(0);
+        this->skip_reproducibility        = static_cast<bool>(0);
+        this->sparsity_pattern_statistics = static_cast<bool>(0);
+        this->filename[0]                 = '\0';
+        this->function[0]                 = '\0';
+        this->name[0]                     = '\0';
+        this->category[0]                 = '\0';
+        this->hardware[0]                 = '\0';
+        this->skip_hardware[0]            = '\0';
     }
 
     this->precision = 's';
@@ -162,17 +186,29 @@ void rocsparse_arguments_config::set_description(options_description& desc)
      value<std::string>(&this->b_matrixmarket)->default_value(""), "read from matrix "
      "market (.mtx) format. This will override parameters -m, -n, and -z.")
 
+    ("smtx",
+     value<std::string>(&this->b_mlcsr)->default_value(""), "read from machine "
+     "learning CSR (.smtx) format. This will override parameters -m, -n, and -z.")
+
+    ("bsmtx",
+     value<std::string>(&this->b_mlbsr)->default_value(""), "read from machine "
+     "learning BSR (.bsmtx) format. This will override parameters -m, -n, and -z.")
+
     ("rocalution",
      value<std::string>(&this->b_rocalution)->default_value(""),
-     "read from rocalution matrix binary file. This will override parameter --mtx")
+     "read from rocalution matrix binary file.")
 
     ("rocsparseio",
      value<std::string>(&this->b_rocsparseio)->default_value(""),
-     "read from rocsparseio matrix binary file. This will override parameter --rocalution")
+     "read from rocsparseio matrix binary file.")
 
     ("file",
      value<std::string>(&this->b_file)->default_value(""),
-     "read from file with file extension detection. This will override parameters --rocsparseio,rocalution,mtx")
+     "read from file with file extension detection.")
+
+    ("matrices-dir",
+     value<std::string>(&this->b_matrices_dir)->default_value(""),
+     "Specify the matrix source directory.")
 
     ("dimx",
      value<rocsparse_int>(&this->dimx)->default_value(0), "assemble "
@@ -250,6 +286,10 @@ void rocsparse_arguments_config::set_description(options_description& desc)
      "0 = rocsparse_hyb_partition_auto, 1 = rocsparse_hyb_partition_user,\n"
      "2 = rocsparse_hyb_partition_max, (default: 0)")
 
+    ("hybellwidth",
+     value<uint32_t>(&this->algo)->default_value(0),
+     "ell width to use when hybpart is set to rocsparse_hyb_partition_user (=1), (default: 0)")
+
     ("matrix_type",
      value<int>(&this->b_matrix_type)->default_value(0),
      "0 = rocsparse_matrix_type_general, 1 = rocsparse_matrix_type_symmetric,\n"
@@ -282,9 +322,9 @@ void rocsparse_arguments_config::set_description(options_description& desc)
      "  Conversion: csr2coo, csr2csc, gebsr2gebsc, csr2ell, csr2hyb, csr2bsr, csr2gebsr\n"
      "              coo2csr, ell2csr, hyb2csr, dense2csr, dense2coo, prune_dense2csr, prune_dense2csr_by_percentage, dense2csc\n"
      "              csr2dense, csc2dense, coo2dense, bsr2csr, gebsr2csr, gebsr2gebsr, csr2csr_compress, prune_csr2csr, prune_csr2csr_by_percentage\n"
-     "              sparse_to_dense_coo, sparse_to_dense_csr, sparse_to_dense_csc, dense_to_sparse_coo, dense_to_sparse_csr, dense_to_sparse_csc\n"
+     "              sparse_to_dense_coo, sparse_to_dense_csr, sparse_to_dense_csc, dense_to_sparse_coo, dense_to_sparse_csr, dense_to_sparse_csc, sparse_to_sparse, extract\n"
      "  Sorting: cscsort, csrsort, coosort\n"
-     "  Misc: identity, nnz\n"
+     "  Misc: identity, inverse_permutation, nnz\n"
      "  Util: check_matrix_csr, check_matrix_csc, check_matrix_coo, check_matrix_gebsr, check_matrix_gebsc, check_matrix_ell, check_matrix_hyb")
 
     ("indextype",
@@ -299,9 +339,17 @@ void rocsparse_arguments_config::set_description(options_description& desc)
      "Validate GPU results with CPU? 0 = No, 1 = Yes (default: No)")
 
     ("iters,i",
-     value<int>(&this->iters)->default_value(10),
+     value<rocsparse_int>(&this->iters)->default_value(10),
      "Iterations to run inside timing loop")
 
+    ("nfreeiter",
+     value<rocsparse_int>(&this->nfreeiter)->default_value(20),
+     "Number of free iterations for iterative algorithms (default: 20)")
+
+    ("nmaxiter",
+     value<rocsparse_int>(&this->nmaxiter)->default_value(1000),
+     "Maximum number of iterations for iterative algorithms (default: 1000)")
+
     ("device,d",
      value<rocsparse_int>(&this->device_id)->default_value(0),
      "Set default device to be used for subsequent program runs")
@@ -314,12 +362,28 @@ void rocsparse_arguments_config::set_description(options_description& desc)
      value<rocsparse_int>(&this->b_order)->default_value(rocsparse_order_column),
      "Indicates whether a dense matrix is laid out in column-major storage: 1, or row-major storage 0 (default: 1)")
 
+    ("orderB",
+     value<rocsparse_int>(&this->b_orderB)->default_value(rocsparse_order_column),
+     "Indicates whether a dense matrix is laid out in column-major storage: 1, or row-major storage 0 (default: 1)")
+
+    ("orderC",
+     value<rocsparse_int>(&this->b_orderC)->default_value(rocsparse_order_column),
+     "Indicates whether a dense matrix is laid out in column-major storage: 1, or row-major storage 0 (default: 1)")
+
     ("format",
-     value<rocsparse_int>(&this->b_format)->default_value(rocsparse_format_coo),
-     "Indicates whether a sparse matrix is laid out in coo format: 0, coo_aos format: 1, csr format: 2, csc format: 3 or ell format: 4 (default:0)")
+     value<rocsparse_int>(&this->b_formatA)->default_value(rocsparse_format_coo),
+     "Indicates whether a sparse matrix is laid out in coo format: 0, coo_aos format: 1, csr format: 2, csc format: 3, ell format: 4, bell format: 5, bsr format: 6 (default:0)")
+
+    ("formatA",
+     value<rocsparse_int>(&this->b_formatA)->default_value(rocsparse_format_coo),
+     "Indicates whether a sparse matrix is laid out in coo format: 0, coo_aos format: 1, csr format: 2, csc format: 3, ell format: 4, bell format: 5, bsr format: 6 (default:0)")
+
+    ("formatB",
+     value<rocsparse_int>(&this->b_formatB)->default_value(rocsparse_format_coo),
+     "Indicates whether a sparse matrix is laid out in coo format: 0, coo_aos format: 1, csr format: 2, csc format: 3, ell format: 4, bell format: 5, bsr format: 6 (default:0)")
 
     ("denseld",
-     value<rocsparse_int>(&this->denseld)->default_value(128),
+     value<int64_t>(&this->denseld)->default_value(128),
      "Indicates the leading dimension of a dense matrix >= M, assuming a column-oriented storage.")
 
     ("batch_count",
@@ -350,7 +414,7 @@ void rocsparse_arguments_config::set_description(options_description& desc)
 
     ("spmv_alg",
       value<rocsparse_int>(&this->b_spmv_alg)->default_value(rocsparse_spmv_alg_default),
-      "Indicates what algorithm to use when running SpMV. Possibly choices are default: 0, COO: 1, CSR adaptive: 2, CSR stream: 3, ELL: 4, COO atomic: 5 (default:0)")
+      "Indicates what algorithm to use when running SpMV. Possibly choices are default: 0, COO: 1, CSR adaptive: 2, CSR stream: 3, ELL: 4, COO atomic: 5, BSR: 6, CSR LRB: 7 (default:0)")
 
     ("itilu0_alg",
       value<rocsparse_int>(&this->b_itilu0_alg)->default_value(rocsparse_itilu0_alg_default),
@@ -358,11 +422,19 @@ void rocsparse_arguments_config::set_description(options_description& desc)
 
     ("spmm_alg",
       value<rocsparse_int>(&this->b_spmm_alg)->default_value(rocsparse_spmm_alg_default),
-      "Indicates what algorithm to use when running SpMM. Possibly choices are default: 0, CSR: 1, COO segmented: 2, COO atomic: 3, CSR row split: 4, CSR merge: 5, COO segmented atomic: 6, BELL: 7 (default:0)")
+      "Indicates what algorithm to use when running SpMM. Possibly choices are default: 0, CSR: 1, COO segmented: 2, COO atomic: 3, CSR row split: 4, CSR merge: 5, COO segmented atomic: 6, BELL: 7, BSR: 8, CSR merge-path: 9, CSR non-zero split: 5 (default:0)")
+
+    ("sddmm_alg",
+      value<rocsparse_int>(&this->b_sddmm_alg)->default_value(rocsparse_sddmm_alg_default),
+      "Indicates what algorithm to use when running SDDMM. Possibly choices are rocsparse_sddmm_alg_default: 0, rocsparse_sddmm_alg_default: 1 (default: 0)")
 
     ("gtsv_interleaved_alg",
       value<rocsparse_int>(&this->b_gtsv_interleaved_alg)->default_value(rocsparse_gtsv_interleaved_alg_default),
-      "Indicates what algorithm to use when running rocsparse_gtsv_interleaved_batch. Possibly choices are thomas: 1, lu: 2, qr: 3 (default:3)");
+      "Indicates what algorithm to use when running rocsparse_gtsv_interleaved_batch. Possibly choices are thomas: 1, lu: 2, qr: 3 (default:3)")
+
+      ("sparsity-pattern-statistics",
+       "enable sparsity pattern statistics: min,max and median of the number of non-zeros per row and per column of the sparsity pattern of the matrix A will be part of benchmarking results.");
+
 }
 
 int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&desc)
@@ -389,14 +461,61 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
     return -1;
   }
 
-  if(this->b_format != rocsparse_format_csr && this->b_format != rocsparse_format_coo
-     && this->b_format != rocsparse_format_coo_aos && this->b_format != rocsparse_format_ell
-     && this->b_format != rocsparse_format_csc)
+  if(this->b_orderB != rocsparse_order_row && this->b_orderB != rocsparse_order_column)
   {
-    std::cerr << "Invalid value for --format" << std::endl;
+    std::cerr << "Invalid value for --orderB" << std::endl;
     return -1;
   }
 
+  if(this->b_orderC != rocsparse_order_row && this->b_orderC != rocsparse_order_column)
+  {
+    std::cerr << "Invalid value for --orderC" << std::endl;
+    return -1;
+  }
+
+  { bool is_format_invalid = true;
+    switch(this->b_formatA)
+      {
+      case rocsparse_format_csr:
+      case rocsparse_format_coo:
+      case rocsparse_format_ell:
+      case rocsparse_format_csc:
+      case rocsparse_format_coo_aos:
+      case rocsparse_format_bell:
+      case rocsparse_format_bsr:
+	{
+	  is_format_invalid = false;
+	  break;
+	}
+      }
+
+    if(is_format_invalid)
+      {
+	std::cerr << "Invalid value for --format" << std::endl;
+	return -1;
+      } }
+  { bool is_format_invalid = true;
+    switch(this->b_formatB)
+      {
+      case rocsparse_format_csr:
+      case rocsparse_format_coo:
+      case rocsparse_format_ell:
+      case rocsparse_format_csc:
+      case rocsparse_format_coo_aos:
+      case rocsparse_format_bell:
+      case rocsparse_format_bsr:
+	{
+	  is_format_invalid = false;
+	  break;
+	}
+      }
+
+    if(is_format_invalid)
+      {
+	std::cerr << "Invalid value for --formatB" << std::endl;
+	return -1;
+      } }
+
   if (rocsparse_itilu0_alg_t::is_invalid(this->b_itilu0_alg))
     {
       std::cerr << "Invalid value '"
@@ -412,7 +531,9 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
        && this->b_spmv_alg != rocsparse_spmv_alg_csr_adaptive
        && this->b_spmv_alg != rocsparse_spmv_alg_csr_stream
        && this->b_spmv_alg != rocsparse_spmv_alg_ell
-       && this->b_spmv_alg != rocsparse_spmv_alg_coo_atomic)
+       && this->b_spmv_alg != rocsparse_spmv_alg_coo_atomic
+       && this->b_spmv_alg != rocsparse_spmv_alg_bsr
+       && this->b_spmv_alg != rocsparse_spmv_alg_csr_lrb)
   {
       std::cerr << "Invalid value for --spmv_alg" << std::endl;
       return -1;
@@ -420,10 +541,12 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
 
   if(this->b_spmm_alg != rocsparse_spmm_alg_default
        && this->b_spmm_alg != rocsparse_spmm_alg_csr
+       && this->b_spmm_alg != rocsparse_spmm_alg_bsr
        && this->b_spmm_alg != rocsparse_spmm_alg_coo_segmented
        && this->b_spmm_alg != rocsparse_spmm_alg_coo_atomic
        && this->b_spmm_alg != rocsparse_spmm_alg_csr_row_split
-       && this->b_spmm_alg != rocsparse_spmm_alg_csr_merge
+       && this->b_spmm_alg != rocsparse_spmm_alg_csr_nnz_split
+       && this->b_spmm_alg != rocsparse_spmm_alg_csr_merge_path
        && this->b_spmm_alg != rocsparse_spmm_alg_coo_segmented_atomic
        && this->b_spmm_alg != rocsparse_spmm_alg_bell)
   {
@@ -431,6 +554,13 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
       return -1;
   }
 
+  if(this->b_sddmm_alg != rocsparse_sddmm_alg_default
+       && this->b_sddmm_alg != rocsparse_sddmm_alg_dense)
+  {
+      std::cerr << "Invalid value for --sddmm_alg" << std::endl;
+      return -1;
+  }
+
   if(this->b_gtsv_interleaved_alg != rocsparse_gtsv_interleaved_alg_default
        && this->b_gtsv_interleaved_alg != rocsparse_gtsv_interleaved_alg_thomas
        && this->b_gtsv_interleaved_alg != rocsparse_gtsv_interleaved_alg_lu
@@ -440,6 +570,11 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
       return -1;
   }
 
+  if(vm.count("sparsity-pattern-statistics"))
+  {
+    this->sparsity_pattern_statistics = true;
+  }
+
   if(this->b_transA == 'N')
   {
     this->transA = rocsparse_operation_none;
@@ -487,10 +622,14 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
   this->direction
     = (this->b_dir == rocsparse_direction_row) ? rocsparse_direction_row : rocsparse_direction_column;
   this->order  = (this->b_order == rocsparse_order_row) ? rocsparse_order_row : rocsparse_order_column;
-  this->format = (rocsparse_format)this->b_format;
+  this->orderB  = (this->b_orderB == rocsparse_order_row) ? rocsparse_order_row : rocsparse_order_column;
+  this->orderC  = (this->b_orderC == rocsparse_order_row) ? rocsparse_order_row : rocsparse_order_column;
+  this->formatA = (rocsparse_format)this->b_formatA;
+  this->formatB = (rocsparse_format)this->b_formatB;
   this->spmv_alg = (rocsparse_spmv_alg)this->b_spmv_alg;
   this->itilu0_alg = (rocsparse_itilu0_alg)this->b_itilu0_alg;
   this->spmm_alg = (rocsparse_spmm_alg)this->b_spmm_alg;
+  this->sddmm_alg = (rocsparse_sddmm_alg)this->b_sddmm_alg;
   this->gtsv_interleaved_alg = (rocsparse_gtsv_interleaved_alg)this->b_gtsv_interleaved_alg;
 
 #ifdef ROCSPARSE_WITH_MEMSTAT
@@ -502,10 +641,58 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
     }
 #endif
 
-  // rocALUTION parameter overrides filename parameter
+  if(this->b_matrices_dir != "")
+  {
+    rocsparse_clients_matrices_dir_set(this->b_matrices_dir.c_str());
+  }
+
   if(this->b_file != "")
   {
     strcpy(this->filename, this->b_file.c_str());
+
+    rocsparse_importer_format_t importer_format;
+    importer_format(this->filename);
+    switch(importer_format.value)
+      {
+      case rocsparse_importer_format_t::matrixmarket:
+	{
+	  this->matrix = rocsparse_matrix_file_mtx;
+	  break;
+	}
+      case rocsparse_importer_format_t::mlcsr:
+	{
+	  this->matrix = rocsparse_matrix_file_smtx;
+	  break;
+	}
+      case rocsparse_importer_format_t::mlbsr:
+	{
+	  this->matrix = rocsparse_matrix_file_bsmtx;
+	  break;
+	}
+      case rocsparse_importer_format_t::rocalution:
+	{
+	  this->matrix = rocsparse_matrix_file_rocalution;
+	  break;
+	}
+      case rocsparse_importer_format_t::rocsparseio:
+	{
+	  this->matrix = rocsparse_matrix_file_rocsparseio;
+	  break;
+	}
+      case rocsparse_importer_format_t::unknown:
+	{
+	  std::cerr << "No extension is detected in the filename '"<< b_file <<"' " << std::endl;
+	  std::cerr << "The list of detectable extensions is : " << std::endl;
+	  for (auto format : rocsparse_importer_format_t::all_formats)
+	    {
+	      if (rocsparse_importer_format_t::unknown != format)
+		{
+		  std::cerr << " - '" << rocsparse_importer_format_t::extension(format) << "'" << std::endl;
+		}
+	    }
+	  return -1;
+	}
+      }
   }
   else if(this->b_rocsparseio != "")
   {
@@ -517,6 +704,21 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
     strcpy(this->filename, this->b_rocalution.c_str());
     this->matrix = rocsparse_matrix_file_rocalution;
   }
+  else if(this->b_matrixmarket != "")
+  {
+    strcpy(this->filename, this->b_matrixmarket.c_str());
+    this->matrix = rocsparse_matrix_file_mtx;
+  }
+  else if(this->b_mlcsr != "")
+  {
+    strcpy(this->filename, this->b_mlcsr.c_str());
+    this->matrix = rocsparse_matrix_file_smtx;
+  }
+  else if(this->b_mlbsr != "")
+  {
+    strcpy(this->filename, this->b_mlbsr.c_str());
+    this->matrix = rocsparse_matrix_file_bsmtx;
+  }
   else if(this->dimx != 0 && this->dimy != 0 && this->dimz != 0)
   {
     this->matrix = rocsparse_matrix_laplace_3d;
@@ -525,11 +727,6 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
   {
     this->matrix = rocsparse_matrix_laplace_2d;
   }
-  else if(this->b_matrixmarket != "")
-  {
-    strcpy(this->filename, this->b_matrixmarket.c_str());
-    this->matrix = rocsparse_matrix_file_mtx;
-  }
   else if(this->ll == 0 && this->l != 0 && this->u != 0 && this->uu == 0)
   {
     this->matrix = rocsparse_matrix_tridiagonal;
@@ -641,278 +838,9 @@ int rocsparse_arguments_config::parse(int&argc,char**&argv, options_description&
       }
     }
 
-  return 0;
-}
-
-int rocsparse_arguments_config::parse_no_default(int&argc,char**&argv, options_description&desc)
-{
-  variables_map vm;
-  store(parse_command_line(argc, argv, desc), vm);
-  notify(vm);
-
-  if(vm.count("help"))
-  {
-    std::cout << desc << std::endl;
-    return -2;
-  }
-
-  if(this->b_dir != rocsparse_direction_row && this->b_dir != rocsparse_direction_column)
-  {
-    std::cerr << "Invalid value for --direction" << std::endl;
-    return -1;
-  }
-
-  if(this->b_order != rocsparse_order_row && this->b_order != rocsparse_order_column)
-  {
-    std::cerr << "Invalid value for --order" << std::endl;
-    return -1;
-  }
-
-  if(this->b_format != rocsparse_format_csr && this->b_format != rocsparse_format_coo
-     && this->b_format != rocsparse_format_coo_aos && this->b_format != rocsparse_format_ell
-     && this->b_format != rocsparse_format_csc)
-  {
-    std::cerr << "Invalid value for --format" << std::endl;
-    return -1;
-  }
-
-  if(this->b_spmv_alg != rocsparse_spmv_alg_default
-       && this->b_spmv_alg != rocsparse_spmv_alg_coo
-       && this->b_spmv_alg != rocsparse_spmv_alg_csr_adaptive
-       && this->b_spmv_alg != rocsparse_spmv_alg_csr_stream
-       && this->b_spmv_alg != rocsparse_spmv_alg_ell
-       && this->b_spmv_alg != rocsparse_spmv_alg_coo_atomic)
-  {
-      std::cerr << "Invalid value for --spmv_alg" << std::endl;
-      return -1;
-  }
-
-  if(this->b_spmm_alg != rocsparse_spmm_alg_default
-       && this->b_spmm_alg != rocsparse_spmm_alg_csr
-       && this->b_spmm_alg != rocsparse_spmm_alg_coo_segmented
-       && this->b_spmm_alg != rocsparse_spmm_alg_coo_atomic
-       && this->b_spmm_alg != rocsparse_spmm_alg_csr_row_split
-       && this->b_spmm_alg != rocsparse_spmm_alg_csr_merge
-       && this->b_spmm_alg != rocsparse_spmm_alg_coo_segmented_atomic
-       && this->b_spmm_alg != rocsparse_spmm_alg_bell)
-  {
-      std::cerr << "Invalid value for --spmm_alg" << std::endl;
-      return -1;
-  }
-
-  if(this->b_gtsv_interleaved_alg != rocsparse_gtsv_interleaved_alg_default
-       && this->b_gtsv_interleaved_alg != rocsparse_gtsv_interleaved_alg_thomas
-       && this->b_gtsv_interleaved_alg != rocsparse_gtsv_interleaved_alg_lu
-       && this->b_gtsv_interleaved_alg != rocsparse_gtsv_interleaved_alg_qr)
-  {
-      std::cerr << "Invalid value for --gtsv_interleaved_alg" << std::endl;
-      return -1;
-  }
-
-  if(b_transA == 'N')
-  {
-    this->transA = rocsparse_operation_none;
-  }
-  else if(b_transA == 'T')
-  {
-    this->transA = rocsparse_operation_transpose;
-  }
-  else if(b_transA == 'C')
-  {
-    this->transA = rocsparse_operation_conjugate_transpose;
-  }
-
-  if(b_transB == 'N')
-  {
-    this->transB = rocsparse_operation_none;
-  }
-  else if(b_transB == 'T')
-  {
-    this->transB = rocsparse_operation_transpose;
-  }
-  else if(b_transB == 'C')
-  {
-    this->transB = rocsparse_operation_conjugate_transpose;
-  }
-
-  sprintf(this->function,"%s",this->function_name.c_str());
-  this->baseA = (b_baseA == 0) ? rocsparse_index_base_zero : rocsparse_index_base_one;
-  this->baseB = (b_baseB == 0) ? rocsparse_index_base_zero : rocsparse_index_base_one;
-  this->baseC = (b_baseC == 0) ? rocsparse_index_base_zero : rocsparse_index_base_one;
-  this->baseD = (b_baseD == 0) ? rocsparse_index_base_zero : rocsparse_index_base_one;
-
-  this->action = (b_action == 0) ? rocsparse_action_numeric : rocsparse_action_symbolic;
-  this->part   = (b_part == 0)   ? rocsparse_hyb_partition_auto
-    : (b_part == 1) ? rocsparse_hyb_partition_user
-    : rocsparse_hyb_partition_max;
-  this->diag   = (b_diag == 'N') ? rocsparse_diag_type_non_unit : rocsparse_diag_type_unit;
-  this->uplo   = (b_uplo == 'L') ? rocsparse_fill_mode_lower : rocsparse_fill_mode_upper;
-  this->storage= (b_storage == 0) ? rocsparse_storage_mode_sorted : rocsparse_storage_mode_unsorted;
-  this->apol   = (b_apol == 'R') ? rocsparse_analysis_policy_reuse : rocsparse_analysis_policy_force;
-  this->spol   = rocsparse_solve_policy_auto;
-  this->direction
-    = (b_dir == rocsparse_direction_row) ? rocsparse_direction_row : rocsparse_direction_column;
-  this->order  = (b_order == rocsparse_order_row) ? rocsparse_order_row : rocsparse_order_column;
-  this->format = (rocsparse_format)b_format;
-  this->spmv_alg = (rocsparse_spmv_alg)this->b_spmv_alg;
-  this->spmm_alg = (rocsparse_spmm_alg)this->b_spmm_alg;
-  this->gtsv_interleaved_alg = (rocsparse_gtsv_interleaved_alg)this->b_gtsv_interleaved_alg;
-
-  // rocALUTION parameter overrides filename parameter
-  if(b_file != "")
-  {
-    strcpy(this->filename, b_file.c_str());
-    const char * p = b_file.c_str();
-    const char * q = nullptr;
-    while(*p!='\0') { if (*p=='.') q = p; ++p;}
-    if (q==nullptr)
-      {
-	std::cerr << "extension is not detected in filename '"<< b_file <<"' " << std::endl;
-	return -1;
-      }
-    if (!strcmp(q,".mtx"))
-      {
-	this->matrix = rocsparse_matrix_file_mtx;
-      }
-    else if (!strcmp(q,".bin"))
-      {
-	this->matrix = rocsparse_matrix_file_rocsparseio;
-      }
-    else if (!strcmp(q,".csr"))
-      {
-	this->matrix = rocsparse_matrix_file_rocalution;
-      }
-  }
-  else if(b_rocsparseio != "")
-  {
-    strcpy(this->filename, b_rocsparseio.c_str());
-    this->matrix = rocsparse_matrix_file_rocsparseio;
-  }
-  else if(b_rocalution != "")
-  {
-    strcpy(this->filename, b_rocalution.c_str());
-    this->matrix = rocsparse_matrix_file_rocalution;
-  }
-  else if(this->dimx != 0 && this->dimy != 0 && this->dimz != 0)
-  {
-    this->matrix = rocsparse_matrix_laplace_3d;
-  }
-  else if(this->dimx != 0 && this->dimy != 0)
-  {
-    this->matrix = rocsparse_matrix_laplace_2d;
-  }
-  else if(b_matrixmarket != "")
-  {
-    strcpy(this->filename, b_matrixmarket.c_str());
-    this->matrix = rocsparse_matrix_file_mtx;
-  }
-  else
-  {
-    this->matrix = rocsparse_matrix_random;
-  }
-
-  this->matrix_init_kind = rocsparse_matrix_init_kind_default;
-  /* ============================================================================================
-   */
-  if(this->M < 0 || this->N < 0)
-  {
-    std::cerr << "Invalid dimension" << std::endl;
-    return -1;
-  }
-
-  if(this->block_dim < 1)
-  {
-    std::cerr << "Invalid value for --blockdim" << std::endl;
-    return -1;
-  }
-
-  if(this->row_block_dimA < 1)
-  {
-    std::cerr << "Invalid value for --row-blockdimA" << std::endl;
-    return -1;
-  }
-
-  if(this->col_block_dimA < 1)
-  {
-    std::cerr << "Invalid value for --col-blockdimA" << std::endl;
-    return -1;
-  }
-
-  if(this->row_block_dimB < 1)
-  {
-    std::cerr << "Invalid value for --row-blockdimB" << std::endl;
-    return -1;
-  }
-
-  if(this->col_block_dimB < 1)
-  {
-    std::cerr << "Invalid value for --col-blockdimB" << std::endl;
-    return -1;
-  }
-
-
-
-  switch(this->indextype)
-    {
-    case 's':
-      {
-	this->index_type_I   = rocsparse_indextype_i32;
-	this->index_type_J   = rocsparse_indextype_i32;
-	break;
-      }
-    case 'd':
-      {
-	this->index_type_I   = rocsparse_indextype_i64;
-	this->index_type_J   = rocsparse_indextype_i64;
-	break;
-      }
-
-    case 'm':
-      {
-	this->index_type_I   = rocsparse_indextype_i64;
-	this->index_type_J   = rocsparse_indextype_i32;
-	break;
-      }
-    default:
-      {
-	std::cerr << "Invalid value for --indextype" << std::endl;
-	return -1;
-      }
-    }
-
-  switch(this->precision)
-    {
-    case 's':
-      {
-	this->compute_type = rocsparse_datatype_f32_r;
-  break;
-      }
-    case 'd':
-      {
-	this->compute_type = rocsparse_datatype_f64_r;
-	break;
-      }
-
-    case 'c':
-      {
-	this->compute_type = rocsparse_datatype_f32_c;
-	break;
-      }
-    case 'z':
-      {
-	this->compute_type = rocsparse_datatype_f64_c;
-	break;
-      }
-    default:
-      {
-	std::cerr << "Invalid value for --precision" << std::endl;
-	return -1;
-      }
-    }
-
+  this->A_row_indextype = this->index_type_I;
+  this->A_col_indextype = this->index_type_J;
 
   return 0;
 }
 
-
-
