@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,8 +27,6 @@
 template <typename X, typename Y>
 rocsparse_status rocsparse_type_conversion(const X& x, Y& y);
 
-#ifdef ROCSPARSEIO
-
 template <typename X, typename Y>
 inline rocsparse_status rocsparse2rocsparseio_convert(const X& x, Y& y);
 
@@ -125,11 +123,9 @@ inline rocsparseio_type rocsparseio_type_convert<rocsparse_double_complex>()
 {
     return rocsparseio_type_complex64;
 };
-#endif
 
 rocsparse_exporter_rocsparseio::~rocsparse_exporter_rocsparseio()
 {
-#ifdef ROCSPARSEIO
     const char* env = getenv("GTEST_LISTENER");
     if(!env || strcmp(env, "NO_PASS_LINE_IN_LOG"))
     {
@@ -140,13 +136,12 @@ rocsparse_exporter_rocsparseio::~rocsparse_exporter_rocsparseio()
     if(istatus != rocsparseio_status_success)
     {
     }
-#endif
 }
 
 rocsparse_exporter_rocsparseio::rocsparse_exporter_rocsparseio(const std::string& filename_)
     : m_filename(filename_)
 {
-#ifdef ROCSPARSEIO
+
     const char* env = getenv("GTEST_LISTENER");
     if(!env || strcmp(env, "NO_PASS_LINE_IN_LOG"))
     {
@@ -160,9 +155,6 @@ rocsparse_exporter_rocsparseio::rocsparse_exporter_rocsparseio(const std::string
         std::cerr << "Problem with rocsparseio_open" << std::endl;
         throw rocsparse_status_internal_error;
     }
-#else
-    throw rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I, typename J>
@@ -175,15 +167,14 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_sparse_csx(rocsparse_dire
                                                                   const T* __restrict__ val_,
                                                                   rocsparse_index_base base_)
 {
-#ifdef ROCSPARSEIO
     const rocsparseio_type ptr_type = rocsparseio_type_convert<I>();
     const rocsparseio_type ind_type = rocsparseio_type_convert<J>();
     const rocsparseio_type val_type = rocsparseio_type_convert<T>();
 
     rocsparseio_direction  dir;
-    size_t                 m;
-    size_t                 n;
-    size_t                 nnz;
+    uint64_t               m;
+    uint64_t               n;
+    uint64_t               nnz;
     rocsparseio_index_base base;
 
     rocsparse_status status;
@@ -220,17 +211,14 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_sparse_csx(rocsparse_dire
         return status;
     }
 
+    const char*        name      = nullptr;
     rocsparseio_status io_status = rocsparseio_write_sparse_csx(
-        this->m_handle, dir, m, n, nnz, ptr_type, ptr_, ind_type, ind_, val_type, val_, base);
+        this->m_handle, dir, m, n, nnz, ptr_type, ptr_, ind_type, ind_, val_type, val_, base, name);
     if(io_status != rocsparseio_status_success)
     {
         return rocsparse_status_internal_error;
     }
     return rocsparse_status_success;
-
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I, typename J>
@@ -246,18 +234,18 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_sparse_gebsx(rocsparse_di
                                                                     const T* __restrict__ val_,
                                                                     rocsparse_index_base base_)
 {
-#ifdef ROCSPARSEIO
+
     const rocsparseio_type ptr_type = rocsparseio_type_convert<I>();
     const rocsparseio_type ind_type = rocsparseio_type_convert<J>();
     const rocsparseio_type val_type = rocsparseio_type_convert<T>();
 
     rocsparseio_direction  dir;
     rocsparseio_direction  dirb;
-    size_t                 mb;
-    size_t                 nb;
-    size_t                 nnzb;
-    size_t                 block_dim_row;
-    size_t                 block_dim_column;
+    uint64_t               mb;
+    uint64_t               nb;
+    uint64_t               nnzb;
+    uint64_t               block_dim_row;
+    uint64_t               block_dim_column;
     rocsparseio_index_base base;
 
     rocsparse_status status;
@@ -313,6 +301,8 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_sparse_gebsx(rocsparse_di
     {
         return status;
     }
+
+    const char*        name      = nullptr;
     rocsparseio_status io_status = rocsparseio_write_sparse_gebsx(this->m_handle,
                                                                   dir,
                                                                   dirb,
@@ -327,25 +317,22 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_sparse_gebsx(rocsparse_di
                                                                   ind_,
                                                                   val_type,
                                                                   val_,
-                                                                  base);
+                                                                  base,
+                                                                  name);
     if(io_status != rocsparseio_status_success)
     {
         return rocsparse_status_internal_error;
     }
     return rocsparse_status_success;
-
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I>
 rocsparse_status
     rocsparse_exporter_rocsparseio::write_dense_vector(I nmemb_, const T* __restrict__ x_, I incx_)
 {
-#ifdef ROCSPARSEIO
+
     const rocsparseio_type val_type = rocsparseio_type_convert<T>();
-    size_t                 nmemb, incx;
+    uint64_t               nmemb, incx;
     rocsparse_status       status;
     status = rocsparse_type_conversion(nmemb_, nmemb);
     if(status != rocsparse_status_success)
@@ -359,25 +346,23 @@ rocsparse_status
         return status;
     }
 
+    const char*        name = nullptr;
     rocsparseio_status io_status
-        = rocsparseio_write_dense_vector(this->m_handle, val_type, nmemb, x_, incx);
+        = rocsparseio_write_dense_vector(this->m_handle, val_type, nmemb, x_, incx, name);
     if(io_status != rocsparseio_status_success)
     {
         return rocsparse_status_internal_error;
     }
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I>
 rocsparse_status rocsparse_exporter_rocsparseio::write_dense_matrix(
     rocsparse_order order_, I m_, I n_, const T* __restrict__ x_, I ld_)
 {
-#ifdef ROCSPARSEIO
+
     rocsparseio_order      order;
-    size_t                 m, n, ld;
+    uint64_t               m, n, ld;
     rocsparse_status       status;
     const rocsparseio_type val_type = rocsparseio_type_convert<T>();
 
@@ -407,17 +392,14 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_dense_matrix(
         return status;
     }
 
+    const char*        name = nullptr;
     rocsparseio_status io_status
-        = rocsparseio_write_dense_matrix(this->m_handle, order, m, n, val_type, x_, ld);
+        = rocsparseio_write_dense_matrix(this->m_handle, order, m, n, val_type, x_, ld, name);
     if(io_status != rocsparseio_status_success)
     {
         return rocsparse_status_internal_error;
     }
     return rocsparse_status_success;
-
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 template <typename T, typename I>
@@ -429,11 +411,11 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_sparse_coo(I m_,
                                                                   const T* __restrict__ val_,
                                                                   rocsparse_index_base base_)
 {
-#ifdef ROCSPARSEIO
+
     const rocsparseio_type ind_type = rocsparseio_type_convert<I>();
     const rocsparseio_type val_type = rocsparseio_type_convert<T>();
 
-    size_t                 m, n, nnz;
+    uint64_t               m, n, nnz;
     rocsparseio_index_base base;
 
     rocsparse_status status;
@@ -462,16 +444,24 @@ rocsparse_status rocsparse_exporter_rocsparseio::write_sparse_coo(I m_,
         return status;
     }
 
-    rocsparseio_status io_status = rocsparseio_write_sparse_coo(
-        this->m_handle, m, n, nnz, ind_type, row_ind_, ind_type, col_ind_, val_type, val_, base);
+    const char*        name      = nullptr;
+    rocsparseio_status io_status = rocsparseio_write_sparse_coo(this->m_handle,
+                                                                m,
+                                                                n,
+                                                                nnz,
+                                                                ind_type,
+                                                                row_ind_,
+                                                                ind_type,
+                                                                col_ind_,
+                                                                val_type,
+                                                                val_,
+                                                                base,
+                                                                name);
     if(io_status != rocsparseio_status_success)
     {
         return rocsparse_status_internal_error;
     }
     return rocsparse_status_success;
-#else
-    return rocsparse_status_not_implemented;
-#endif
 }
 
 #define INSTANTIATE_TIJ(T, I, J)                                                  \
