@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,24 +24,28 @@
 
 #include "rocsparse_matrix_factory.hpp"
 #include "rocsparse_clients_envariables.hpp"
+#include "rocsparse_clients_matrices_dir.hpp"
+#include "rocsparse_importer_format_t.hpp"
 #include "rocsparse_init.hpp"
 
-static void get_matrix_full_filename(std::string&       full_filename_,
-                                     const std::string& filename_,
-                                     const std::string& extension_,
-                                     const bool         timing_)
+static void get_matrix_full_filename(const Arguments& arg_,
+                                     const char*      extension_,
+                                     std::string&     full_filename_)
 {
-    const bool envar_is_defined
-        = rocsparse_clients_envariables::is_defined(rocsparse_clients_envariables::MATRICES_DIR);
-    std::string path = rocsparse_exepath() + "../matrices/";
-    if(envar_is_defined)
-    {
-        path = rocsparse_clients_envariables::get(rocsparse_clients_envariables::MATRICES_DIR);
-        path += "/";
-    }
 
-    full_filename_ = timing_ ? ((envar_is_defined) ? (path + filename_) : filename_)
-                             : (path + filename_ + extension_);
+    if(arg_.timing)
+    {
+        static constexpr bool use_default = false;
+        full_filename_                    = rocsparse_clients_matrices_dir_get(use_default);
+        full_filename_ += arg_.filename;
+    }
+    else
+    {
+        static constexpr bool use_default = true;
+        full_filename_                    = rocsparse_clients_matrices_dir_get(use_default);
+        full_filename_ += arg_.filename;
+        full_filename_ += extension_;
+    }
 }
 
 //
@@ -116,7 +120,10 @@ rocsparse_matrix_factory<T, I, J>::rocsparse_matrix_factory(const Arguments&
     case rocsparse_matrix_file_rocalution:
     {
         std::string full_filename;
-        get_matrix_full_filename(full_filename, arg.filename, ".csr", arg.timing);
+        get_matrix_full_filename(
+            arg,
+            rocsparse_importer_format_t::extension(rocsparse_importer_format_t::rocalution),
+            full_filename);
 
         this->m_instance
             = new rocsparse_matrix_factory_rocalution<T, I, J>(full_filename.c_str(), to_int);
@@ -126,7 +133,10 @@ rocsparse_matrix_factory<T, I, J>::rocsparse_matrix_factory(const Arguments&
     case rocsparse_matrix_file_rocsparseio:
     {
         std::string full_filename;
-        get_matrix_full_filename(full_filename, arg.filename, ".bin", arg.timing);
+        get_matrix_full_filename(
+            arg,
+            rocsparse_importer_format_t::extension(rocsparse_importer_format_t::rocsparseio),
+            full_filename);
         this->m_instance
             = new rocsparse_matrix_factory_rocsparseio<T, I, J>(full_filename.c_str(), to_int);
         break;
@@ -135,24 +145,49 @@ rocsparse_matrix_factory<T, I, J>::rocsparse_matrix_factory(const Arguments&
     case rocsparse_matrix_file_mtx:
     {
         std::string full_filename;
-        get_matrix_full_filename(full_filename, arg.filename, ".mtx", arg.timing);
+        get_matrix_full_filename(
+            arg,
+            rocsparse_importer_format_t::extension(rocsparse_importer_format_t::matrixmarket),
+            full_filename);
         this->m_instance = new rocsparse_matrix_factory_mtx<T, I, J>(full_filename.c_str());
         break;
     }
 
+    case rocsparse_matrix_file_smtx:
+    {
+        std::string full_filename;
+        get_matrix_full_filename(
+            arg,
+            rocsparse_importer_format_t::extension(rocsparse_importer_format_t::mlcsr),
+            full_filename);
+        this->m_instance = new rocsparse_matrix_factory_smtx<T, I, J>(full_filename.c_str());
+        break;
+    }
+
+    case rocsparse_matrix_file_bsmtx:
+    {
+        std::string full_filename;
+        get_matrix_full_filename(
+            arg,
+            rocsparse_importer_format_t::extension(rocsparse_importer_format_t::mlbsr),
+            full_filename);
+        this->m_instance = new rocsparse_matrix_factory_bsmtx<T, I, J>(full_filename.c_str());
+        break;
+    }
+
     case rocsparse_matrix_zero:
     {
         this->m_instance = new rocsparse_matrix_factory_zero<T, I, J>();
         break;
     }
+    }
 
-    default:
+    if(this->m_instance == nullptr)
     {
-        this->m_instance = nullptr;
-        break;
+        std::cerr << "rocsparse_matrix_factory constructor failed, at line " << __LINE__
+                  << std::endl;
+        throw(1);
     }
-    }
-    assert(this->m_instance != nullptr);
 }
 
 //
@@ -843,12 +878,12 @@ struct traits_init_hyb<T,
         nnz = hA.nnz;
 
         // ELL width limit
-        rocsparse_int width_limit = 2 * (hA.nnz - 1) / M + 1;
+        rocsparse_int width_limit = (M != 0) ? (2 * (hA.nnz - 1) / M + 1) : 0;
 
         // Limit ELL user width
         if(part == rocsparse_hyb_partition_user)
         {
-            user_ell_width *= (hA.nnz / M);
+            user_ell_width *= (M != 0) ? (hA.nnz / M) : 0;
             user_ell_width = std::min(width_limit, user_ell_width);
         }
 
