@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -78,6 +78,18 @@ struct rocsparse_init_file_traits<rocsparse_matrix_file_mtx>
     using importer_t = rocsparse_importer_matrixmarket;
 };
 
+template <>
+struct rocsparse_init_file_traits<rocsparse_matrix_file_smtx>
+{
+    using importer_t = rocsparse_importer_mlcsr;
+};
+
+template <>
+struct rocsparse_init_file_traits<rocsparse_matrix_file_bsmtx>
+{
+    using importer_t = rocsparse_importer_mlbsr;
+};
+
 template <>
 struct rocsparse_init_file_traits<rocsparse_matrix_file_rocsparseio>
 {
@@ -141,11 +153,11 @@ struct spec
             bsr_row_ptr, bsr_col_ind, bsr_val, Mb, Nb, nnzb, base, matrix_type, uplo, storage);
 
         // Then temporarily skip the values.
-        I nvalues = nnzb * row_block_dim * col_block_dim;
+        size_t nvalues = size_t(nnzb) * row_block_dim * col_block_dim;
         bsr_val.resize(nvalues);
         for(I i = 0; i < nvalues; ++i)
         {
-            bsr_val[i] = random_generator<T>();
+            bsr_val[i] = random_cached_generator<T>();
         }
     }
 };
@@ -259,6 +271,34 @@ void rocsparse_matrix_factory_file<MATRIX_INIT, T, I, J>::init_gebsr(
                                  base);
         break;
     }
+    case rocsparse_matrix_file_smtx:
+    {
+        rocsparse_init_gebsr_smtx(this->m_filename.c_str(),
+                                  bsr_row_ptr,
+                                  bsr_col_ind,
+                                  bsr_val,
+                                  Mb,
+                                  Nb,
+                                  nnzb,
+                                  row_block_dim,
+                                  col_block_dim,
+                                  base);
+        break;
+    }
+    case rocsparse_matrix_file_bsmtx:
+    {
+        rocsparse_init_gebsr_bsmtx(this->m_filename.c_str(),
+                                   bsr_row_ptr,
+                                   bsr_col_ind,
+                                   bsr_val,
+                                   Mb,
+                                   Nb,
+                                   nnzb,
+                                   row_block_dim,
+                                   col_block_dim,
+                                   base);
+        break;
+    }
     case rocsparse_matrix_file_rocalution:
     {
         spec<T, I, J>::init_gebsr_rocalution(*this,
@@ -351,6 +391,16 @@ void rocsparse_matrix_factory_file<MATRIX_INIT, T, I, J>::init_csr(
         rocsparse_init_csr_mtx(this->m_filename.c_str(), row_ptr, col_ind, val, M, N, nnz, base);
         break;
     }
+    case rocsparse_matrix_file_smtx:
+    {
+        rocsparse_init_csr_smtx(this->m_filename.c_str(), row_ptr, col_ind, val, M, N, nnz, base);
+        break;
+    }
+    case rocsparse_matrix_file_bsmtx:
+    {
+        rocsparse_init_csr_bsmtx(this->m_filename.c_str(), row_ptr, col_ind, val, M, N, nnz, base);
+        break;
+    }
     }
 
     switch(matrix_type)
@@ -438,6 +488,20 @@ void rocsparse_matrix_factory_file<MATRIX_INIT, T, I, J>::init_coo(
         break;
     }
 
+    case rocsparse_matrix_file_smtx:
+    {
+        rocsparse_init_coo_smtx(this->m_filename.c_str(), row_ind, col_ind, val, M, N, nnz, base);
+
+        break;
+    }
+
+    case rocsparse_matrix_file_bsmtx:
+    {
+        rocsparse_init_coo_bsmtx(this->m_filename.c_str(), row_ind, col_ind, val, M, N, nnz, base);
+
+        break;
+    }
+
     case rocsparse_matrix_file_rocsparseio:
     {
         rocsparse_init_coo_rocsparseio(
@@ -661,3 +725,97 @@ template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocsparseio,
                                               rocsparse_double_complex,
                                               int64_t,
                                               int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, int8_t, int32_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, int8_t, int64_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, int8_t, int64_t, int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, float, int32_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, float, int64_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, float, int64_t, int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, double, int32_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, double, int64_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, double, int64_t, int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_float_complex,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_float_complex,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_float_complex,
+                                              int64_t,
+                                              int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_double_complex,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_double_complex,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_double_complex,
+                                              int64_t,
+                                              int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              int8_t,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              int8_t,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              int8_t,
+                                              int64_t,
+                                              int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx, float, int32_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx, float, int64_t, int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx, float, int64_t, int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              double,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              double,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              double,
+                                              int64_t,
+                                              int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_float_complex,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_float_complex,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_float_complex,
+                                              int64_t,
+                                              int64_t>;
+
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_double_complex,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_double_complex,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_double_complex,
+                                              int64_t,
+                                              int64_t>;
