@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -70,6 +70,15 @@ struct Arguments
     rocsparse_datatype y_type;
     rocsparse_datatype compute_type;
 
+    rocsparse_indextype A_row_indextype;
+    rocsparse_indextype A_col_indextype;
+
+    rocsparse_indextype B_row_indextype;
+    rocsparse_indextype B_col_indextype;
+
+    rocsparse_indextype C_row_indextype;
+    rocsparse_indextype C_col_indextype;
+
     double alpha;
     double alphai;
     double beta;
@@ -93,7 +102,10 @@ struct Arguments
     rocsparse_solve_policy         spol;
     rocsparse_direction            direction;
     rocsparse_order                order;
-    rocsparse_format               format;
+    rocsparse_order                orderB;
+    rocsparse_order                orderC;
+    rocsparse_format               formatA;
+    rocsparse_format               formatB;
     rocsparse_itilu0_alg           itilu0_alg;
     rocsparse_sddmm_alg            sddmm_alg;
     rocsparse_spmv_alg             spmv_alg;
@@ -113,13 +125,17 @@ struct Arguments
     rocsparse_int unit_check;
     rocsparse_int timing;
     rocsparse_int iters;
+    rocsparse_int nfreeiter;
+    rocsparse_int nmaxiter;
 
-    rocsparse_int denseld;
+    int64_t       denseld;
     rocsparse_int batch_count;
     rocsparse_int batch_count_A;
     rocsparse_int batch_count_B;
     rocsparse_int batch_count_C;
     rocsparse_int batch_stride;
+    rocsparse_int ld_multiplier_B;
+    rocsparse_int ld_multiplier_C;
 
     uint32_t algo;
 
@@ -131,12 +147,16 @@ struct Arguments
     double tolm;
 
     bool graph_test;
-
+    bool skip_reproducibility;
+    bool sparsity_pattern_statistics;
     char filename[128];
     char function[64];
     char name[64];
     char category[32];
     char hardware[32];
+    char skip_hardware[32];
+
+    uint32_t req_memory;
 
     // Validate input format.
     // rocsparse_gentest.py is expected to conform to this format.
@@ -163,11 +183,11 @@ struct Arguments
         else if(strcmp(trailer, "ROCsparse"))
             error("trailer");
 
-        auto check_func = [&, sig = (unsigned char)0](const auto& elem, auto name) mutable {
+        auto check_func = [&, sig = (uint8_t)0](const auto& elem, auto name) mutable {
             static_assert(sizeof(elem) <= 255,
                           "One of the fields of Arguments is too large (> 255 bytes)");
-            for(unsigned char i = 0; i < sizeof(elem); ++i)
-                if(reinterpret_cast<const unsigned char*>(&elem)[i] ^ sig ^ i)
+            for(uint8_t i = 0; i < sizeof(elem); ++i)
+                if(reinterpret_cast<const uint8_t*>(&elem)[i] ^ sig ^ i)
                     error(name);
             sig += 89;
         };
@@ -199,6 +219,12 @@ struct Arguments
         ROCSPARSE_FORMAT_CHECK(x_type);
         ROCSPARSE_FORMAT_CHECK(y_type);
         ROCSPARSE_FORMAT_CHECK(compute_type);
+        ROCSPARSE_FORMAT_CHECK(A_row_indextype);
+        ROCSPARSE_FORMAT_CHECK(A_col_indextype);
+        ROCSPARSE_FORMAT_CHECK(B_row_indextype);
+        ROCSPARSE_FORMAT_CHECK(B_col_indextype);
+        ROCSPARSE_FORMAT_CHECK(C_row_indextype);
+        ROCSPARSE_FORMAT_CHECK(C_col_indextype);
         ROCSPARSE_FORMAT_CHECK(alpha);
         ROCSPARSE_FORMAT_CHECK(alphai);
         ROCSPARSE_FORMAT_CHECK(beta);
@@ -221,7 +247,10 @@ struct Arguments
         ROCSPARSE_FORMAT_CHECK(spol);
         ROCSPARSE_FORMAT_CHECK(direction);
         ROCSPARSE_FORMAT_CHECK(order);
-        ROCSPARSE_FORMAT_CHECK(format);
+        ROCSPARSE_FORMAT_CHECK(orderB);
+        ROCSPARSE_FORMAT_CHECK(orderC);
+        ROCSPARSE_FORMAT_CHECK(formatA);
+        ROCSPARSE_FORMAT_CHECK(formatB);
         ROCSPARSE_FORMAT_CHECK(itilu0_alg);
         ROCSPARSE_FORMAT_CHECK(sddmm_alg);
         ROCSPARSE_FORMAT_CHECK(spmv_alg);
@@ -239,12 +268,16 @@ struct Arguments
         ROCSPARSE_FORMAT_CHECK(unit_check);
         ROCSPARSE_FORMAT_CHECK(timing);
         ROCSPARSE_FORMAT_CHECK(iters);
+        ROCSPARSE_FORMAT_CHECK(nfreeiter);
+        ROCSPARSE_FORMAT_CHECK(nmaxiter);
         ROCSPARSE_FORMAT_CHECK(denseld);
         ROCSPARSE_FORMAT_CHECK(batch_count);
         ROCSPARSE_FORMAT_CHECK(batch_count_A);
         ROCSPARSE_FORMAT_CHECK(batch_count_B);
         ROCSPARSE_FORMAT_CHECK(batch_count_C);
         ROCSPARSE_FORMAT_CHECK(batch_stride);
+        ROCSPARSE_FORMAT_CHECK(ld_multiplier_B);
+        ROCSPARSE_FORMAT_CHECK(ld_multiplier_C);
         ROCSPARSE_FORMAT_CHECK(algo);
         ROCSPARSE_FORMAT_CHECK(numericboost);
         ROCSPARSE_FORMAT_CHECK(boosttol);
@@ -252,11 +285,15 @@ struct Arguments
         ROCSPARSE_FORMAT_CHECK(boostvali);
         ROCSPARSE_FORMAT_CHECK(tolm);
         ROCSPARSE_FORMAT_CHECK(graph_test);
+        ROCSPARSE_FORMAT_CHECK(skip_reproducibility);
+        ROCSPARSE_FORMAT_CHECK(sparsity_pattern_statistics);
         ROCSPARSE_FORMAT_CHECK(filename);
         ROCSPARSE_FORMAT_CHECK(function);
         ROCSPARSE_FORMAT_CHECK(name);
         ROCSPARSE_FORMAT_CHECK(category);
         ROCSPARSE_FORMAT_CHECK(hardware);
+        ROCSPARSE_FORMAT_CHECK(skip_hardware);
+        ROCSPARSE_FORMAT_CHECK(req_memory);
     }
 
     template <typename T>
@@ -389,6 +426,12 @@ private:
         print("x_type", rocsparse_datatype2string(arg.x_type));
         print("y_type", rocsparse_datatype2string(arg.y_type));
         print("compute_type", rocsparse_datatype2string(arg.compute_type));
+        print("A_row_indextype", rocsparse_indextype2string(arg.A_row_indextype));
+        print("A_col_indextype", rocsparse_indextype2string(arg.A_col_indextype));
+        print("B_row_indextype", rocsparse_indextype2string(arg.B_row_indextype));
+        print("B_col_indextype", rocsparse_indextype2string(arg.B_col_indextype));
+        print("C_row_indextype", rocsparse_indextype2string(arg.C_row_indextype));
+        print("C_col_indextype", rocsparse_indextype2string(arg.C_col_indextype));
         print("transA", rocsparse_operation2string(arg.transA));
         print("transB", rocsparse_operation2string(arg.transB));
         print("baseA", rocsparse_indexbase2string(arg.baseA));
@@ -427,7 +470,10 @@ private:
         print("solve_policy", rocsparse_solve2string(arg.spol));
         print("direction", rocsparse_direction2string(arg.direction));
         print("order", rocsparse_order2string(arg.order));
-        print("format", rocsparse_format2string(arg.format));
+        print("orderB", rocsparse_order2string(arg.orderB));
+        print("orderC", rocsparse_order2string(arg.orderC));
+        print("formatA", rocsparse_format2string(arg.formatA));
+        print("formatB", rocsparse_format2string(arg.formatB));
         print("itilu0_alg", rocsparse_itilu0alg2string(arg.itilu0_alg));
         print("sddmm_alg", rocsparse_sddmmalg2string(arg.sddmm_alg));
         print("spmv_alg", rocsparse_spmvalg2string(arg.spmv_alg));
@@ -451,18 +497,26 @@ private:
         print("boost_vali", arg.boostvali);
         print("tolm", arg.tolm);
         print("graph_test", arg.graph_test);
+        print("skip_reproducibility", arg.skip_reproducibility);
+        print("sparsity_pattern_statistics", arg.sparsity_pattern_statistics);
         print("name", arg.name);
         print("category", arg.category);
         print("hardware", arg.hardware);
+        print("skip_hardware", arg.skip_hardware);
+        print("req_memory", arg.req_memory);
         print("unit_check", arg.unit_check);
         print("timing", arg.timing);
         print("iters", arg.iters);
+        print("nfreeiter", arg.nfreeiter);
+        print("nmaxiter", arg.nmaxiter);
         print("denseld", arg.denseld);
         print("batch_count", arg.batch_count);
         print("batch_count_A", arg.batch_count_A);
         print("batch_count_B", arg.batch_count_B);
         print("batch_count_C", arg.batch_count_C);
         print("batch_stride", arg.batch_stride);
+        print("ld_multiplier_B", arg.ld_multiplier_B);
+        print("ld_multiplier_C", arg.ld_multiplier_C);
         return str << " }\n";
     }
 };
@@ -479,6 +533,8 @@ inline bool rocsparse_arguments_has_datafile(const Arguments& arg)
 {
     return (arg.matrix == rocsparse_matrix_file_rocalution)
            || (arg.matrix == rocsparse_matrix_file_mtx)
+           || (arg.matrix == rocsparse_matrix_file_smtx)
+           || (arg.matrix == rocsparse_matrix_file_bsmtx)
            || (arg.matrix == rocsparse_matrix_file_rocsparseio);
 }
 
