@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -49,9 +49,9 @@ struct rocsparse_host
                         I                    nnz,
                         const T*             alpha,
                         const T*             A,
-                        J                    lda,
+                        int64_t              lda,
                         const T*             B,
-                        J                    ldb,
+                        int64_t              ldb,
                         const T*             beta,
                         const I*             csr_row_ptr_C,
                         const J*             csr_col_ind_C,
@@ -68,9 +68,9 @@ struct rocsparse_host
                         I                    nnz,
                         const T*             alpha,
                         const T*             A,
-                        J                    lda,
+                        int64_t              lda,
                         const T*             B,
-                        J                    ldb,
+                        int64_t              ldb,
                         const T*             beta,
                         const I*             csr_row_ptr_C,
                         const J*             csr_col_ind_C,
@@ -87,9 +87,9 @@ struct rocsparse_host
                         I                    nnz,
                         const T*             alpha,
                         const T*             A,
-                        J                    lda,
+                        int64_t              lda,
                         const T*             B,
-                        J                    ldb,
+                        int64_t              ldb,
                         const T*             beta,
                         const I*             coo_row_ind_C,
                         const I*             coo_col_ind_C,
@@ -106,9 +106,9 @@ struct rocsparse_host
                            I                    nnz,
                            const T*             alpha,
                            const T*             A,
-                           J                    lda,
+                           int64_t              lda,
                            const T*             B,
-                           J                    ldb,
+                           int64_t              ldb,
                            const T*             beta,
                            const I*             coo_row_ind_C,
                            const I*             coo_col_ind_C,
@@ -125,9 +125,9 @@ struct rocsparse_host
                         I                    nnz,
                         const T*             alpha,
                         const T*             A,
-                        J                    lda,
+                        int64_t              lda,
                         const T*             B,
-                        J                    ldb,
+                        int64_t              ldb,
                         const T*             beta,
                         const J              ell_width,
                         const I*             ell_ind_C,
@@ -156,12 +156,12 @@ void host_axpby(I                    size,
                 T*                   y,
                 rocsparse_index_base base);
 
-template <typename I, typename T>
+template <typename I, typename X, typename Y, typename T>
 void host_doti(
-    I nnz, const T* x_val, const I* x_ind, const T* y, T* result, rocsparse_index_base base);
-template <typename I, typename T>
+    I nnz, const X* x_val, const I* x_ind, const Y* y, T* result, rocsparse_index_base base);
+template <typename I, typename X, typename Y, typename T>
 void host_dotci(
-    I nnz, const T* x_val, const I* x_ind, const T* y, T* result, rocsparse_index_base base);
+    I nnz, const X* x_val, const I* x_ind, const Y* y, T* result, rocsparse_index_base base);
 template <typename I, typename T>
 void host_gthr(I nnz, const T* y, T* x_val, const I* x_ind, rocsparse_index_base base);
 template <typename T>
@@ -180,6 +180,23 @@ void host_sctr(I nnz, const T* x_val, const I* x_ind, T* y, rocsparse_index_base
  * ===========================================================================
  */
 template <typename T, typename I, typename J, typename A, typename X, typename Y>
+void host_bsrmv(rocsparse_direction  dir,
+                rocsparse_operation  trans,
+                J                    mb,
+                J                    nb,
+                I                    nnzb,
+                T                    alpha,
+                const I*             bsr_row_ptr,
+                const I*             bsr_end_ptr,
+                const J*             bsr_col_ind,
+                const A*             bsr_val,
+                J                    bsr_dim,
+                const X*             x,
+                T                    beta,
+                Y*                   y,
+                rocsparse_index_base base);
+
+template <typename T, typename I, typename J, typename A, typename X, typename Y>
 void host_bsrmv(rocsparse_direction  dir,
                 rocsparse_operation  trans,
                 J                    mb,
@@ -301,6 +318,7 @@ void host_csrsv(rocsparse_operation  trans,
                 const J*             csr_col_ind,
                 const T*             csr_val,
                 const T*             x,
+                int64_t              x_inc,
                 T*                   y,
                 rocsparse_diag_type  diag_type,
                 rocsparse_fill_mode  fill_mode,
@@ -309,20 +327,20 @@ void host_csrsv(rocsparse_operation  trans,
                 J*                   numeric_pivot);
 
 template <typename I, typename T>
-void host_coosv(rocsparse_operation   trans,
-                I                     M,
-                int64_t               nnz,
-                T                     alpha,
-                const std::vector<I>& coo_row_ind,
-                const std::vector<I>& coo_col_ind,
-                const std::vector<T>& coo_val,
-                const std::vector<T>& x,
-                std::vector<T>&       y,
-                rocsparse_diag_type   diag_type,
-                rocsparse_fill_mode   fill_mode,
-                rocsparse_index_base  base,
-                I*                    struct_pivot,
-                I*                    numeric_pivot);
+void host_coosv(rocsparse_operation  trans,
+                I                    M,
+                int64_t              nnz,
+                T                    alpha,
+                const I*             coo_row_ind,
+                const I*             coo_col_ind,
+                const T*             coo_val,
+                const T*             x,
+                T*                   y,
+                rocsparse_diag_type  diag_type,
+                rocsparse_fill_mode  fill_mode,
+                rocsparse_index_base base,
+                I*                   struct_pivot,
+                I*                   numeric_pivot);
 
 template <typename T, typename I, typename A, typename X, typename Y>
 void host_ellmv(rocsparse_operation  trans,
@@ -377,7 +395,7 @@ void host_gemvi(I                    M,
                 I                    N,
                 T                    alpha,
                 const T*             A,
-                I                    lda,
+                int64_t              lda,
                 I                    nnz,
                 const T*             x_val,
                 const I*             x_ind,
@@ -389,26 +407,28 @@ void host_gemvi(I                    M,
  *    level 3 SPARSE
  * ===========================================================================
  */
-template <typename T>
-void host_bsrmm(rocsparse_handle          handle,
-                rocsparse_direction       dir,
-                rocsparse_operation       trans_A,
-                rocsparse_operation       trans_B,
-                rocsparse_int             mb,
-                rocsparse_int             n,
-                rocsparse_int             kb,
-                rocsparse_int             nnzb,
-                const T*                  alpha,
-                const rocsparse_mat_descr descr,
-                const T*                  bsr_val,
-                const rocsparse_int*      bsr_row_ptr,
-                const rocsparse_int*      bsr_col_ind,
-                rocsparse_int             block_dim,
-                const T*                  B,
-                rocsparse_int             ldb,
-                const T*                  beta,
-                T*                        C,
-                rocsparse_int             ldc);
+template <typename T, typename I, typename J, typename A, typename B, typename C>
+void host_bsrmm(rocsparse_handle     handle,
+                rocsparse_direction  dir,
+                rocsparse_operation  trans_A,
+                rocsparse_operation  trans_B,
+                J                    mb,
+                J                    n,
+                J                    kb,
+                I                    nnzb,
+                T                    alpha,
+                const A*             bsr_val,
+                const I*             bsr_row_ptr,
+                const J*             bsr_col_ind,
+                J                    block_dim,
+                const B*             dense_B,
+                int64_t              ldb,
+                rocsparse_order      order_B,
+                T                    beta,
+                C*                   dense_C,
+                int64_t              ldc,
+                rocsparse_order      order_C,
+                rocsparse_index_base base);
 
 template <typename T>
 void host_gebsrmm(rocsparse_handle          handle,
@@ -427,12 +447,12 @@ void host_gebsrmm(rocsparse_handle          handle,
                   rocsparse_int             row_block_dim,
                   rocsparse_int             col_block_dim,
                   const T*                  B,
-                  rocsparse_int             ldb,
+                  int64_t                   ldb,
                   const T*                  beta,
                   T*                        C,
-                  rocsparse_int             ldc);
+                  int64_t                   ldc);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_csrmm(J                    M,
                 J                    N,
                 J                    K,
@@ -441,43 +461,45 @@ void host_csrmm(J                    M,
                 T                    alpha,
                 const I*             csr_row_ptr_A,
                 const J*             csr_col_ind_A,
-                const T*             csr_val_A,
-                const T*             B,
-                J                    ldb,
+                const A*             csr_val_A,
+                const B*             dense_B,
+                int64_t              ldb,
+                rocsparse_order      order_B,
                 T                    beta,
-                T*                   C,
-                J                    ldc,
-                rocsparse_order      order,
+                C*                   dense_C,
+                int64_t              ldc,
+                rocsparse_order      order_C,
                 rocsparse_index_base base,
                 bool                 force_conj_A);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_csrmm_batched(J                    M,
                         J                    N,
                         J                    K,
                         J                    batch_count_A,
-                        I                    offsets_batch_stride_A,
-                        I                    columns_values_batch_stride_A,
+                        int64_t              offsets_batch_stride_A,
+                        int64_t              columns_values_batch_stride_A,
                         rocsparse_operation  transA,
                         rocsparse_operation  transB,
                         T                    alpha,
                         const I*             csr_row_ptr_A,
                         const J*             csr_col_ind_A,
-                        const T*             csr_val_A,
-                        const T*             B,
-                        J                    ldb,
+                        const A*             csr_val_A,
+                        const B*             dense_B,
+                        int64_t              ldb,
                         J                    batch_count_B,
-                        I                    batch_stride_B,
+                        int64_t              batch_stride_B,
+                        rocsparse_order      order_B,
                         T                    beta,
-                        T*                   C,
-                        J                    ldc,
+                        C*                   dense_C,
+                        int64_t              ldc,
                         J                    batch_count_C,
-                        I                    batch_stride_C,
-                        rocsparse_order      order,
+                        int64_t              batch_stride_C,
+                        rocsparse_order      order_C,
                         rocsparse_index_base base,
                         bool                 force_conj_A);
 
-template <typename T, typename I>
+template <typename T, typename I, typename A, typename B, typename C>
 void host_coomm(I                    M,
                 I                    N,
                 I                    K,
@@ -487,16 +509,17 @@ void host_coomm(I                    M,
                 T                    alpha,
                 const I*             coo_row_ind_A,
                 const I*             coo_col_ind_A,
-                const T*             coo_val_A,
-                const T*             B,
-                I                    ldb,
+                const A*             coo_val_A,
+                const B*             dense_B,
+                int64_t              ldb,
+                rocsparse_order      order_B,
                 T                    beta,
-                T*                   C,
-                I                    ldc,
-                rocsparse_order      order,
+                C*                   dense_C,
+                int64_t              ldc,
+                rocsparse_order      order_C,
                 rocsparse_index_base base);
 
-template <typename T, typename I>
+template <typename T, typename I, typename A, typename B, typename C>
 void host_coomm_batched(I                    M,
                         I                    N,
                         I                    K,
@@ -508,20 +531,21 @@ void host_coomm_batched(I                    M,
                         T                    alpha,
                         const I*             coo_row_ind_A,
                         const I*             coo_col_ind_A,
-                        const T*             coo_val_A,
-                        const T*             B,
-                        I                    ldb,
+                        const A*             coo_val_A,
+                        const B*             dense_B,
+                        int64_t              ldb,
                         I                    batch_count_B,
                         int64_t              batch_stride_B,
+                        rocsparse_order      order_B,
                         T                    beta,
-                        T*                   C,
-                        I                    ldc,
+                        C*                   dense_C,
+                        int64_t              ldc,
                         I                    batch_count_C,
                         int64_t              batch_stride_C,
-                        rocsparse_order      order,
+                        rocsparse_order      order_C,
                         rocsparse_index_base base);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_cscmm(J                   M,
                 J                   N,
                 J                   K,
@@ -530,38 +554,40 @@ void host_cscmm(J                   M,
                 T                   alpha,
                 const I* __restrict csc_col_ptr_A,
                 const J* __restrict csc_row_ind_A,
-                const T* __restrict csc_val_A,
-                const T* __restrict B,
-                J ldb,
-                T beta,
-                T* __restrict C,
-                J                    ldc,
-                rocsparse_order      order,
+                const A* __restrict csc_val_A,
+                const B* __restrict dense_B,
+                int64_t         ldb,
+                rocsparse_order order_B,
+                T               beta,
+                C* __restrict dense_C,
+                int64_t              ldc,
+                rocsparse_order      order_C,
                 rocsparse_index_base base);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_cscmm_batched(J                    M,
                         J                    N,
                         J                    K,
                         J                    batch_count_A,
-                        I                    offsets_batch_stride_A,
-                        I                    rows_values_batch_stride_A,
+                        int64_t              offsets_batch_stride_A,
+                        int64_t              rows_values_batch_stride_A,
                         rocsparse_operation  transA,
                         rocsparse_operation  transB,
                         T                    alpha,
                         const I*             csc_col_ptr_A,
                         const J*             csc_row_ind_A,
-                        const T*             csc_val_A,
-                        const T*             B,
-                        J                    ldb,
+                        const A*             csc_val_A,
+                        const B*             dense_B,
+                        int64_t              ldb,
                         J                    batch_count_B,
-                        I                    batch_stride_B,
+                        int64_t              batch_stride_B,
+                        rocsparse_order      order_B,
                         T                    beta,
-                        T*                   C,
-                        J                    ldc,
+                        C*                   dense_C,
+                        int64_t              ldc,
                         J                    batch_count_C,
-                        I                    batch_stride_C,
-                        rocsparse_order      order,
+                        int64_t              batch_stride_C,
+                        rocsparse_order      order_C,
                         rocsparse_index_base base);
 
 template <typename I, typename J, typename T>
@@ -575,7 +601,8 @@ void host_csrsm(J                    M,
                 const J*             csr_col_ind,
                 const T*             csr_val,
                 T*                   B,
-                J                    ldb,
+                int64_t              ldb,
+                rocsparse_order      order_B,
                 rocsparse_diag_type  diag_type,
                 rocsparse_fill_mode  fill_mode,
                 rocsparse_index_base base,
@@ -593,7 +620,8 @@ void host_coosm(I                    M,
                 const I*             coo_col_ind,
                 const T*             coo_val,
                 T*                   B,
-                I                    ldb,
+                int64_t              ldb,
+                rocsparse_order      order_B,
                 rocsparse_diag_type  diag_type,
                 rocsparse_fill_mode  fill_mode,
                 rocsparse_index_base base,
@@ -613,9 +641,9 @@ void host_bsrsm(rocsparse_int       mb,
                 const T* __restrict bsr_val,
                 rocsparse_int        bsr_dim,
                 const T*             B,
-                rocsparse_int        ldb,
+                int64_t              ldb,
                 T*                   X,
-                rocsparse_int        ldx,
+                int64_t              ldx,
                 rocsparse_diag_type  diag_type,
                 rocsparse_fill_mode  fill_mode,
                 rocsparse_index_base base,
@@ -629,13 +657,13 @@ void host_gemmi(rocsparse_int        M,
                 rocsparse_operation  transB,
                 T                    alpha,
                 const T*             A,
-                rocsparse_int        lda,
+                int64_t              lda,
                 const rocsparse_int* csr_row_ptr,
                 const rocsparse_int* csr_col_ind,
                 const T*             csr_val,
                 T                    beta,
                 T*                   C,
-                rocsparse_int        ldc,
+                int64_t              ldc,
                 rocsparse_index_base base);
 
 /*
@@ -837,7 +865,9 @@ void host_csric0(rocsparse_int                     M,
                  std::vector<T>&                   csr_val,
                  rocsparse_index_base              base,
                  rocsparse_int*                    struct_pivot,
-                 rocsparse_int*                    numeric_pivot);
+                 rocsparse_int*                    numeric_pivot,
+                 rocsparse_int*                    singular_pivot,
+                 double                            tol);
 
 template <typename T, typename U>
 void host_csrilu0(rocsparse_int                     M,
@@ -847,6 +877,8 @@ void host_csrilu0(rocsparse_int                     M,
                   rocsparse_index_base              base,
                   rocsparse_int*                    struct_pivot,
                   rocsparse_int*                    numeric_pivot,
+                  rocsparse_int*                    singular_pivot,
+                  double                            tol,
                   bool                              boost,
                   U                                 boost_tol,
                   T                                 boost_val);
@@ -901,7 +933,7 @@ rocsparse_status host_nnz(rocsparse_direction dirA,
                           rocsparse_int       m,
                           rocsparse_int       n,
                           const T*            A,
-                          rocsparse_int       lda,
+                          int64_t             lda,
                           rocsparse_int*      nnz_per_row_columns,
                           rocsparse_int*      nnz_total_dev_host_ptr);
 
@@ -909,7 +941,7 @@ template <typename T>
 void host_prune_dense2csr(rocsparse_int               m,
                           rocsparse_int               n,
                           const std::vector<T>&       A,
-                          rocsparse_int               lda,
+                          int64_t                     lda,
                           rocsparse_index_base        base,
                           T                           threshold,
                           rocsparse_int&              nnz,
@@ -921,7 +953,7 @@ template <typename T>
 void host_prune_dense2csr_by_percentage(rocsparse_int               m,
                                         rocsparse_int               n,
                                         const std::vector<T>&       A,
-                                        rocsparse_int               lda,
+                                        int64_t                     lda,
                                         rocsparse_index_base        base,
                                         T                           percentage,
                                         rocsparse_int&              nnz,
@@ -934,7 +966,7 @@ void host_dense2csx(J                    m,
                     J                    n,
                     rocsparse_index_base base,
                     const T*             A,
-                    I                    ld,
+                    int64_t              ld,
                     rocsparse_order      order,
                     const I*             nnz_per_row_columns,
                     T*                   csx_val,
@@ -950,14 +982,14 @@ void host_csx2dense(J                    m,
                     const I*             csx_row_col_ptr,
                     const J*             csx_col_row_ind,
                     T*                   A,
-                    I                    ld);
+                    int64_t              ld);
 
 template <typename I, typename T>
 void host_dense_to_coo(I                     m,
                        I                     n,
                        rocsparse_index_base  base,
                        const std::vector<T>& A,
-                       I                     ld,
+                       int64_t               ld,
                        rocsparse_order       order,
                        const std::vector<I>& nnz_per_row,
                        std::vector<T>&       coo_val,
@@ -967,13 +999,13 @@ void host_dense_to_coo(I                     m,
 template <typename I, typename T>
 void host_coo_to_dense(I                     m,
                        I                     n,
-                       I                     nnz,
+                       int64_t               nnz,
                        rocsparse_index_base  base,
                        const std::vector<T>& coo_val,
                        const std::vector<I>& coo_row_ind,
                        const std::vector<I>& coo_col_ind,
                        std::vector<T>&       A,
-                       I                     ld,
+                       int64_t               ld,
                        rocsparse_order       order);
 
 template <typename I, typename J>
@@ -1137,6 +1169,7 @@ void host_csr_to_ell(J                     M,
 
 template <typename T>
 void host_csr_to_hyb(rocsparse_int                     M,
+                     rocsparse_int                     N,
                      rocsparse_int                     nnz,
                      const std::vector<rocsparse_int>& csr_row_ptr,
                      const std::vector<rocsparse_int>& csr_col_ind,
@@ -1195,8 +1228,7 @@ void host_prune_csr_to_csr_by_percentage(rocsparse_int                     M,
                                          T                                 percentage);
 
 template <typename I, typename J>
-void host_coo_to_csr(
-    J M, I nnz, const J* coo_row_ind, std::vector<I>& csr_row_ptr, rocsparse_index_base base);
+void host_coo_to_csr(J M, I nnz, const J* coo_row_ind, I* csr_row_ptr, rocsparse_index_base base);
 
 template <typename T>
 void host_ell_to_csr(rocsparse_int                     M,
