@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -595,6 +595,11 @@ struct rocsparse_matrix_utils
                                     (hsolve_pivot_1[0] != -1) ? rocsparse_status_zero_pivot
                                                               : rocsparse_status_success);
 
+            rocsparse_int hsolve_pivot_neg[1];
+            {
+                auto st = rocsparse_csric0_singular_pivot(handle, info, hsolve_pivot_neg);
+                EXPECT_ROCSPARSE_STATUS(st, rocsparse_status_success);
+            }
             break;
         }
         }
@@ -1033,23 +1038,12 @@ struct rocsparse_matrix_utils
         //
         // Compute (A + A^T) / 2
         //
-        J*   blank;
-        auto err = rocsparse_hipHostMalloc(&blank, M * sizeof(J));
-        if(err != hipSuccess)
+        host_dense_vector<J> blank(M);
+        for(size_t i = 0; i < blank.size(); i++)
         {
-            return rocsparse_status_memory_error;
-        }
-        for(J i = 0; i < M; ++i)
-        {
-            blank[i] = static_cast<J>(0);
-        }
-
-        J* select;
-        err = rocsparse_hipHostMalloc(&select, M * sizeof(J));
-        if(err != hipSuccess)
-        {
-            return rocsparse_status_memory_error;
+            blank[i] = 0;
         }
+        host_dense_vector<J> select(M);
 
         symA.define(M, M, 0, base);
 
@@ -1134,7 +1128,7 @@ struct rocsparse_matrix_utils
                 }
             }
 
-            std::sort(select, select + select_n);
+            std::sort(select.data(), select.data() + select_n);
 
             for(J k = 0; k < select_n; ++k)
             {
@@ -1160,9 +1154,6 @@ struct rocsparse_matrix_utils
             }
         }
 
-        rocsparse_hipHostFree(select);
-        rocsparse_hipHostFree(blank);
-
         return rocsparse_status_success;
     }
 };
