@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the Software), to deal
@@ -22,8 +22,6 @@
  * ************************************************************************ */
 
 #pragma once
-#ifndef TESTING_SDDMM_DISPATCH_HPP
-#define TESTING_SDDMM_DISPATCH_HPP
 
 #include "auto_testing_bad_arg.hpp"
 
@@ -353,47 +351,104 @@ private:
 public:
     static void testing_sddmm_bad_arg(const Arguments& arg)
     {
-        T alpha = 0.6;
-        T beta  = 0.1;
+        const T     local_alpha = 0.6;
+        const T     local_beta  = 0.1;
+        const void* alpha       = (const void*)&local_alpha;
+        const void* beta        = (const void*)&local_beta;
 
         rocsparse_local_handle local_handle;
 
         rocsparse_handle    handle  = local_handle;
         rocsparse_operation trans_A = rocsparse_operation_none;
         rocsparse_operation trans_B = rocsparse_operation_none;
-        const void*         p_alpha = (const void*)&alpha;
-        const void*         p_beta  = (const void*)&beta;
         rocsparse_sddmm_alg alg     = rocsparse_sddmm_alg_default;
-        size_t              buffer_size;
-        size_t*             p_buffer_size = &buffer_size;
-        void*               temp_buffer   = (void*)0x4;
-        rocsparse_datatype  ttype         = get_datatype<T>();
+        size_t              local_buffer_size;
+        size_t*             buffer_size  = &local_buffer_size;
+        void*               temp_buffer  = (void*)0x4;
+        rocsparse_datatype  compute_type = get_datatype<T>();
 
-#define PARAMS_BUFFER_SIZE                                                              \
-    handle, trans_A, trans_B, p_alpha, (const rocsparse_dnmat_descr&)A,                 \
-        (const rocsparse_dnmat_descr&)B, p_beta, (rocsparse_spmat_descr&)C, ttype, alg, \
-        p_buffer_size
+#define PARAMS_BUFFER_SIZE \
+    handle, trans_A, trans_B, alpha, A, B, beta, C, compute_type, alg, buffer_size
 
-#define PARAMS                                                                          \
-    handle, trans_A, trans_B, p_alpha, (const rocsparse_dnmat_descr&)A,                 \
-        (const rocsparse_dnmat_descr&)B, p_beta, (rocsparse_spmat_descr&)C, ttype, alg, \
-        temp_buffer
+#define PARAMS handle, trans_A, trans_B, alpha, A, B, beta, C, compute_type, alg, temp_buffer
 
         //
         // AUTOMATIC BAD ARGS.
         //
         {
-            device_dense_matrix<T> dA, dB;
-            rocsparse_local_dnmat  A(dA), B(dB);
-
+            device_dense_matrix<T>  dA, dB;
+            rocsparse_local_dnmat   local_A(dA), local_B(dB);
             device_sparse_matrix<T> dC;
-            rocsparse_local_spmat   C(dC);
+            rocsparse_local_spmat   local_C(dC);
+
+            rocsparse_dnmat_descr A = local_A;
+            rocsparse_dnmat_descr B = local_B;
+            rocsparse_spmat_descr C = local_C;
 
             static constexpr int num_to_exclude             = 1;
             static constexpr int to_exclude[num_to_exclude] = {10};
-            auto_testing_bad_arg(rocsparse_sddmm_buffer_size, PARAMS_BUFFER_SIZE);
-            auto_testing_bad_arg(rocsparse_sddmm_preprocess, num_to_exclude, to_exclude, PARAMS);
-            auto_testing_bad_arg(rocsparse_sddmm, num_to_exclude, to_exclude, PARAMS);
+            bad_arg_analysis(rocsparse_sddmm_buffer_size, PARAMS_BUFFER_SIZE);
+            select_bad_arg_analysis(rocsparse_sddmm_preprocess, num_to_exclude, to_exclude, PARAMS);
+            select_bad_arg_analysis(rocsparse_sddmm, num_to_exclude, to_exclude, PARAMS);
+        }
+
+        // conjugate transpose
+        {
+            device_dense_matrix<T> dA, dB;
+            rocsparse_local_dnmat  local_A(dA), local_B(dB);
+
+            device_sparse_matrix<T> dC;
+            rocsparse_local_spmat   local_C(dC);
+
+            rocsparse_dnmat_descr A = local_A;
+            rocsparse_dnmat_descr B = local_B;
+            rocsparse_spmat_descr C = local_C;
+
+            trans_A = rocsparse_operation_conjugate_transpose;
+            trans_B = rocsparse_operation_none;
+
+            EXPECT_ROCSPARSE_STATUS(rocsparse_sddmm_buffer_size(PARAMS_BUFFER_SIZE),
+                                    rocsparse_status_not_implemented);
+
+            EXPECT_ROCSPARSE_STATUS(rocsparse_sddmm_preprocess(PARAMS),
+                                    rocsparse_status_not_implemented);
+
+            EXPECT_ROCSPARSE_STATUS(rocsparse_sddmm(PARAMS), rocsparse_status_not_implemented);
+
+            trans_A = rocsparse_operation_none;
+            trans_B = rocsparse_operation_conjugate_transpose;
+
+            EXPECT_ROCSPARSE_STATUS(rocsparse_sddmm_buffer_size(PARAMS_BUFFER_SIZE),
+                                    rocsparse_status_not_implemented);
+
+            EXPECT_ROCSPARSE_STATUS(rocsparse_sddmm_preprocess(PARAMS),
+                                    rocsparse_status_not_implemented);
+
+            EXPECT_ROCSPARSE_STATUS(rocsparse_sddmm(PARAMS), rocsparse_status_not_implemented);
+
+            trans_A = rocsparse_operation_none;
+            trans_B = rocsparse_operation_none;
+        }
+
+        // buffer size
+        {
+            device_dense_matrix<T> dA, dB;
+            rocsparse_local_dnmat  local_A(dA), local_B(dB);
+
+            device_sparse_matrix<T> dC(10, 10, 10, rocsparse_index_base_zero);
+            rocsparse_local_spmat   local_C(dC);
+
+            rocsparse_dnmat_descr A = local_A;
+            rocsparse_dnmat_descr B = local_B;
+            rocsparse_spmat_descr C = local_C;
+
+            alg         = rocsparse_sddmm_alg_dense;
+            temp_buffer = nullptr;
+
+            EXPECT_ROCSPARSE_STATUS(rocsparse_sddmm(PARAMS), rocsparse_status_invalid_pointer);
+
+            alg         = rocsparse_sddmm_alg_default;
+            temp_buffer = (void*)0x4;
         }
 
 #undef PARAMS
@@ -408,17 +463,29 @@ public:
         rocsparse_operation  trans_A = arg.transA;
         rocsparse_operation  trans_B = arg.transB;
         rocsparse_index_base base    = arg.baseA;
-        rocsparse_sddmm_alg  alg     = rocsparse_sddmm_alg_default;
+        rocsparse_sddmm_alg  alg     = arg.sddmm_alg;
         rocsparse_datatype   ttype   = get_datatype<T>();
         rocsparse_order      order_A = arg.order;
         rocsparse_order      order_B = arg.order;
-
         // Create rocsparse handle
         rocsparse_local_handle handle(arg);
 
+#ifndef ROCSPARSE_WITH_ROCBLAS
+        if(arg.sddmm_alg == rocsparse_sddmm_alg_dense)
+        {
+            std::cerr
+                << "No BLAS implementation is available, skipping rocsparse_sddmm_alg_dense test."
+                << std::endl;
+            return;
+        }
+#endif
+
         host_scalar<T> h_alpha(arg.get_alpha<T>());
         host_scalar<T> h_beta(arg.get_beta<T>());
 
+        device_scalar<T> d_alpha(h_alpha);
+        device_scalar<T> d_beta(h_beta);
+
 #define PARAMS_BUFFER_SIZE(alpha_, A_, B_, beta_, C_)                                          \
     handle, trans_A, trans_B, alpha_, (const rocsparse_dnmat_descr&)A_,                        \
         (const rocsparse_dnmat_descr&)B_, beta_, (const rocsparse_spmat_descr&)C_, ttype, alg, \
@@ -513,6 +580,10 @@ public:
             CHECK_ROCSPARSE_ERROR(rocsparse_sddmm_preprocess(PARAMS(h_alpha, A, B, h_beta, C)));
 
             CHECK_ROCSPARSE_ERROR(testing::rocsparse_sddmm(PARAMS(h_alpha, A, B, h_beta, C)));
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("C pointer mode host", dC);
+            }
 
             {
                 host_vector<T> hC_val_copy(hC.val);
@@ -534,12 +605,15 @@ public:
 
             // Pointer mode device
             {
-                device_scalar<T> d_alpha(h_alpha), d_beta(h_beta);
                 CHECK_ROCSPARSE_ERROR(
                     rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
                 CHECK_ROCSPARSE_ERROR(rocsparse_sddmm_preprocess(PARAMS(d_alpha, A, B, d_beta, C)));
                 CHECK_ROCSPARSE_ERROR(testing::rocsparse_sddmm(PARAMS(d_alpha, A, B, d_beta, C)));
             }
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("C pointer mode device", dC);
+            }
 
             hC.near_check(dC);
         }
@@ -576,29 +650,31 @@ public:
             double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
             double gpu_gbyte  = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-            display_timing_info("format",
+            display_timing_info(display_key_t::format,
                                 rocsparse_format2string(FORMAT),
-                                "transA",
+                                display_key_t::trans_A,
                                 rocsparse_operation2string(trans_A),
-                                "transB",
+                                display_key_t::trans_B,
                                 rocsparse_operation2string(trans_B),
-                                "M",
+                                display_key_t::M,
                                 M,
-                                "N",
+                                display_key_t::N,
                                 N,
-                                "K",
+                                display_key_t::K,
                                 K,
-                                "nnz",
+                                display_key_t::nnz,
                                 dC.nnz,
-                                "alpha",
+                                display_key_t::alpha,
                                 *h_alpha,
-                                "beta",
+                                display_key_t::beta,
                                 *h_beta,
-                                s_timing_info_perf,
+                                display_key_t::algorithm,
+                                rocsparse_sddmmalg2string(alg),
+                                display_key_t::gflops,
                                 gpu_gflops,
-                                s_timing_info_bandwidth,
+                                display_key_t::bandwidth,
                                 gpu_gbyte,
-                                s_timing_info_time,
+                                display_key_t::time_ms,
                                 get_gpu_time_msec(gpu_time_used));
         }
 
@@ -606,5 +682,3 @@ public:
         return;
     }
 };
-
-#endif // TESTING_SDDMM_DISPATCH_HPP
