@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the Software), to deal
@@ -22,10 +22,9 @@
  * ************************************************************************ */
 
 #pragma once
-#ifndef TESTING_SPMV_HPP
-#define TESTING_SPMV_HPP
 
 #include "auto_testing_bad_arg.hpp"
+#include "rocsparse_matrix_statistics.hpp"
 
 template <rocsparse_format FORMAT, typename I, typename J, typename T>
 struct testing_matrix_type_traits;
@@ -217,6 +216,7 @@ struct testing_spmv_dispatch_traits<rocsparse_format_csc, I, J, A, X, Y, T>
                              device_sparse_matrix<A>& dA,
                              Ts&&... ts)
     {
+
         display_timing_info(trans,
                             trans_value,
                             display_key_t::M,
@@ -568,24 +568,24 @@ private:
 public:
     static void testing_spmv_bad_arg(const Arguments& arg)
     {
-        T alpha = 0.6;
-        T beta  = 0.1;
+        const T local_alpha = static_cast<T>(6);
+        const T local_beta  = static_cast<T>(2);
 
         rocsparse_local_handle local_handle;
 
-        rocsparse_handle    handle  = local_handle;
-        rocsparse_operation trans   = rocsparse_operation_none;
-        const void*         p_alpha = (const void*)&alpha;
-        const void*         p_beta  = (const void*)&beta;
-        rocsparse_spmv_alg  alg     = rocsparse_spmv_alg_default;
-        size_t              buffer_size;
-        size_t*             p_buffer_size = &buffer_size;
-        void*               temp_buffer   = (void*)0x4;
-        rocsparse_datatype  ttype         = get_datatype<T>();
+        rocsparse_handle     handle = local_handle;
+        rocsparse_operation  trans  = rocsparse_operation_none;
+        const void*          alpha  = (const void*)&local_alpha;
+        const void*          beta   = (const void*)&local_beta;
+        rocsparse_spmv_alg   alg    = rocsparse_spmv_alg_default;
+        rocsparse_spmv_stage stage  = rocsparse_spmv_stage_compute;
+        size_t               local_buffer_size;
+        size_t*              buffer_size  = &local_buffer_size;
+        void*                temp_buffer  = (void*)0x4;
+        rocsparse_datatype   compute_type = get_datatype<T>();
 
-#define PARAMS                                                                                   \
-    handle, trans, p_alpha, (const rocsparse_spmat_descr&)matA, (const rocsparse_dnvec_descr&)x, \
-        p_beta, (rocsparse_dnvec_descr&)y, ttype, alg, p_buffer_size, temp_buffer
+#define PARAMS \
+    handle, trans, alpha, mat, x, beta, y, compute_type, alg, stage, buffer_size, temp_buffer
 
         //
         // AUTOMATIC BAD ARGS.
@@ -594,19 +594,23 @@ public:
             device_dense_matrix<X>  dx;
             device_dense_matrix<Y>  dy;
             device_sparse_matrix<A> dA;
-            rocsparse_local_spmat   matA(dA);
-            rocsparse_local_dnvec   x(dx);
-            rocsparse_local_dnvec   y(dy);
+            rocsparse_local_spmat   local_mat(dA);
+            rocsparse_local_dnvec   local_x(dx);
+            rocsparse_local_dnvec   local_y(dy);
+
+            rocsparse_spmat_descr mat = local_mat;
+            rocsparse_dnvec_descr x   = local_x;
+            rocsparse_dnvec_descr y   = local_y;
 
             //
             // WITH 2 ARGUMENTS BEING SKIPPED DURING THE CHECK.
             //
             static const int nex   = 2;
-            static const int ex[2] = {9, 10};
-            auto_testing_bad_arg(rocsparse_spmv, nex, ex, PARAMS);
+            static const int ex[2] = {10, 11};
+            select_bad_arg_analysis(rocsparse_spmv, nex, ex, PARAMS);
 
-            p_buffer_size = nullptr;
-            temp_buffer   = nullptr;
+            buffer_size = nullptr;
+            temp_buffer = nullptr;
             EXPECT_ROCSPARSE_STATUS(rocsparse_spmv(PARAMS), rocsparse_status_invalid_pointer);
         }
 
@@ -631,69 +635,23 @@ public:
         host_scalar<T> h_alpha(arg.get_alpha<T>());
         host_scalar<T> h_beta(arg.get_beta<T>());
 
-#define PARAMS(alpha_, A_, x_, beta_, y_) \
-    handle, trans, alpha_, A_, x_, beta_, y_, ttype, alg, &buffer_size, dbuffer
+        device_scalar<T> d_alpha(h_alpha);
+        device_scalar<T> d_beta(h_beta);
 
-        // Argument sanity check before allocating invalid memory
-
-        // Allocate memory on device
-        // Pointer mode
-
-        // Check structures
-        // Check SpMV when structures can be created
-        if(M <= 0 || N <= 0 || (matrix_type == rocsparse_matrix_type_symmetric && M != N)
-           || (matrix_type == rocsparse_matrix_type_triangular && M != N))
-        {
-            if(M == 0 || N == 0)
-            {
-                CHECK_ROCSPARSE_ERROR(
-                    rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-                device_sparse_matrix<A> dA;
-                device_dense_matrix<X>  dx;
-                device_dense_matrix<Y>  dy;
-
-                rocsparse_local_spmat matA(dA);
-                rocsparse_local_dnvec x(dx);
-                rocsparse_local_dnvec y(dy);
-
-                EXPECT_ROCSPARSE_STATUS(
-                    rocsparse_spmat_set_attribute(
-                        matA, rocsparse_spmat_matrix_type, &matrix_type, sizeof(matrix_type)),
-                    rocsparse_status_success);
-
-                EXPECT_ROCSPARSE_STATUS(rocsparse_spmat_set_attribute(
-                                            matA, rocsparse_spmat_fill_mode, &uplo, sizeof(uplo)),
-                                        rocsparse_status_success);
-
-                EXPECT_ROCSPARSE_STATUS(
-                    rocsparse_spmat_set_attribute(
-                        matA, rocsparse_spmat_storage_mode, &storage, sizeof(storage)),
-                    rocsparse_status_success);
-
-                size_t buffer_size;
-                void*  dbuffer = nullptr;
-                EXPECT_ROCSPARSE_STATUS(rocsparse_spmv(PARAMS(h_alpha, matA, x, h_beta, y)),
-                                        rocsparse_status_success);
-                CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, 10));
-                EXPECT_ROCSPARSE_STATUS(rocsparse_spmv(PARAMS(h_alpha, matA, x, h_beta, y)),
-                                        rocsparse_status_success);
-                CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
-                return;
-            }
-            return;
-        }
-        // Wavefront size
-        int dev;
-        hipGetDevice(&dev);
-
-        hipDeviceProp_t prop;
-        hipGetDeviceProperties(&prop, dev);
+#define PARAMS(alpha_, A_, x_, beta_, y_, stage) \
+    handle, trans, alpha_, A_, x_, beta_, y_, ttype, alg, stage, &buffer_size, dbuffer
 
         //
         // INITIALIZATE THE SPARSE MATRIX
         //
         host_sparse_matrix<A> hA;
         {
+            int dev;
+            hipGetDevice(&dev);
+
+            hipDeviceProp_t prop;
+            hipGetDeviceProperties(&prop, dev);
+
             const bool has_datafile = rocsparse_arguments_has_datafile(arg);
             bool       to_int       = false;
             to_int |= (prop.warpSize == 32);
@@ -739,50 +697,47 @@ public:
                                     matA, rocsparse_spmat_storage_mode, &storage, sizeof(storage)),
                                 rocsparse_status_success);
 
+        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
+
+        // Run buffer size
         void*  dbuffer     = nullptr;
         size_t buffer_size = 0;
-        CHECK_ROCSPARSE_ERROR(rocsparse_spmv(PARAMS(h_alpha, matA, x, h_beta, y)));
+        CHECK_ROCSPARSE_ERROR(
+            rocsparse_spmv(PARAMS(h_alpha, matA, x, h_beta, y, rocsparse_spmv_stage_buffer_size)));
         CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, buffer_size));
 
+        // Run preprocess
+        CHECK_ROCSPARSE_ERROR(
+            rocsparse_spmv(PARAMS(h_alpha, matA, x, h_beta, y, rocsparse_spmv_stage_preprocess)));
+
         if(arg.unit_check)
         {
-#define EX_PARAMS(alpha_, A_, x_, beta_, y_, stage_) \
-    handle, trans, alpha_, A_, x_, beta_, y_, ttype, alg, stage_, &buffer_size, dbuffer
+            // Run solve
+            CHECK_ROCSPARSE_ERROR(testing::rocsparse_spmv(
+                PARAMS(h_alpha, matA, x, h_beta, y, rocsparse_spmv_stage_compute)));
 
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmv_ex(
-                EX_PARAMS(h_alpha, matA, x, h_beta, y, rocsparse_spmv_stage_preprocess)));
-
-            // Pointer mode host
-            CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-            CHECK_ROCSPARSE_ERROR(testing::rocsparse_spmv_ex(
-                EX_PARAMS(h_alpha, matA, x, h_beta, y, rocsparse_spmv_stage_compute)));
-
-            //
-            // CPU spmv
-            //
-            {
-                host_dense_matrix<Y> hy_copy(hy);
-                //
-                // HOST CALCULATION
-                //
-                traits::host_calculation(trans, h_alpha, hA, hx, h_beta, hy, alg, matrix_type);
-
-                hy.near_check(dy);
-                dy.transfer_from(hy_copy);
-            }
-
-            //
-            // Pointer mode device
-            //
-            {
-                device_scalar<T> d_alpha(h_alpha), d_beta(h_beta);
-                CHECK_ROCSPARSE_ERROR(
-                    rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
-                CHECK_ROCSPARSE_ERROR(testing::rocsparse_spmv_ex(
-                    EX_PARAMS(d_alpha, matA, x, d_beta, y, rocsparse_spmv_stage_compute)));
-            }
+            host_dense_matrix<Y> hy_copy(hy);
+            traits::host_calculation(trans, h_alpha, hA, hx, h_beta, hy, alg, matrix_type);
 
             hy.near_check(dy);
+
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("Y_pointer_mode_host", dy);
+            }
+
+            dy.transfer_from(hy_copy);
+            CHECK_ROCSPARSE_ERROR(
+                rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
+            CHECK_ROCSPARSE_ERROR(testing::rocsparse_spmv(
+                PARAMS(d_alpha, matA, x, d_beta, y, rocsparse_spmv_stage_compute)));
+            CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
+
+            hy.near_check(dy);
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("Y_pointer_mode_device", dy);
+            }
         }
 
         if(arg.timing)
@@ -790,12 +745,11 @@ public:
             const int number_cold_calls = 2;
             const int number_hot_calls  = arg.iters;
 
-            CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
             // Warm up
             for(int iter = 0; iter < number_cold_calls; ++iter)
             {
-                CHECK_ROCSPARSE_ERROR(rocsparse_spmv(PARAMS(h_alpha, matA, x, h_beta, y)));
+                CHECK_ROCSPARSE_ERROR(rocsparse_spmv(
+                    PARAMS(h_alpha, matA, x, h_beta, y, rocsparse_spmv_stage_compute)));
             }
 
             double gpu_time_used = get_time_us();
@@ -803,7 +757,8 @@ public:
             // Performance run
             for(int iter = 0; iter < number_hot_calls; ++iter)
             {
-                CHECK_ROCSPARSE_ERROR(rocsparse_spmv(PARAMS(h_alpha, matA, x, h_beta, y)));
+                CHECK_ROCSPARSE_ERROR(rocsparse_spmv(
+                    PARAMS(h_alpha, matA, x, h_beta, y, rocsparse_spmv_stage_compute)));
             }
 
             gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
@@ -814,27 +769,69 @@ public:
             const double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
             const double gpu_gbyte  = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-            traits::display_info(arg,
-                                 display_key_t::trans_A,
-                                 rocsparse_operation2string(trans),
-                                 dA,
-                                 display_key_t::alpha,
-                                 *h_alpha,
-                                 display_key_t::beta,
-                                 *h_beta,
-                                 display_key_t::algorithm,
-                                 rocsparse_spmvalg2string(alg),
-                                 display_key_t::gflops,
-                                 gpu_gflops,
-                                 display_key_t::bandwidth,
-                                 gpu_gbyte,
-                                 display_key_t::time_ms,
-                                 get_gpu_time_msec(gpu_time_used));
+            if(arg.sparsity_pattern_statistics)
+            {
+                int64_t min_nnz_row;
+                int64_t median_nnz_row;
+                int64_t max_nnz_row;
+                rocsparse_matrix_statistics::get_nnz_per_row(
+                    dA, min_nnz_row, median_nnz_row, max_nnz_row);
+
+                int64_t min_nnz_col;
+                int64_t median_nnz_col;
+                int64_t max_nnz_col;
+                rocsparse_matrix_statistics::get_nnz_per_column(
+                    dA, min_nnz_col, median_nnz_col, max_nnz_col);
+                traits::display_info(arg,
+                                     display_key_t::trans_A,
+                                     rocsparse_operation2string(trans),
+                                     dA,
+                                     display_key_t::min_nnz_per_row,
+                                     min_nnz_row,
+                                     display_key_t::max_nnz_per_row,
+                                     max_nnz_row,
+                                     display_key_t::median_nnz_per_row,
+                                     median_nnz_row,
+                                     display_key_t::min_nnz_per_col,
+                                     min_nnz_col,
+                                     display_key_t::max_nnz_per_col,
+                                     max_nnz_col,
+                                     display_key_t::median_nnz_per_col,
+                                     median_nnz_col,
+                                     display_key_t::alpha,
+                                     *h_alpha,
+                                     display_key_t::beta,
+                                     *h_beta,
+                                     display_key_t::algorithm,
+                                     rocsparse_spmvalg2string(alg),
+                                     display_key_t::gflops,
+                                     gpu_gflops,
+                                     display_key_t::bandwidth,
+                                     gpu_gbyte,
+                                     display_key_t::time_ms,
+                                     get_gpu_time_msec(gpu_time_used));
+            }
+            else
+            {
+                traits::display_info(arg,
+                                     display_key_t::trans_A,
+                                     rocsparse_operation2string(trans),
+                                     dA,
+                                     display_key_t::alpha,
+                                     *h_alpha,
+                                     display_key_t::beta,
+                                     *h_beta,
+                                     display_key_t::algorithm,
+                                     rocsparse_spmvalg2string(alg),
+                                     display_key_t::gflops,
+                                     gpu_gflops,
+                                     display_key_t::bandwidth,
+                                     gpu_gbyte,
+                                     display_key_t::time_ms,
+                                     get_gpu_time_msec(gpu_time_used));
+            }
         }
 
         CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
-        return;
     }
 };
-
-#endif // TESTING_SPMV_HPP
