@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,7 @@
 template <typename T>
 void testing_bsrgeam_bad_arg(const Arguments& arg)
 {
-    static const size_t safe_size = 100;
+    static const size_t safe_size = 1;
     T                   h_alpha   = 0.6;
     T                   h_beta    = 0.2;
 
@@ -60,9 +60,15 @@ void testing_bsrgeam_bad_arg(const Arguments& arg)
     const rocsparse_int*      bsr_col_ind_B = (const rocsparse_int*)0x4;
     const rocsparse_mat_descr descr_C       = local_descr_C;
     T*                        bsr_val_C     = (T*)0x4;
-    rocsparse_int*            bsr_row_ptr_C = (rocsparse_int*)0x4;
-    rocsparse_int*            bsr_col_ind_C = (rocsparse_int*)0x4;
-    rocsparse_int*            nnzb_C        = (rocsparse_int*)0x4;
+
+    host_dense_vector<rocsparse_int> hbsr_row_ptr_C(safe_size + 1);
+    hbsr_row_ptr_C[0] = 0;
+    hbsr_row_ptr_C[1] = 1;
+    device_dense_vector<rocsparse_int> dbsr_row_ptr_C(hbsr_row_ptr_C);
+
+    rocsparse_int* bsr_row_ptr_C = (rocsparse_int*)dbsr_row_ptr_C;
+    rocsparse_int* bsr_col_ind_C = (rocsparse_int*)0x4;
+    rocsparse_int* nnzb_C        = (rocsparse_int*)0x4;
 
 #define PARAMS_NNZB                                                                         \
     handle, dir, mb, nb, block_dim, descr_A, nnzb_A, bsr_row_ptr_A, bsr_col_ind_A, descr_B, \
@@ -73,9 +79,8 @@ void testing_bsrgeam_bad_arg(const Arguments& arg)
         bsr_col_ind_A, beta, descr_B, nnzb_B, bsr_val_B, bsr_row_ptr_B, bsr_col_ind_B, descr_C, \
         bsr_val_C, bsr_row_ptr_C, bsr_col_ind_C
 
-    auto_testing_bad_arg(rocsparse_bsrgeam_nnzb, PARAMS_NNZB);
-    auto_testing_bad_arg(rocsparse_bsrgeam<T>, PARAMS);
-
+    bad_arg_analysis(rocsparse_bsrgeam_nnzb, PARAMS_NNZB);
+    bad_arg_analysis(rocsparse_bsrgeam<T>, PARAMS);
     for(auto val : rocsparse_matrix_type_t::values)
     {
         if(val != rocsparse_matrix_type_general)
@@ -89,11 +94,26 @@ void testing_bsrgeam_bad_arg(const Arguments& arg)
         }
     }
 
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_A, rocsparse_matrix_type_general));
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_B, rocsparse_matrix_type_general));
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_C, rocsparse_matrix_type_general));
+
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_A, rocsparse_matrix_type_general));
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_B, rocsparse_matrix_type_general));
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_C, rocsparse_matrix_type_general));
+
+    // Check block_dim == 0
+    block_dim = 0;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrgeam_nnzb(PARAMS_NNZB), rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrgeam<T>(PARAMS), rocsparse_status_invalid_size);
+    block_dim = safe_size;
+
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr_A, rocsparse_storage_mode_unsorted));
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr_B, rocsparse_storage_mode_unsorted));
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr_C, rocsparse_storage_mode_unsorted));
-    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrgeam_nnzb(PARAMS_NNZB), rocsparse_status_not_implemented);
-    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrgeam<T>(PARAMS), rocsparse_status_not_implemented);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrgeam_nnzb(PARAMS_NNZB),
+                            rocsparse_status_requires_sorted_storage);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrgeam<T>(PARAMS), rocsparse_status_requires_sorted_storage);
 
 #undef PARAMS
 #undef PARAMS_NNZB
@@ -113,13 +133,8 @@ void testing_bsrgeam(const Arguments& arg)
     rocsparse_matrix_factory<T>        matrix_factory(arg, arg.timing ? false : true, full_rank);
     rocsparse_matrix_factory_random<T> matrix_factory_random(full_rank);
 
-    rocsparse_int Mb = -1;
-    rocsparse_int Nb = -1;
-    if(block_dim > 0)
-    {
-        Mb = (M + block_dim - 1) / block_dim;
-        Nb = (N + block_dim - 1) / block_dim;
-    }
+    rocsparse_int Mb = (M + block_dim - 1) / block_dim;
+    rocsparse_int Nb = (N + block_dim - 1) / block_dim;
 
     host_scalar<T> h_alpha(arg.get_alpha<T>());
     host_scalar<T> h_beta(arg.get_beta<T>());
@@ -137,74 +152,6 @@ void testing_bsrgeam(const Arguments& arg)
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_index_base(descrB, baseB));
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_index_base(descrC, baseC));
 
-    // Argument sanity check before allocating invalid memory
-    if(Mb <= 0 || Nb <= 0)
-    {
-        static const size_t safe_size = 100;
-
-        device_vector<rocsparse_int> dbsr_row_ptr_A;
-        device_vector<rocsparse_int> dbsr_col_ind_A;
-        device_vector<T>             dbsr_val_A;
-        device_vector<rocsparse_int> dbsr_row_ptr_B;
-        device_vector<rocsparse_int> dbsr_col_ind_B;
-        device_vector<T>             dbsr_val_B;
-        device_vector<rocsparse_int> dbsr_row_ptr_C;
-        device_vector<rocsparse_int> dbsr_col_ind_C;
-        device_vector<T>             dbsr_val_C;
-
-        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
-        rocsparse_int nnzb_C;
-
-        rocsparse_status status_1 = rocsparse_bsrgeam_nnzb(handle,
-                                                           dir,
-                                                           Mb,
-                                                           Nb,
-                                                           block_dim,
-                                                           descrA,
-                                                           safe_size,
-                                                           dbsr_row_ptr_A,
-                                                           dbsr_col_ind_A,
-                                                           descrB,
-                                                           safe_size,
-                                                           dbsr_row_ptr_B,
-                                                           dbsr_col_ind_B,
-                                                           descrC,
-                                                           dbsr_row_ptr_C,
-                                                           &nnzb_C);
-        rocsparse_status status_2 = rocsparse_bsrgeam<T>(handle,
-                                                         dir,
-                                                         Mb,
-                                                         Nb,
-                                                         block_dim,
-                                                         h_alpha,
-                                                         descrA,
-                                                         safe_size,
-                                                         dbsr_val_A,
-                                                         dbsr_row_ptr_A,
-                                                         dbsr_col_ind_A,
-                                                         h_beta,
-                                                         descrB,
-                                                         safe_size,
-                                                         dbsr_val_B,
-                                                         dbsr_row_ptr_B,
-                                                         dbsr_col_ind_B,
-                                                         descrC,
-                                                         dbsr_val_C,
-                                                         dbsr_row_ptr_C,
-                                                         dbsr_col_ind_C);
-
-        // alpha == nullptr && beta != nullptr
-        EXPECT_ROCSPARSE_STATUS(status_1,
-                                (Mb < 0 || Nb < 0) ? rocsparse_status_invalid_size
-                                                   : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(status_2,
-                                (Mb < 0 || Nb < 0) ? rocsparse_status_invalid_size
-                                                   : rocsparse_status_success);
-
-        return;
-    }
-
     // Allocate host memory for matrices
     host_vector<rocsparse_int> hbsr_row_ptr_A;
     host_vector<rocsparse_int> hbsr_col_ind_A;
@@ -214,8 +161,8 @@ void testing_bsrgeam(const Arguments& arg)
     host_vector<T>             hbsr_val_B;
 
     // Sample matrix
-    rocsparse_int nnzb_A = 4;
-    rocsparse_int nnzb_B = 4;
+    rocsparse_int nnzb_A;
+    rocsparse_int nnzb_B;
     rocsparse_int hnnzb_C_gold;
     rocsparse_int hnnzb_C_1;
     rocsparse_int hnnzb_C_2;
@@ -242,10 +189,10 @@ void testing_bsrgeam(const Arguments& arg)
     // Allocate device memory
     device_vector<rocsparse_int> dbsr_row_ptr_A(Mb + 1);
     device_vector<rocsparse_int> dbsr_col_ind_A(nnzb_A);
-    device_vector<T>             dbsr_val_A(block_dim * block_dim * nnzb_A);
+    device_vector<T>             dbsr_val_A(size_t(nnzb_A) * block_dim * block_dim);
     device_vector<rocsparse_int> dbsr_row_ptr_B(Mb + 1);
     device_vector<rocsparse_int> dbsr_col_ind_B(nnzb_B);
-    device_vector<T>             dbsr_val_B(block_dim * block_dim * nnzb_B);
+    device_vector<T>             dbsr_val_B(size_t(nnzb_B) * block_dim * block_dim);
     device_scalar<T>             d_alpha(h_alpha);
     device_scalar<T>             d_beta(h_beta);
     device_vector<rocsparse_int> dbsr_row_ptr_C_1(Mb + 1);
@@ -341,8 +288,8 @@ void testing_bsrgeam(const Arguments& arg)
         // Allocate device memory for C
         device_vector<rocsparse_int> dbsr_col_ind_C_1(hnnzb_C_1);
         device_vector<rocsparse_int> dbsr_col_ind_C_2(hnnzb_C_2);
-        device_vector<T>             dbsr_val_C_1(block_dim * block_dim * hnnzb_C_1);
-        device_vector<T>             dbsr_val_C_2(block_dim * block_dim * hnnzb_C_2);
+        device_vector<T>             dbsr_val_C_1(size_t(hnnzb_C_1) * block_dim * block_dim);
+        device_vector<T>             dbsr_val_C_2(size_t(hnnzb_C_2) * block_dim * block_dim);
 
         // Perform matrix matrix multiplication
 
@@ -397,8 +344,8 @@ void testing_bsrgeam(const Arguments& arg)
         // Copy output to host
         host_vector<rocsparse_int> hbsr_col_ind_C_1(hnnzb_C_1);
         host_vector<rocsparse_int> hbsr_col_ind_C_2(hnnzb_C_2);
-        host_vector<T>             hbsr_val_C_1(block_dim * block_dim * hnnzb_C_1);
-        host_vector<T>             hbsr_val_C_2(block_dim * block_dim * hnnzb_C_2);
+        host_vector<T>             hbsr_val_C_1(size_t(hnnzb_C_1) * block_dim * block_dim);
+        host_vector<T>             hbsr_val_C_2(size_t(hnnzb_C_2) * block_dim * block_dim);
 
         hbsr_col_ind_C_1.transfer_from(dbsr_col_ind_C_1);
         hbsr_col_ind_C_2.transfer_from(dbsr_col_ind_C_2);
@@ -407,7 +354,7 @@ void testing_bsrgeam(const Arguments& arg)
 
         // CPU bsrgemm
         host_vector<rocsparse_int> hbsr_col_ind_C_gold(hnnzb_C_gold);
-        host_vector<T>             hbsr_val_C_gold(block_dim * block_dim * hnnzb_C_gold);
+        host_vector<T>             hbsr_val_C_gold(size_t(hnnzb_C_gold) * block_dim * block_dim);
         host_bsrgeam<T>(dir,
                         Mb,
                         Nb,
@@ -428,6 +375,17 @@ void testing_bsrgeam(const Arguments& arg)
                         baseC);
 
         // Check C
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("hbsr_row_ptr_C_1", hbsr_row_ptr_C_1);
+            rocsparse_reproducibility::save("hbsr_col_ind_C_1", hbsr_col_ind_C_1);
+            rocsparse_reproducibility::save("hbsr_val_C_1", hbsr_val_C_1);
+
+            rocsparse_reproducibility::save("hbsr_row_ptr_C_2", hbsr_row_ptr_C_2);
+            rocsparse_reproducibility::save("hbsr_col_ind_C_2", hbsr_col_ind_C_2);
+            rocsparse_reproducibility::save("hbsr_val_C_2", hbsr_val_C_2);
+        }
+
         hbsr_col_ind_C_gold.unit_check(hbsr_col_ind_C_1);
         hbsr_col_ind_C_gold.unit_check(hbsr_col_ind_C_2);
         hbsr_val_C_gold.near_check(hbsr_val_C_1);
@@ -506,7 +464,7 @@ void testing_bsrgeam(const Arguments& arg)
                                                          &nnzb_C));
         }
 
-        gpu_analysis_time_used = get_time_us() - gpu_analysis_time_used;
+        gpu_analysis_time_used = (get_time_us() - gpu_analysis_time_used) / number_hot_calls;
 
         // Warm up
         for(int iter = 0; iter < number_cold_calls; ++iter)
@@ -572,29 +530,29 @@ void testing_bsrgeam(const Arguments& arg)
         double gpu_gflops = get_gpu_gflops(gpu_solve_time_used, gflop_count);
         double gpu_gbyte  = get_gpu_gbyte(gpu_solve_time_used, gbyte_count);
 
-        display_timing_info("Mb",
+        display_timing_info(display_key_t::Mb,
                             Mb,
-                            "Nb",
+                            display_key_t::Nb,
                             Nb,
-                            "block_dim",
+                            display_key_t::bdim,
                             block_dim,
-                            "nnzb_A",
+                            display_key_t::nnzb_A,
                             nnzb_A,
-                            "nnzb_B",
+                            display_key_t::nnzb_B,
                             nnzb_B,
-                            "nnzb_C",
+                            display_key_t::nnzb_C,
                             nnzb_C,
-                            "alpha",
+                            display_key_t::alpha,
                             *h_alpha,
-                            "beta",
+                            display_key_t::beta,
                             *h_beta,
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            "analysis msec",
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
     }
 }
