@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,13 +25,21 @@
 #include "auto_testing_bad_arg.hpp"
 #include "testing.hpp"
 
+//
+//
+//
 template <typename T>
 void testing_bsrgemm_bad_arg(const Arguments& arg)
 {
-    static const size_t safe_size = 100;
+    static const size_t safe_size = 1;
     T                   h_alpha   = 0.6;
     T                   h_beta    = 0.2;
 
+    host_dense_vector<rocsparse_int> hbsr_row_ptr_C(safe_size + 1);
+    hbsr_row_ptr_C[0] = 0;
+    hbsr_row_ptr_C[1] = 1;
+    device_dense_vector<rocsparse_int> dbsr_row_ptr_C(hbsr_row_ptr_C);
+
     // Create rocsparse handle
     rocsparse_local_handle local_handle;
     // Create matrix descriptors
@@ -57,9 +65,9 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
     // C matrix
     const rocsparse_mat_descr descr_C       = local_descr_C;
     T*                        bsr_val_C     = (T*)0x4;
-    rocsparse_int*            bsr_row_ptr_C = (rocsparse_int*)0x4;
+    rocsparse_int*            bsr_row_ptr_C = (rocsparse_int*)dbsr_row_ptr_C;
     rocsparse_int*            bsr_col_ind_C = (rocsparse_int*)0x4;
-    rocsparse_int*            pnnzb_C       = (rocsparse_int*)0x4;
+    rocsparse_int*            nnzb_C        = (rocsparse_int*)0x4;
 
 #define PARAMS_BUFFER_SIZE                                                                       \
     handle, dir, trans_A, trans_B, mb, nb, kb, block_dim, alpha, descr_A, nnzb_A, bsr_row_ptr_A, \
@@ -69,7 +77,7 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
 #define PARAMS_NNZB                                                                       \
     handle, dir, trans_A, trans_B, mb, nb, kb, block_dim, descr_A, nnzb_A, bsr_row_ptr_A, \
         bsr_col_ind_A, descr_B, nnzb_B, bsr_row_ptr_B, bsr_col_ind_B, descr_D, nnzb_D,    \
-        bsr_row_ptr_D, bsr_col_ind_D, descr_C, bsr_row_ptr_C, pnnzb_C, info_C, temp_buffer
+        bsr_row_ptr_D, bsr_col_ind_D, descr_C, bsr_row_ptr_C, nnzb_C, info_C, temp_buffer
 
 #define PARAMS                                                                                  \
     handle, dir, trans_A, trans_B, mb, nb, kb, block_dim, alpha, descr_A, nnzb_A, bsr_val_A,    \
@@ -87,18 +95,18 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
     // ###############################################
     // Scenario 1: alpha == nullptr && beta == nullptr
     // ###############################################
-
     {
         // In this scenario matrices A == B == D == nullptr
-        int nargs_to_exclude_buffer_size = 14;
-        int nargs_to_exclude_nnzb        = 12;
-        int nargs_to_exclude             = 17;
+        static constexpr int nargs_to_exclude_buffer_size = 14;
+        static constexpr int nargs_to_exclude_nnzb        = 16;
+        static constexpr int nargs_to_exclude             = 22;
 
-        const int args_to_exclude_buffer_size[14]
+        const int args_to_exclude_buffer_size[nargs_to_exclude_buffer_size]
             = {8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21};
-        const int args_to_exclude_nnzb[12] = {8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19};
-        const int args_to_exclude[17]
-            = {8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24};
+        const int args_to_exclude_nnzb[nargs_to_exclude_nnzb]
+            = {8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 24};
+        const int args_to_exclude[nargs_to_exclude] = {8,  9,  10, 11, 12, 13, 14, 15, 16, 17, 18,
+                                                       19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 30};
 
         const T* alpha = (const T*)nullptr;
         const T* beta  = (const T*)nullptr;
@@ -124,16 +132,15 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
         const T*                  bsr_val_D     = (const T*)nullptr;
         const rocsparse_int*      bsr_row_ptr_D = (const rocsparse_int*)nullptr;
         const rocsparse_int*      bsr_col_ind_D = (const rocsparse_int*)nullptr;
+        select_bad_arg_analysis(rocsparse_bsrgemm_buffer_size<T>,
+                                nargs_to_exclude_buffer_size,
+                                args_to_exclude_buffer_size,
+                                PARAMS_BUFFER_SIZE);
 
-        auto_testing_bad_arg(rocsparse_bsrgemm_buffer_size<T>,
-                             nargs_to_exclude_buffer_size,
-                             args_to_exclude_buffer_size,
-                             PARAMS_BUFFER_SIZE);
-
-        auto_testing_bad_arg(
+        select_bad_arg_analysis(
             rocsparse_bsrgemm_nnzb, nargs_to_exclude_nnzb, args_to_exclude_nnzb, PARAMS_NNZB);
 
-        auto_testing_bad_arg(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -141,13 +148,14 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
     // ###############################################
     {
         // In this scenario matrices A != B != nullptr and D == nullptr
-        int nargs_to_exclude_buffer_size = 5;
-        int nargs_to_exclude_nnzb        = 4;
-        int nargs_to_exclude             = 6;
+        static constexpr int nargs_to_exclude_buffer_size = 6;
+        static constexpr int nargs_to_exclude_nnzb        = 5;
+        static constexpr int nargs_to_exclude             = 7;
 
-        const int args_to_exclude_buffer_size[5] = {17, 18, 19, 20, 21};
-        const int args_to_exclude_nnzb[4]        = {16, 17, 18, 19};
-        const int args_to_exclude[6]             = {19, 20, 21, 22, 23, 24};
+        const int args_to_exclude_buffer_size[nargs_to_exclude_buffer_size]
+            = {8, 17, 18, 19, 20, 21};
+        const int args_to_exclude_nnzb[nargs_to_exclude_nnzb] = {16, 17, 18, 19, 24};
+        const int args_to_exclude[nargs_to_exclude]           = {19, 20, 21, 22, 23, 24, 30};
 
         const T* alpha = &h_alpha;
         const T* beta  = (const T*)nullptr;
@@ -174,14 +182,13 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
         const rocsparse_int*      bsr_row_ptr_D = (const rocsparse_int*)nullptr;
         const rocsparse_int*      bsr_col_ind_D = (const rocsparse_int*)nullptr;
 
-        auto_testing_bad_arg(rocsparse_bsrgemm_buffer_size<T>,
-                             nargs_to_exclude_buffer_size,
-                             args_to_exclude_buffer_size,
-                             PARAMS_BUFFER_SIZE);
-        auto_testing_bad_arg(
+        select_bad_arg_analysis(rocsparse_bsrgemm_buffer_size<T>,
+                                nargs_to_exclude_buffer_size,
+                                args_to_exclude_buffer_size,
+                                PARAMS_BUFFER_SIZE);
+        select_bad_arg_analysis(
             rocsparse_bsrgemm_nnzb, nargs_to_exclude_nnzb, args_to_exclude_nnzb, PARAMS_NNZB);
-
-        auto_testing_bad_arg(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -190,13 +197,15 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
 
     {
         // In this scenario matrices A == B == nullptr and D != nullptr
-        int nargs_to_exclude_buffer_size = 9;
-        int nargs_to_exclude_nnzb        = 8;
-        int nargs_to_exclude             = 11;
+        static constexpr int nargs_to_exclude_buffer_size = 11;
+        static constexpr int nargs_to_exclude_nnzb        = 9;
+        static constexpr int nargs_to_exclude             = 12;
 
-        const int args_to_exclude_buffer_size[9] = {8, 9, 10, 11, 12, 13, 14, 15, 16};
-        const int args_to_exclude_nnzb[8]        = {8, 9, 10, 11, 12, 13, 14, 15};
-        const int args_to_exclude[11]            = {8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18};
+        const int args_to_exclude_buffer_size[nargs_to_exclude_buffer_size]
+            = {6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17};
+        const int args_to_exclude_nnzb[nargs_to_exclude_nnzb] = {8, 9, 10, 11, 12, 13, 14, 15, 24};
+        const int args_to_exclude[nargs_to_exclude]
+            = {8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 30};
 
         const T* alpha = (const T*)nullptr;
         const T* beta  = &h_beta;
@@ -223,14 +232,13 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
         const rocsparse_int*      bsr_row_ptr_D = (const rocsparse_int*)0x4;
         const rocsparse_int*      bsr_col_ind_D = (const rocsparse_int*)0x4;
 
-        auto_testing_bad_arg(rocsparse_bsrgemm_buffer_size<T>,
-                             nargs_to_exclude_buffer_size,
-                             args_to_exclude_buffer_size,
-                             PARAMS_BUFFER_SIZE);
-        auto_testing_bad_arg(
+        select_bad_arg_analysis(rocsparse_bsrgemm_buffer_size<T>,
+                                nargs_to_exclude_buffer_size,
+                                args_to_exclude_buffer_size,
+                                PARAMS_BUFFER_SIZE);
+        select_bad_arg_analysis(
             rocsparse_bsrgemm_nnzb, nargs_to_exclude_nnzb, args_to_exclude_nnzb, PARAMS_NNZB);
-
-        auto_testing_bad_arg(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -270,12 +278,12 @@ void testing_bsrgemm_bad_arg(const Arguments& arg)
         const rocsparse_int*      bsr_row_ptr_D = (const rocsparse_int*)0x4;
         const rocsparse_int*      bsr_col_ind_D = (const rocsparse_int*)0x4;
 
-        auto_testing_bad_arg(rocsparse_bsrgemm_buffer_size<T>,
-                             nargs_to_exclude_buffer_size,
-                             args_to_exclude_buffer_size,
-                             PARAMS_BUFFER_SIZE);
-        auto_testing_bad_arg(rocsparse_bsrgemm_nnzb, PARAMS_NNZB);
-        auto_testing_bad_arg(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_bsrgemm_buffer_size<T>,
+                                nargs_to_exclude_buffer_size,
+                                args_to_exclude_buffer_size,
+                                PARAMS_BUFFER_SIZE);
+        bad_arg_analysis(rocsparse_bsrgemm_nnzb, PARAMS_NNZB);
+        select_bad_arg_analysis(rocsparse_bsrgemm<T>, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // Not implemented cases.
@@ -390,15 +398,9 @@ void testing_bsrgemm(const Arguments& arg)
     rocsparse_index_base  baseD     = arg.baseD;
     static constexpr bool full_rank = false;
 
-    rocsparse_int Mb = -1;
-    rocsparse_int Nb = -1;
-    rocsparse_int Kb = -1;
-    if(block_dim > 0)
-    {
-        Mb = (M + block_dim - 1) / block_dim;
-        Nb = (N + block_dim - 1) / block_dim;
-        Kb = (K + block_dim - 1) / block_dim;
-    }
+    rocsparse_int Mb = (M + block_dim - 1) / block_dim;
+    rocsparse_int Nb = (N + block_dim - 1) / block_dim;
+    rocsparse_int Kb = (K + block_dim - 1) / block_dim;
 
     T v_alpha = arg.get_alpha<T>(), v_beta = arg.get_beta<T>();
 
@@ -487,45 +489,6 @@ void testing_bsrgemm(const Arguments& arg)
     }
     }
 
-    // Argument sanity check before allocating invalid memory
-    if((Mb <= 0 || Nb <= 0 || Kb <= 0 || block_dim <= 0)
-       || scenario == testing_bsrgemm_scenario_none)
-    {
-        device_gebsr_matrix<T> d_A, d_B, d_C, d_D;
-        d_A.define(dir, Mb, Kb, 0, block_dim, block_dim, baseA);
-        d_B.define(dir, Kb, Nb, 0, block_dim, block_dim, baseB);
-        d_C.define(dir, Mb, Nb, 0, block_dim, block_dim, baseC);
-        d_D.define(dir, Mb, Nb, 0, block_dim, block_dim, baseD);
-
-        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
-        size_t           buffer_size;
-        rocsparse_int    nnzb;
-        rocsparse_status status_1 = rocsparse_bsrgemm_buffer_size<T>(
-            PARAMS_BUFFER_SIZE(h_alpha, h_beta, d_A, d_B, d_C, d_D, buffer_size));
-        rocsparse_status status_2 = rocsparse_bsrgemm_nnzb(PARAMS_NNZB(d_A, d_B, d_C, d_D, &nnzb));
-        rocsparse_status status_3
-            = rocsparse_bsrgemm<T>(PARAMS(h_alpha, h_beta, d_A, d_B, d_C, d_D));
-
-        EXPECT_ROCSPARSE_STATUS(
-            status_1,
-            (Mb < 0 || Nb < 0 || Kb < 0 || block_dim <= 0) ? rocsparse_status_invalid_size
-            : scenario == testing_bsrgemm_scenario_none    ? rocsparse_status_invalid_pointer
-                                                           : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(
-            status_2,
-            (Mb < 0 || Nb < 0 || Kb < 0 || block_dim <= 0) ? rocsparse_status_invalid_size
-            : scenario == testing_bsrgemm_scenario_none    ? rocsparse_status_invalid_pointer
-                                                           : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(
-            status_3,
-            (Mb < 0 || Nb < 0 || Kb < 0 || block_dim <= 0) ? rocsparse_status_invalid_size
-            : scenario == testing_bsrgemm_scenario_none    ? rocsparse_status_invalid_pointer
-                                                           : rocsparse_status_success);
-        return;
-    }
-
     // Declare host and device objects.
     host_gebsr_matrix<T>   h_A, h_B, h_C, h_D;
     device_gebsr_matrix<T> d_A;
@@ -633,6 +596,11 @@ void testing_bsrgemm(const Arguments& arg)
 
     if(arg.unit_check)
     {
+        // normalize
+        rocsparse_vector_utils<T>::normalize(h_A.val);
+
+        d_A.val.transfer_from(h_A.val);
+
         // Host calculation
         rocsparse_int out_nnzb;
 
@@ -685,10 +653,16 @@ void testing_bsrgemm(const Arguments& arg)
         // GPU with pointer mode host
         host_scalar<rocsparse_int> h_out_nnz;
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
+
         CHECK_ROCSPARSE_ERROR(rocsparse_bsrgemm_nnzb(PARAMS_NNZB(d_A, d_B, d_C, d_D, h_out_nnz)));
+
         d_C.define(
             d_C.dir, d_C.mb, d_C.nb, *h_out_nnz, d_C.row_block_dim, d_C.col_block_dim, d_C.base);
         CHECK_ROCSPARSE_ERROR(rocsparse_bsrgemm<T>(PARAMS(h_alpha, h_beta, d_A, d_B, d_C, d_D)));
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("C pointer mode host", d_C);
+        }
         h_C.near_check(d_C);
 
         // GPU with pointer mode host
@@ -699,6 +673,10 @@ void testing_bsrgemm(const Arguments& arg)
         d_C.define(
             d_C.dir, d_C.mb, d_C.nb, *h_out_nnz2, d_C.row_block_dim, d_C.col_block_dim, d_C.base);
         CHECK_ROCSPARSE_ERROR(rocsparse_bsrgemm<T>(PARAMS(d_alpha, d_beta, d_A, d_B, d_C, d_D)));
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("C pointer mode device", d_C);
+        }
         h_C.near_check(d_C);
     }
 
@@ -758,35 +736,35 @@ void testing_bsrgemm(const Arguments& arg)
             sprintf(beta, "%s", ss.str().c_str());
         }
 
-        display_timing_info("opA",
+        display_timing_info(display_key_t::trans_A,
                             rocsparse_operation2string(transA),
-                            "opB",
+                            display_key_t::trans_B,
                             rocsparse_operation2string(transB),
-                            "Mb",
+                            display_key_t::Mb,
                             Mb,
-                            "Nb",
+                            display_key_t::Nb,
                             Nb,
-                            "Kb",
+                            display_key_t::Kb,
                             Kb,
-                            "block_dim",
+                            display_key_t::bdim,
                             block_dim,
-                            "nnzb_A",
+                            display_key_t::nnzb_A,
                             d_A.nnzb,
-                            "nnzb_B",
+                            display_key_t::nnzb_B,
                             d_B.nnzb,
-                            "nnzb_C",
+                            display_key_t::nnzb_C,
                             d_C.nnzb,
-                            "nnzb_D",
+                            display_key_t::nnzb_D,
                             d_D.nnzb,
-                            "alpha",
+                            display_key_t::alpha,
                             alpha,
-                            "beta",
+                            display_key_t::beta,
                             beta,
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
     }
 
