@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -53,6 +53,7 @@ void testing_bsrilu0_bad_arg(const Arguments& arg)
     rocsparse_mat_info        info        = local_info;
     rocsparse_analysis_policy analysis    = rocsparse_analysis_policy_force;
     rocsparse_solve_policy    solve       = rocsparse_solve_policy_auto;
+    rocsparse_solve_policy    policy      = rocsparse_solve_policy_auto;
     size_t*                   buffer_size = &h_buffer_size;
     void*                     temp_buffer = (void*)0x4;
 
@@ -63,22 +64,31 @@ void testing_bsrilu0_bad_arg(const Arguments& arg)
     handle, dir, mb, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, block_dim, info, analysis, \
         solve, temp_buffer
 
-#define PARAMS                                                                               \
-    handle, dir, mb, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, block_dim, info, solve, \
+#define PARAMS                                                                                \
+    handle, dir, mb, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, block_dim, info, policy, \
         temp_buffer
 
-    auto_testing_bad_arg(rocsparse_bsrilu0_buffer_size<T>, PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(rocsparse_bsrilu0_analysis<T>, PARAMS_ANALYSIS);
-    auto_testing_bad_arg(rocsparse_bsrilu0<T>, PARAMS);
+    bad_arg_analysis(rocsparse_bsrilu0_buffer_size<T>, PARAMS_BUFFER_SIZE);
+    bad_arg_analysis(rocsparse_bsrilu0_analysis<T>, PARAMS_ANALYSIS);
+    bad_arg_analysis(rocsparse_bsrilu0<T>, PARAMS);
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_unsorted));
     EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_buffer_size<T>(PARAMS_BUFFER_SIZE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_analysis<T>(PARAMS_ANALYSIS),
-                            rocsparse_status_not_implemented);
-    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0<T>(PARAMS), rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0<T>(PARAMS), rocsparse_status_requires_sorted_storage);
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_sorted));
 
+    // block_dim == 0
+    block_dim = 0;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_buffer_size<T>(PARAMS_BUFFER_SIZE),
+                            rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_analysis<T>(PARAMS_ANALYSIS),
+                            rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0<T>(PARAMS), rocsparse_status_invalid_size);
+    block_dim = safe_size;
+
 #undef PARAMS_BUFFER_SIZE
 #undef PARAMS_ANALYSIS
 #undef PARAMS
@@ -157,13 +167,8 @@ void testing_bsrilu0(const Arguments& arg)
 
     T h_boost_val = arg.get_boostval<T>();
 
-    rocsparse_int Mb = -1;
-    rocsparse_int Nb = -1;
-    if(block_dim > 0)
-    {
-        Mb = (M + block_dim - 1) / block_dim;
-        Nb = (N + block_dim - 1) / block_dim;
-    }
+    rocsparse_int Mb = (M + block_dim - 1) / block_dim;
+    rocsparse_int Nb = (N + block_dim - 1) / block_dim;
 
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
@@ -177,62 +182,6 @@ void testing_bsrilu0(const Arguments& arg)
     // Set matrix index base
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_index_base(descr, base));
 
-    // Argument sanity check before allocating invalid memory
-    if(Mb <= 0 || block_dim <= 0)
-    {
-        static const size_t safe_size = 100;
-        size_t              buffer_size;
-        rocsparse_int       pivot;
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_buffer_size<T>(handle,
-                                                                 direction,
-                                                                 Mb,
-                                                                 safe_size,
-                                                                 descr,
-                                                                 nullptr,
-                                                                 nullptr,
-                                                                 nullptr,
-                                                                 safe_size,
-                                                                 info,
-                                                                 &buffer_size),
-                                (Mb < 0 || block_dim <= 0) ? rocsparse_status_invalid_size
-                                                           : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_analysis<T>(handle,
-                                                              direction,
-                                                              Mb,
-                                                              safe_size,
-                                                              descr,
-                                                              nullptr,
-                                                              nullptr,
-                                                              nullptr,
-                                                              safe_size,
-                                                              info,
-                                                              apol,
-                                                              spol,
-                                                              nullptr),
-                                (Mb < 0 || block_dim <= 0) ? rocsparse_status_invalid_size
-                                                           : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0<T>(handle,
-                                                     direction,
-                                                     Mb,
-                                                     safe_size,
-                                                     descr,
-                                                     nullptr,
-                                                     nullptr,
-                                                     nullptr,
-                                                     safe_size,
-                                                     info,
-                                                     spol,
-                                                     nullptr),
-                                (Mb < 0 || block_dim <= 0) ? rocsparse_status_invalid_size
-                                                           : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_zero_pivot(handle, info, &pivot),
-                                rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrilu0_clear(handle, info), rocsparse_status_success);
-
-        return;
-    }
-
     if(M != N)
     {
         return;
@@ -453,9 +402,29 @@ void testing_bsrilu0(const Arguments& arg)
         // Check solution vector if no pivot has been found
         if(hanalysis_pivot_gold[0] == -1 && hsolve_pivot_gold[0] == -1)
         {
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save(
+                    "P pointer mode host", hbsr_val_1, "P pointer mode device", hbsr_val_2);
+            }
+
             hbsr_val_gold.near_check(hbsr_val_1);
             hbsr_val_gold.near_check(hbsr_val_2);
         }
+        else
+        {
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("Pivot analysis pointer mode host",
+                                                hanalysis_pivot_1,
+                                                "Pivot analysis pointer mode device",
+                                                hanalysis_pivot_2,
+                                                "Pivot solve pointer mode host",
+                                                hsolve_pivot_1,
+                                                "Pivot solve pointer mode device",
+                                                hsolve_pivot_2);
+            }
+        }
     }
 
     if(arg.timing)
@@ -581,25 +550,25 @@ void testing_bsrilu0(const Arguments& arg)
 
         double gpu_gbyte = get_gpu_gbyte(gpu_solve_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "nnzb",
+                            display_key_t::nnzb,
                             nnzb,
-                            "block_dim",
+                            display_key_t::bdim,
                             block_dim,
-                            "pivot",
+                            display_key_t::pivot,
                             pivot,
-                            "direction",
+                            display_key_t::bdir,
                             rocsparse_direction2string(direction),
-                            "analysis policy",
+                            display_key_t::analysis_policy,
                             rocsparse_analysis2string(apol),
-                            "solve policy",
+                            display_key_t::solve_policy,
                             rocsparse_solve2string(spol),
-                            "analysis time",
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
     }
 
