@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -64,6 +64,7 @@ void testing_bsrsm_bad_arg(const Arguments& arg)
     rocsparse_mat_info        info        = local_info;
     rocsparse_analysis_policy analysis    = rocsparse_analysis_policy_force;
     rocsparse_solve_policy    solve       = rocsparse_solve_policy_auto;
+    rocsparse_solve_policy    policy      = rocsparse_solve_policy_auto;
     size_t*                   buffer_size = &h_buffer_size;
     void*                     temp_buffer = (void*)0x4;
     rocsparse_int*            position    = &h_position;
@@ -72,7 +73,7 @@ void testing_bsrsm_bad_arg(const Arguments& arg)
 #define PARAMS_BUFFER_SIZE                                                                   \
     handle, dir, trans_A, trans_X, mb, nrhs, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, \
         block_dim, info, buffer_size
-    auto_testing_bad_arg(rocsparse_bsrsm_buffer_size<T>, PARAMS_BUFFER_SIZE);
+    bad_arg_analysis(rocsparse_bsrsm_buffer_size<T>, PARAMS_BUFFER_SIZE);
 
     // Invalid size
     {
@@ -87,7 +88,7 @@ void testing_bsrsm_bad_arg(const Arguments& arg)
 #define PARAMS_ANALYSIS                                                                      \
     handle, dir, trans_A, trans_X, mb, nrhs, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, \
         block_dim, info, analysis, solve, temp_buffer
-    auto_testing_bad_arg(rocsparse_bsrsm_analysis<T>, PARAMS_ANALYSIS);
+    bad_arg_analysis(rocsparse_bsrsm_analysis<T>, PARAMS_ANALYSIS);
 
     // Invalid size
     {
@@ -101,8 +102,8 @@ void testing_bsrsm_bad_arg(const Arguments& arg)
     // bsrsm_solve
 #define PARAMS_SOLVE                                                                   \
     handle, dir, trans_A, trans_X, mb, nrhs, nnzb, alpha, descr, bsr_val, bsr_row_ptr, \
-        bsr_col_ind, block_dim, info, B, ldb, X, ldx, solve, temp_buffer
-    auto_testing_bad_arg(rocsparse_bsrsm_solve<T>, PARAMS_SOLVE);
+        bsr_col_ind, block_dim, info, B, ldb, X, ldx, policy, temp_buffer
+    bad_arg_analysis(rocsparse_bsrsm_solve<T>, PARAMS_SOLVE);
 
     // Invalid size
     {
@@ -146,10 +147,10 @@ void testing_bsrsm_bad_arg(const Arguments& arg)
     }
 
     // bsrsm_zero_pivot
-    auto_testing_bad_arg(rocsparse_bsrsm_zero_pivot, handle, info, position);
+    bad_arg_analysis(rocsparse_bsrsm_zero_pivot, handle, info, position);
 
     // bsrsm_clear
-    auto_testing_bad_arg(rocsparse_bsrsm_clear, handle, info);
+    bad_arg_analysis(rocsparse_bsrsm_clear, handle, info);
 
     // Matrix types different from general
     for(auto val : rocsparse_matrix_type_t::values)
@@ -169,11 +170,11 @@ void testing_bsrsm_bad_arg(const Arguments& arg)
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_unsorted));
     EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_buffer_size<T>(PARAMS_BUFFER_SIZE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_analysis<T>(PARAMS_ANALYSIS),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_solve<T>(PARAMS_SOLVE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_sorted));
 
 #undef PARAMS_BUFFER_SIZE
@@ -254,12 +255,10 @@ void testing_bsrsm(const Arguments& arg)
     rocsparse_index_base      base      = arg.baseA;
 
     // BSR dimension
-    rocsparse_int mb = (block_dim > 0) ? (m + block_dim - 1) / block_dim : -1;
+    rocsparse_int mb = (m + block_dim - 1) / block_dim;
 
     // Scalar
-    host_scalar<T> h_alpha;
-
-    *h_alpha = arg.get_alpha<T>();
+    host_scalar<T> h_alpha(arg.get_alpha<T>());
 
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
@@ -279,84 +278,6 @@ void testing_bsrsm(const Arguments& arg)
     // Set matrix index base
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_index_base(descr, base));
 
-    // Argument sanity check before allocating invalid memory
-    if(mb <= 0 || nrhs <= 0)
-    {
-        static const size_t safe_size = 100;
-        size_t              buffer_size;
-        rocsparse_int       pivot;
-
-        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_buffer_size<T>(handle,
-                                                               dir,
-                                                               trans_A,
-                                                               trans_X,
-                                                               mb,
-                                                               nrhs,
-                                                               safe_size,
-                                                               descr,
-                                                               nullptr,
-                                                               nullptr,
-                                                               nullptr,
-                                                               block_dim,
-                                                               info,
-                                                               &buffer_size),
-                                (mb < 0 || nrhs < 0 || block_dim <= 0)
-                                    ? rocsparse_status_invalid_size
-                                    : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_analysis<T>(handle,
-                                                            dir,
-                                                            trans_A,
-                                                            trans_X,
-                                                            mb,
-                                                            nrhs,
-                                                            safe_size,
-                                                            descr,
-                                                            nullptr,
-                                                            nullptr,
-                                                            nullptr,
-                                                            block_dim,
-                                                            info,
-                                                            apol,
-                                                            spol,
-                                                            nullptr),
-                                (mb < 0 || nrhs < 0 || block_dim <= 0)
-                                    ? rocsparse_status_invalid_size
-                                    : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_solve<T>(handle,
-                                                         dir,
-                                                         trans_A,
-                                                         trans_X,
-                                                         mb,
-                                                         nrhs,
-                                                         safe_size,
-                                                         h_alpha,
-                                                         descr,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr,
-                                                         block_dim,
-                                                         info,
-                                                         nullptr,
-                                                         safe_size,
-                                                         nullptr,
-                                                         safe_size,
-                                                         spol,
-                                                         nullptr),
-                                (mb < 0 || nrhs < 0 || block_dim <= 0)
-                                    ? rocsparse_status_invalid_size
-                                    : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_zero_pivot(handle, info, &pivot),
-                                rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsm_clear(handle, info), rocsparse_status_success);
-
-        return;
-    }
-
     // Allocate host memory for BSR matrix A
     rocsparse_matrix_factory<T> matrix_factory(arg);
 
@@ -623,38 +544,38 @@ void testing_bsrsm(const Arguments& arg)
                                   ? std::min(*analysis_pivot_gold, *solve_pivot_gold)
                                   : std::max(*analysis_pivot_gold, *solve_pivot_gold);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             m,
-                            "nnz",
+                            display_key_t::nnz,
                             size_t(hA.nnzb) * block_dim * block_dim,
-                            "nrhs",
+                            display_key_t::nrhs,
                             nrhs,
-                            "block_dim",
+                            display_key_t::bdim,
                             block_dim,
-                            "alpha",
+                            display_key_t::alpha,
                             *h_alpha,
-                            "pivot",
+                            display_key_t::pivot,
                             pivot,
-                            "op(A)",
+                            display_key_t::trans_A,
                             rocsparse_operation2string(trans_A),
-                            "op(B/X)",
+                            display_key_t::trans_X,
                             rocsparse_operation2string(trans_X),
-                            "diag_type",
+                            display_key_t::diag_type,
                             rocsparse_diagtype2string(diag),
-                            "fill",
+                            display_key_t::fill_mode,
                             rocsparse_fillmode2string(uplo),
-                            "analysis",
+                            display_key_t::analysis_policy,
                             rocsparse_analysis2string(apol),
-                            "solve",
+                            display_key_t::solve_policy,
                             rocsparse_solve2string(spol),
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            "analysis_msec",
-                            gpu_analysis_time_used / 1e3,
-                            "solve_msec",
-                            gpu_solve_time_used / 1e3);
+                            display_key_t::analysis_time_ms,
+                            get_gpu_time_msec(gpu_analysis_time_used),
+                            display_key_t::time_ms,
+                            get_gpu_time_msec(gpu_solve_time_used));
     }
 
     // Clear bsrsm meta data
