@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,44 +43,45 @@ void testing_bsrsv_bad_arg(const Arguments& arg)
     // Create matrix info
     rocsparse_local_mat_info local_info;
 
-    rocsparse_handle          handle      = local_handle;
-    rocsparse_direction       dir         = rocsparse_direction_row;
-    rocsparse_operation       trans       = rocsparse_operation_none;
-    rocsparse_int             mb          = safe_size;
-    rocsparse_int             nnzb        = safe_size;
-    const T*                  alpha       = &h_alpha;
-    const rocsparse_mat_descr descr       = local_descr;
-    const T*                  bsr_val     = (const T*)0x4;
-    const rocsparse_int*      bsr_row_ptr = (const rocsparse_int*)0x4;
-    const rocsparse_int*      bsr_col_ind = (const rocsparse_int*)0x4;
-    rocsparse_mat_info        info        = local_info;
-    rocsparse_int             block_dim   = safe_size;
-    const T*                  x           = (const T*)0x4;
-    T*                        y           = (T*)0x4;
-    rocsparse_analysis_policy analysis    = rocsparse_analysis_policy_reuse;
-    rocsparse_solve_policy    solve       = rocsparse_solve_policy_auto;
-    size_t                    buffer_size;
-    size_t*                   p_buffer_size = &buffer_size;
-    void*                     temp_buffer   = (void*)0x4;
+    rocsparse_handle          handle            = local_handle;
+    rocsparse_direction       dir               = rocsparse_direction_row;
+    rocsparse_operation       trans             = rocsparse_operation_none;
+    rocsparse_int             mb                = safe_size;
+    rocsparse_int             nnzb              = safe_size;
+    const T*                  alpha_device_host = &h_alpha;
+    const rocsparse_mat_descr descr             = local_descr;
+    const T*                  bsr_val           = (const T*)0x4;
+    const rocsparse_int*      bsr_row_ptr       = (const rocsparse_int*)0x4;
+    const rocsparse_int*      bsr_col_ind       = (const rocsparse_int*)0x4;
+    rocsparse_mat_info        info              = local_info;
+    rocsparse_int             block_dim         = safe_size;
+    const T*                  x                 = (const T*)0x4;
+    T*                        y                 = (T*)0x4;
+    rocsparse_analysis_policy analysis          = rocsparse_analysis_policy_reuse;
+    rocsparse_solve_policy    policy            = rocsparse_solve_policy_auto;
+    rocsparse_solve_policy    solve             = rocsparse_solve_policy_auto;
+    size_t                    buffer_size_value;
+    size_t*                   buffer_size = &buffer_size_value;
+    void*                     temp_buffer = (void*)0x4;
 
 #define PARAMS_BUFFER_SIZE                                                                   \
     handle, dir, trans, mb, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, block_dim, info, \
-        p_buffer_size
+        buffer_size
 
 #define PARAMS_ANALYSIS                                                                      \
     handle, dir, trans, mb, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, block_dim, info, \
         analysis, solve, temp_buffer
 
-#define PARAMS_SOLVE                                                                          \
-    handle, dir, trans, mb, nnzb, alpha, descr, bsr_val, bsr_row_ptr, bsr_col_ind, block_dim, \
-        info, x, y, solve, temp_buffer
+#define PARAMS_SOLVE                                                                           \
+    handle, dir, trans, mb, nnzb, alpha_device_host, descr, bsr_val, bsr_row_ptr, bsr_col_ind, \
+        block_dim, info, x, y, policy, temp_buffer
 
     //
     // Call solve before analysis
     //
-    auto_testing_bad_arg(rocsparse_bsrsv_buffer_size<T>, PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(rocsparse_bsrsv_analysis<T>, PARAMS_ANALYSIS);
-    auto_testing_bad_arg(rocsparse_bsrsv_solve<T>, PARAMS_SOLVE);
+    bad_arg_analysis(rocsparse_bsrsv_buffer_size<T>, PARAMS_BUFFER_SIZE);
+    bad_arg_analysis(rocsparse_bsrsv_analysis<T>, PARAMS_ANALYSIS);
+    bad_arg_analysis(rocsparse_bsrsv_solve<T>, PARAMS_SOLVE);
 
     //
     // Not implemented cases.
@@ -116,11 +117,18 @@ void testing_bsrsv_bad_arg(const Arguments& arg)
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_unsorted));
     EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_analysis<T>(PARAMS_ANALYSIS),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_solve<T>(PARAMS_SOLVE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_sorted));
 
+    // block_dim == 0
+    block_dim = 0;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_analysis<T>(PARAMS_ANALYSIS),
+                            rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_solve<T>(PARAMS_SOLVE), rocsparse_status_invalid_size);
+    block_dim = safe_size;
+
 #undef PARAMS_BUFFER_SIZE
 #undef PARAMS_ANALYSIS
 #undef PARAMS_SOLVE
@@ -145,7 +153,7 @@ void testing_bsrsv_bad_arg(const Arguments& arg)
                                                      trans,
                                                      mb,
                                                      nnzb,
-                                                     alpha,
+                                                     alpha_device_host,
                                                      descr,
                                                      nullptr,
                                                      bsr_row_ptr,
@@ -154,7 +162,7 @@ void testing_bsrsv_bad_arg(const Arguments& arg)
                                                      info,
                                                      x,
                                                      y,
-                                                     solve,
+                                                     policy,
                                                      temp_buffer),
                             rocsparse_status_invalid_pointer);
 }
@@ -209,53 +217,10 @@ void testing_bsrsv(const Arguments& arg)
     handle, dir, trans, A_.mb, A_.nnzb, alpha_, descr, A_.val, A_.ptr, A_.ind, A_.row_block_dim, \
         info, x_, y_, spol, dbuffer
 
-    // Set matrix diag type
-    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_diag_type(descr, diag));
-
-    // Set matrix fill mode
-    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_fill_mode(descr, uplo));
-
-    // Set matrix index base
-    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_index_base(descr, base));
-
     // BSR dimensions
-    const size_t  safe_size = 16;
-    rocsparse_int mb        = (block_dim > 0) ? ((M + block_dim - 1) / block_dim) : safe_size;
-    rocsparse_int nb        = (block_dim > 0) ? ((N + block_dim - 1) / block_dim) : safe_size;
+    rocsparse_int mb = (M + block_dim - 1) / block_dim;
+    rocsparse_int nb = (N + block_dim - 1) / block_dim;
 
-    // Argument sanity check before allocating invalid memory
-    if(mb <= 0 || nb <= 0 || M <= 0 || N <= 0 || block_dim <= 0)
-    {
-        rocsparse_int          pivot;
-        size_t                 buffer_size;
-        device_vector<T>       dx, dy, dbuffer;
-        device_gebsr_matrix<T> dA;
-        dA.mb            = mb;
-        dA.nb            = nb;
-        dA.nnzb          = 10;
-        dA.row_block_dim = block_dim;
-        dA.col_block_dim = block_dim;
-
-        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_buffer_size<T>(PARAMS_BUFFER_SIZE(dA)),
-                                (mb < 0 || nb < 0 || block_dim < 0) ? rocsparse_status_invalid_size
-                                                                    : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_analysis<T>(PARAMS_ANALYSIS(dA)),
-                                (mb < 0 || nb < 0 || block_dim < 0) ? rocsparse_status_invalid_size
-                                                                    : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
-                                (mb < 0 || nb < 0 || block_dim < 0) ? rocsparse_status_invalid_size
-                                                                    : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_zero_pivot(handle, info, &pivot),
-                                rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_clear(handle, info), rocsparse_status_success);
-        return;
-    }
     // Allocate host memory for matrix
     host_gebsr_matrix<T>   hA;
     device_gebsr_matrix<T> dA;
@@ -275,7 +240,6 @@ void testing_bsrsv(const Arguments& arg)
     // Allocate host memory for vectors
     host_dense_matrix<T> hx(M, 1);
     rocsparse_matrix_utils::init_exact(hx);
-    //    rocsparse_init<T>(hx.val,hx.m,1,1,0,1,static_cast<T>(-1),static_cast<T>(1));
 
     device_dense_matrix<T>     dx(hx), dy(M, 1);
     host_scalar<rocsparse_int> h_analysis_pivot, h_solve_pivot;
@@ -323,9 +287,9 @@ void testing_bsrsv(const Arguments& arg)
             //
             // Call before analysis
             //
-            EXPECT_ROCSPARSE_STATUS(
-                testing::rocsparse_bsrsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
-                rocsparse_status_invalid_pointer);
+            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
+                                    (M == 0) ? rocsparse_status_success
+                                             : rocsparse_status_invalid_pointer);
 
             // Call it twice.
             CHECK_ROCSPARSE_ERROR(rocsparse_bsrsv_analysis<T>(PARAMS_ANALYSIS(dA)));
@@ -374,9 +338,9 @@ void testing_bsrsv(const Arguments& arg)
             //
             // Call before analysis
             //
-            EXPECT_ROCSPARSE_STATUS(
-                testing::rocsparse_bsrsv_solve<T>(PARAMS_SOLVE(d_alpha, dA, dx, dy)),
-                rocsparse_status_invalid_pointer);
+            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
+                                    (M == 0) ? rocsparse_status_success
+                                             : rocsparse_status_invalid_pointer);
 
             // Call it twice.
             CHECK_ROCSPARSE_ERROR(rocsparse_bsrsv_analysis<T>(PARAMS_ANALYSIS(dA)));
@@ -515,31 +479,31 @@ void testing_bsrsv(const Arguments& arg)
         double gpu_gflops = get_gpu_gflops(gpu_solve_time_used, gflop_count);
         double gpu_gbyte  = get_gpu_gbyte(gpu_solve_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "nnz",
+                            display_key_t::nnz,
                             size_t(dA.nnzb) * dA.row_block_dim * dA.row_block_dim,
-                            "alpha",
+                            display_key_t::alpha,
                             h_alpha,
-                            "pivot",
+                            display_key_t::pivot,
                             std::min(*h_analysis_pivot, *h_solve_pivot),
-                            "operation",
+                            display_key_t::trans,
                             rocsparse_operation2string(trans),
-                            "diag_type",
+                            display_key_t::diag_type,
                             rocsparse_diagtype2string(diag),
-                            "fill_mode",
+                            display_key_t::fill_mode,
                             rocsparse_fillmode2string(uplo),
-                            "analysis_policy",
+                            display_key_t::analysis_policy,
                             rocsparse_analysis2string(apol),
-                            "solve_policy",
+                            display_key_t::solve_policy,
                             rocsparse_solve2string(spol),
-                            "analysis time",
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
     }
 
