@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,7 +30,7 @@ void testing_check_matrix_csr_bad_arg(const Arguments& arg)
     rocsparse_int          m           = 100;
     rocsparse_int          n           = 100;
     rocsparse_int          nnz         = 100;
-    rocsparse_index_base   base        = rocsparse_index_base_zero;
+    rocsparse_index_base   idx_base    = rocsparse_index_base_zero;
     rocsparse_matrix_type  matrix_type = rocsparse_matrix_type_general;
     rocsparse_fill_mode    uplo        = rocsparse_fill_mode_lower;
     rocsparse_storage_mode storage     = rocsparse_storage_mode_sorted;
@@ -39,26 +39,27 @@ void testing_check_matrix_csr_bad_arg(const Arguments& arg)
     rocsparse_local_handle local_handle;
     rocsparse_handle       handle = local_handle;
 
-    const rocsparse_int*  csr_row_ptr = (const rocsparse_int*)0x4;
-    const rocsparse_int*  csr_col_ind = (const rocsparse_int*)0x4;
-    const T*              csr_val     = (const T*)0x4;
-    void*                 temp_buffer = (void*)0x4;
-    size_t                buffer_size;
-    rocsparse_data_status data_status;
+    const rocsparse_int*   csr_row_ptr = (const rocsparse_int*)0x4;
+    const rocsparse_int*   csr_col_ind = (const rocsparse_int*)0x4;
+    const T*               csr_val     = (const T*)0x4;
+    void*                  temp_buffer = (void*)0x4;
+    size_t*                buffer_size = (size_t*)0x4;
+    rocsparse_data_status* data_status = (rocsparse_data_status*)0x4;
 
     int       nargs_to_exclude   = 3;
     const int args_to_exclude[3] = {4, 5, 6};
-#define PARAMS_BUFFER_SIZE                                                                  \
-    handle, m, n, nnz, csr_val, csr_row_ptr, csr_col_ind, base, matrix_type, uplo, storage, \
-        &buffer_size
-#define PARAMS                                                                              \
-    handle, m, n, nnz, csr_val, csr_row_ptr, csr_col_ind, base, matrix_type, uplo, storage, \
-        &data_status, temp_buffer
-    auto_testing_bad_arg(rocsparse_check_matrix_csr_buffer_size<T>,
-                         nargs_to_exclude,
-                         args_to_exclude,
-                         PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(rocsparse_check_matrix_csr<T>, nargs_to_exclude, args_to_exclude, PARAMS);
+#define PARAMS_BUFFER_SIZE                                                                      \
+    handle, m, n, nnz, csr_val, csr_row_ptr, csr_col_ind, idx_base, matrix_type, uplo, storage, \
+        buffer_size
+#define PARAMS                                                                                  \
+    handle, m, n, nnz, csr_val, csr_row_ptr, csr_col_ind, idx_base, matrix_type, uplo, storage, \
+        data_status, temp_buffer
+    select_bad_arg_analysis(rocsparse_check_matrix_csr_buffer_size<T>,
+                            nargs_to_exclude,
+                            args_to_exclude,
+                            PARAMS_BUFFER_SIZE);
+    select_bad_arg_analysis(
+        rocsparse_check_matrix_csr<T>, nargs_to_exclude, args_to_exclude, PARAMS);
 #undef PARAMS_BUFFER_SIZE
 #undef PARAMS
 }
@@ -409,15 +410,15 @@ void testing_check_matrix_csr(const Arguments& arg)
         double gbyte_count = check_matrix_csr_gbyte_count<T>(m, nnz);
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             m,
-                            "N",
+                            display_key_t::N,
                             n,
-                            "nnz",
+                            display_key_t::nnz,
                             nnz,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 
