@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,8 +32,8 @@ void testing_check_matrix_gebsr_bad_arg(const Arguments& arg)
     rocsparse_int          nnzb          = 100;
     rocsparse_int          row_block_dim = 2;
     rocsparse_int          col_block_dim = 2;
-    rocsparse_direction    direction     = rocsparse_direction_row;
-    rocsparse_index_base   base          = rocsparse_index_base_zero;
+    rocsparse_direction    dir           = rocsparse_direction_row;
+    rocsparse_index_base   idx_base      = rocsparse_index_base_zero;
     rocsparse_matrix_type  matrix_type   = rocsparse_matrix_type_general;
     rocsparse_fill_mode    uplo          = rocsparse_fill_mode_lower;
     rocsparse_storage_mode storage       = rocsparse_storage_mode_sorted;
@@ -42,28 +42,52 @@ void testing_check_matrix_gebsr_bad_arg(const Arguments& arg)
     rocsparse_local_handle local_handle;
     rocsparse_handle       handle = local_handle;
 
-    const rocsparse_int*  bsr_row_ptr = (const rocsparse_int*)0x4;
-    const rocsparse_int*  bsr_col_ind = (const rocsparse_int*)0x4;
-    const T*              bsr_val     = (const T*)0x4;
-    void*                 temp_buffer = (void*)0x4;
-    size_t                buffer_size;
-    rocsparse_data_status data_status;
+    const rocsparse_int*   bsr_row_ptr = (const rocsparse_int*)0x4;
+    const rocsparse_int*   bsr_col_ind = (const rocsparse_int*)0x4;
+    const T*               bsr_val     = (const T*)0x4;
+    void*                  temp_buffer = (void*)0x4;
+    size_t*                buffer_size = (size_t*)0x4;
+    rocsparse_data_status* data_status = (rocsparse_data_status*)0x4;
 
     int       nargs_to_exclude   = 3;
     const int args_to_exclude[3] = {7, 8, 9};
 
-#define PARAMS_BUFFER_SIZE                                                               \
-    handle, direction, mb, nb, nnzb, row_block_dim, col_block_dim, bsr_val, bsr_row_ptr, \
-        bsr_col_ind, base, matrix_type, uplo, storage, &buffer_size
-#define PARAMS                                                                           \
-    handle, direction, mb, nb, nnzb, row_block_dim, col_block_dim, bsr_val, bsr_row_ptr, \
-        bsr_col_ind, base, matrix_type, uplo, storage, &data_status, temp_buffer
-    auto_testing_bad_arg(rocsparse_check_matrix_gebsr_buffer_size<T>,
-                         nargs_to_exclude,
-                         args_to_exclude,
-                         PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(
+#define PARAMS_BUFFER_SIZE                                                                      \
+    handle, dir, mb, nb, nnzb, row_block_dim, col_block_dim, bsr_val, bsr_row_ptr, bsr_col_ind, \
+        idx_base, matrix_type, uplo, storage, buffer_size
+#define PARAMS                                                                                  \
+    handle, dir, mb, nb, nnzb, row_block_dim, col_block_dim, bsr_val, bsr_row_ptr, bsr_col_ind, \
+        idx_base, matrix_type, uplo, storage, data_status, temp_buffer
+    select_bad_arg_analysis(rocsparse_check_matrix_gebsr_buffer_size<T>,
+                            nargs_to_exclude,
+                            args_to_exclude,
+                            PARAMS_BUFFER_SIZE);
+    select_bad_arg_analysis(
         rocsparse_check_matrix_gebsr<T>, nargs_to_exclude, args_to_exclude, PARAMS);
+
+    // row_block_dim == 0
+    row_block_dim = 0;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_check_matrix_gebsr_buffer_size<T>(PARAMS_BUFFER_SIZE),
+                            rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_check_matrix_gebsr<T>(PARAMS), rocsparse_status_invalid_size);
+    row_block_dim = 2;
+
+    // col_block_dim == 0
+    col_block_dim = 0;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_check_matrix_gebsr_buffer_size<T>(PARAMS_BUFFER_SIZE),
+                            rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_check_matrix_gebsr<T>(PARAMS), rocsparse_status_invalid_size);
+    col_block_dim = 2;
+
+    // row_block_dim == 0 && col_block_dim == 0
+    row_block_dim = 0;
+    col_block_dim = 0;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_check_matrix_gebsr_buffer_size<T>(PARAMS_BUFFER_SIZE),
+                            rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_check_matrix_gebsr<T>(PARAMS), rocsparse_status_invalid_size);
+    row_block_dim = 2;
+    col_block_dim = 2;
+
 #undef PARAMS_BUFFER_SIZE
 #undef PARAMS
 }
@@ -466,15 +490,15 @@ void testing_check_matrix_gebsr(const Arguments& arg)
             = check_matrix_gebsr_gbyte_count<T>(mb, nnzb, row_block_dim, col_block_dim);
         double gpu_gbyte = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("Mb",
+        display_timing_info(display_key_t::Mb,
                             mb,
-                            "Nb",
+                            display_key_t::Nb,
                             nb,
-                            "nnzb",
+                            display_key_t::nnzb,
                             nnzb,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 
