@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,7 +46,11 @@ void testing_coo2dense_bad_arg(const Arguments& arg)
     rocsparse_int             lda         = safe_size;
 
 #define PARAMS handle, m, n, nnz, descr, coo_val, coo_row_ind, coo_col_ind, A, lda
-    auto_testing_bad_arg(rocsparse_coo2dense<T>, PARAMS);
+    bad_arg_analysis(rocsparse_coo2dense<T>, PARAMS);
+
+    // Check lda < m
+    lda = m / 2;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_coo2dense<T>(PARAMS), rocsparse_status_invalid_size);
 #undef PARAMS
 }
 
@@ -65,17 +69,8 @@ void testing_coo2dense(const Arguments& arg)
     // Set matrix storage mode
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, arg.storage));
 
-    // Argument sanity check before allocating invalid memory
-    if(M <= 0 || N <= 0 || LD < M)
+    if(LD < M)
     {
-        rocsparse_status expected_status = (((M == 0 && N >= 0) || (M >= 0 && N == 0)) && (LD >= M))
-                                               ? rocsparse_status_success
-                                               : rocsparse_status_invalid_size;
-
-        EXPECT_ROCSPARSE_STATUS(
-            rocsparse_coo2dense<T>(
-                handle, M, N, 100, descr, nullptr, nullptr, nullptr, (T*)nullptr, LD),
-            expected_status);
         return;
     }
 
@@ -135,6 +130,11 @@ void testing_coo2dense(const Arguments& arg)
                                                               (T*)d_dense_val,
                                                               LD));
 
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("d_dense_val", d_dense_val);
+        }
+
         host_vector<T> gpu_dense_val(LD * N);
         CHECK_HIP_ERROR(
             hipMemcpy(gpu_dense_val, d_dense_val, sizeof(T) * LD * N, hipMemcpyDeviceToHost));
@@ -162,7 +162,6 @@ void testing_coo2dense(const Arguments& arg)
         // Warm-up
         for(int iter = 0; iter < number_cold_calls; ++iter)
         {
-
             CHECK_ROCSPARSE_ERROR(rocsparse_coo2dense<T>(handle,
                                                          M,
                                                          N,
@@ -197,17 +196,17 @@ void testing_coo2dense(const Arguments& arg)
         double gbyte_count = coo2dense_gbyte_count<T>(M, N, nnz);
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "N",
+                            display_key_t::N,
                             N,
-                            "LD",
+                            display_key_t::LD,
                             LD,
-                            "nnz",
+                            display_key_t::nnz,
                             nnz,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 }
