@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,7 @@
 template <typename T>
 void testing_csrgeam_bad_arg(const Arguments& arg)
 {
-    static const size_t safe_size = 100;
+    static const size_t safe_size = 1;
     T                   h_alpha   = 0.6;
     T                   h_beta    = 0.2;
 
@@ -56,9 +56,15 @@ void testing_csrgeam_bad_arg(const Arguments& arg)
     const rocsparse_int*      csr_col_ind_B = (const rocsparse_int*)0x4;
     const rocsparse_mat_descr descr_C       = local_descr_C;
     T*                        csr_val_C     = (T*)0x4;
-    rocsparse_int*            csr_row_ptr_C = (rocsparse_int*)0x4;
-    rocsparse_int*            csr_col_ind_C = (rocsparse_int*)0x4;
-    rocsparse_int*            nnz_C         = (rocsparse_int*)0x4;
+
+    host_dense_vector<rocsparse_int> hcsr_row_ptr_C(safe_size + 1);
+    hcsr_row_ptr_C[0] = 0;
+    hcsr_row_ptr_C[1] = 1;
+    device_dense_vector<rocsparse_int> dcsr_row_ptr_C(hcsr_row_ptr_C);
+
+    rocsparse_int* csr_row_ptr_C = (rocsparse_int*)dcsr_row_ptr_C;
+    rocsparse_int* csr_col_ind_C = (rocsparse_int*)0x4;
+    rocsparse_int* nnz_C         = (rocsparse_int*)0x4;
 
 #define PARAMS_NNZ                                                                             \
     handle, m, n, descr_A, nnz_A, csr_row_ptr_A, csr_col_ind_A, descr_B, nnz_B, csr_row_ptr_B, \
@@ -69,8 +75,8 @@ void testing_csrgeam_bad_arg(const Arguments& arg)
         nnz_B, csr_val_B, csr_row_ptr_B, csr_col_ind_B, descr_C, csr_val_C, csr_row_ptr_C,       \
         csr_col_ind_C
 
-    auto_testing_bad_arg(rocsparse_csrgeam_nnz, PARAMS_NNZ);
-    auto_testing_bad_arg(rocsparse_csrgeam<T>, PARAMS);
+    bad_arg_analysis(rocsparse_csrgeam_nnz, PARAMS_NNZ);
+    bad_arg_analysis(rocsparse_csrgeam<T>, PARAMS);
 
     for(auto val : rocsparse_matrix_type_t::values)
     {
@@ -84,12 +90,16 @@ void testing_csrgeam_bad_arg(const Arguments& arg)
             EXPECT_ROCSPARSE_STATUS(rocsparse_csrgeam<T>(PARAMS), rocsparse_status_not_implemented);
         }
     }
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_A, rocsparse_matrix_type_general));
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_B, rocsparse_matrix_type_general));
+    CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr_C, rocsparse_matrix_type_general));
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr_A, rocsparse_storage_mode_unsorted));
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr_B, rocsparse_storage_mode_unsorted));
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr_C, rocsparse_storage_mode_unsorted));
-    EXPECT_ROCSPARSE_STATUS(rocsparse_csrgeam_nnz(PARAMS_NNZ), rocsparse_status_not_implemented);
-    EXPECT_ROCSPARSE_STATUS(rocsparse_csrgeam<T>(PARAMS), rocsparse_status_not_implemented);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_csrgeam_nnz(PARAMS_NNZ),
+                            rocsparse_status_requires_sorted_storage);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_csrgeam<T>(PARAMS), rocsparse_status_requires_sorted_storage);
 
 #undef PARAMS
 #undef PARAMS_NNZ
@@ -392,6 +402,17 @@ void testing_csrgeam(const Arguments& arg)
                         baseB,
                         baseC);
 
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("hcsr_row_ptr_C_1", hcsr_row_ptr_C_1);
+            rocsparse_reproducibility::save("hcsr_col_ind_C_1", hcsr_col_ind_C_1);
+            rocsparse_reproducibility::save("hcsr_val_C_1", hcsr_val_C_1);
+
+            rocsparse_reproducibility::save("hcsr_row_ptr_C_2", hcsr_row_ptr_C_2);
+            rocsparse_reproducibility::save("hcsr_col_ind_C_2", hcsr_col_ind_C_2);
+            rocsparse_reproducibility::save("hcsr_val_C_2", hcsr_val_C_2);
+        }
+
         // Check C
         hcsr_col_ind_C_gold.unit_check(hcsr_col_ind_C_1);
         hcsr_col_ind_C_gold.unit_check(hcsr_col_ind_C_2);
@@ -466,7 +487,7 @@ void testing_csrgeam(const Arguments& arg)
                                                         &nnz_C));
         }
 
-        gpu_analysis_time_used = get_time_us() - gpu_analysis_time_used;
+        gpu_analysis_time_used = (get_time_us() - gpu_analysis_time_used) / number_hot_calls;
 
         // Warm up
         for(int iter = 0; iter < number_cold_calls; ++iter)
@@ -526,27 +547,27 @@ void testing_csrgeam(const Arguments& arg)
         double gpu_gflops = get_gpu_gflops(gpu_solve_time_used, gflop_count);
         double gpu_gbyte  = get_gpu_gbyte(gpu_solve_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "N",
+                            display_key_t::N,
                             N,
-                            "nnz_A",
+                            display_key_t::nnz_A,
                             nnz_A,
-                            "nnz_B",
+                            display_key_t::nnz_B,
                             nnz_B,
-                            "nnz_C",
+                            display_key_t::nnz_C,
                             nnz_C,
-                            "alpha",
+                            display_key_t::alpha,
                             *h_alpha,
-                            "beta",
+                            display_key_t::beta,
                             *h_beta,
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            "nnz msec",
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
     }
 }
