@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -70,6 +70,10 @@ void testing_csrilusv(const Arguments& arg)
     host_vector<rocsparse_int> h_numeric_pivot_U_1(1);
     host_vector<rocsparse_int> h_numeric_pivot_U_2(1);
 
+    host_vector<rocsparse_int> h_singular_pivot_gold(1);
+    host_vector<rocsparse_int> h_singular_pivot_1(1);
+    host_vector<rocsparse_int> h_singular_pivot_2(1);
+
     // Sample matrix
     rocsparse_int nnz;
     matrix_factory.init_csr(hcsr_row_ptr, hcsr_col_ind, hcsr_val_gold, M, N, nnz, base);
@@ -83,6 +87,8 @@ void testing_csrilusv(const Arguments& arg)
     device_vector<rocsparse_int> d_numeric_pivot_L_2(1);
     device_vector<rocsparse_int> d_numeric_pivot_U_2(1);
 
+    device_vector<rocsparse_int> d_singular_pivot_2(1);
+
     if(!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val || !d_struct_pivot_2 || !d_numeric_pivot_2
        || !d_numeric_pivot_L_2 || !d_numeric_pivot_U_2)
     {
@@ -98,16 +104,23 @@ void testing_csrilusv(const Arguments& arg)
     CHECK_HIP_ERROR(hipMemcpy(dcsr_val, hcsr_val_gold, sizeof(T) * nnz, hipMemcpyHostToDevice));
 
     // Compute reference incomplete LU factorization on host
-    host_csrilu0<T>(M,
-                    hcsr_row_ptr,
-                    hcsr_col_ind,
-                    hcsr_val_gold,
-                    base,
-                    h_struct_pivot_gold,
-                    h_numeric_pivot_gold,
-                    false,
-                    std::real(static_cast<T>(0)),
-                    static_cast<T>(0));
+    {
+        double tol = 0;
+        CHECK_ROCSPARSE_ERROR(rocsparse_csrilu0_get_tolerance(handle, info, &tol));
+
+        host_csrilu0<T>(M,
+                        hcsr_row_ptr,
+                        hcsr_col_ind,
+                        hcsr_val_gold,
+                        base,
+                        h_struct_pivot_gold,
+                        h_numeric_pivot_gold,
+                        h_singular_pivot_gold,
+                        tol,
+                        false,
+                        std::real(static_cast<T>(0)),
+                        static_cast<T>(0));
+    }
 
     // Obtain csrilu0 buffer size
     size_t buffer_size;
@@ -115,14 +128,7 @@ void testing_csrilusv(const Arguments& arg)
         handle, M, nnz, descrM, dcsr_val, dcsr_row_ptr, dcsr_col_ind, info, &buffer_size));
 
     // Allocate buffer
-    void* dbuffer;
-    CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, buffer_size));
-
-    if(!dbuffer)
-    {
-        CHECK_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
+    device_dense_vector<char> dbuffer(buffer_size);
 
     // csrilu0 analysis
     CHECK_ROCSPARSE_ERROR(rocsparse_csrilu0_analysis<T>(
@@ -165,22 +171,37 @@ void testing_csrilusv(const Arguments& arg)
                             (h_numeric_pivot_gold[0] != -1) ? rocsparse_status_zero_pivot
                                                             : rocsparse_status_success);
 
+    {
+        auto st = rocsparse_csrilu0_singular_pivot(handle, info, h_singular_pivot_1);
+        EXPECT_ROCSPARSE_STATUS(st, rocsparse_status_success);
+    }
     // Check for structural zero pivot using device pointer mode
     CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
     EXPECT_ROCSPARSE_STATUS(rocsparse_csrilu0_zero_pivot(handle, info, d_numeric_pivot_2),
                             (h_numeric_pivot_gold[0] != -1) ? rocsparse_status_zero_pivot
                                                             : rocsparse_status_success);
 
+    {
+
+        auto st = rocsparse_csrilu0_singular_pivot(handle, info, d_singular_pivot_2);
+        EXPECT_ROCSPARSE_STATUS(st, rocsparse_status_success);
+    }
+
     // Copy output to CPU
     host_vector<T> hcsr_val(nnz);
     CHECK_HIP_ERROR(hipMemcpy(
         h_numeric_pivot_2, d_numeric_pivot_2, sizeof(rocsparse_int), hipMemcpyDeviceToHost));
+    CHECK_HIP_ERROR(hipMemcpy(
+        h_singular_pivot_2, d_singular_pivot_2, sizeof(rocsparse_int), hipMemcpyDeviceToHost));
     CHECK_HIP_ERROR(hipMemcpy(hcsr_val, dcsr_val, sizeof(T) * nnz, hipMemcpyDeviceToHost));
 
     // Check pivot results
     h_numeric_pivot_gold.unit_check(h_numeric_pivot_1);
     h_numeric_pivot_gold.unit_check(h_numeric_pivot_2);
 
+    h_singular_pivot_gold.unit_check(h_singular_pivot_1);
+    h_singular_pivot_gold.unit_check(h_singular_pivot_2);
+
     // If numerical pivot has been found, we are done
     if(h_numeric_pivot_gold[0] != -1)
     {
@@ -239,6 +260,7 @@ void testing_csrilusv(const Arguments& arg)
                                                 hcsr_col_ind,
                                                 hcsr_val_gold,
                                                 hx,
+                                                (int64_t)1,
                                                 hz_gold,
                                                 rocsparse_diag_type_unit,
                                                 rocsparse_fill_mode_lower,
@@ -253,6 +275,7 @@ void testing_csrilusv(const Arguments& arg)
                                                 hcsr_col_ind,
                                                 hcsr_val_gold,
                                                 hz_gold,
+                                                (int64_t)1,
                                                 hy_gold,
                                                 rocsparse_diag_type_non_unit,
                                                 rocsparse_fill_mode_upper,
@@ -479,9 +502,6 @@ void testing_csrilusv(const Arguments& arg)
     CHECK_ROCSPARSE_ERROR(rocsparse_csrsv_clear(handle, descrL, info));
     CHECK_ROCSPARSE_ERROR(rocsparse_csrsv_clear(handle, descrU, info));
     CHECK_ROCSPARSE_ERROR(rocsparse_csrilu0_clear(handle, info));
-
-    // Free buffer
-    CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
 }
 
 #define INSTANTIATE(TYPE)                                       \
