@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -85,7 +85,6 @@ rocsparse_status rocsparse_host_csritsv_ptr_end(rocsparse_fill_mode fill_mode_,
                 if(!mark)
                 {
                     zero_pivot[0] = std::min(zero_pivot[0], i + base_);
-                    ;
                     ++count_symbolic_diag;
                 }
             }
@@ -96,9 +95,6 @@ rocsparse_status rocsparse_host_csritsv_ptr_end(rocsparse_fill_mode fill_mode_,
 
             if(count_symbolic_diag > 0)
             {
-                std::cout << "host csritsv, " << count_symbolic_diag
-                          << "  zero pivots detected, first index is " << zero_pivot[0]
-                          << std::endl;
                 return rocsparse_status_success;
             }
             break;
@@ -159,9 +155,6 @@ rocsparse_status rocsparse_host_csritsv_ptr_end(rocsparse_fill_mode fill_mode_,
 
             if(count_symbolic_diag > 0)
             {
-                std::cout << "host csritsv, " << count_symbolic_diag
-                          << "  zero pivots detected, first index is " << zero_pivot[0]
-                          << std::endl;
                 return rocsparse_status_success;
             }
             break;
@@ -536,6 +529,7 @@ rocsparse_status rocsparse_host_csritsv_solve(rocsparse_handle          handle,
 template <typename T>
 void testing_csritsv_bad_arg(const Arguments& arg)
 {
+
     static constexpr bool verbose = false;
     // Create rocsparse handle
     rocsparse_local_handle local_handle;
@@ -548,67 +542,91 @@ void testing_csritsv_bad_arg(const Arguments& arg)
 
     const T h_alpha = static_cast<T>(1);
 
-    rocsparse_handle          handle        = local_handle;
-    rocsparse_int*            host_nmaxiter = (rocsparse_int*)0x4;
-    const floating_data_t<T>* host_tol      = (const floating_data_t<T>*)0x4;
-    floating_data_t<T>*       host_history  = (floating_data_t<T>*)0x4;
-    rocsparse_operation       trans         = rocsparse_operation_none;
-    rocsparse_int             m             = 32;
-    rocsparse_int             nnz           = 32;
-    const T*                  alpha         = &h_alpha;
-    const rocsparse_mat_descr descr         = local_descr;
-    const T*                  csr_val       = (const T*)0x4;
-    const rocsparse_int*      csr_row_ptr   = (const rocsparse_int*)0x4;
-    const rocsparse_int*      csr_col_ind   = (const rocsparse_int*)0x4;
-    rocsparse_mat_info        info          = local_info;
-    const T*                  x             = (const T*)0x4;
-    T*                        y             = (T*)0x4;
-    rocsparse_solve_policy    solve         = rocsparse_solve_policy_auto;
-    rocsparse_analysis_policy analysis      = rocsparse_analysis_policy_force;
-    void*                     temp_buffer   = (void*)0x4;
-    size_t*                   buffer_size   = (size_t*)0x4;
+    rocsparse_handle          handle            = local_handle;
+    rocsparse_int*            host_nmaxiter     = (rocsparse_int*)0x4;
+    rocsparse_int             host_nfreeiter    = 2;
+    const floating_data_t<T>* host_tol          = (const floating_data_t<T>*)0x4;
+    floating_data_t<T>*       host_history      = (floating_data_t<T>*)0x4;
+    rocsparse_operation       trans             = rocsparse_operation_none;
+    rocsparse_int             m                 = 32;
+    rocsparse_int             nnz               = 32;
+    const T*                  alpha_device_host = &h_alpha;
+    const rocsparse_mat_descr descr             = local_descr;
+    const T*                  csr_val           = (const T*)0x4;
+    const rocsparse_int*      csr_row_ptr       = (const rocsparse_int*)0x4;
+    const rocsparse_int*      csr_col_ind       = (const rocsparse_int*)0x4;
+    rocsparse_mat_info        info              = local_info;
+    const T*                  x                 = (const T*)0x4;
+    T*                        y                 = (T*)0x4;
+    rocsparse_solve_policy    solve             = rocsparse_solve_policy_auto;
+    rocsparse_solve_policy    policy            = rocsparse_solve_policy_auto;
+    rocsparse_analysis_policy analysis          = rocsparse_analysis_policy_force;
+    void*                     temp_buffer       = (void*)0x4;
+    size_t*                   buffer_size       = (size_t*)0x4;
+
 #define PARAMS_BUFFER_SIZE \
     handle, trans, m, nnz, descr, csr_val, csr_row_ptr, csr_col_ind, info, buffer_size
 
 #define PARAMS_ANALYSIS                                                                     \
     handle, trans, m, nnz, descr, csr_val, csr_row_ptr, csr_col_ind, info, analysis, solve, \
         temp_buffer
-#define PARAMS_SOLVE                                                                     \
-    handle, host_nmaxiter, host_tol, host_history, trans, m, nnz, alpha, descr, csr_val, \
-        csr_row_ptr, csr_col_ind, info, x, y, solve, temp_buffer
+#define PARAMS_SOLVE                                                                        \
+    handle, host_nmaxiter, host_tol, host_history, trans, m, nnz, alpha_device_host, descr, \
+        csr_val, csr_row_ptr, csr_col_ind, info, x, y, policy, temp_buffer
+
+#define PARAMSX_SOLVE                                                                    \
+    handle, host_nmaxiter, host_nfreeiter, host_tol, host_history, trans, m, nnz,        \
+        alpha_device_host, descr, csr_val, csr_row_ptr, csr_col_ind, info, x, y, policy, \
+        temp_buffer
 
     //
     //
     //
     if(verbose)
     {
-        std::cout << "auto_testing_bad_arg(rocsparse_csritsv_buffer_size<T>, PARAMS_BUFFER_SIZE)"
+        std::cout << "bad_arg_analysis(rocsparse_csritsv_buffer_size<T>, PARAMS_BUFFER_SIZE)"
                   << std::endl;
     }
-    auto_testing_bad_arg(rocsparse_csritsv_buffer_size<T>, PARAMS_BUFFER_SIZE);
+    bad_arg_analysis(rocsparse_csritsv_buffer_size<T>, PARAMS_BUFFER_SIZE);
 
     //
     //
     //
     if(verbose)
     {
-        std::cout << "auto_testing_bad_arg(rocsparse_csritsv_analysis<T>, PARAMS_ANALYSIS)"
+        std::cout << "bad_arg_analysis(rocsparse_csritsv_analysis<T>, PARAMS_ANALYSIS)"
                   << std::endl;
     }
-    auto_testing_bad_arg(rocsparse_csritsv_analysis<T>, PARAMS_ANALYSIS);
+    bad_arg_analysis(rocsparse_csritsv_analysis<T>, PARAMS_ANALYSIS);
 
     //
     //
     //
     if(verbose)
     {
-        std::cout << "auto_testing_bad_arg(rocsparse_csritsv_solve<T>, PARAMS_SOLVE)" << std::endl;
+        std::cout << "bad_arg_analysis(rocsparse_csritsv_solve<T>, PARAMS_SOLVE)" << std::endl;
     }
-    static constexpr int nargs_to_exclude                  = 2;
-    static constexpr int args_to_exclude[nargs_to_exclude] = {2, 3};
 
-    auto_testing_bad_arg(
-        rocsparse_csritsv_solve<T>, nargs_to_exclude, args_to_exclude, PARAMS_SOLVE);
+    //
+    //
+    //
+    {
+        static constexpr int nargs_to_exclude                  = 2;
+        static constexpr int args_to_exclude[nargs_to_exclude] = {2, 3};
+
+        select_bad_arg_analysis(
+            rocsparse_csritsv_solve<T>, nargs_to_exclude, args_to_exclude, PARAMS_SOLVE);
+    }
+
+    //
+    //
+    //
+    {
+        static constexpr int nargs_to_exclude                  = 2;
+        static constexpr int args_to_exclude[nargs_to_exclude] = {3, 4};
+        select_bad_arg_analysis(
+            rocsparse_csritsv_solve_ex<T>, nargs_to_exclude, args_to_exclude, PARAMSX_SOLVE);
+    }
 
     for(auto matrix_type : rocsparse_matrix_type_t::values)
     {
@@ -622,6 +640,8 @@ void testing_csritsv_bad_arg(const Arguments& arg)
                                     rocsparse_status_not_implemented);
             EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve<T>(PARAMS_SOLVE),
                                     rocsparse_status_not_implemented);
+            EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve_ex<T>(PARAMSX_SOLVE),
+                                    rocsparse_status_not_implemented);
         }
     }
 
@@ -630,19 +650,23 @@ void testing_csritsv_bad_arg(const Arguments& arg)
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_unsorted));
 
     EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_buffer_size<T>(PARAMS_BUFFER_SIZE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
 
     EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_analysis<T>(PARAMS_ANALYSIS),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
 
     EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve<T>(PARAMS_SOLVE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
+
+    EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve_ex<T>(PARAMSX_SOLVE),
+                            rocsparse_status_requires_sorted_storage);
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_sorted));
 
 #undef PARAMS_BUFFER_SIZE
 #undef PARAMS_ANALYSIS
 #undef PARAMS_SOLVE
+#undef PARAMSX_SOLVE
 
     // Test rocsparse_csritsv_zero_pivot()
     rocsparse_int position;
@@ -670,8 +694,8 @@ void testing_csritsv(const Arguments& arg)
     //
     // Set nmaxiter.
     //
-    static constexpr rocsparse_int s_nmaxiter       = 200;
-    rocsparse_int                  host_nmaxiter[1] = {s_nmaxiter};
+    rocsparse_int host_nmaxiter[1] = {arg.nmaxiter};
+    rocsparse_int host_nfreeiter   = arg.nfreeiter;
 
     //
     // Tolerance for the iterative method.
@@ -695,7 +719,7 @@ void testing_csritsv(const Arguments& arg)
     //
     // Convergence history.
     //
-    floating_data_t<T> host_history[s_nmaxiter];
+    host_vector<floating_data_t<T>> host_history(host_nmaxiter[0]);
 
     //
     // Grab parametrization.
@@ -742,13 +766,20 @@ void testing_csritsv(const Arguments& arg)
     //
 #define PARAMS_BUFFER_SIZE(A_) \
     handle, trans, A_.m, A_.nnz, descr, A_.val, A_.ptr, A_.ind, info, &buffer_size
+
 #define PARAMS_ANALYSIS(A_) \
     handle, trans, A_.m, A_.nnz, descr, A_.val, A_.ptr, A_.ind, info, apol, spol, dbuffer
+
 #define PARAMS_SOLVE(alpha_, A_, x_, y_)                                                       \
     handle, host_nmaxiter, host_tol, host_history, trans, A_.m, A_.nnz, alpha_, descr, A_.val, \
         A_.ptr, A_.ind, info, x_, y_, spol, dbuffer
 
+#define PARAMSX_SOLVE(alpha_, A_, x_, y_)                                                       \
+    handle, host_nmaxiter, host_nfreeiter, host_tol, host_history, trans, A_.m, A_.nnz, alpha_, \
+        descr, A_.val, A_.ptr, A_.ind, info, x_, y_, spol, dbuffer
+
     rocsparse_int host_zero_pivot;
+
 #define HOST_PARAMS_SOLVE(alpha_, A_, x_, y_)                                                  \
     handle, host_nmaxiter, host_tol, host_history, trans, A_.m, A_.nnz, alpha_, descr, A_.val, \
         A_.ptr, A_.ind, x_, y_, hbuffer, &host_zero_pivot
@@ -785,6 +816,9 @@ void testing_csritsv(const Arguments& arg)
         EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
                                 rocsparse_status_success);
 
+        EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve_ex<T>(PARAMSX_SOLVE(h_alpha, dA, dx, dy)),
+                                rocsparse_status_success);
+
         EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_zero_pivot(handle, descr, info, &pivot),
                                 rocsparse_status_success);
 
@@ -799,12 +833,6 @@ void testing_csritsv(const Arguments& arg)
     //
     host_csr_matrix<T> hA;
 
-    //
-    // Configure matrix A.
-    //
-    {
-    }
-
     if(diag == rocsparse_diag_type_unit && rocsparse_matrix_type_triangular == arg.matrix_type)
     {
         host_csr_matrix<T>          hB;
@@ -884,18 +912,7 @@ void testing_csritsv(const Arguments& arg)
         rocsparse_matrix_factory<T> matrix_factory(arg, to_int, full_rank);
         matrix_factory.init_csr(hA, M, N);
     }
-#if 0
-  if (hA.nnz > hA.m)
-    {
-      hB.ptr[0] = hB.base;
-      for (rocsparse_int i=0;i<hA.m;++i)
-	hB.ptr[i+1] = i;
-      for (rocsparse_int i=1;i<hA.m;++i)
-	hB.ptr[i+1] -= i;
-      host_csr_matrix<T> hh(hA.m,hA.n,hA.nnz - hA.m);
-    }
-  exit(1);
-#endif
+
     //
     // Again, since we import or generate the matrix, Non-squared matrices are not supported.
     //
@@ -913,7 +930,6 @@ void testing_csritsv(const Arguments& arg)
     //
     // Let's normalize ... it helps on horrible matrices.
     //
-
     floating_data_t<T> mx = 0;
     for(rocsparse_int i = 0; i < hA.nnz; ++i)
         mx = std::max(mx, std::abs(hA.val[i]));
@@ -928,20 +944,7 @@ void testing_csritsv(const Arguments& arg)
     //
     host_dense_matrix<T> hx(M, 1);
     rocsparse_matrix_utils::init(hx);
-#if 0
-  { floating_data_t<T> frob = static_cast<floating_data_t<T>>(0);
-    for (rocsparse_int i=0;i<hA.nnz;++i)
-      frob += std::abs(hA.val[i]*hA.val[i]);
-    frob = std::sqrt(frob);
 
-    std::cout << "frob " << frob << std::endl;
-    floating_data_t<T> mx = static_cast<floating_data_t<T>>(0);
-    for (rocsparse_int i=0;i<M;++i)
-      mx = std::max(mx,std::abs(hx[i]));
-    mx *=std::abs( h_alpha[0]);
-    std::cout << "adjusted tolerance " << host_tol[0]*mx * frob << " from " << host_tol[0]<<std::endl;
-    host_tol[0] *= mx * frob; }
-#endif
     //
     // Define and transfer A and X from host to device.
     //
@@ -1019,7 +1022,7 @@ void testing_csritsv(const Arguments& arg)
         CHECK_ROCSPARSE_ERROR(
             (rocsparse_host_csritsv_solve<rocsparse_int, rocsparse_int, T>)(HOST_PARAMS_SOLVE(
                 h_alpha, hA, hx, hy_iterative)));
-        const bool host_iterative_convergence = (host_nmaxiter[0] < s_nmaxiter);
+        const bool host_iterative_convergence = (host_nmaxiter[0] < arg.nmaxiter);
         if(false == host_iterative_convergence)
         {
             //
@@ -1046,6 +1049,7 @@ void testing_csritsv(const Arguments& arg)
                                                     hA.ind,
                                                     hA.val,
                                                     hx,
+                                                    (int64_t)1,
                                                     hy_direct,
                                                     diag,
                                                     uplo,
@@ -1115,7 +1119,7 @@ void testing_csritsv(const Arguments& arg)
         //
         // Call before analysis
         //
-        host_nmaxiter[0] = s_nmaxiter;
+        host_nmaxiter[0] = arg.nmaxiter;
         EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
                                 rocsparse_status_invalid_pointer);
         if(verbose)
@@ -1140,46 +1144,22 @@ void testing_csritsv(const Arguments& arg)
                                                             : rocsparse_status_success);
         }
 
-        CHECK_HIP_ERROR(hipDeviceSynchronize());
         if(verbose)
         {
             std::cout << " - compute device iterative, call solve" << std::endl;
         }
 
         CHECK_HIP_ERROR(hipMemset(dy, 0, sizeof(T) * M));
-        host_nmaxiter[0] = s_nmaxiter;
+        host_nmaxiter[0] = arg.nmaxiter;
         CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)));
         {
             auto st = rocsparse_csritsv_zero_pivot(handle, descr, info, solve_pivot);
             EXPECT_ROCSPARSE_STATUS(
                 st, (*solve_pivot != -1) ? rocsparse_status_zero_pivot : rocsparse_status_success);
         }
-        CHECK_HIP_ERROR(hipDeviceSynchronize());
-        const bool device_iterative_convergence = (host_nmaxiter[0] < s_nmaxiter);
 
-        if(device_iterative_convergence != host_iterative_convergence)
-        {
-            if(trans == rocsparse_operation_none)
-            {
-                std::cout << " - WARNING host and device iterative convergece differs on "
-                             "NonTranspose case ... it happens rarely but it does."
-                          << std::endl;
-                // Let's not be too restrictive ...
-                // CHECK_ROCSPARSE_ERROR(rocsparse_status_internal_error);
-            }
-            else
-            {
-                std::cout << " - WARNING host and device iterative convergece differs, it happens "
-                             "with transpose cases since csrmv uses atomics."
-                          << std::endl;
-            }
-        }
-        if(verbose)
-        {
-            std::cout << " - compute device iterative, check pivot." << std::endl;
-            std::cout << "h_analysis_pivot " << *h_analysis_pivot << std::endl;
-            std::cout << "h_solve_pivot " << *h_solve_pivot << std::endl;
-        }
+        const bool device_iterative_convergence = (host_nmaxiter[0] < arg.nmaxiter);
+
         h_analysis_pivot.unit_check(analysis_pivot);
         h_solve_pivot.unit_check(solve_pivot);
 
@@ -1197,6 +1177,76 @@ void testing_csritsv(const Arguments& arg)
             }
         }
 
+        host_nmaxiter[0] = arg.nmaxiter;
+        CHECK_HIP_ERROR(hipMemset(dy, 0, sizeof(T) * M));
+        CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_solve_ex<T>(PARAMSX_SOLVE(h_alpha, dA, dx, dy)));
+        {
+            auto st = rocsparse_csritsv_zero_pivot(handle, descr, info, solve_pivot);
+            EXPECT_ROCSPARSE_STATUS(
+                st, (*solve_pivot != -1) ? rocsparse_status_zero_pivot : rocsparse_status_success);
+        }
+        const bool device_iterative_convergence_ex = (host_nmaxiter[0] < arg.nmaxiter);
+        if(*h_analysis_pivot == -1 && *h_solve_pivot == -1)
+        {
+            if(verbose)
+            {
+                std::cout
+                    << " - compute device iterative_ex, compare solutions between host and device."
+                    << std::endl;
+            }
+            if(host_iterative_convergence)
+            {
+                hy_iterative.near_check(dy, tol_compare);
+            }
+        }
+
+        if(device_iterative_convergence != host_iterative_convergence)
+        {
+            if(verbose)
+            {
+                if(trans == rocsparse_operation_none)
+                {
+                    std::cout << " - WARNING host and device iterative convergence differs on "
+                                 "NonTranspose case ... it happens rarely but it does."
+                              << std::endl;
+                }
+                else
+                {
+                    std::cout
+                        << " - WARNING host and device iterative convergence differs, it happens "
+                           "with transpose cases since csrmv uses atomics."
+                        << std::endl;
+                }
+            }
+        }
+
+        if(device_iterative_convergence_ex != host_iterative_convergence)
+        {
+            if(verbose)
+            {
+                if(trans == rocsparse_operation_none)
+                {
+                    std::cout << " - WARNING host and device iterative_ex convergence differs on "
+                                 "NonTranspose case ... it happens rarely but it does."
+                              << std::endl;
+                }
+                else
+                {
+                    std::cout << " - WARNING host and device iterative_ex convergence differs, it "
+                                 "happens "
+                                 "with transpose cases since csrmv uses atomics."
+                              << std::endl;
+                }
+            }
+        }
+
+        if(verbose)
+        {
+            std::cout << " - compute device iterative, check pivot." << std::endl;
+            std::cout << "h_analysis_pivot " << *h_analysis_pivot << std::endl;
+            std::cout << "h_solve_pivot " << *h_solve_pivot << std::endl;
+        }
+
         //
         // RESET MAT INFO.
         //
@@ -1210,52 +1260,77 @@ void testing_csritsv(const Arguments& arg)
             std::cout << " - compute device iterative" << std::endl;
         }
 
+        device_scalar<rocsparse_int> d_analysis_pivot;
+        device_scalar<rocsparse_int> d_solve_pivot;
+        device_scalar<T>             d_alpha(h_alpha);
+
+        //
+        // POINTER MODE DEVICE
+        //
+        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
+
+        //
+        // CHECK IF DEFAULT ZERO PIVOT IS -1
+        //
+        EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_zero_pivot(handle, descr, info, d_analysis_pivot),
+                                rocsparse_status_success);
+        analysis_no_pivot.unit_check(d_analysis_pivot);
+
+        //
+        // Call before analysis
+        //
+        host_nmaxiter[0] = arg.nmaxiter;
+        EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
+                                rocsparse_status_invalid_pointer);
+
+        //
+        // Call it twice.
+        //
+        CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_analysis<T>(PARAMS_ANALYSIS(dA)));
+        CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_analysis<T>(PARAMS_ANALYSIS(dA)));
+        EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_zero_pivot(handle, descr, info, d_analysis_pivot),
+                                (*h_analysis_pivot != -1) ? rocsparse_status_zero_pivot
+                                                          : rocsparse_status_success);
+
+        CHECK_HIP_ERROR(hipMemset(dy, 0, sizeof(T) * M));
+        host_nmaxiter[0] = arg.nmaxiter;
+        CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(d_alpha, dA, dx, dy)));
+        EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_zero_pivot(handle, descr, info, d_solve_pivot),
+                                (*h_solve_pivot != -1) ? rocsparse_status_zero_pivot
+                                                       : rocsparse_status_success);
+
+        h_analysis_pivot.unit_check(d_analysis_pivot);
+        h_solve_pivot.unit_check(d_solve_pivot);
+
+        if(*h_analysis_pivot == -1 && *h_solve_pivot == -1)
         {
-            device_scalar<rocsparse_int> d_analysis_pivot;
-            device_scalar<rocsparse_int> d_solve_pivot;
-            device_scalar<T>             d_alpha(h_alpha);
-
-            //
-            // POINTER MODE DEVICE
-            //
-            CHECK_ROCSPARSE_ERROR(
-                rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
-
-            //
-            // CHECK IF DEFAULT ZERO PIVOT IS -1
-            //
-            EXPECT_ROCSPARSE_STATUS(
-                rocsparse_csritsv_zero_pivot(handle, descr, info, d_analysis_pivot),
-                rocsparse_status_success);
-            analysis_no_pivot.unit_check(d_analysis_pivot);
-
-            //
-            // Call before analysis
-            //
-            host_nmaxiter[0] = s_nmaxiter;
-            EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)),
-                                    rocsparse_status_invalid_pointer);
-
-            //
-            // Call it twice.
-            //
-            CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_analysis<T>(PARAMS_ANALYSIS(dA)));
-            CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_analysis<T>(PARAMS_ANALYSIS(dA)));
-            EXPECT_ROCSPARSE_STATUS(
-                rocsparse_csritsv_zero_pivot(handle, descr, info, d_analysis_pivot),
-                (*h_analysis_pivot != -1) ? rocsparse_status_zero_pivot : rocsparse_status_success);
-            CHECK_HIP_ERROR(hipDeviceSynchronize());
-
-            CHECK_HIP_ERROR(hipMemset(dy, 0, sizeof(T) * M));
-            host_nmaxiter[0] = s_nmaxiter;
-            CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(d_alpha, dA, dx, dy)));
-            EXPECT_ROCSPARSE_STATUS(
-                rocsparse_csritsv_zero_pivot(handle, descr, info, d_solve_pivot),
-                (*h_solve_pivot != -1) ? rocsparse_status_zero_pivot : rocsparse_status_success);
-            CHECK_HIP_ERROR(hipDeviceSynchronize());
-            h_analysis_pivot.unit_check(d_analysis_pivot);
-            h_solve_pivot.unit_check(d_solve_pivot);
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("Y", dy);
+            }
+            if(host_iterative_convergence)
+            {
+                hy_iterative.near_check(dy, tol_compare);
+            }
         }
+        else
+        {
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save(
+                    "analysis_pivot", d_analysis_pivot, "solve_pivot", d_solve_pivot);
+            }
+        }
+
+        CHECK_HIP_ERROR(hipMemset(dy, 0, sizeof(T) * M));
+        host_nmaxiter[0] = arg.nmaxiter;
+        CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_solve_ex<T>(PARAMSX_SOLVE(d_alpha, dA, dx, dy)));
+        EXPECT_ROCSPARSE_STATUS(rocsparse_csritsv_zero_pivot(handle, descr, info, d_solve_pivot),
+                                (*h_solve_pivot != -1) ? rocsparse_status_zero_pivot
+                                                       : rocsparse_status_success);
+
+        h_analysis_pivot.unit_check(d_analysis_pivot);
+        h_solve_pivot.unit_check(d_solve_pivot);
 
         if(*h_analysis_pivot == -1 && *h_solve_pivot == -1)
         {
@@ -1275,6 +1350,8 @@ void testing_csritsv(const Arguments& arg)
     if(arg.timing)
     {
 
+        host_nfreeiter = arg.nfreeiter;
+
         int number_cold_calls = 2;
         int number_hot_calls  = arg.iters;
 
@@ -1286,7 +1363,8 @@ void testing_csritsv(const Arguments& arg)
             CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_analysis<T>(PARAMS_ANALYSIS(dA)));
             CHECK_ROCSPARSE_ERROR(
                 rocsparse_csritsv_zero_pivot(handle, descr, info, h_analysis_pivot));
-            CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)));
+            CHECK_ROCSPARSE_ERROR(
+                rocsparse_csritsv_solve_ex<T>(PARAMSX_SOLVE(h_alpha, dA, dx, dy)));
             CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_zero_pivot(handle, descr, info, h_solve_pivot));
             CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_clear(handle, descr, info));
         }
@@ -1301,10 +1379,11 @@ void testing_csritsv(const Arguments& arg)
         for(int iter = 0; iter < number_hot_calls; ++iter)
         {
             CHECK_HIP_ERROR(hipMemset(dy, 0, sizeof(T) * M));
-            host_nmaxiter[0]             = s_nmaxiter;
+            host_nmaxiter[0]             = arg.nmaxiter;
             double gpu_solve_time_used_1 = get_time_us();
 
-            CHECK_ROCSPARSE_ERROR(rocsparse_csritsv_solve<T>(PARAMS_SOLVE(h_alpha, dA, dx, dy)));
+            CHECK_ROCSPARSE_ERROR(
+                rocsparse_csritsv_solve_ex<T>(PARAMSX_SOLVE(h_alpha, dA, dx, dy)));
             gpu_solve_time_used_1 = (get_time_us() - gpu_solve_time_used_1);
             gpu_solve_time_used += gpu_solve_time_used_1;
         }
@@ -1321,13 +1400,14 @@ void testing_csritsv(const Arguments& arg)
                             dA.nnz,
                             display_key_t::alpha,
                             *h_alpha,
-
-                            "pivot",
+                            display_key_t::nfreeiter,
+                            host_nfreeiter,
+                            display_key_t::nmaxiter,
+                            host_nmaxiter[0],
+                            display_key_t::pivot,
                             std::min(*h_analysis_pivot, *h_solve_pivot),
-
                             display_key_t::trans,
                             rocsparse_operation2string(trans),
-
                             display_key_t::diag_type,
                             rocsparse_diagtype2string(diag),
                             display_key_t::fill_mode,
@@ -1336,7 +1416,6 @@ void testing_csritsv(const Arguments& arg)
                             rocsparse_analysis2string(apol),
                             display_key_t::solve_policy,
                             rocsparse_solve2string(spol),
-
                             display_key_t::gflops,
                             gpu_gflops,
                             display_key_t::bandwidth,
