@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,39 +40,41 @@ void testing_csrsm_bad_arg(const Arguments& arg)
     // Create matrix info
     rocsparse_local_mat_info local_info;
 
-    rocsparse_handle          handle            = local_handle;
-    rocsparse_operation       trans_A           = rocsparse_operation_none;
-    rocsparse_operation       trans_B           = rocsparse_operation_none;
-    rocsparse_int             m                 = safe_size;
-    rocsparse_int             nrhs              = safe_size;
-    rocsparse_int             nnz               = safe_size;
-    const T*                  alpha_device_host = &h_alpha;
-    const rocsparse_mat_descr descr             = local_descr;
-    const T*                  csr_val           = (const T*)0x4;
-    const rocsparse_int*      csr_row_ptr       = (const rocsparse_int*)0x4;
-    const rocsparse_int*      csr_col_ind       = (const rocsparse_int*)0x4;
-    T*                        B                 = (T*)0x4;
-    rocsparse_int             ldb               = safe_size;
-    rocsparse_mat_info        info              = local_info;
-    rocsparse_analysis_policy analysis          = rocsparse_analysis_policy_force;
-    rocsparse_solve_policy    solve             = rocsparse_solve_policy_auto;
-    size_t*                   buffer_size       = (size_t*)0x4;
-    void*                     temp_buffer       = (void*)0x4;
+    rocsparse_handle          handle      = local_handle;
+    rocsparse_operation       trans_A     = rocsparse_operation_none;
+    rocsparse_operation       trans_B     = rocsparse_operation_none;
+    rocsparse_int             m           = safe_size;
+    rocsparse_int             nrhs        = safe_size;
+    rocsparse_int             nnz         = safe_size;
+    const T*                  alpha       = &h_alpha;
+    const rocsparse_mat_descr descr       = local_descr;
+    const T*                  csr_val     = (const T*)0x4;
+    const rocsparse_int*      csr_row_ptr = (const rocsparse_int*)0x4;
+    const rocsparse_int*      csr_col_ind = (const rocsparse_int*)0x4;
+    T*                        B           = (T*)0x4;
+    rocsparse_int             ldb         = safe_size;
+    rocsparse_mat_info        info        = local_info;
+    rocsparse_analysis_policy analysis    = rocsparse_analysis_policy_force;
+    rocsparse_solve_policy    solve       = rocsparse_solve_policy_auto;
+    rocsparse_solve_policy    policy      = rocsparse_solve_policy_auto;
+    size_t                    local_buffer_size;
+    size_t*                   buffer_size = &local_buffer_size;
+    void*                     temp_buffer = (void*)0x4;
 
-#define PARAMS_BUFFER_SIZE                                                                  \
-    handle, trans_A, trans_B, m, nrhs, nnz, alpha_device_host, descr, csr_val, csr_row_ptr, \
-        csr_col_ind, B, ldb, info, solve, buffer_size
-    auto_testing_bad_arg(rocsparse_csrsm_buffer_size<T>, PARAMS_BUFFER_SIZE);
+#define PARAMS_BUFFER_SIZE                                                                      \
+    handle, trans_A, trans_B, m, nrhs, nnz, alpha, descr, csr_val, csr_row_ptr, csr_col_ind, B, \
+        ldb, info, policy, buffer_size
+    bad_arg_analysis(rocsparse_csrsm_buffer_size<T>, PARAMS_BUFFER_SIZE);
 
-#define PARAMS_ANALYSIS                                                                     \
-    handle, trans_A, trans_B, m, nrhs, nnz, alpha_device_host, descr, csr_val, csr_row_ptr, \
-        csr_col_ind, B, ldb, info, analysis, solve, temp_buffer
-    auto_testing_bad_arg(rocsparse_csrsm_analysis<T>, PARAMS_ANALYSIS);
+#define PARAMS_ANALYSIS                                                                         \
+    handle, trans_A, trans_B, m, nrhs, nnz, alpha, descr, csr_val, csr_row_ptr, csr_col_ind, B, \
+        ldb, info, analysis, solve, temp_buffer
+    bad_arg_analysis(rocsparse_csrsm_analysis<T>, PARAMS_ANALYSIS);
 
-#define PARAMS_SOLVE                                                                        \
-    handle, trans_A, trans_B, m, nrhs, nnz, alpha_device_host, descr, csr_val, csr_row_ptr, \
-        csr_col_ind, B, ldb, info, solve, temp_buffer
-    auto_testing_bad_arg(rocsparse_csrsm_solve<T>, PARAMS_SOLVE);
+#define PARAMS_SOLVE                                                                            \
+    handle, trans_A, trans_B, m, nrhs, nnz, alpha, descr, csr_val, csr_row_ptr, csr_col_ind, B, \
+        ldb, info, policy, temp_buffer
+    bad_arg_analysis(rocsparse_csrsm_solve<T>, PARAMS_SOLVE);
 
     for(auto matrix_type : rocsparse_matrix_type_t::values)
     {
@@ -91,11 +93,11 @@ void testing_csrsm_bad_arg(const Arguments& arg)
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_unsorted));
     EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_buffer_size<T>(PARAMS_BUFFER_SIZE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_analysis<T>(PARAMS_ANALYSIS),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_solve<T>(PARAMS_SOLVE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_sorted));
 
     // Test rocsparse_csrsm_zero_pivot()
@@ -120,20 +122,20 @@ void testing_csrsm_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_csrsm(const Arguments& arg)
 {
-    rocsparse_operation         transA    = arg.transA;
-    rocsparse_operation         transB    = arg.transB;
-    rocsparse_int               M         = arg.M;
-    rocsparse_int               nrhs      = arg.K;
-    rocsparse_diag_type         diag      = arg.diag;
-    rocsparse_fill_mode         uplo      = arg.uplo;
-    rocsparse_analysis_policy   apol      = arg.apol;
-    rocsparse_solve_policy      spol      = arg.spol;
-    rocsparse_index_base        base      = arg.baseA;
-    static constexpr bool       full_rank = true;
+    rocsparse_operation       transA    = arg.transA;
+    rocsparse_operation       transB    = arg.transB;
+    rocsparse_int             M         = arg.M;
+    rocsparse_int             nrhs      = arg.K;
+    rocsparse_diag_type       diag      = arg.diag;
+    rocsparse_fill_mode       uplo      = arg.uplo;
+    rocsparse_analysis_policy apol      = arg.apol;
+    rocsparse_solve_policy    spol      = arg.spol;
+    rocsparse_index_base      base      = arg.baseA;
+    static constexpr bool     full_rank = true;
+
     rocsparse_matrix_factory<T> matrix_factory(arg, false, full_rank);
 
-    host_scalar<T> h_alpha;
-    *h_alpha = arg.get_alpha<T>();
+    host_scalar<T> h_alpha(arg.get_alpha<T>());
 
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
@@ -155,90 +157,17 @@ void testing_csrsm(const Arguments& arg)
     // Set matrix index base
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_index_base(descr, base));
 
-    // Argument sanity check before allocating invalid memory
-    if(M <= 0 || nrhs <= 0)
-    {
-
-        static const size_t safe_size = 100;
-        size_t              buffer_size;
-        rocsparse_int       pivot;
-        rocsparse_int       ldb = (transB == rocsparse_operation_none) ? M : nrhs;
-
-        // Allocate memory on device
-        rocsparse_int* dcsr_row_ptr = (rocsparse_int*)0x4;
-        rocsparse_int* dcsr_col_ind = (rocsparse_int*)0x4;
-        T*             dcsr_val     = (T*)0x4;
-        T*             dB           = (T*)0x4;
-        T*             dbuffer      = (T*)0x4;
-
-        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-        EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_buffer_size<T>(handle,
-                                                               transA,
-                                                               transB,
-                                                               M,
-                                                               nrhs,
-                                                               safe_size,
-                                                               h_alpha,
-                                                               descr,
-                                                               dcsr_val,
-                                                               dcsr_row_ptr,
-                                                               dcsr_col_ind,
-                                                               dB,
-                                                               ldb,
-                                                               info,
-                                                               spol,
-                                                               &buffer_size),
-                                (M < 0 || nrhs < 0) ? rocsparse_status_invalid_size
-                                                    : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_analysis<T>(handle,
-                                                            transA,
-                                                            transB,
-                                                            M,
-                                                            nrhs,
-                                                            safe_size,
-                                                            h_alpha,
-                                                            descr,
-                                                            dcsr_val,
-                                                            dcsr_row_ptr,
-                                                            dcsr_col_ind,
-                                                            dB,
-                                                            ldb,
-                                                            info,
-                                                            apol,
-                                                            spol,
-                                                            dbuffer),
-                                (M < 0 || nrhs < 0) ? rocsparse_status_invalid_size
-                                                    : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_solve<T>(handle,
-                                                         transA,
-                                                         transB,
-                                                         M,
-                                                         nrhs,
-                                                         safe_size,
-                                                         h_alpha,
-                                                         descr,
-                                                         dcsr_val,
-                                                         dcsr_row_ptr,
-                                                         dcsr_col_ind,
-                                                         dB,
-                                                         ldb,
-                                                         info,
-                                                         spol,
-                                                         dbuffer),
-                                (M < 0 || nrhs < 0) ? rocsparse_status_invalid_size
-                                                    : rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_zero_pivot(handle, info, &pivot),
-                                rocsparse_status_success);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_csrsm_clear(handle, info), rocsparse_status_success);
-
-        return;
-    }
-
     host_csr_matrix<T> hcsr;
     matrix_factory.init_csr(hcsr);
 
-    rocsparse_int nnz  = hcsr.nnz;
-    M                  = hcsr.m;
+    //
+    // Scale the matrix.
+    //
+    hcsr.scale();
+
+    rocsparse_int nnz = hcsr.nnz;
+    M                 = hcsr.m;
+
     rocsparse_int hB_m = (transB == rocsparse_operation_none) ? M : nrhs;
     rocsparse_int hB_n = (transB == rocsparse_operation_none) ? nrhs : M;
 
@@ -391,11 +320,12 @@ void testing_csrsm(const Arguments& arg)
                                                         hcsr.val,
                                                         hB,
                                                         hB.ld,
+                                                        rocsparse_order_column,
                                                         diag,
                                                         uplo,
                                                         base,
-                                                        (rocsparse_int*)h_analysis_pivot,
-                                                        (rocsparse_int*)h_solve_pivot);
+                                                        h_analysis_pivot,
+                                                        h_solve_pivot);
 
             //
             // CHECK PIVOTS
@@ -510,35 +440,35 @@ void testing_csrsm(const Arguments& arg)
         double gpu_gflops = get_gpu_gflops(gpu_solve_time_used, gflop_count);
         double gpu_gbyte  = get_gpu_gbyte(gpu_solve_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "nnz",
+                            display_key_t::nnz,
                             nnz,
-                            "nrhs",
+                            display_key_t::nrhs,
                             nrhs,
-                            "alpha",
+                            display_key_t::alpha,
                             *h_alpha,
-                            "pivot",
+                            display_key_t::pivot,
                             std::min(*h_analysis_pivot, *h_solve_pivot),
-                            "op(A)",
+                            display_key_t::trans_A,
                             rocsparse_operation2string(transA),
-                            "op(B)",
+                            display_key_t::trans_B,
                             rocsparse_operation2string(transB),
-                            "diag_type",
+                            display_key_t::diag_type,
                             rocsparse_diagtype2string(diag),
-                            "fill_mode",
+                            display_key_t::fill_mode,
                             rocsparse_fillmode2string(uplo),
-                            "analysis_policy",
+                            display_key_t::analysis_policy,
                             rocsparse_analysis2string(apol),
-                            "solve_policy",
+                            display_key_t::solve_policy,
                             rocsparse_solve2string(spol),
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            "analysis msec",
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
     }
 
