@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,22 +40,23 @@ void testing_csrsv_bad_arg(const Arguments& arg)
     // Create matrix info
     rocsparse_local_mat_info local_info;
 
-    rocsparse_handle          handle      = local_handle;
-    rocsparse_operation       trans       = rocsparse_operation_none;
-    rocsparse_int             m           = safe_size;
-    rocsparse_int             nnz         = safe_size;
-    const T*                  alpha       = &h_alpha;
-    const rocsparse_mat_descr descr       = local_descr;
-    const T*                  csr_val     = (const T*)0x4;
-    const rocsparse_int*      csr_row_ptr = (const rocsparse_int*)0x4;
-    const rocsparse_int*      csr_col_ind = (const rocsparse_int*)0x4;
-    rocsparse_mat_info        info        = local_info;
-    const T*                  x           = (const T*)0x4;
-    T*                        y           = (T*)0x4;
-    rocsparse_analysis_policy analysis    = rocsparse_analysis_policy_reuse;
-    rocsparse_solve_policy    solve       = rocsparse_solve_policy_auto;
-    size_t*                   buffer_size = (size_t*)0x4;
-    void*                     temp_buffer = (void*)0x4;
+    rocsparse_handle          handle            = local_handle;
+    rocsparse_operation       trans             = rocsparse_operation_none;
+    rocsparse_int             m                 = safe_size;
+    rocsparse_int             nnz               = safe_size;
+    const T*                  alpha_device_host = &h_alpha;
+    const rocsparse_mat_descr descr             = local_descr;
+    const T*                  csr_val           = (const T*)0x4;
+    const rocsparse_int*      csr_row_ptr       = (const rocsparse_int*)0x4;
+    const rocsparse_int*      csr_col_ind       = (const rocsparse_int*)0x4;
+    rocsparse_mat_info        info              = local_info;
+    const T*                  x                 = (const T*)0x4;
+    T*                        y                 = (T*)0x4;
+    rocsparse_analysis_policy analysis          = rocsparse_analysis_policy_reuse;
+    rocsparse_solve_policy    solve             = rocsparse_solve_policy_auto;
+    rocsparse_solve_policy    policy            = rocsparse_solve_policy_auto;
+    size_t*                   buffer_size       = (size_t*)0x4;
+    void*                     temp_buffer       = (void*)0x4;
 
 #define PARAMS_BUFFER_SIZE \
     handle, trans, m, nnz, descr, csr_val, csr_row_ptr, csr_col_ind, info, buffer_size
@@ -64,17 +65,18 @@ void testing_csrsv_bad_arg(const Arguments& arg)
     handle, trans, m, nnz, descr, csr_val, csr_row_ptr, csr_col_ind, info, analysis, solve, \
         temp_buffer
 
-#define PARAMS_SOLVE                                                                           \
-    handle, trans, m, nnz, alpha, descr, csr_val, csr_row_ptr, csr_col_ind, info, x, y, solve, \
-        temp_buffer
+#define PARAMS_SOLVE                                                                             \
+    handle, trans, m, nnz, alpha_device_host, descr, csr_val, csr_row_ptr, csr_col_ind, info, x, \
+        y, policy, temp_buffer
 
-    auto_testing_bad_arg(rocsparse_csrsv_buffer_size<T>, PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(rocsparse_csrsv_analysis<T>, PARAMS_ANALYSIS);
-    auto_testing_bad_arg(rocsparse_csrsv_solve<T>, PARAMS_SOLVE);
+    bad_arg_analysis(rocsparse_csrsv_buffer_size<T>, PARAMS_BUFFER_SIZE);
+    bad_arg_analysis(rocsparse_csrsv_analysis<T>, PARAMS_ANALYSIS);
+    bad_arg_analysis(rocsparse_csrsv_solve<T>, PARAMS_SOLVE);
 
     for(auto matrix_type : rocsparse_matrix_type_t::values)
     {
-        if(matrix_type != rocsparse_matrix_type_general)
+        if(matrix_type != rocsparse_matrix_type_general
+           && matrix_type != rocsparse_matrix_type_triangular)
         {
             CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr, matrix_type));
             EXPECT_ROCSPARSE_STATUS(rocsparse_csrsv_buffer_size<T>(PARAMS_BUFFER_SIZE),
@@ -89,11 +91,11 @@ void testing_csrsv_bad_arg(const Arguments& arg)
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_unsorted));
     EXPECT_ROCSPARSE_STATUS(rocsparse_csrsv_buffer_size<T>(PARAMS_BUFFER_SIZE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_csrsv_analysis<T>(PARAMS_ANALYSIS),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_csrsv_solve<T>(PARAMS_SOLVE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_sorted));
 
 #undef PARAMS_BUFFER_SIZE
@@ -235,6 +237,7 @@ void testing_csrsv(const Arguments& arg)
                                                     hA.ind,
                                                     hA.val,
                                                     hx,
+                                                    (int64_t)1,
                                                     hy,
                                                     diag,
                                                     uplo,
@@ -558,13 +561,10 @@ void testing_csrsv(const Arguments& arg)
                             dA.nnz,
                             display_key_t::alpha,
                             *h_alpha,
-
-                            "pivot",
+                            display_key_t::pivot,
                             std::min(*h_analysis_pivot, *h_solve_pivot),
-
                             display_key_t::trans,
                             rocsparse_operation2string(trans),
-
                             display_key_t::diag_type,
                             rocsparse_diagtype2string(diag),
                             display_key_t::fill_mode,
@@ -573,7 +573,6 @@ void testing_csrsv(const Arguments& arg)
                             rocsparse_analysis2string(apol),
                             display_key_t::solve_policy,
                             rocsparse_solve2string(spol),
-
                             display_key_t::gflops,
                             gpu_gflops,
                             display_key_t::bandwidth,
