@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -43,7 +43,7 @@ void testing_prune_dense2csr_bad_arg(const Arguments& arg)
     rocsparse_int             m                      = safe_size;
     rocsparse_int             n                      = safe_size;
     const T*                  A                      = (const T*)0x4;
-    rocsparse_int             ld                     = safe_size;
+    rocsparse_int             lda                    = safe_size;
     const rocsparse_mat_descr descr                  = local_descr;
     T*                        csr_val                = (T*)0x4;
     rocsparse_int*            csr_row_ptr            = (rocsparse_int*)0x4;
@@ -52,29 +52,39 @@ void testing_prune_dense2csr_bad_arg(const Arguments& arg)
     size_t*                   buffer_size            = &h_buffer_size;
     void*                     temp_buffer            = (void*)0x4;
 
-    int       nargs_to_exclude   = 1;
-    const int args_to_exclude[1] = {5};
-    const T*  threshold          = &h_threshold;
+    static constexpr int nargs_to_exclude_buffer_size                              = 3;
+    const int            args_to_exclude_buffer_size[nargs_to_exclude_buffer_size] = {5, 7, 9};
+
+    static constexpr int nargs_to_exclude_nnz                      = 2;
+    const int            args_to_exclude_nnz[nargs_to_exclude_nnz] = {5, 9};
+
+    static constexpr int nargs_to_exclude_solve                        = 4;
+    const int            args_to_exclude_solve[nargs_to_exclude_solve] = {5, 7, 9, 10};
+
+    const T* threshold = &h_threshold;
 
 #define PARAMS_BUFFER_SIZE \
-    handle, m, n, A, ld, threshold, descr, csr_val, csr_row_ptr, csr_col_ind, buffer_size
+    handle, m, n, A, lda, threshold, descr, csr_val, csr_row_ptr, csr_col_ind, buffer_size
 #define PARAMS_NNZ \
-    handle, m, n, A, ld, threshold, descr, csr_row_ptr, nnz_total_dev_host_ptr, temp_buffer
-#define PARAMS handle, m, n, A, ld, threshold, descr, csr_val, csr_row_ptr, csr_col_ind, temp_buffer
-    auto_testing_bad_arg(rocsparse_prune_dense2csr_buffer_size<T>,
-                         nargs_to_exclude,
-                         args_to_exclude,
-                         PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(
-        rocsparse_prune_dense2csr_nnz<T>, nargs_to_exclude, args_to_exclude, PARAMS_NNZ);
-    auto_testing_bad_arg(rocsparse_prune_dense2csr<T>, nargs_to_exclude, args_to_exclude, PARAMS);
+    handle, m, n, A, lda, threshold, descr, csr_row_ptr, nnz_total_dev_host_ptr, temp_buffer
+#define PARAMS \
+    handle, m, n, A, lda, threshold, descr, csr_val, csr_row_ptr, csr_col_ind, temp_buffer
+    select_bad_arg_analysis(rocsparse_prune_dense2csr_buffer_size<T>,
+                            nargs_to_exclude_buffer_size,
+                            args_to_exclude_buffer_size,
+                            PARAMS_BUFFER_SIZE);
+    select_bad_arg_analysis(
+        rocsparse_prune_dense2csr_nnz<T>, nargs_to_exclude_nnz, args_to_exclude_nnz, PARAMS_NNZ);
+    select_bad_arg_analysis(
+        rocsparse_prune_dense2csr<T>, nargs_to_exclude_solve, args_to_exclude_solve, PARAMS);
 
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, rocsparse_storage_mode_unsorted));
     EXPECT_ROCSPARSE_STATUS(rocsparse_prune_dense2csr_buffer_size<T>(PARAMS_BUFFER_SIZE),
-                            rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
     EXPECT_ROCSPARSE_STATUS(rocsparse_prune_dense2csr_nnz<T>(PARAMS_NNZ),
-                            rocsparse_status_not_implemented);
-    EXPECT_ROCSPARSE_STATUS(rocsparse_prune_dense2csr<T>(PARAMS), rocsparse_status_not_implemented);
+                            rocsparse_status_requires_sorted_storage);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_prune_dense2csr<T>(PARAMS),
+                            rocsparse_status_requires_sorted_storage);
 #undef PARAMS
 #undef PARAMS_NNZ
 #undef PARAMS_BUFFER_SIZE
@@ -137,7 +147,7 @@ void testing_prune_dense2csr(const Arguments& arg)
     {
         for(rocsparse_int i = 0; i < M; ++i)
         {
-            h_A[j * LDA + i] = random_generator_normal<T>();
+            h_A[j * LDA + i] = random_cached_generator_normal<T>();
         }
     }
 
@@ -283,19 +293,19 @@ void testing_prune_dense2csr(const Arguments& arg)
         double gbyte_count = prune_dense2csr_gbyte_count<T>(M, N, h_nnz_total_dev_host_ptr[0]);
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "N",
+                            display_key_t::N,
                             N,
-                            "denseld",
+                            display_key_t::LD,
                             LDA,
-                            "nnz",
+                            display_key_t::nnz,
                             h_nnz_total_dev_host_ptr[0],
-                            "threshold",
+                            display_key_t::threshold,
                             *h_threshold,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 
