@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,9 @@ rocsparse_status rocsparse_bsr_set_pointers(rocsparse_spmat_descr         descr,
     return rocsparse_bsr_set_pointers(descr, bsr_matrix.ptr, bsr_matrix.ind, bsr_matrix.val);
 }
 
+//
+//
+//
 template <typename I, typename J, typename T>
 void testing_spgemm_bsr_bad_arg(const Arguments& arg)
 {
@@ -39,14 +42,15 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
     // Create rocsparse handle
     rocsparse_local_handle local_handle;
 
-    rocsparse_handle handle = local_handle;
-    J                mb     = safe_size;
-    J                nb     = safe_size;
-    J                kb     = safe_size;
-    I                nnzb_A = safe_size;
-    I                nnzb_B = safe_size;
-    I                nnzb_C = safe_size;
-    I                nnzb_D = safe_size;
+    rocsparse_handle handle    = local_handle;
+    J                mb        = safe_size;
+    J                nb        = safe_size;
+    J                kb        = safe_size;
+    I                nnzb_A    = safe_size;
+    I                nnzb_B    = safe_size;
+    I                nnzb_C    = safe_size;
+    I                nnzb_D    = safe_size;
+    J                block_dim = safe_size;
 
     void* bsr_row_ptr_A = (void*)0x4;
     void* bsr_col_ind_A = (void*)0x4;
@@ -61,67 +65,76 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
     void* bsr_col_ind_D = (void*)0x4;
     void* bsr_val_D     = (void*)0x4;
 
+    rocsparse_direction    dir     = rocsparse_direction_row;
     rocsparse_operation    trans_A = rocsparse_operation_none;
     rocsparse_operation    trans_B = rocsparse_operation_none;
     rocsparse_index_base   base    = rocsparse_index_base_zero;
     rocsparse_spgemm_alg   alg     = rocsparse_spgemm_alg_default;
-    rocsparse_spgemm_stage stage   = rocsparse_spgemm_stage_auto;
+    rocsparse_spgemm_stage stage   = rocsparse_spgemm_stage_compute;
 
     // Index and data type
-    rocsparse_indextype itype = get_indextype<I>();
-    rocsparse_indextype jtype = get_indextype<J>();
-    rocsparse_datatype  ttype = get_datatype<T>();
+    rocsparse_indextype itype        = get_indextype<I>();
+    rocsparse_indextype jtype        = get_indextype<J>();
+    rocsparse_datatype  compute_type = get_datatype<T>();
 
     // SpGEMM structures
-    rocsparse_local_spmat local_mat_A(mb,
-                                      kb,
-                                      nnzb_A,
-                                      bsr_row_ptr_A,
-                                      bsr_col_ind_A,
-                                      bsr_val_A,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_bsr);
-    rocsparse_local_spmat local_mat_B(kb,
-                                      nb,
-                                      nnzb_B,
-                                      bsr_row_ptr_B,
-                                      bsr_col_ind_B,
-                                      bsr_val_B,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_bsr);
-    rocsparse_local_spmat local_mat_C(mb,
-                                      nb,
-                                      nnzb_C,
-                                      bsr_row_ptr_C,
-                                      bsr_col_ind_C,
-                                      bsr_val_C,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_bsr);
-    rocsparse_local_spmat local_mat_D(mb,
-                                      nb,
-                                      nnzb_D,
-                                      bsr_row_ptr_D,
-                                      bsr_col_ind_D,
-                                      bsr_val_D,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_bsr);
+    rocsparse_local_spmat local_A(mb,
+                                  kb,
+                                  nnzb_A,
+                                  dir,
+                                  block_dim,
+                                  bsr_row_ptr_A,
+                                  bsr_col_ind_A,
+                                  bsr_val_A,
+                                  itype,
+                                  jtype,
+                                  base,
+                                  compute_type,
+                                  rocsparse_format_bsr);
+    rocsparse_local_spmat local_B(kb,
+                                  nb,
+                                  nnzb_B,
+                                  dir,
+                                  block_dim,
+                                  bsr_row_ptr_B,
+                                  bsr_col_ind_B,
+                                  bsr_val_B,
+                                  itype,
+                                  jtype,
+                                  base,
+                                  compute_type,
+                                  rocsparse_format_bsr);
+    rocsparse_local_spmat local_C(mb,
+                                  nb,
+                                  nnzb_C,
+                                  dir,
+                                  block_dim,
+                                  bsr_row_ptr_C,
+                                  bsr_col_ind_C,
+                                  bsr_val_C,
+                                  itype,
+                                  jtype,
+                                  base,
+                                  compute_type,
+                                  rocsparse_format_bsr);
+    rocsparse_local_spmat local_D(mb,
+                                  nb,
+                                  nnzb_D,
+                                  dir,
+                                  block_dim,
+                                  bsr_row_ptr_D,
+                                  bsr_col_ind_D,
+                                  bsr_val_D,
+                                  itype,
+                                  jtype,
+                                  base,
+                                  compute_type,
+                                  rocsparse_format_bsr);
 
-    rocsparse_spmat_descr mat_A = local_mat_A;
-    rocsparse_spmat_descr mat_B = local_mat_B;
-    rocsparse_spmat_descr mat_C = local_mat_C;
-    rocsparse_spmat_descr mat_D = local_mat_D;
+    rocsparse_spmat_descr A = local_A;
+    rocsparse_spmat_descr B = local_B;
+    rocsparse_spmat_descr C = local_C;
+    rocsparse_spmat_descr D = local_D;
 
     int       nargs_to_exclude   = 4;
     const int args_to_exclude[4] = {3, 6, 12, 13};
@@ -133,9 +146,9 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
     // Scenario 3: alpha == nullptr && beta != nullptr
     // Scenario 4: alpha != nullptr && beta != nullptr
 
-#define PARAMS                                                                            \
-    handle, trans_A, trans_B, alpha, mat_A, mat_B, beta, mat_D, mat_C, ttype, alg, stage, \
-        buffer_size, temp_buffer
+#define PARAMS                                                                                \
+    handle, trans_A, trans_B, alpha, A, B, beta, D, C, compute_type, alg, stage, buffer_size, \
+        temp_buffer
     // ###############################################
     // Scenario 1: alpha == nullptr && beta == nullptr
     // ###############################################
@@ -145,19 +158,19 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
 
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -169,19 +182,19 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
 
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -193,19 +206,19 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
 
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -217,19 +230,19 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
 
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 #undef PARAMS
 
@@ -239,14 +252,14 @@ void testing_spgemm_bsr_bad_arg(const Arguments& arg)
                                              trans_A,
                                              trans_B,
                                              alpha,
-                                             mat_A,
-                                             mat_B,
+                                             A,
+                                             B,
                                              beta,
-                                             mat_D,
-                                             mat_C,
-                                             ttype,
+                                             D,
+                                             C,
+                                             compute_type,
                                              alg,
-                                             stage,
+                                             rocsparse_spgemm_stage_buffer_size,
                                              nullptr,
                                              nullptr),
                             rocsparse_status_invalid_pointer);
@@ -269,15 +282,9 @@ void testing_spgemm_bsr(const Arguments& arg)
     rocsparse_spgemm_alg  alg       = arg.spgemm_alg;
     static constexpr bool full_rank = false;
 
-    J Mb = -1;
-    J Nb = -1;
-    J Kb = -1;
-    if(block_dim > 0)
-    {
-        Mb = (M + block_dim - 1) / block_dim;
-        Nb = (N + block_dim - 1) / block_dim;
-        Kb = (K + block_dim - 1) / block_dim;
-    }
+    J Mb = (M + block_dim - 1) / block_dim;
+    J Nb = (N + block_dim - 1) / block_dim;
+    J Kb = (K + block_dim - 1) / block_dim;
 
     T h_alpha = arg.get_alpha<T>();
     T h_beta  = arg.get_beta<T>();
@@ -472,6 +479,11 @@ void testing_spgemm_bsr(const Arguments& arg)
                                                  dbuffer),
                                 rocsparse_status_success);
 
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("dC pointer mode host", dC);
+        }
+
         // Check
         hC.near_check(dC);
 
@@ -493,6 +505,11 @@ void testing_spgemm_bsr(const Arguments& arg)
                                                  dbuffer),
                                 rocsparse_status_success);
 
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("dC pointer mode device", dC);
+        }
+
         // Check
         hC.near_check(dC);
     }
@@ -599,37 +616,37 @@ void testing_spgemm_bsr(const Arguments& arg)
         double gpu_gbyte  = get_gpu_gbyte(gpu_solve_time_used, gbyte_count);
         double gpu_gflops = get_gpu_gflops(gpu_solve_time_used, gflop_count);
 
-        display_timing_info("opA",
+        display_timing_info(display_key_t::trans_A,
                             rocsparse_operation2string(trans_A),
-                            "opB",
+                            display_key_t::trans_B,
                             rocsparse_operation2string(trans_B),
-                            "Mb",
+                            display_key_t::Mb,
                             Mb,
-                            "Nb",
+                            display_key_t::Nb,
                             Nb,
-                            "Kb",
+                            display_key_t::Kb,
                             Kb,
-                            "block_dim",
+                            display_key_t::bdim,
                             block_dim,
-                            "nnzb_A",
+                            display_key_t::nnzb_A,
                             dA.nnzb,
-                            "nnzb_B",
+                            display_key_t::nnzb_B,
                             dB.nnzb,
-                            "nnzb_C",
+                            display_key_t::nnzb_C,
                             dC.nnzb,
-                            "nnzb_D",
+                            display_key_t::nnzb_D,
                             dD.nnzb,
-                            "alpha",
+                            display_key_t::alpha,
                             h_alpha,
-                            "beta",
+                            display_key_t::beta,
                             h_beta,
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            "analysis msec",
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
     }
 
