@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,9 @@ rocsparse_status rocsparse_csr_set_pointers(rocsparse_spmat_descr       descr,
     return rocsparse_csr_set_pointers(descr, csr_matrix.ptr, csr_matrix.ind, csr_matrix.val);
 }
 
+//
+//
+//
 template <typename I, typename J, typename T>
 void testing_spgemm_csr_bad_arg(const Arguments& arg)
 {
@@ -64,63 +67,27 @@ void testing_spgemm_csr_bad_arg(const Arguments& arg)
     rocsparse_operation    trans_B = rocsparse_operation_none;
     rocsparse_index_base   base    = rocsparse_index_base_zero;
     rocsparse_spgemm_alg   alg     = rocsparse_spgemm_alg_default;
-    rocsparse_spgemm_stage stage   = rocsparse_spgemm_stage_auto;
+    rocsparse_spgemm_stage stage   = rocsparse_spgemm_stage_compute;
 
     // Index and data type
-    rocsparse_indextype itype = get_indextype<I>();
-    rocsparse_indextype jtype = get_indextype<J>();
-    rocsparse_datatype  ttype = get_datatype<T>();
+    rocsparse_indextype itype        = get_indextype<I>();
+    rocsparse_indextype jtype        = get_indextype<J>();
+    rocsparse_datatype  compute_type = get_datatype<T>();
 
     // SpGEMM structures
-    rocsparse_local_spmat local_mat_A(m,
-                                      k,
-                                      nnz_A,
-                                      csr_row_ptr_A,
-                                      csr_col_ind_A,
-                                      csr_val_A,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_csr);
-    rocsparse_local_spmat local_mat_B(k,
-                                      n,
-                                      nnz_B,
-                                      csr_row_ptr_B,
-                                      csr_col_ind_B,
-                                      csr_val_B,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_csr);
-    rocsparse_local_spmat local_mat_C(m,
-                                      n,
-                                      nnz_C,
-                                      csr_row_ptr_C,
-                                      csr_col_ind_C,
-                                      csr_val_C,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_csr);
-    rocsparse_local_spmat local_mat_D(m,
-                                      n,
-                                      nnz_D,
-                                      csr_row_ptr_D,
-                                      csr_col_ind_D,
-                                      csr_val_D,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_csr);
+    rocsparse_local_spmat local_A(
+        m, k, nnz_A, csr_row_ptr_A, csr_col_ind_A, csr_val_A, itype, jtype, base, compute_type);
+    rocsparse_local_spmat local_B(
+        k, n, nnz_B, csr_row_ptr_B, csr_col_ind_B, csr_val_B, itype, jtype, base, compute_type);
+    rocsparse_local_spmat local_C(
+        m, n, nnz_C, csr_row_ptr_C, csr_col_ind_C, csr_val_C, itype, jtype, base, compute_type);
+    rocsparse_local_spmat local_D(
+        m, n, nnz_D, csr_row_ptr_D, csr_col_ind_D, csr_val_D, itype, jtype, base, compute_type);
 
-    rocsparse_spmat_descr mat_A = local_mat_A;
-    rocsparse_spmat_descr mat_B = local_mat_B;
-    rocsparse_spmat_descr mat_C = local_mat_C;
-    rocsparse_spmat_descr mat_D = local_mat_D;
+    rocsparse_spmat_descr A = local_A;
+    rocsparse_spmat_descr B = local_B;
+    rocsparse_spmat_descr C = local_C;
+    rocsparse_spmat_descr D = local_D;
 
     int       nargs_to_exclude   = 4;
     const int args_to_exclude[4] = {3, 6, 12, 13};
@@ -132,31 +99,30 @@ void testing_spgemm_csr_bad_arg(const Arguments& arg)
     // Scenario 3: alpha == nullptr && beta != nullptr
     // Scenario 4: alpha != nullptr && beta != nullptr
 
-#define PARAMS                                                                            \
-    handle, trans_A, trans_B, alpha, mat_A, mat_B, beta, mat_D, mat_C, ttype, alg, stage, \
-        buffer_size, temp_buffer
+#define PARAMS                                                                                \
+    handle, trans_A, trans_B, alpha, A, B, beta, D, C, compute_type, alg, stage, buffer_size, \
+        temp_buffer
     // ###############################################
     // Scenario 1: alpha == nullptr && beta == nullptr
     // ###############################################
     {
-        const T* alpha = (const T*)nullptr;
-        const T* beta  = (const T*)nullptr;
-
-        size_t* buffer_size = (size_t*)0x4;
-        void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        const T* alpha       = (const T*)nullptr;
+        const T* beta        = (const T*)nullptr;
+        size_t*  buffer_size = (size_t*)0x4;
+        void*    temp_buffer = (void*)0x4;
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -168,19 +134,19 @@ void testing_spgemm_csr_bad_arg(const Arguments& arg)
 
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -192,19 +158,19 @@ void testing_spgemm_csr_bad_arg(const Arguments& arg)
 
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     // ###############################################
@@ -216,19 +182,19 @@ void testing_spgemm_csr_bad_arg(const Arguments& arg)
 
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = (size_t*)0x4;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
 
         buffer_size = nullptr;
         temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spgemm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 #undef PARAMS
 
@@ -238,14 +204,14 @@ void testing_spgemm_csr_bad_arg(const Arguments& arg)
                                              trans_A,
                                              trans_B,
                                              alpha,
-                                             mat_A,
-                                             mat_B,
+                                             A,
+                                             B,
                                              beta,
-                                             mat_D,
-                                             mat_C,
-                                             ttype,
+                                             D,
+                                             C,
+                                             compute_type,
                                              alg,
-                                             stage,
+                                             rocsparse_spgemm_stage_buffer_size,
                                              nullptr,
                                              nullptr),
                             rocsparse_status_invalid_pointer);
@@ -275,17 +241,20 @@ void testing_spgemm_csr(const Arguments& arg)
     // Index and data type
     rocsparse_datatype ttype = get_datatype<T>();
 
-    // SpGEMM stage
-    rocsparse_spgemm_stage stage = rocsparse_spgemm_stage_auto;
-
     // Create rocsparse handle
     rocsparse_local_handle handle;
     using host_csr   = host_csr_matrix<T, I, J>;
     using device_csr = device_csr_matrix<T, I, J>;
 
-#define PARAMS(alpha_, A_, B_, D_, beta_, C_, buffer_)                                        \
-    handle, trans_A, trans_B, alpha_, A_, B_, beta_, D_, C_, ttype, alg, stage, &buffer_size, \
-        buffer_
+#define PARAMS_BUFFER_SIZE(alpha_, A_, B_, D_, beta_, C_, buffer_)       \
+    handle, trans_A, trans_B, alpha_, A_, B_, beta_, D_, C_, ttype, alg, \
+        rocsparse_spgemm_stage_buffer_size, &buffer_size, buffer_
+#define PARAMS_NNZ(alpha_, A_, B_, D_, beta_, C_, buffer_)               \
+    handle, trans_A, trans_B, alpha_, A_, B_, beta_, D_, C_, ttype, alg, \
+        rocsparse_spgemm_stage_nnz, &buffer_size, buffer_
+#define PARAMS_COMPUTE(alpha_, A_, B_, D_, beta_, C_, buffer_)           \
+    handle, trans_A, trans_B, alpha_, A_, B_, beta_, D_, C_, ttype, alg, \
+        rocsparse_spgemm_stage_compute, &buffer_size, buffer_
 
     // Argument sanity check before allocating invalid memory
     if((M <= 0 || N <= 0 || K <= 0) && (M <= 0 || N <= 0 || K != 0 || h_beta_ptr == nullptr))
@@ -331,13 +300,13 @@ void testing_spgemm_csr(const Arguments& arg)
         size_t buffer_size;
         void*  dbuffer = nullptr;
         EXPECT_ROCSPARSE_STATUS(
-            rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)),
+            rocsparse_spgemm(PARAMS_BUFFER_SIZE(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)),
             rocsparse_status_success);
 
         CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, safe_size));
 
         EXPECT_ROCSPARSE_STATUS(
-            rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)),
+            rocsparse_spgemm(PARAMS_NNZ(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)),
             rocsparse_status_success);
 
         // Verify that nnz_C is equal to zero
@@ -459,15 +428,15 @@ void testing_spgemm_csr(const Arguments& arg)
                 size_t buffer_size;
                 void*  dbuffer = nullptr;
 
-                CHECK_ROCSPARSE_ERROR(
-                    rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                CHECK_ROCSPARSE_ERROR(rocsparse_spgemm(
+                    PARAMS_BUFFER_SIZE(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
                 CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, buffer_size));
 
                 //
                 // Compute symbolic C.
                 //
                 CHECK_ROCSPARSE_ERROR(
-                    rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                    rocsparse_spgemm(PARAMS_NNZ(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
 
                 //
                 // Update memory.
@@ -483,10 +452,15 @@ void testing_spgemm_csr(const Arguments& arg)
                 // Compute numeric C.
                 //
                 CHECK_ROCSPARSE_ERROR(
-                    rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                    rocsparse_spgemm(PARAMS_COMPUTE(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
                 CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
             }
 
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("dC pointer mode host", dC);
+            }
+
             //
             // Check
             //
@@ -519,15 +493,15 @@ void testing_spgemm_csr(const Arguments& arg)
                     size_t buffer_size;
                     void*  dbuffer = nullptr;
 
-                    CHECK_ROCSPARSE_ERROR(
-                        rocsparse_spgemm(PARAMS(d_alpha_ptr, A, B, D, d_beta_ptr, C, dbuffer)));
+                    CHECK_ROCSPARSE_ERROR(rocsparse_spgemm(
+                        PARAMS_BUFFER_SIZE(d_alpha_ptr, A, B, D, d_beta_ptr, C, dbuffer)));
                     CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, buffer_size));
 
                     //
                     // Compute symbolic C.
                     //
                     CHECK_ROCSPARSE_ERROR(
-                        rocsparse_spgemm(PARAMS(d_alpha_ptr, A, B, D, d_beta_ptr, C, dbuffer)));
+                        rocsparse_spgemm(PARAMS_NNZ(d_alpha_ptr, A, B, D, d_beta_ptr, C, dbuffer)));
 
                     //
                     // Update memory.
@@ -542,11 +516,16 @@ void testing_spgemm_csr(const Arguments& arg)
                     //
                     // Compute numeric C.
                     //
-                    CHECK_ROCSPARSE_ERROR(
-                        rocsparse_spgemm(PARAMS(d_alpha_ptr, A, B, D, d_beta_ptr, C, dbuffer)));
+                    CHECK_ROCSPARSE_ERROR(rocsparse_spgemm(
+                        PARAMS_COMPUTE(d_alpha_ptr, A, B, D, d_beta_ptr, C, dbuffer)));
                     CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
                 }
 
+                if(ROCSPARSE_REPRODUCIBILITY)
+                {
+                    rocsparse_reproducibility::save("dC pointer mode device", dC);
+                }
+
                 if((!h_alpha_ptr || std::abs(*h_alpha_ptr) == ((I)std::abs(*h_alpha_ptr)))
                    && (!h_beta_ptr || std::abs(*h_beta_ptr) == ((I)std::abs(*h_beta_ptr))))
                 {
@@ -586,13 +565,13 @@ void testing_spgemm_csr(const Arguments& arg)
                 size_t buffer_size;
                 void*  dbuffer = nullptr;
                 //
-                CHECK_ROCSPARSE_ERROR(
-                    rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                CHECK_ROCSPARSE_ERROR(rocsparse_spgemm(
+                    PARAMS_BUFFER_SIZE(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
                 //
                 CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, buffer_size));
                 //
                 CHECK_ROCSPARSE_ERROR(
-                    rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                    rocsparse_spgemm(PARAMS_NNZ(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
                 //
                 {
                     int64_t C_m, C_n, C_nnz;
@@ -602,7 +581,7 @@ void testing_spgemm_csr(const Arguments& arg)
                 }
                 //
                 CHECK_ROCSPARSE_ERROR(
-                    rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                    rocsparse_spgemm(PARAMS_COMPUTE(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
                 //
                 CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
             }
@@ -625,12 +604,12 @@ void testing_spgemm_csr(const Arguments& arg)
             size_t buffer_size;
             void*  dbuffer = nullptr;
             CHECK_ROCSPARSE_ERROR(
-                rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                rocsparse_spgemm(PARAMS_BUFFER_SIZE(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
             //
             CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, buffer_size));
             //
             CHECK_ROCSPARSE_ERROR(
-                rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                rocsparse_spgemm(PARAMS_NNZ(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
             //
 
             gpu_analysis_time_used = get_time_us() - gpu_analysis_time_used;
@@ -650,7 +629,7 @@ void testing_spgemm_csr(const Arguments& arg)
             for(int iter = 0; iter < number_hot_calls; ++iter)
             {
                 CHECK_ROCSPARSE_ERROR(
-                    rocsparse_spgemm(PARAMS(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
+                    rocsparse_spgemm(PARAMS_COMPUTE(h_alpha_ptr, A, B, D, h_beta_ptr, C, dbuffer)));
             }
 
             gpu_solve_time_used = (get_time_us() - gpu_solve_time_used) / number_hot_calls;
