@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -27,68 +27,65 @@ template <typename I, typename J, typename T>
 void testing_spitsv_csr_bad_arg(const Arguments& arg)
 {
 
-    J m     = 100;
-    J n     = 100;
-    I nnz   = 100;
-    T alpha = 0.6;
-
+    J                         m             = 100;
+    J                         n             = 100;
+    I                         nnz           = 100;
+    const T                   local_alpha   = T(0.6);
+    const T*                  alpha         = &local_alpha;
     rocsparse_int*            host_nmaxiter = (rocsparse_int*)0x4;
     const floating_data_t<T>* host_tol      = (const floating_data_t<T>*)0x4;
     floating_data_t<T>*       host_history  = (floating_data_t<T>*)0x4;
 
-    rocsparse_operation  trans_A = rocsparse_operation_none;
-    rocsparse_index_base base    = rocsparse_index_base_zero;
-    rocsparse_spitsv_alg alg     = rocsparse_spitsv_alg_default;
+    rocsparse_operation  trans = rocsparse_operation_none;
+    rocsparse_index_base base  = rocsparse_index_base_zero;
+    rocsparse_spitsv_alg alg   = rocsparse_spitsv_alg_default;
 
     // Index and data type
-    rocsparse_indextype itype = get_indextype<I>();
-    rocsparse_indextype jtype = get_indextype<J>();
-    rocsparse_datatype  ttype = get_datatype<T>();
+    rocsparse_indextype itype        = get_indextype<I>();
+    rocsparse_indextype jtype        = get_indextype<J>();
+    rocsparse_datatype  compute_type = get_datatype<T>();
 
     // Create rocsparse handle
     rocsparse_local_handle local_handle;
 
     // Spitsv structures
-    rocsparse_local_spmat local_A(m,
-                                  n,
-                                  nnz,
-                                  (void*)0x4,
-                                  (void*)0x4,
-                                  (void*)0x4,
-                                  itype,
-                                  jtype,
-                                  base,
-                                  ttype,
-                                  rocsparse_format_csr);
-    rocsparse_local_dnvec local_x(m, (void*)0x4, ttype);
-    rocsparse_local_dnvec local_y(m, (void*)0x4, ttype);
+    rocsparse_local_spmat local_A(
+        m, n, nnz, (void*)0x4, (void*)0x4, (void*)0x4, itype, jtype, base, compute_type);
+
+    rocsparse_local_dnvec local_x(m, (void*)0x4, compute_type);
+    rocsparse_local_dnvec local_y(m, (void*)0x4, compute_type);
 
     int       nargs_to_exclude   = 4;
     const int args_to_exclude[4] = {2, 3, 12, 13};
 
     rocsparse_handle      handle = local_handle;
-    rocsparse_spmat_descr A      = local_A;
+    rocsparse_spmat_descr mat    = local_A;
     rocsparse_dnvec_descr x      = local_x;
     rocsparse_dnvec_descr y      = local_y;
 
-    size_t buffer_size;
-    void*  temp_buffer = (void*)0x4;
+    size_t  local_buffer_size;
+    size_t* buffer_size = &local_buffer_size;
+    void*   temp_buffer = (void*)0x4;
 
-#define PARAMS_BUFFER_SIZE                                                               \
-    handle, host_nmaxiter, host_tol, host_history, trans_A, &alpha, A, x, y, ttype, alg, \
-        rocsparse_spitsv_stage_buffer_size, &buffer_size, temp_buffer
+#define PARAMS_BUFFER_SIZE                                                                     \
+    handle, host_nmaxiter, host_tol, host_history, trans, alpha, mat, x, y, compute_type, alg, \
+        stage, buffer_size, temp_buffer
 
-#define PARAMS_ANALYSIS                                                                  \
-    handle, host_nmaxiter, host_tol, host_history, trans_A, &alpha, A, x, y, ttype, alg, \
-        rocsparse_spitsv_stage_preprocess, &buffer_size, temp_buffer
+#define PARAMS_ANALYSIS                                                                        \
+    handle, host_nmaxiter, host_tol, host_history, trans, alpha, mat, x, y, compute_type, alg, \
+        stage, buffer_size, temp_buffer
 
-#define PARAMS_SOLVE                                                                     \
-    handle, host_nmaxiter, host_tol, host_history, trans_A, &alpha, A, x, y, ttype, alg, \
-        rocsparse_spitsv_stage_compute, &buffer_size, temp_buffer
+#define PARAMS_SOLVE                                                                           \
+    handle, host_nmaxiter, host_tol, host_history, trans, alpha, mat, x, y, compute_type, alg, \
+        stage, buffer_size, temp_buffer
 
-    auto_testing_bad_arg(rocsparse_spitsv, nargs_to_exclude, args_to_exclude, PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(rocsparse_spitsv, nargs_to_exclude, args_to_exclude, PARAMS_ANALYSIS);
-    auto_testing_bad_arg(rocsparse_spitsv, nargs_to_exclude, args_to_exclude, PARAMS_SOLVE);
+    rocsparse_spitsv_stage stage = rocsparse_spitsv_stage_buffer_size;
+    select_bad_arg_analysis(
+        rocsparse_spitsv, nargs_to_exclude, args_to_exclude, PARAMS_BUFFER_SIZE);
+    stage = rocsparse_spitsv_stage_preprocess;
+    select_bad_arg_analysis(rocsparse_spitsv, nargs_to_exclude, args_to_exclude, PARAMS_ANALYSIS);
+    stage = rocsparse_spitsv_stage_compute;
+    select_bad_arg_analysis(rocsparse_spitsv, nargs_to_exclude, args_to_exclude, PARAMS_SOLVE);
 
 #undef PARAMS_BUFFER_SIZE
 #undef PARAMS_ANALYSIS
@@ -98,7 +95,7 @@ void testing_spitsv_csr_bad_arg(const Arguments& arg)
 template <typename I, typename J, typename T>
 void testing_spitsv_csr(const Arguments& arg)
 {
-
+    auto tol = get_near_check_tol<T>(arg);
     //
     // Set nmaxiter.
     //
@@ -122,10 +119,6 @@ void testing_spitsv_csr(const Arguments& arg)
     rocsparse_diag_type  diag    = arg.diag;
     rocsparse_fill_mode  uplo    = arg.uplo;
 
-    rocsparse_spitsv_stage buffersize = rocsparse_spitsv_stage_buffer_size;
-    rocsparse_spitsv_stage preprocess = rocsparse_spitsv_stage_preprocess;
-    rocsparse_spitsv_stage compute    = rocsparse_spitsv_stage_compute;
-
     T halpha = arg.get_alpha<T>();
 
     // Index and data type
@@ -136,114 +129,6 @@ void testing_spitsv_csr(const Arguments& arg)
     // Create rocsparse handle
     rocsparse_local_handle handle;
 
-    // Argument sanity check before allocating invalid memory
-    if(M <= 0)
-    {
-        // M == 0 means nnz can only be 0, too
-
-        static const I safe_size = 100;
-
-        // Allocate memory on device
-        device_vector<I> dcsr_row_ptr(safe_size);
-        device_vector<J> dcsr_col_ind(safe_size);
-        device_vector<T> dcsr_val(safe_size);
-        device_vector<T> dx(safe_size);
-        device_vector<T> dy(safe_size);
-
-        if(!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val || !dx || !dy)
-        {
-            CHECK_HIP_ERROR(hipErrorOutOfMemory);
-            return;
-        }
-
-        // Check Spitsv when structures can be created
-        if(M == 0 && M == N)
-        {
-            // Pointer mode
-            CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
-            // Check structures
-            I                     nnz_A = 0;
-            rocsparse_local_spmat A(M,
-                                    N,
-                                    nnz_A,
-                                    dcsr_row_ptr,
-                                    dcsr_col_ind,
-                                    dcsr_val,
-                                    itype,
-                                    jtype,
-                                    base,
-                                    ttype,
-                                    rocsparse_format_csr);
-
-            rocsparse_local_dnvec x(M, dx, ttype);
-            rocsparse_local_dnvec y(M, dy, ttype);
-
-            EXPECT_ROCSPARSE_STATUS(
-                rocsparse_spmat_set_attribute(A, rocsparse_spmat_fill_mode, &uplo, sizeof(uplo)),
-                rocsparse_status_success);
-
-            EXPECT_ROCSPARSE_STATUS(
-                rocsparse_spmat_set_attribute(A, rocsparse_spmat_diag_type, &diag, sizeof(diag)),
-                rocsparse_status_success);
-
-            size_t buffer_size;
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spitsv(handle,
-                                                     host_nmaxiter,
-                                                     host_tol,
-                                                     host_history,
-                                                     trans_A,
-                                                     &halpha,
-                                                     A,
-                                                     x,
-                                                     y,
-                                                     ttype,
-                                                     alg,
-                                                     buffersize,
-                                                     &buffer_size,
-                                                     nullptr),
-                                    rocsparse_status_success);
-
-            void* dbuffer;
-            CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, safe_size));
-
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spitsv(handle,
-                                                     host_nmaxiter,
-                                                     host_tol,
-                                                     host_history,
-                                                     trans_A,
-                                                     &halpha,
-                                                     A,
-                                                     x,
-                                                     y,
-                                                     ttype,
-                                                     alg,
-                                                     preprocess,
-                                                     nullptr,
-                                                     dbuffer),
-                                    rocsparse_status_success);
-
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spitsv(handle,
-                                                     host_nmaxiter,
-                                                     host_tol,
-                                                     host_history,
-                                                     trans_A,
-                                                     &halpha,
-                                                     A,
-                                                     x,
-                                                     y,
-                                                     ttype,
-                                                     alg,
-                                                     compute,
-                                                     &buffer_size,
-                                                     dbuffer),
-                                    rocsparse_status_success);
-            CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
-        }
-
-        return;
-    }
-
     rocsparse_matrix_factory<T, I, J> matrix_factory(arg, false, true);
 
     // Allocate host memory for matrix
@@ -292,12 +177,6 @@ void testing_spitsv_csr(const Arguments& arg)
     device_vector<T> dy_2(M);
     device_vector<T> dalpha(1);
 
-    if(!dcsr_row_ptr || !dcsr_col_ind || !dcsr_val || !dx || !dy_1 || !dy_2 || !dalpha)
-    {
-        CHECK_HIP_ERROR(hipErrorOutOfMemory);
-        return;
-    }
-
     // Copy data from CPU to device
     CHECK_HIP_ERROR(
         hipMemcpy(dcsr_row_ptr, hcsr_row_ptr.data(), sizeof(I) * (M + 1), hipMemcpyHostToDevice));
@@ -310,17 +189,9 @@ void testing_spitsv_csr(const Arguments& arg)
     CHECK_HIP_ERROR(hipMemcpy(dalpha, &halpha, sizeof(T), hipMemcpyHostToDevice));
 
     // Create descriptors
-    rocsparse_local_spmat A(M,
-                            N,
-                            nnz_A,
-                            dcsr_row_ptr,
-                            dcsr_col_ind,
-                            dcsr_val,
-                            itype,
-                            jtype,
-                            base,
-                            ttype,
-                            rocsparse_format_csr);
+    rocsparse_local_spmat A(
+        M, N, nnz_A, dcsr_row_ptr, dcsr_col_ind, dcsr_val, itype, jtype, base, ttype);
+
     rocsparse_local_dnvec x(M, dx, ttype);
     rocsparse_local_dnvec y1(M, dy_1, ttype);
     rocsparse_local_dnvec y2(M, dy_2, ttype);
@@ -344,7 +215,7 @@ void testing_spitsv_csr(const Arguments& arg)
                                            y1,
                                            ttype,
                                            alg,
-                                           rocsparse_spitsv_stage_auto /*buffersize*/,
+                                           rocsparse_spitsv_stage_buffer_size,
                                            &buffer_size,
                                            nullptr));
 
@@ -365,7 +236,7 @@ void testing_spitsv_csr(const Arguments& arg)
                                            y1,
                                            ttype,
                                            alg,
-                                           rocsparse_spitsv_stage_auto /*preprocess*/,
+                                           rocsparse_spitsv_stage_preprocess,
                                            nullptr,
                                            dbuffer));
 
@@ -382,7 +253,7 @@ void testing_spitsv_csr(const Arguments& arg)
                                            y2,
                                            ttype,
                                            alg,
-                                           rocsparse_spitsv_stage_auto /*preprocess*/,
+                                           rocsparse_spitsv_stage_preprocess,
                                            nullptr,
                                            dbuffer));
 
@@ -404,7 +275,7 @@ void testing_spitsv_csr(const Arguments& arg)
                                                y1,
                                                ttype,
                                                alg,
-                                               rocsparse_spitsv_stage_auto /*compute*/,
+                                               rocsparse_spitsv_stage_compute,
                                                &buffer_size,
                                                dbuffer));
 
@@ -423,7 +294,7 @@ void testing_spitsv_csr(const Arguments& arg)
                                                y2,
                                                ttype,
                                                alg,
-                                               rocsparse_spitsv_stage_auto /*compute*/,
+                                               rocsparse_spitsv_stage_compute,
                                                &buffer_size,
                                                dbuffer));
 
@@ -444,6 +315,7 @@ void testing_spitsv_csr(const Arguments& arg)
                             hcsr_col_ind,
                             hcsr_val,
                             hx,
+                            (int64_t)1,
                             hy_gold,
                             diag,
                             uplo,
@@ -453,8 +325,13 @@ void testing_spitsv_csr(const Arguments& arg)
 
         if(analysis_pivot == -1 && solve_pivot == -1)
         {
-            hy_gold.near_check(hy_1);
-            hy_gold.near_check(hy_2);
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save(
+                    "Y pointe mode host", hy_1, "Y pointe mode device", hy_2);
+            }
+            hy_gold.near_check(hy_1, tol);
+            hy_gold.near_check(hy_2, tol);
         }
     }
 
@@ -482,7 +359,7 @@ void testing_spitsv_csr(const Arguments& arg)
                                                    y1,
                                                    ttype,
                                                    alg,
-                                                   compute,
+                                                   rocsparse_spitsv_stage_compute,
                                                    &buffer_size,
                                                    dbuffer));
         }
@@ -506,7 +383,7 @@ void testing_spitsv_csr(const Arguments& arg)
                                                    y1,
                                                    ttype,
                                                    alg,
-                                                   compute,
+                                                   rocsparse_spitsv_stage_compute,
                                                    &buffer_size,
                                                    dbuffer));
             gpu_time_used_iter = (get_time_us() - gpu_time_used_iter);
