@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 
 #include <tuple>
 
-template <typename I, typename J, typename T>
+template <typename I, typename J, typename A, typename B, typename C, typename T>
 void testing_spmm_batched_csr_bad_arg(const Arguments& arg)
 {
     static const size_t safe_size = 100;
@@ -41,38 +41,33 @@ void testing_spmm_batched_csr_bad_arg(const Arguments& arg)
     void*                csr_val     = (void*)0x4;
     void*                csr_row_ptr = (void*)0x4;
     void*                csr_col_ind = (void*)0x4;
-    void*                B           = (void*)0x4;
-    void*                C           = (void*)0x4;
+    void*                dense_B     = (void*)0x4;
+    void*                dense_C     = (void*)0x4;
     size_t*              buffer_size = (size_t*)0x4;
     void*                temp_buffer = (void*)0x4;
     rocsparse_operation  trans_A     = rocsparse_operation_none;
     rocsparse_operation  trans_B     = rocsparse_operation_none;
     rocsparse_index_base base        = rocsparse_index_base_zero;
-    rocsparse_order      order       = rocsparse_order_column;
+    rocsparse_order      order_B     = rocsparse_order_column;
+    rocsparse_order      order_C     = rocsparse_order_column;
     rocsparse_spmm_alg   alg         = rocsparse_spmm_alg_default;
-    rocsparse_spmm_stage stage       = rocsparse_spmm_stage_auto;
+    rocsparse_spmm_stage stage       = rocsparse_spmm_stage_compute;
 
     rocsparse_indextype itype = get_indextype<I>();
     rocsparse_indextype jtype = get_indextype<J>();
+    rocsparse_datatype  atype = get_datatype<A>();
+    rocsparse_datatype  btype = get_datatype<B>();
+    rocsparse_datatype  ctype = get_datatype<C>();
     rocsparse_datatype  ttype = get_datatype<T>();
 
     T alpha = static_cast<T>(1.0);
     T beta  = static_cast<T>(0.0);
 
     // SpMM structures
-    rocsparse_local_spmat local_mat_A(m,
-                                      k,
-                                      nnz,
-                                      csr_row_ptr,
-                                      csr_col_ind,
-                                      csr_val,
-                                      itype,
-                                      jtype,
-                                      base,
-                                      ttype,
-                                      rocsparse_format_csr);
-    rocsparse_local_dnmat local_mat_B(k, n, k, B, ttype, order);
-    rocsparse_local_dnmat local_mat_C(m, n, m, C, ttype, order);
+    rocsparse_local_spmat local_mat_A(
+        m, k, nnz, csr_row_ptr, csr_col_ind, csr_val, itype, jtype, base, atype);
+    rocsparse_local_dnmat local_mat_B(k, n, k, dense_B, btype, order_B);
+    rocsparse_local_dnmat local_mat_C(m, n, m, dense_C, ctype, order_C);
 
     rocsparse_spmat_descr mat_A = local_mat_A;
     rocsparse_dnmat_descr mat_B = local_mat_B;
@@ -82,13 +77,13 @@ void testing_spmm_batched_csr_bad_arg(const Arguments& arg)
     handle, trans_A, trans_B, &alpha, mat_A, mat_B, &beta, mat_C, ttype, alg, stage, buffer_size, \
         temp_buffer
 
-    int     batch_count_A;
-    int     batch_count_B;
-    int     batch_count_C;
-    int64_t offsets_batch_stride_A;
-    int64_t columns_values_batch_stride_A;
-    int64_t batch_stride_B;
-    int64_t batch_stride_C;
+    rocsparse_int batch_count_A;
+    rocsparse_int batch_count_B;
+    rocsparse_int batch_count_C;
+    int64_t       offsets_batch_stride_A;
+    int64_t       columns_values_batch_stride_A;
+    int64_t       batch_stride_B;
+    int64_t       batch_stride_C;
 
     // C_i = A * B_i
     batch_count_A                 = 1;
@@ -149,17 +144,20 @@ void testing_spmm_batched_csr_bad_arg(const Arguments& arg)
 #undef PARAMS
 }
 
-template <typename I, typename J, typename T>
+template <typename I, typename J, typename A, typename B, typename C, typename T>
 void testing_spmm_batched_csr(const Arguments& arg)
 {
-    J                    M       = arg.M;
-    J                    N       = arg.N;
-    J                    K       = arg.K;
-    rocsparse_operation  trans_A = arg.transA;
-    rocsparse_operation  trans_B = arg.transB;
-    rocsparse_index_base base    = arg.baseA;
-    rocsparse_spmm_alg   alg     = arg.spmm_alg;
-    rocsparse_order      order   = arg.order;
+    J                    M               = arg.M;
+    J                    N               = arg.N;
+    J                    K               = arg.K;
+    rocsparse_operation  trans_A         = arg.transA;
+    rocsparse_operation  trans_B         = arg.transB;
+    rocsparse_index_base base            = arg.baseA;
+    rocsparse_spmm_alg   alg             = arg.spmm_alg;
+    rocsparse_order      order_B         = arg.orderB;
+    rocsparse_order      order_C         = arg.orderC;
+    rocsparse_int        ld_multiplier_B = arg.ld_multiplier_B;
+    rocsparse_int        ld_multiplier_C = arg.ld_multiplier_C;
 
     J batch_count_A = arg.batch_count_A;
     J batch_count_B = arg.batch_count_B;
@@ -171,16 +169,14 @@ void testing_spmm_batched_csr(const Arguments& arg)
     // Index and data type
     rocsparse_indextype itype = get_indextype<I>();
     rocsparse_indextype jtype = get_indextype<J>();
+    rocsparse_datatype  atype = get_datatype<A>();
+    rocsparse_datatype  btype = get_datatype<B>();
+    rocsparse_datatype  ctype = get_datatype<C>();
     rocsparse_datatype  ttype = get_datatype<T>();
 
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
 
-    if(M <= 0 || N <= 0 || K <= 0)
-    {
-        return;
-    }
-
     bool Ci_A_Bi  = (batch_count_A == 1 && batch_count_B == batch_count_C);
     bool Ci_Ai_B  = (batch_count_B == 1 && batch_count_A == batch_count_C);
     bool Ci_Ai_Bi = (batch_count_A == batch_count_C && batch_count_A == batch_count_B);
@@ -191,12 +187,12 @@ void testing_spmm_batched_csr(const Arguments& arg)
     }
 
     // Allocate host memory for matrix
-    rocsparse_matrix_factory<T, I, J> matrix_factory(arg);
+    rocsparse_matrix_factory<A, I, J> matrix_factory(arg);
 
     // Generate single batch of A matrix
     host_vector<I> hcsr_row_ptr_temp;
     host_vector<J> hcsr_col_ind_temp;
-    host_vector<T> hcsr_val_temp;
+    host_vector<A> hcsr_val_temp;
 
     I nnz_A;
     matrix_factory.init_csr(hcsr_row_ptr_temp,
@@ -215,28 +211,31 @@ void testing_spmm_batched_csr(const Arguments& arg)
     J C_m = M;
     J C_n = N;
 
-    J ldb = (order == rocsparse_order_column)
-                ? ((trans_B == rocsparse_operation_none) ? (2 * K) : (2 * N))
-                : ((trans_B == rocsparse_operation_none) ? (2 * N) : (2 * K));
-    J ldc = (order == rocsparse_order_column) ? (2 * M) : (2 * N);
+    int64_t ldb = (order_B == rocsparse_order_column)
+                      ? ((trans_B == rocsparse_operation_none) ? (int64_t(ld_multiplier_B) * K)
+                                                               : (int64_t(ld_multiplier_B) * N))
+                      : ((trans_B == rocsparse_operation_none) ? (int64_t(ld_multiplier_B) * N)
+                                                               : (int64_t(ld_multiplier_B) * K));
+    int64_t ldc = (order_C == rocsparse_order_column) ? (int64_t(ld_multiplier_C) * M)
+                                                      : (int64_t(ld_multiplier_C) * N);
 
-    J nrowB = (order == rocsparse_order_column) ? ldb : B_m;
-    J ncolB = (order == rocsparse_order_column) ? B_n : ldb;
-    J nrowC = (order == rocsparse_order_column) ? ldc : C_m;
-    J ncolC = (order == rocsparse_order_column) ? C_n : ldc;
+    int64_t nrowB = (order_B == rocsparse_order_column) ? ldb : B_m;
+    int64_t ncolB = (order_B == rocsparse_order_column) ? B_n : ldb;
+    int64_t nrowC = (order_C == rocsparse_order_column) ? ldc : C_m;
+    int64_t ncolC = (order_C == rocsparse_order_column) ? C_n : ldc;
 
-    I nnz_B = nrowB * ncolB;
-    I nnz_C = nrowC * ncolC;
+    int64_t nnz_B = nrowB * ncolB;
+    int64_t nnz_C = nrowC * ncolC;
 
-    I offsets_batch_stride_A        = (batch_count_A > 1) ? (A_m + 1) : 0;
-    I columns_values_batch_stride_A = (batch_count_A > 1) ? nnz_A : 0;
-    I batch_stride_B                = (batch_count_B > 1) ? nnz_B : 0;
-    I batch_stride_C                = (batch_count_C > 1) ? nnz_C : 0;
+    int64_t offsets_batch_stride_A        = (batch_count_A > 1) ? (A_m + 1) : 0;
+    int64_t columns_values_batch_stride_A = (batch_count_A > 1) ? nnz_A : 0;
+    int64_t batch_stride_B                = (batch_count_B > 1) ? nnz_B : 0;
+    int64_t batch_stride_C                = (batch_count_C > 1) ? nnz_C : 0;
 
     // Allocate host memory for all batches of A matrix
     host_vector<I> hcsr_row_ptr(batch_count_A * (A_m + 1));
     host_vector<J> hcsr_col_ind(batch_count_A * nnz_A);
-    host_vector<T> hcsr_val(batch_count_A * nnz_A);
+    host_vector<A> hcsr_val(batch_count_A * nnz_A);
 
     for(J i = 0; i < batch_count_A; i++)
     {
@@ -253,14 +252,14 @@ void testing_spmm_batched_csr(const Arguments& arg)
     }
 
     // Allocate host memory for vectors
-    host_vector<T> hB(batch_count_B * nnz_B);
-    host_vector<T> hC_1(batch_count_C * nnz_C);
-    host_vector<T> hC_2(batch_count_C * nnz_C);
-    host_vector<T> hC_gold(batch_count_C * nnz_C);
+    host_vector<B> hB(batch_count_B * nnz_B);
+    host_vector<C> hC_1(batch_count_C * nnz_C);
+    host_vector<C> hC_2(batch_count_C * nnz_C);
+    host_vector<C> hC_gold(batch_count_C * nnz_C);
 
     // Initialize data on CPU
-    rocsparse_init<T>(hB, batch_count_B * nnz_B, 1, 1);
-    rocsparse_init<T>(hC_1, batch_count_C * nnz_C, 1, 1);
+    rocsparse_init<B>(hB, batch_count_B * nnz_B, 1, 1);
+    rocsparse_init<C>(hC_1, batch_count_C * nnz_C, 1, 1);
 
     hC_2    = hC_1;
     hC_gold = hC_1;
@@ -268,10 +267,10 @@ void testing_spmm_batched_csr(const Arguments& arg)
     // Allocate device memory
     device_vector<I> dcsr_row_ptr(hcsr_row_ptr);
     device_vector<J> dcsr_col_ind(hcsr_col_ind);
-    device_vector<T> dcsr_val(hcsr_val);
-    device_vector<T> dB(hB);
-    device_vector<T> dC_1(hC_1);
-    device_vector<T> dC_2(hC_2);
+    device_vector<A> dcsr_val(hcsr_val);
+    device_vector<B> dB(hB);
+    device_vector<C> dC_1(hC_1);
+    device_vector<C> dC_2(hC_2);
     device_vector<T> dalpha(1);
     device_vector<T> dbeta(1);
 
@@ -279,26 +278,21 @@ void testing_spmm_batched_csr(const Arguments& arg)
     CHECK_HIP_ERROR(hipMemcpy(dbeta, &hbeta, sizeof(T), hipMemcpyHostToDevice));
 
     // Create descriptors
-    rocsparse_local_spmat A(A_m,
-                            A_n,
-                            nnz_A,
-                            dcsr_row_ptr,
-                            dcsr_col_ind,
-                            dcsr_val,
-                            itype,
-                            jtype,
-                            base,
-                            ttype,
-                            rocsparse_format_csr);
-    rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, order);
-    rocsparse_local_dnmat C1(C_m, C_n, ldc, dC_1, ttype, order);
-    rocsparse_local_dnmat C2(C_m, C_n, ldc, dC_2, ttype, order);
+    rocsparse_local_spmat mat_A(
+        A_m, A_n, nnz_A, dcsr_row_ptr, dcsr_col_ind, dcsr_val, itype, jtype, base, atype);
+
+    ldb = std::max(int64_t(1), ldb);
+    ldc = std::max(int64_t(1), ldc);
+
+    rocsparse_local_dnmat mat_B(B_m, B_n, ldb, dB, btype, order_B);
+    rocsparse_local_dnmat mat_C1(C_m, C_n, ldc, dC_1, ctype, order_C);
+    rocsparse_local_dnmat mat_C2(C_m, C_n, ldc, dC_2, ctype, order_C);
 
     CHECK_ROCSPARSE_ERROR(rocsparse_csr_set_strided_batch(
-        A, batch_count_A, offsets_batch_stride_A, columns_values_batch_stride_A));
-    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(B, batch_count_B, batch_stride_B));
-    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(C1, batch_count_C, batch_stride_C));
-    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(C2, batch_count_C, batch_stride_C));
+        mat_A, batch_count_A, offsets_batch_stride_A, columns_values_batch_stride_A));
+    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(mat_B, batch_count_B, batch_stride_B));
+    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(mat_C1, batch_count_C, batch_stride_C));
+    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(mat_C2, batch_count_C, batch_stride_C));
 
     // Query SpMM buffer
     size_t buffer_size;
@@ -306,10 +300,10 @@ void testing_spmm_batched_csr(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_buffer_size,
@@ -324,10 +318,10 @@ void testing_spmm_batched_csr(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_preprocess,
@@ -342,10 +336,10 @@ void testing_spmm_batched_csr(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       &halpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       &hbeta,
-                                                      C1,
+                                                      mat_C1,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
@@ -358,10 +352,10 @@ void testing_spmm_batched_csr(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       dalpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       dbeta,
-                                                      C2,
+                                                      mat_C2,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
@@ -373,30 +367,31 @@ void testing_spmm_batched_csr(const Arguments& arg)
         hC_2.transfer_from(dC_2);
 
         // CPU csrmm_batched
-        host_csrmm_batched<T, I, J>(A_m,
-                                    N,
-                                    A_n,
-                                    batch_count_A,
-                                    offsets_batch_stride_A,
-                                    columns_values_batch_stride_A,
-                                    trans_A,
-                                    trans_B,
-                                    halpha,
-                                    hcsr_row_ptr,
-                                    hcsr_col_ind,
-                                    hcsr_val,
-                                    hB,
-                                    ldb,
-                                    batch_count_B,
-                                    batch_stride_B,
-                                    hbeta,
-                                    hC_gold,
-                                    ldc,
-                                    batch_count_C,
-                                    batch_stride_C,
-                                    order,
-                                    base,
-                                    false);
+        host_csrmm_batched<T, I, J, A, B, C>(A_m,
+                                             N,
+                                             A_n,
+                                             batch_count_A,
+                                             offsets_batch_stride_A,
+                                             columns_values_batch_stride_A,
+                                             trans_A,
+                                             trans_B,
+                                             halpha,
+                                             hcsr_row_ptr,
+                                             hcsr_col_ind,
+                                             hcsr_val,
+                                             hB,
+                                             ldb,
+                                             batch_count_B,
+                                             batch_stride_B,
+                                             order_B,
+                                             hbeta,
+                                             hC_gold,
+                                             ldc,
+                                             batch_count_C,
+                                             batch_stride_C,
+                                             order_C,
+                                             base,
+                                             false);
 
         hC_gold.near_check(hC_1);
         hC_gold.near_check(hC_2);
@@ -416,10 +411,10 @@ void testing_spmm_batched_csr(const Arguments& arg)
                                                  trans_A,
                                                  trans_B,
                                                  &halpha,
-                                                 A,
-                                                 B,
+                                                 mat_A,
+                                                 mat_B,
                                                  &hbeta,
-                                                 C1,
+                                                 mat_C1,
                                                  ttype,
                                                  alg,
                                                  rocsparse_spmm_stage_compute,
@@ -436,10 +431,10 @@ void testing_spmm_batched_csr(const Arguments& arg)
                                                  trans_A,
                                                  trans_B,
                                                  &halpha,
-                                                 A,
-                                                 B,
+                                                 mat_A,
+                                                 mat_B,
                                                  &hbeta,
-                                                 C1,
+                                                 mat_C1,
                                                  ttype,
                                                  alg,
                                                  rocsparse_spmm_stage_compute,
@@ -464,40 +459,47 @@ void testing_spmm_batched_csr(const Arguments& arg)
                                                           hbeta != static_cast<T>(0));
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "N",
+                            display_key_t::N,
                             N,
-                            "K",
+                            display_key_t::K,
                             K,
-                            "nnz_A",
+                            display_key_t::nnz_A,
                             nnz_A,
-                            "batch_count_A",
+                            display_key_t::batch_count_A,
                             batch_count_A,
-                            "batch_count_B",
+                            display_key_t::batch_count_B,
                             batch_count_B,
-                            "batch_count_C",
+                            display_key_t::batch_count_C,
                             batch_count_C,
-                            "alpha",
+                            display_key_t::alpha,
                             halpha,
-                            "beta",
+                            display_key_t::beta,
                             hbeta,
-                            "Algorithm",
+                            display_key_t::algorithm,
                             rocsparse_spmmalg2string(alg),
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 
     CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
 }
 
-#define INSTANTIATE(ITYPE, JTYPE, TTYPE)                                                       \
-    template void testing_spmm_batched_csr_bad_arg<ITYPE, JTYPE, TTYPE>(const Arguments& arg); \
-    template void testing_spmm_batched_csr<ITYPE, JTYPE, TTYPE>(const Arguments& arg)
+#define INSTANTIATE(ITYPE, JTYPE, TTYPE)                                                      \
+    template void testing_spmm_batched_csr_bad_arg<ITYPE, JTYPE, TTYPE, TTYPE, TTYPE, TTYPE>( \
+        const Arguments& arg);                                                                \
+    template void testing_spmm_batched_csr<ITYPE, JTYPE, TTYPE, TTYPE, TTYPE, TTYPE>(         \
+        const Arguments& arg)
+#define INSTANTIATE_MIXED(ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE)                           \
+    template void testing_spmm_batched_csr_bad_arg<ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE>( \
+        const Arguments& arg);                                                                \
+    template void testing_spmm_batched_csr<ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE>(         \
+        const Arguments& arg)
 
 INSTANTIATE(int32_t, int32_t, float);
 INSTANTIATE(int32_t, int32_t, double);
@@ -511,4 +513,12 @@ INSTANTIATE(int64_t, int64_t, float);
 INSTANTIATE(int64_t, int64_t, double);
 INSTANTIATE(int64_t, int64_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
+
+INSTANTIATE_MIXED(int32_t, int32_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int64_t, int32_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int64_t, int64_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int32_t, int32_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int64_t, int32_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int64_t, int64_t, int8_t, int8_t, float, float);
+
 void testing_spmm_batched_csr_extra(const Arguments& arg) {}
