@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -29,27 +29,42 @@ void testing_spmm_bell_bad_arg(const Arguments& arg)
     // Create rocsparse handle
     rocsparse_local_handle local_handle;
 
-    rocsparse_handle      handle      = local_handle;
-    rocsparse_operation   trans_A     = rocsparse_operation_none;
-    rocsparse_operation   trans_B     = rocsparse_operation_none;
-    void*                 alpha       = (void*)0x4;
-    rocsparse_spmat_descr A           = (rocsparse_spmat_descr)0x4;
-    rocsparse_dnmat_descr B           = (rocsparse_dnmat_descr)0x4;
-    void*                 beta        = (void*)0x4;
-    rocsparse_dnmat_descr C           = (rocsparse_dnmat_descr)0x4;
-    rocsparse_datatype    ttype       = rocsparse_datatype_f32_r;
-    rocsparse_spmm_alg    alg         = rocsparse_spmm_alg_bell;
-    rocsparse_spmm_stage  stage       = rocsparse_spmm_stage_auto;
-    size_t*               buffer_size = (size_t*)0x4;
-    void*                 buffer      = (void*)0x4;
+    rocsparse_handle     handle       = local_handle;
+    rocsparse_operation  trans_A      = rocsparse_operation_none;
+    rocsparse_operation  trans_B      = rocsparse_operation_none;
+    void*                alpha        = (void*)0x4;
+    void*                beta         = (void*)0x4;
+    rocsparse_datatype   compute_type = rocsparse_datatype_f32_r;
+    rocsparse_spmm_alg   alg          = rocsparse_spmm_alg_bell;
+    rocsparse_spmm_stage stage        = rocsparse_spmm_stage_compute;
+    size_t*              buffer_size  = (size_t*)0x4;
+    void*                buffer       = (void*)0x4;
 
-#define PARAMS \
-    handle, trans_A, trans_B, &alpha, A, B, &beta, C, ttype, alg, stage, buffer_size, buffer
+    rocsparse_local_spmat local_mat_A(4,
+                                      4,
+                                      rocsparse_direction_row,
+                                      2,
+                                      2,
+                                      (void*)0x4,
+                                      (void*)0x4,
+                                      rocsparse_indextype_i32,
+                                      rocsparse_index_base_zero,
+                                      compute_type);
+    rocsparse_local_dnmat local_mat_B(4, 4, 4, (void*)0x4, compute_type, rocsparse_order_column);
+    rocsparse_local_dnmat local_mat_C(4, 4, 4, (void*)0x4, compute_type, rocsparse_order_column);
+
+    rocsparse_spmat_descr mat_A = local_mat_A;
+    rocsparse_dnmat_descr mat_B = local_mat_B;
+    rocsparse_dnmat_descr mat_C = local_mat_C;
+
+#define PARAMS                                                                            \
+    handle, trans_A, trans_B, alpha, mat_A, mat_B, beta, mat_C, compute_type, alg, stage, \
+        buffer_size, buffer
 
     static const int nargs_to_exclude                  = 2;
     static const int args_to_exclude[nargs_to_exclude] = {11, 12};
 
-    auto_testing_bad_arg(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
+    select_bad_arg_analysis(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
 
 #undef PARAMS
 }
@@ -70,14 +85,16 @@ void testing_spmm_bell(const Arguments& arg)
     rocsparse_direction  direction = arg.direction;
     rocsparse_index_base base      = arg.baseA;
     rocsparse_spmm_alg   alg       = arg.spmm_alg;
-    rocsparse_order      order     = arg.order;
+    rocsparse_order      order_B   = arg.orderB;
+    rocsparse_order      order_C   = arg.orderC;
 
     I Mb = -1, Nb = -1, Kb = -1;
 
-    host_scalar<T> h_alpha, h_beta;
+    host_scalar<T> h_alpha(arg.get_alpha<T>());
+    host_scalar<T> h_beta(arg.get_beta<T>());
 
-    *h_alpha = arg.get_alpha<T>();
-    *h_beta  = arg.get_beta<T>();
+    device_scalar<T> d_alpha(h_alpha);
+    device_scalar<T> d_beta(h_beta);
 
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
@@ -89,97 +106,6 @@ void testing_spmm_bell(const Arguments& arg)
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_index_base(descr, base));
     CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-    // Argument sanity check before allocating invalid memory
-    if(M <= 0 || N <= 0 || K <= 0 || block_dim <= 0)
-    {
-        const rocsparse_int safe_width = 4;
-        // Pointer mode
-        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
-        I nrow_A = trans_A == rocsparse_operation_none ? M : K;
-        I ncol_A = trans_A == rocsparse_operation_none ? K : M;
-
-        rocsparse_local_spmat A(nrow_A,
-                                ncol_A,
-                                direction,
-                                block_dim,
-                                std::min(safe_width * block_dim, ncol_A),
-                                (I*)0x4,
-                                (T*)0x4,
-                                itype,
-                                base,
-                                ttype);
-        I                     ldb = order == rocsparse_order_column
-                                        ? (trans_B == rocsparse_operation_none ? 2 * K : 2 * N)
-                                        : (trans_B == rocsparse_operation_none ? 2 * N : 2 * K);
-
-        I nrow_B = trans_B == rocsparse_operation_none ? K : N;
-        I ncol_B = trans_B == rocsparse_operation_none ? N : K;
-
-        I                     ldc    = order == rocsparse_order_column ? 2 * M : 2 * N;
-        I                     nrow_C = M;
-        I                     ncol_C = N;
-        rocsparse_local_dnmat B(nrow_B, ncol_B, ldb, (void*)0x4, ttype, order);
-        rocsparse_local_dnmat C(nrow_C, ncol_C, ldc, (void*)0x4, ttype, order);
-
-        void*  dbuffer     = nullptr;
-        size_t buffer_size = sizeof(I);
-        EXPECT_ROCSPARSE_STATUS(rocsparse_spmm(handle,
-                                               trans_A,
-                                               trans_B,
-                                               h_alpha,
-                                               A,
-                                               B,
-                                               h_beta,
-                                               C,
-                                               ttype,
-                                               alg,
-                                               rocsparse_spmm_stage_buffer_size,
-                                               &buffer_size,
-                                               dbuffer),
-                                (M < 0 || N < 0 || K < 0 || block_dim <= 0)
-                                    ? rocsparse_status_invalid_pointer
-                                    : rocsparse_status_success);
-
-        CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, buffer_size));
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_spmm(handle,
-                                               trans_A,
-                                               trans_B,
-                                               h_alpha,
-                                               A,
-                                               B,
-                                               h_beta,
-                                               C,
-                                               ttype,
-                                               alg,
-                                               rocsparse_spmm_stage_preprocess,
-                                               &buffer_size,
-                                               dbuffer),
-                                (M < 0 || N < 0 || K < 0 || block_dim <= 0)
-                                    ? rocsparse_status_invalid_pointer
-                                    : rocsparse_status_success);
-
-        EXPECT_ROCSPARSE_STATUS(rocsparse_spmm(handle,
-                                               trans_A,
-                                               trans_B,
-                                               h_alpha,
-                                               A,
-                                               B,
-                                               h_beta,
-                                               C,
-                                               ttype,
-                                               alg,
-                                               rocsparse_spmm_stage_compute,
-                                               &buffer_size,
-                                               dbuffer),
-                                (M < 0 || N < 0 || K < 0 || block_dim <= 0)
-                                    ? rocsparse_status_invalid_pointer
-                                    : rocsparse_status_success);
-        CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
-        return;
-    }
-
     rocsparse_matrix_factory<T, I, I> matrix_factory(arg);
 
     host_ell_matrix<T, I> hA;
@@ -223,10 +149,19 @@ void testing_spmm_bell(const Arguments& arg)
                             ttype);
 
     rocsparse_local_dnmat B(
-        dB.m, dB.n, (order == rocsparse_order_column) ? dB.m : dB.n, dB, ttype, order);
+        dB.m,
+        dB.n,
+        std::max(rocsparse_int(1), (order_B == rocsparse_order_column) ? dB.m : dB.n),
+        dB,
+        ttype,
+        order_B);
     rocsparse_local_dnmat C(
-        dC.m, dC.n, (order == rocsparse_order_column) ? dC.m : dC.n, dC, ttype, order);
-
+        dC.m,
+        dC.n,
+        std::max(rocsparse_int(1), (order_C == rocsparse_order_column) ? dC.m : dC.n),
+        dC,
+        ttype,
+        order_C);
     // Query SpMM buffer
     size_t buffer_size;
     CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
@@ -281,6 +216,12 @@ void testing_spmm_bell(const Arguments& arg)
                                                       rocsparse_spmm_stage_compute,
                                                       &buffer_size,
                                                       dbuffer));
+
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("C pointer mode host", dC);
+        }
+
         {
             host_dense_matrix<T> hC_copy(hC);
 
@@ -345,23 +286,24 @@ void testing_spmm_bell(const Arguments& arg)
                 }
             }
 
-            host_coomm<T, I>(M,
-                             N,
-                             K,
-                             nnzb * block_dim * block_dim,
-                             trans_A,
-                             trans_B,
-                             *h_alpha,
-                             coo_row,
-                             coo_col,
-                             coo_val,
-                             hB,
-                             (order == rocsparse_order_column) ? hB.m : hB.n,
-                             *h_beta,
-                             hC,
-                             (order == rocsparse_order_column) ? hC.m : hC.n,
-                             order,
-                             base);
+            host_coomm<T, I, T, T, T>(M,
+                                      N,
+                                      K,
+                                      nnzb * block_dim * block_dim,
+                                      trans_A,
+                                      trans_B,
+                                      *h_alpha,
+                                      coo_row,
+                                      coo_col,
+                                      coo_val,
+                                      hB,
+                                      (order_B == rocsparse_order_column) ? hB.m : hB.n,
+                                      order_B,
+                                      *h_beta,
+                                      hC,
+                                      (order_C == rocsparse_order_column) ? hC.m : hC.n,
+                                      order_C,
+                                      base);
 
             delete[] coo_val;
             delete[] coo_col;
@@ -378,7 +320,6 @@ void testing_spmm_bell(const Arguments& arg)
 
             CHECK_ROCSPARSE_ERROR(
                 rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
-            device_scalar<T> d_alpha(h_alpha), d_beta(h_beta);
             CHECK_ROCSPARSE_ERROR(testing::rocsparse_spmm(handle,
                                                           trans_A,
                                                           trans_B,
@@ -392,6 +333,10 @@ void testing_spmm_bell(const Arguments& arg)
                                                           rocsparse_spmm_stage_compute,
                                                           &buffer_size,
                                                           dbuffer));
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("C pointer mode device", dC);
+            }
         }
     }
 
@@ -452,26 +397,26 @@ void testing_spmm_bell(const Arguments& arg)
                                                   *h_beta != static_cast<T>(0));
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "N",
+                            display_key_t::N,
                             N,
-                            "K",
+                            display_key_t::K,
                             K,
-                            "nnz",
+                            display_key_t::nnz,
                             dA.nnz,
-                            "alpha",
+                            display_key_t::alpha,
                             *h_alpha,
-                            "beta",
+                            display_key_t::beta,
                             *h_beta,
-                            "Algorithm",
+                            display_key_t::algorithm,
                             rocsparse_spmmalg2string(alg),
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
-                            gpu_time_used / 1e3);
+                            display_key_t::time_ms,
+                            get_gpu_time_msec(gpu_time_used));
     }
 
     CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
