@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 
 #include <tuple>
 
-template <typename I, typename J, typename T>
+template <typename I, typename J, typename A, typename B, typename C, typename T>
 void testing_spmm_csc_bad_arg(const Arguments& arg)
 {
     static const size_t safe_size = 100;
@@ -43,20 +43,25 @@ void testing_spmm_csc_bad_arg(const Arguments& arg)
     void*                csc_val     = (void*)0x4;
     void*                csc_col_ptr = (void*)0x4;
     void*                csc_row_ind = (void*)0x4;
-    void*                B           = (void*)0x4;
-    void*                C           = (void*)0x4;
+    void*                dense_B     = (void*)0x4;
+    void*                dense_C     = (void*)0x4;
     rocsparse_operation  trans_A     = rocsparse_operation_none;
     rocsparse_operation  trans_B     = rocsparse_operation_none;
     rocsparse_index_base base        = rocsparse_index_base_zero;
-    rocsparse_order      order       = rocsparse_order_column;
+    rocsparse_order      order_B     = rocsparse_order_column;
+    rocsparse_order      order_C     = rocsparse_order_column;
     rocsparse_spmm_alg   alg         = rocsparse_spmm_alg_default;
-    rocsparse_spmm_stage stage       = rocsparse_spmm_stage_auto;
+    rocsparse_spmm_stage stage       = rocsparse_spmm_stage_compute;
 
-    rocsparse_indextype itype = get_indextype<I>();
-    rocsparse_indextype jtype = get_indextype<J>();
-    rocsparse_datatype  ttype = get_datatype<T>();
+    rocsparse_indextype itype        = get_indextype<I>();
+    rocsparse_indextype jtype        = get_indextype<J>();
+    rocsparse_datatype  atype        = get_datatype<A>();
+    rocsparse_datatype  btype        = get_datatype<B>();
+    rocsparse_datatype  ctype        = get_datatype<C>();
+    rocsparse_datatype  compute_type = get_datatype<T>();
 
     // SpMM structures
+    static const bool     csc_format_required = true;
     rocsparse_local_spmat local_mat_A(m,
                                       k,
                                       nnz,
@@ -66,10 +71,10 @@ void testing_spmm_csc_bad_arg(const Arguments& arg)
                                       itype,
                                       jtype,
                                       base,
-                                      ttype,
-                                      rocsparse_format_csc);
-    rocsparse_local_dnmat local_mat_B(k, n, k, B, ttype, order);
-    rocsparse_local_dnmat local_mat_C(m, n, m, C, ttype, order);
+                                      atype,
+                                      csc_format_required);
+    rocsparse_local_dnmat local_mat_B(k, n, k, dense_B, btype, order_B);
+    rocsparse_local_dnmat local_mat_C(m, n, m, dense_C, ctype, order_C);
 
     rocsparse_spmat_descr mat_A = local_mat_A;
     rocsparse_dnmat_descr mat_B = local_mat_B;
@@ -78,61 +83,49 @@ void testing_spmm_csc_bad_arg(const Arguments& arg)
     int       nargs_to_exclude   = 2;
     const int args_to_exclude[2] = {11, 12};
 
-#define PARAMS                                                                                  \
-    handle, trans_A, trans_B, alpha, mat_A, mat_B, beta, mat_C, ttype, alg, stage, buffer_size, \
-        temp_buffer
+#define PARAMS                                                                            \
+    handle, trans_A, trans_B, alpha, mat_A, mat_B, beta, mat_C, compute_type, alg, stage, \
+        buffer_size, temp_buffer
     {
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     {
         size_t* buffer_size = (size_t*)0x4;
         void*   temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     {
         size_t* buffer_size = nullptr;
         void*   temp_buffer = (void*)0x4;
-        auto_testing_bad_arg(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 
     {
         size_t* buffer_size = nullptr;
         void*   temp_buffer = nullptr;
-        auto_testing_bad_arg(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
+        select_bad_arg_analysis(rocsparse_spmm, nargs_to_exclude, args_to_exclude, PARAMS);
     }
 #undef PARAMS
-
-    EXPECT_ROCSPARSE_STATUS(rocsparse_spmm(handle,
-                                           trans_A,
-                                           trans_B,
-                                           alpha,
-                                           mat_A,
-                                           mat_B,
-                                           beta,
-                                           mat_C,
-                                           ttype,
-                                           alg,
-                                           stage,
-                                           nullptr,
-                                           nullptr),
-                            rocsparse_status_invalid_pointer);
 }
 
-template <typename I, typename J, typename T>
+template <typename I, typename J, typename A, typename B, typename C, typename T>
 void testing_spmm_csc(const Arguments& arg)
 {
-    J                    M       = arg.M;
-    J                    N       = arg.N;
-    J                    K       = arg.K;
-    rocsparse_operation  trans_A = arg.transA;
-    rocsparse_operation  trans_B = arg.transB;
-    rocsparse_index_base base    = arg.baseA;
-    rocsparse_spmm_alg   alg     = arg.spmm_alg;
-    rocsparse_order      order   = arg.order;
+    J                    M               = arg.M;
+    J                    N               = arg.N;
+    J                    K               = arg.K;
+    rocsparse_operation  trans_A         = arg.transA;
+    rocsparse_operation  trans_B         = arg.transB;
+    rocsparse_index_base base            = arg.baseA;
+    rocsparse_spmm_alg   alg             = arg.spmm_alg;
+    rocsparse_order      order_B         = arg.orderB;
+    rocsparse_order      order_C         = arg.orderC;
+    rocsparse_int        ld_multiplier_B = arg.ld_multiplier_B;
+    rocsparse_int        ld_multiplier_C = arg.ld_multiplier_C;
 
     T halpha = arg.get_alpha<T>();
     T hbeta  = arg.get_beta<T>();
@@ -140,119 +133,21 @@ void testing_spmm_csc(const Arguments& arg)
     // Index and data type
     rocsparse_indextype itype = get_indextype<I>();
     rocsparse_indextype jtype = get_indextype<J>();
+    rocsparse_datatype  atype = get_datatype<A>();
+    rocsparse_datatype  btype = get_datatype<B>();
+    rocsparse_datatype  ctype = get_datatype<C>();
     rocsparse_datatype  ttype = get_datatype<T>();
 
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
 
-    // Argument sanity check before allocating invalid memory
-    if(M <= 0 || N <= 0 || K <= 0)
-    {
-        // M == N == 0 means nnz can only be 0, too
-        static const I safe_size = 100;
-
-        // Allocate memory on device
-        device_vector<I> dcsc_col_ptr(safe_size);
-        device_vector<J> dcsc_row_ind(safe_size);
-        device_vector<T> dcsc_val(safe_size);
-        device_vector<T> dB(safe_size);
-        device_vector<T> dC(safe_size);
-
-        // Check SpMM when structures can be created
-        if(M == 0 && N == 0 && K == 0)
-        {
-            // Pointer mode
-            CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
-            J A_m = 0;
-            J A_n = 0;
-            J B_m = 0;
-            J B_n = 0;
-            J C_m = 0;
-            J C_n = 0;
-
-            J ldb = 0;
-            J ldc = 0;
-
-            // Check structures
-            I nnz_A = 0;
-
-            rocsparse_local_spmat A(A_m,
-                                    A_n,
-                                    nnz_A,
-                                    dcsc_col_ptr,
-                                    dcsc_row_ind,
-                                    dcsc_val,
-                                    itype,
-                                    jtype,
-                                    base,
-                                    ttype,
-                                    rocsparse_format_csc);
-            rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, order);
-            rocsparse_local_dnmat C(C_m, C_n, ldc, dC, ttype, order);
-
-            size_t buffer_size;
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spmm(handle,
-                                                   trans_A,
-                                                   trans_B,
-                                                   &halpha,
-                                                   A,
-                                                   B,
-                                                   &hbeta,
-                                                   C,
-                                                   ttype,
-                                                   alg,
-                                                   rocsparse_spmm_stage_buffer_size,
-                                                   &buffer_size,
-                                                   nullptr),
-                                    rocsparse_status_success);
-
-            void* dbuffer;
-            CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, safe_size));
-
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spmm(handle,
-                                                   trans_A,
-                                                   trans_B,
-                                                   &halpha,
-                                                   A,
-                                                   B,
-                                                   &hbeta,
-                                                   C,
-                                                   ttype,
-                                                   alg,
-                                                   rocsparse_spmm_stage_preprocess,
-                                                   &buffer_size,
-                                                   dbuffer),
-                                    rocsparse_status_success);
-
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spmm(handle,
-                                                   trans_A,
-                                                   trans_B,
-                                                   &halpha,
-                                                   A,
-                                                   B,
-                                                   &hbeta,
-                                                   C,
-                                                   ttype,
-                                                   alg,
-                                                   rocsparse_spmm_stage_compute,
-                                                   &buffer_size,
-                                                   dbuffer),
-                                    rocsparse_status_success);
-
-            CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
-        }
-
-        return;
-    }
-
     // Allocate host memory for matrix
     host_vector<I> hcsc_col_ptr;
     host_vector<J> hcsc_row_ind;
-    host_vector<T> hcsc_val;
+    host_vector<A> hcsc_val;
 
     // Allocate host memory for matrix
-    rocsparse_matrix_factory<T, I, J> matrix_factory(arg);
+    rocsparse_matrix_factory<A, I, J> matrix_factory(arg);
 
     I nnz_A;
     matrix_factory.init_csc(hcsc_col_ptr,
@@ -271,28 +166,31 @@ void testing_spmm_csc(const Arguments& arg)
     J C_m = M;
     J C_n = N;
 
-    J ldb = (order == rocsparse_order_column)
-                ? ((trans_B == rocsparse_operation_none) ? (2 * K) : (2 * N))
-                : ((trans_B == rocsparse_operation_none) ? (2 * N) : (2 * K));
-    J ldc = (order == rocsparse_order_column) ? (2 * M) : (2 * N);
+    int64_t ldb = (order_B == rocsparse_order_column)
+                      ? ((trans_B == rocsparse_operation_none) ? (int64_t(ld_multiplier_B) * K)
+                                                               : (int64_t(ld_multiplier_B) * N))
+                      : ((trans_B == rocsparse_operation_none) ? (int64_t(ld_multiplier_B) * N)
+                                                               : (int64_t(ld_multiplier_B) * K));
+    int64_t ldc = (order_C == rocsparse_order_column) ? (int64_t(ld_multiplier_C) * M)
+                                                      : (int64_t(ld_multiplier_C) * N);
 
-    J nrowB = (order == rocsparse_order_column) ? ldb : B_m;
-    J ncolB = (order == rocsparse_order_column) ? B_n : ldb;
-    J nrowC = (order == rocsparse_order_column) ? ldc : C_m;
-    J ncolC = (order == rocsparse_order_column) ? C_n : ldc;
+    int64_t nrowB = (order_B == rocsparse_order_column) ? ldb : B_m;
+    int64_t ncolB = (order_B == rocsparse_order_column) ? B_n : ldb;
+    int64_t nrowC = (order_C == rocsparse_order_column) ? ldc : C_m;
+    int64_t ncolC = (order_C == rocsparse_order_column) ? C_n : ldc;
 
-    I nnz_B = nrowB * ncolB;
-    I nnz_C = nrowC * ncolC;
+    int64_t nnz_B = nrowB * ncolB;
+    int64_t nnz_C = nrowC * ncolC;
 
     // Allocate host memory for vectors
-    host_vector<T> hB(nnz_B);
-    host_vector<T> hC_1(nnz_C);
-    host_vector<T> hC_2(nnz_C);
-    host_vector<T> hC_gold(nnz_C);
+    host_vector<B> hB(nnz_B);
+    host_vector<C> hC_1(nnz_C);
+    host_vector<C> hC_2(nnz_C);
+    host_vector<C> hC_gold(nnz_C);
 
     // Initialize data on CPU
-    rocsparse_init<T>(hB, nnz_B, 1, 1);
-    rocsparse_init<T>(hC_1, nnz_C, 1, 1);
+    rocsparse_init<B>(hB, nnz_B, 1, 1);
+    rocsparse_init<C>(hC_1, nnz_C, 1, 1);
 
     hC_2    = hC_1;
     hC_gold = hC_1;
@@ -300,10 +198,10 @@ void testing_spmm_csc(const Arguments& arg)
     // Allocate device memory
     device_vector<I> dcsc_col_ptr(A_n + 1);
     device_vector<J> dcsc_row_ind(nnz_A);
-    device_vector<T> dcsc_val(nnz_A);
-    device_vector<T> dB(nnz_B);
-    device_vector<T> dC_1(nnz_C);
-    device_vector<T> dC_2(nnz_C);
+    device_vector<A> dcsc_val(nnz_A);
+    device_vector<B> dB(nnz_B);
+    device_vector<C> dC_1(nnz_C);
+    device_vector<C> dC_2(nnz_C);
     device_vector<T> dalpha(1);
     device_vector<T> dbeta(1);
 
@@ -312,28 +210,33 @@ void testing_spmm_csc(const Arguments& arg)
         hipMemcpy(dcsc_col_ptr, hcsc_col_ptr.data(), sizeof(I) * (A_n + 1), hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(
         hipMemcpy(dcsc_row_ind, hcsc_row_ind.data(), sizeof(J) * nnz_A, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dcsc_val, hcsc_val.data(), sizeof(T) * nnz_A, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dB, hB, sizeof(T) * nnz_B, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dC_1, hC_1, sizeof(T) * nnz_C, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dC_2, hC_2, sizeof(T) * nnz_C, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dcsc_val, hcsc_val.data(), sizeof(A) * nnz_A, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dB, hB, sizeof(B) * nnz_B, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dC_1, hC_1, sizeof(C) * nnz_C, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dC_2, hC_2, sizeof(C) * nnz_C, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dalpha, &halpha, sizeof(T), hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dbeta, &hbeta, sizeof(T), hipMemcpyHostToDevice));
 
     // Create descriptors
-    rocsparse_local_spmat A(A_m,
-                            A_n,
-                            nnz_A,
-                            dcsc_col_ptr,
-                            dcsc_row_ind,
-                            dcsc_val,
-                            itype,
-                            jtype,
-                            base,
-                            ttype,
-                            rocsparse_format_csc);
-    rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, order);
-    rocsparse_local_dnmat C1(C_m, C_n, ldc, dC_1, ttype, order);
-    rocsparse_local_dnmat C2(C_m, C_n, ldc, dC_2, ttype, order);
+    static const bool     csc_format_required = true;
+    rocsparse_local_spmat mat_A(A_m,
+                                A_n,
+                                nnz_A,
+                                dcsc_col_ptr,
+                                dcsc_row_ind,
+                                dcsc_val,
+                                itype,
+                                jtype,
+                                base,
+                                atype,
+                                csc_format_required);
+
+    ldb = std::max(int64_t(1), ldb);
+    ldc = std::max(int64_t(1), ldc);
+
+    rocsparse_local_dnmat mat_B(B_m, B_n, ldb, dB, btype, order_B);
+    rocsparse_local_dnmat mat_C1(C_m, C_n, ldc, dC_1, ctype, order_C);
+    rocsparse_local_dnmat mat_C2(C_m, C_n, ldc, dC_2, ctype, order_C);
 
     // Query SpMM buffer
     size_t buffer_size;
@@ -341,10 +244,10 @@ void testing_spmm_csc(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_buffer_size,
@@ -359,10 +262,10 @@ void testing_spmm_csc(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_preprocess,
@@ -379,15 +282,19 @@ void testing_spmm_csc(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       &halpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       &hbeta,
-                                                      C1,
+                                                      mat_C1,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
                                                       &buffer_size,
                                                       dbuffer));
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("dC_1", dC_1);
+        }
 
         // Pointer mode device
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
@@ -395,40 +302,46 @@ void testing_spmm_csc(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       dalpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       dbeta,
-                                                      C2,
+                                                      mat_C2,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
                                                       &buffer_size,
                                                       dbuffer));
 
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("dC_2", dC_2);
+        }
+
         // Copy output to host
-        CHECK_HIP_ERROR(hipMemcpy(hC_1, dC_1, sizeof(T) * nnz_C, hipMemcpyDeviceToHost));
-        CHECK_HIP_ERROR(hipMemcpy(hC_2, dC_2, sizeof(T) * nnz_C, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(hC_1, dC_1, sizeof(C) * nnz_C, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(hC_2, dC_2, sizeof(C) * nnz_C, hipMemcpyDeviceToHost));
 
         // CPU cscmm
-        host_cscmm<T, I, J>(A_m,
-                            N,
-                            A_n,
-                            trans_A,
-                            trans_B,
-                            halpha,
-                            hcsc_col_ptr,
-                            hcsc_row_ind,
-                            hcsc_val,
-                            hB,
-                            ldb,
-                            hbeta,
-                            hC_gold,
-                            ldc,
-                            order,
-                            base);
+        host_cscmm<T, I, J, A, B, C>(A_m,
+                                     N,
+                                     A_n,
+                                     trans_A,
+                                     trans_B,
+                                     halpha,
+                                     hcsc_col_ptr,
+                                     hcsc_row_ind,
+                                     hcsc_val,
+                                     hB,
+                                     ldb,
+                                     order_B,
+                                     hbeta,
+                                     hC_gold,
+                                     ldc,
+                                     order_C,
+                                     base);
 
-        hC_gold.near_check(hC_1);
-        hC_gold.near_check(hC_2);
+        hC_gold.near_check(hC_1, get_near_check_tol<C>(arg));
+        hC_gold.near_check(hC_2, get_near_check_tol<C>(arg));
     }
 
     if(arg.timing)
@@ -445,10 +358,10 @@ void testing_spmm_csc(const Arguments& arg)
                                                  trans_A,
                                                  trans_B,
                                                  &halpha,
-                                                 A,
-                                                 B,
+                                                 mat_A,
+                                                 mat_B,
                                                  &hbeta,
-                                                 C1,
+                                                 mat_C1,
                                                  ttype,
                                                  alg,
                                                  rocsparse_spmm_stage_compute,
@@ -465,10 +378,10 @@ void testing_spmm_csc(const Arguments& arg)
                                                  trans_A,
                                                  trans_B,
                                                  &halpha,
-                                                 A,
-                                                 B,
+                                                 mat_A,
+                                                 mat_B,
                                                  &hbeta,
-                                                 C1,
+                                                 mat_C1,
                                                  ttype,
                                                  alg,
                                                  rocsparse_spmm_stage_compute,
@@ -486,34 +399,39 @@ void testing_spmm_csc(const Arguments& arg)
             A_n, nnz_A, (I)B_m * (I)B_n, (I)C_m * (I)C_n, hbeta != static_cast<T>(0));
         double gpu_gbyte = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "N",
+                            display_key_t::N,
                             N,
-                            "K",
+                            display_key_t::K,
                             K,
-                            "nnz_A",
+                            display_key_t::nnz_A,
                             nnz_A,
-                            "alpha",
+                            display_key_t::alpha,
                             halpha,
-                            "beta",
+                            display_key_t::beta,
                             hbeta,
-                            "Algorithm",
+                            display_key_t::algorithm,
                             rocsparse_spmmalg2string(alg),
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 
     CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
 }
 
-#define INSTANTIATE(ITYPE, JTYPE, TTYPE)                                               \
-    template void testing_spmm_csc_bad_arg<ITYPE, JTYPE, TTYPE>(const Arguments& arg); \
-    template void testing_spmm_csc<ITYPE, JTYPE, TTYPE>(const Arguments& arg)
+#define INSTANTIATE(ITYPE, JTYPE, TTYPE)                                              \
+    template void testing_spmm_csc_bad_arg<ITYPE, JTYPE, TTYPE, TTYPE, TTYPE, TTYPE>( \
+        const Arguments& arg);                                                        \
+    template void testing_spmm_csc<ITYPE, JTYPE, TTYPE, TTYPE, TTYPE, TTYPE>(const Arguments& arg)
+#define INSTANTIATE_MIXED(ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE)                   \
+    template void testing_spmm_csc_bad_arg<ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE>( \
+        const Arguments& arg);                                                        \
+    template void testing_spmm_csc<ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE>(const Arguments& arg)
 
 INSTANTIATE(int32_t, int32_t, float);
 INSTANTIATE(int32_t, int32_t, double);
@@ -527,4 +445,12 @@ INSTANTIATE(int64_t, int64_t, float);
 INSTANTIATE(int64_t, int64_t, double);
 INSTANTIATE(int64_t, int64_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
+
+INSTANTIATE_MIXED(int32_t, int32_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int64_t, int32_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int64_t, int64_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int32_t, int32_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int64_t, int32_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int64_t, int64_t, int8_t, int8_t, float, float);
+
 void testing_spmm_csc_extra(const Arguments& arg) {}
