@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -26,11 +26,12 @@
 template <typename I, typename T>
 void testing_spsm_coo_bad_arg(const Arguments& arg)
 {
-    I       m     = 100;
-    I       n     = 100;
-    I       k     = 16;
-    int64_t nnz   = 100;
-    T       alpha = 0.6;
+    I        m           = 100;
+    I        n           = 100;
+    I        k           = 16;
+    int64_t  nnz         = 100;
+    const T  local_alpha = 0.6;
+    const T* alpha       = &local_alpha;
 
     rocsparse_operation  trans_A = rocsparse_operation_none;
     rocsparse_operation  trans_B = rocsparse_operation_none;
@@ -38,44 +39,60 @@ void testing_spsm_coo_bad_arg(const Arguments& arg)
     rocsparse_spsm_alg   alg     = rocsparse_spsm_alg_default;
 
     // Index and data type
-    rocsparse_indextype itype = get_indextype<I>();
-    rocsparse_datatype  ttype = get_datatype<T>();
+    rocsparse_indextype itype        = get_indextype<I>();
+    rocsparse_datatype  compute_type = get_datatype<T>();
 
     // Create rocsparse handle
     rocsparse_local_handle local_handle;
 
     // SpSM structures
     rocsparse_local_spmat local_A(
-        m, n, nnz, (void*)0x4, (void*)0x4, (void*)0x4, itype, base, ttype);
-    rocsparse_local_dnmat local_B(m, k, m, (void*)0x4, ttype, rocsparse_order_column);
-    rocsparse_local_dnmat local_C(m, k, m, (void*)0x4, ttype, rocsparse_order_column);
-
-    int       nargs_to_exclude   = 2;
-    const int args_to_exclude[2] = {10, 11};
+        m, n, nnz, (void*)0x4, (void*)0x4, (void*)0x4, itype, base, compute_type);
+    rocsparse_local_dnmat local_B(m, k, m, (void*)0x4, compute_type, rocsparse_order_column);
+    rocsparse_local_dnmat local_C(m, k, m, (void*)0x4, compute_type, rocsparse_order_column);
 
     rocsparse_handle      handle = local_handle;
-    rocsparse_spmat_descr A      = local_A;
-    rocsparse_dnmat_descr B      = local_B;
-    rocsparse_dnmat_descr C      = local_C;
+    rocsparse_spmat_descr matA   = local_A;
+    rocsparse_dnmat_descr matB   = local_B;
+    rocsparse_dnmat_descr matC   = local_C;
 
-    size_t buffer_size;
-    void*  temp_buffer = (void*)0x4;
+    size_t  local_buffer_size;
+    size_t* buffer_size = &local_buffer_size;
+    void*   temp_buffer = (void*)0x4;
 
-#define PARAMS_BUFFER_SIZE                                                                   \
-    handle, trans_A, trans_B, &alpha, A, B, C, ttype, alg, rocsparse_spsm_stage_buffer_size, \
-        &buffer_size, temp_buffer
+#define PARAMS_BUFFER_SIZE                                                                    \
+    handle, trans_A, trans_B, alpha, matA, matB, matC, compute_type, alg, stage, buffer_size, \
+        temp_buffer
 
-#define PARAMS_ANALYSIS                                                                     \
-    handle, trans_A, trans_B, &alpha, A, B, C, ttype, alg, rocsparse_spsm_stage_preprocess, \
-        &buffer_size, temp_buffer
+#define PARAMS_ANALYSIS                                                                       \
+    handle, trans_A, trans_B, alpha, matA, matB, matC, compute_type, alg, stage, buffer_size, \
+        temp_buffer
 
-#define PARAMS_SOLVE                                                                     \
-    handle, trans_A, trans_B, &alpha, A, B, C, ttype, alg, rocsparse_spsm_stage_compute, \
-        &buffer_size, temp_buffer
+#define PARAMS_SOLVE                                                                          \
+    handle, trans_A, trans_B, alpha, matA, matB, matC, compute_type, alg, stage, buffer_size, \
+        temp_buffer
 
-    auto_testing_bad_arg(rocsparse_spsm, nargs_to_exclude, args_to_exclude, PARAMS_BUFFER_SIZE);
-    auto_testing_bad_arg(rocsparse_spsm, nargs_to_exclude, args_to_exclude, PARAMS_ANALYSIS);
-    auto_testing_bad_arg(rocsparse_spsm, nargs_to_exclude, args_to_exclude, PARAMS_SOLVE);
+    {
+        static constexpr int       nargs_to_exclude                  = 1;
+        const int                  args_to_exclude[nargs_to_exclude] = {11};
+        const rocsparse_spsm_stage stage = rocsparse_spsm_stage_buffer_size;
+        select_bad_arg_analysis(
+            rocsparse_spsm, nargs_to_exclude, args_to_exclude, PARAMS_BUFFER_SIZE);
+    }
+
+    {
+        static constexpr int       nargs_to_exclude                  = 2;
+        const int                  args_to_exclude[nargs_to_exclude] = {10, 11};
+        const rocsparse_spsm_stage stage = rocsparse_spsm_stage_preprocess;
+        select_bad_arg_analysis(rocsparse_spsm, nargs_to_exclude, args_to_exclude, PARAMS_ANALYSIS);
+    }
+
+    {
+        static constexpr int       nargs_to_exclude                  = 2;
+        const int                  args_to_exclude[nargs_to_exclude] = {10, 11};
+        const rocsparse_spsm_stage stage                             = rocsparse_spsm_stage_compute;
+        select_bad_arg_analysis(rocsparse_spsm, nargs_to_exclude, args_to_exclude, PARAMS_SOLVE);
+    }
 
 #undef PARAMS_BUFFER_SIZE
 #undef PARAMS_ANALYSIS
@@ -85,15 +102,22 @@ void testing_spsm_coo_bad_arg(const Arguments& arg)
 template <typename I, typename T>
 void testing_spsm_coo(const Arguments& arg)
 {
-    I                    M       = arg.M;
-    I                    N       = arg.N;
-    I                    K       = arg.K;
-    rocsparse_operation  trans_A = arg.transA;
-    rocsparse_operation  trans_B = arg.transB;
-    rocsparse_index_base base    = arg.baseA;
-    rocsparse_spsm_alg   alg     = arg.spsm_alg;
-    rocsparse_diag_type  diag    = arg.diag;
-    rocsparse_fill_mode  uplo    = arg.uplo;
+    I                    M               = arg.M;
+    I                    N               = arg.N;
+    I                    K               = arg.K;
+    rocsparse_operation  trans_A         = arg.transA;
+    rocsparse_operation  trans_B         = arg.transB;
+    rocsparse_index_base base            = arg.baseA;
+    rocsparse_spsm_alg   alg             = arg.spsm_alg;
+    rocsparse_diag_type  diag            = arg.diag;
+    rocsparse_fill_mode  uplo            = arg.uplo;
+    rocsparse_order      order_B         = arg.orderB;
+    rocsparse_order      order_C         = arg.orderC;
+    rocsparse_int        ld_multiplier_B = arg.ld_multiplier_B;
+    rocsparse_int        ld_multiplier_C = arg.ld_multiplier_C;
+
+    // In the generic routines, C is always non-transposed (op(A) * C = op(B))
+    rocsparse_operation trans_C = rocsparse_operation_none;
 
     rocsparse_spsm_stage buffersize = rocsparse_spsm_stage_buffer_size;
     rocsparse_spsm_stage preprocess = rocsparse_spsm_stage_preprocess;
@@ -108,104 +132,7 @@ void testing_spsm_coo(const Arguments& arg)
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
 
-    // Argument sanity check before allocating invalid memory
-    if(M <= 0 || K <= 0)
-    {
-        // M == 0 means nnz can only be 0, too
-
-        static const I safe_size = 100;
-
-        // Allocate memory on device
-        device_vector<I> dcoo_row_ind(safe_size);
-        device_vector<I> dcoo_col_ind(safe_size);
-        device_vector<T> dcoo_val(safe_size);
-        device_vector<T> dB(safe_size);
-        device_vector<T> dC(safe_size);
-
-        // Check SpSM when structures can be created
-        if(M >= 0 && K >= 0 && M == N)
-        {
-            // Pointer mode
-            CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-
-            I B_m = (trans_B == rocsparse_operation_none) ? M : K;
-            I B_n = (trans_B == rocsparse_operation_none) ? K : M;
-
-            I C_m = (trans_B == rocsparse_operation_none) ? M : K;
-            I C_n = (trans_B == rocsparse_operation_none) ? K : M;
-
-            I ldb = (trans_B == rocsparse_operation_none) ? M : K;
-            I ldc = (trans_B == rocsparse_operation_none) ? M : K;
-
-            // Check structures
-            int64_t               nnz_A = 0;
-            rocsparse_local_spmat A(
-                M, N, nnz_A, dcoo_row_ind, dcoo_col_ind, dcoo_val, itype, base, ttype);
-
-            rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, rocsparse_order_column);
-
-            rocsparse_local_dnmat C(C_m, C_n, ldc, dB, ttype, rocsparse_order_column);
-
-            EXPECT_ROCSPARSE_STATUS(
-                rocsparse_spmat_set_attribute(A, rocsparse_spmat_fill_mode, &uplo, sizeof(uplo)),
-                rocsparse_status_success);
-
-            EXPECT_ROCSPARSE_STATUS(
-                rocsparse_spmat_set_attribute(A, rocsparse_spmat_diag_type, &diag, sizeof(diag)),
-                rocsparse_status_success);
-
-            size_t buffer_size;
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spsm(handle,
-                                                   trans_A,
-                                                   trans_B,
-                                                   &halpha,
-                                                   A,
-                                                   B,
-                                                   C,
-                                                   ttype,
-                                                   alg,
-                                                   buffersize,
-                                                   &buffer_size,
-                                                   nullptr),
-                                    rocsparse_status_success);
-
-            void* dbuffer;
-            CHECK_HIP_ERROR(rocsparse_hipMalloc(&dbuffer, safe_size));
-
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spsm(handle,
-                                                   trans_A,
-                                                   trans_B,
-                                                   &halpha,
-                                                   A,
-                                                   B,
-                                                   C,
-                                                   ttype,
-                                                   alg,
-                                                   preprocess,
-                                                   nullptr,
-                                                   dbuffer),
-                                    rocsparse_status_success);
-
-            EXPECT_ROCSPARSE_STATUS(rocsparse_spsm(handle,
-                                                   trans_A,
-                                                   trans_B,
-                                                   &halpha,
-                                                   A,
-                                                   B,
-                                                   C,
-                                                   ttype,
-                                                   alg,
-                                                   compute,
-                                                   &buffer_size,
-                                                   dbuffer),
-                                    rocsparse_status_success);
-            CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
-        }
-
-        return;
-    }
-
-    rocsparse_matrix_factory<T, I> matrix_factory(arg);
+    rocsparse_matrix_factory<T, I, I> matrix_factory(arg);
 
     // Allocate host memory for matrix
     host_vector<I> hcoo_row_ind;
@@ -216,14 +143,44 @@ void testing_spsm_coo(const Arguments& arg)
     int64_t nnz_A;
     matrix_factory.init_coo(hcoo_row_ind, hcoo_col_ind, hcoo_val, M, N, nnz_A, base);
 
+    //
+    // Scale values.
+    //
+    {
+        const size_t       size = hcoo_val.size();
+        floating_data_t<T> mx   = floating_data_t<T>(0);
+        for(size_t i = 0; i < size; ++i)
+        {
+            mx = std::max(mx, std::abs(hcoo_val[i]));
+        }
+        mx = floating_data_t<T>(1.0) / mx;
+        for(size_t i = 0; i < size; ++i)
+        {
+            hcoo_val[i] *= mx;
+        }
+    }
+
     I B_m = (trans_B == rocsparse_operation_none) ? M : K;
     I B_n = (trans_B == rocsparse_operation_none) ? K : M;
 
-    I C_m = (trans_B == rocsparse_operation_none) ? M : K;
-    I C_n = (trans_B == rocsparse_operation_none) ? K : M;
+    I C_m = M;
+    I C_n = K;
 
-    I ldb = (trans_B == rocsparse_operation_none) ? M : K;
-    I ldc = (trans_B == rocsparse_operation_none) ? M : K;
+    int64_t ldb = (order_B == rocsparse_order_column)
+                      ? ((trans_B == rocsparse_operation_none) ? (int64_t(ld_multiplier_B) * M)
+                                                               : (int64_t(ld_multiplier_B) * K))
+                      : ((trans_B == rocsparse_operation_none) ? (int64_t(ld_multiplier_B) * K)
+                                                               : (int64_t(ld_multiplier_B) * M));
+    int64_t ldc = (order_C == rocsparse_order_column) ? (int64_t(ld_multiplier_C) * M)
+                                                      : (int64_t(ld_multiplier_C) * K);
+
+    ldb = std::max(int64_t(1), ldb);
+    ldc = std::max(int64_t(1), ldc);
+
+    int64_t nrowB = (order_B == rocsparse_order_column) ? ldb : B_m;
+    int64_t ncolB = (order_B == rocsparse_order_column) ? B_n : ldb;
+    int64_t nrowC = (order_C == rocsparse_order_column) ? ldc : C_m;
+    int64_t ncolC = (order_C == rocsparse_order_column) ? C_n : ldc;
 
     // Non-squared matrices are not supported
     if(M != N)
@@ -232,25 +189,164 @@ void testing_spsm_coo(const Arguments& arg)
     }
 
     // Allocate host memory for vectors
-    host_dense_matrix<T> hB(B_m, B_n);
-    host_dense_matrix<T> hC_1(C_m, C_n);
-    host_dense_matrix<T> hC_2(C_m, C_n);
-    host_dense_matrix<T> hC_gold(C_m, C_n);
+    host_dense_matrix<T> htemp(B_m, B_n);
+    host_dense_matrix<T> hB(nrowB, ncolB);
+    host_dense_matrix<T> hC_1(nrowC, ncolC);
+    host_dense_matrix<T> hC_2(nrowC, ncolC);
+    host_dense_matrix<T> hC_gold(nrowC, ncolC);
 
-    // Initialize data on CPU
-    rocsparse_matrix_utils::init(hB);
+    rocsparse_matrix_utils::init(htemp);
 
-    hC_1    = hB;
-    hC_2    = hB;
-    hC_gold = hB;
+    // Copy B to C
+    if(order_B == rocsparse_order_column)
+    {
+        for(I j = 0; j < B_n; j++)
+        {
+            for(I i = 0; i < B_m; i++)
+            {
+                hB[i + ldb * j] = htemp[i + B_m * j];
+            }
+        }
+
+        if(trans_B == rocsparse_operation_none)
+        {
+            if(order_C == rocsparse_order_column)
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i + ldc * j] = hB[i + ldb * j];
+                    }
+                }
+            }
+            else
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i * ldc + j] = hB[i + ldb * j];
+                    }
+                }
+            }
+        }
+        else
+        {
+            if(order_C == rocsparse_order_column)
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i * ldc + j] = hB[i + ldb * j];
+                    }
+                }
+            }
+            else
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i + ldc * j] = hB[i + ldb * j];
+                    }
+                }
+            }
+        }
+    }
+    else
+    {
+        for(I i = 0; i < B_m; i++)
+        {
+            for(I j = 0; j < B_n; j++)
+            {
+                hB[ldb * i + j] = htemp[B_n * i + j];
+            }
+        }
+
+        if(trans_B == rocsparse_operation_none)
+        {
+            if(order_C == rocsparse_order_column)
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i + ldc * j] = hB[ldb * i + j];
+                    }
+                }
+            }
+            else
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i * ldc + j] = hB[ldb * i + j];
+                    }
+                }
+            }
+        }
+        else
+        {
+            if(order_C == rocsparse_order_column)
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i * ldc + j] = hB[ldb * i + j];
+                    }
+                }
+            }
+            else
+            {
+                for(I j = 0; j < B_n; j++)
+                {
+                    for(I i = 0; i < B_m; i++)
+                    {
+                        hC_1[i + ldc * j] = hB[ldb * i + j];
+                    }
+                }
+            }
+        }
+    }
+
+    hC_2    = hC_1;
+    hC_gold = hC_1;
+
+    if(trans_B == rocsparse_operation_conjugate_transpose)
+    {
+        if(order_C == rocsparse_order_column)
+        {
+            for(I j = 0; j < C_n; j++)
+            {
+                for(I i = 0; i < C_m; i++)
+                {
+                    hC_gold[i + ldc * j] = rocsparse_conj<T>(hC_gold[i + ldc * j]);
+                }
+            }
+        }
+        else
+        {
+            for(I i = 0; i < C_m; i++)
+            {
+                for(I j = 0; j < C_n; j++)
+                {
+                    hC_gold[ldc * i + j] = rocsparse_conj<T>(hC_gold[ldc * i + j]);
+                }
+            }
+        }
+    }
 
     // Allocate device memory
     device_vector<I>       dcoo_row_ind(nnz_A);
     device_vector<I>       dcoo_col_ind(nnz_A);
     device_vector<T>       dcoo_val(nnz_A);
-    device_dense_matrix<T> dB(B_m, B_n);
-    device_dense_matrix<T> dC_1(C_m, C_n);
-    device_dense_matrix<T> dC_2(C_m, C_n);
+    device_dense_matrix<T> dB(nrowB, ncolB);
+    device_dense_matrix<T> dC_1(nrowC, ncolC);
+    device_dense_matrix<T> dC_2(nrowC, ncolC);
     device_vector<T>       dalpha(1);
 
     // Copy data from CPU to device
@@ -259,16 +355,17 @@ void testing_spsm_coo(const Arguments& arg)
     CHECK_HIP_ERROR(
         hipMemcpy(dcoo_col_ind, hcoo_col_ind.data(), sizeof(I) * nnz_A, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dcoo_val, hcoo_val.data(), sizeof(T) * nnz_A, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dB, hB, sizeof(T) * B_m * B_n, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dC_1, hC_1, sizeof(T) * C_m * C_n, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dC_2, hC_2, sizeof(T) * C_m * C_n, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dB, hB, sizeof(T) * nrowB * ncolB, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dC_1, hC_1, sizeof(T) * nrowC * ncolC, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dC_2, hC_2, sizeof(T) * nrowC * ncolC, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dalpha, &halpha, sizeof(T), hipMemcpyHostToDevice));
 
     // Create descriptors
     rocsparse_local_spmat A(M, N, nnz_A, dcoo_row_ind, dcoo_col_ind, dcoo_val, itype, base, ttype);
-    rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, rocsparse_order_column);
-    rocsparse_local_dnmat C1(C_m, C_n, ldc, dC_1, ttype, rocsparse_order_column);
-    rocsparse_local_dnmat C2(C_m, C_n, ldc, dC_2, ttype, rocsparse_order_column);
+
+    rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, order_B);
+    rocsparse_local_dnmat C1(C_m, C_n, ldc, dC_1, ttype, order_C);
+    rocsparse_local_dnmat C2(C_m, C_n, ldc, dC_2, ttype, order_C);
 
     CHECK_ROCSPARSE_ERROR(
         rocsparse_spmat_set_attribute(A, rocsparse_spmat_fill_mode, &uplo, sizeof(uplo)));
@@ -287,7 +384,7 @@ void testing_spsm_coo(const Arguments& arg)
                                          C1,
                                          ttype,
                                          alg,
-                                         rocsparse_spsm_stage_auto /*buffersize*/,
+                                         buffersize,
                                          &buffer_size,
                                          nullptr));
 
@@ -297,33 +394,13 @@ void testing_spsm_coo(const Arguments& arg)
 
     // Perform analysis on host
     CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-    CHECK_ROCSPARSE_ERROR(rocsparse_spsm(handle,
-                                         trans_A,
-                                         trans_B,
-                                         &halpha,
-                                         A,
-                                         B,
-                                         C1,
-                                         ttype,
-                                         alg,
-                                         rocsparse_spsm_stage_auto /*preprocess*/,
-                                         nullptr,
-                                         dbuffer));
+    CHECK_ROCSPARSE_ERROR(rocsparse_spsm(
+        handle, trans_A, trans_B, &halpha, A, B, C1, ttype, alg, preprocess, nullptr, dbuffer));
 
     // Perform analysis on device
     CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
-    CHECK_ROCSPARSE_ERROR(rocsparse_spsm(handle,
-                                         trans_A,
-                                         trans_B,
-                                         dalpha,
-                                         A,
-                                         B,
-                                         C2,
-                                         ttype,
-                                         alg,
-                                         rocsparse_spsm_stage_auto /*preprocess*/,
-                                         nullptr,
-                                         dbuffer));
+    CHECK_ROCSPARSE_ERROR(rocsparse_spsm(
+        handle, trans_A, trans_B, dalpha, A, B, C2, ttype, alg, preprocess, nullptr, dbuffer));
 
     if(arg.unit_check)
     {
@@ -338,7 +415,7 @@ void testing_spsm_coo(const Arguments& arg)
                                                       C1,
                                                       ttype,
                                                       alg,
-                                                      rocsparse_spsm_stage_auto /*compute*/,
+                                                      compute,
                                                       &buffer_size,
                                                       dbuffer));
 
@@ -353,30 +430,41 @@ void testing_spsm_coo(const Arguments& arg)
                                                       C2,
                                                       ttype,
                                                       alg,
-                                                      rocsparse_spsm_stage_auto /*compute*/,
+                                                      compute,
                                                       &buffer_size,
                                                       dbuffer));
 
         CHECK_HIP_ERROR(hipDeviceSynchronize());
 
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("dC_1", dC_1);
+        }
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("dC_2", dC_2);
+        }
+
         // Copy output to host
-        CHECK_HIP_ERROR(hipMemcpy(hC_1, dC_1, sizeof(T) * C_m * C_n, hipMemcpyDeviceToHost));
-        CHECK_HIP_ERROR(hipMemcpy(hC_2, dC_2, sizeof(T) * C_m * C_n, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(hC_1, dC_1, sizeof(T) * nrowC * ncolC, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(hC_2, dC_2, sizeof(T) * nrowC * ncolC, hipMemcpyDeviceToHost));
 
         // CPU coosm
         I analysis_pivot = -1;
         I solve_pivot    = -1;
+
         host_coosm<I, T>(M,
                          K,
                          nnz_A,
                          trans_A,
-                         trans_B,
+                         trans_C,
                          halpha,
                          hcoo_row_ind,
                          hcoo_col_ind,
                          hcoo_val,
                          hC_gold,
                          ldc,
+                         order_C,
                          diag,
                          uplo,
                          base,
@@ -441,21 +529,21 @@ void testing_spsm_coo(const Arguments& arg)
         double gbyte_count = coosv_gbyte_count<T>(M, nnz_A) * K;
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "nnz_A",
+                            display_key_t::nnz_A,
                             nnz_A,
-                            "nrhs",
+                            display_key_t::nrhs,
                             K,
-                            "alpha",
+                            display_key_t::alpha,
                             halpha,
-                            "Algorithm",
+                            display_key_t::algorithm,
                             rocsparse_spsmalg2string(alg),
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 
