@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2022 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -29,8 +29,12 @@ struct rocsparse_test_dispatch_enum
     {
         t,
         it,
+        it_plus_int8,
         ijt,
+        ixyt,
         iaxyt,
-        ijaxyt
+        ijaxyt,
+        iabct,
+        ijabct
     } value_type;
 };
