@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,4 +24,5 @@
 #include "test.hpp"
 #include "testing_scatter.hpp"
 
-TEST_ROUTINE_WITH_CONFIG(scatter, level1, rocsparse_test_config_it, arg.M, arg.nnz, arg.baseA);
+TEST_ROUTINE_WITH_CONFIG(
+    scatter, level1, rocsparse_test_config_it_plus_int8, arg.M, arg.nnz, arg.baseA, arg.graph_test);
