@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,8 +25,8 @@
  * \brief rocsparse-types.h defines data types used by rocsparse
  */
 
-#ifndef _ROCSPARSE_TYPES_H_
-#define _ROCSPARSE_TYPES_H_
+#ifndef ROCSPARSE_TYPES_H
+#define ROCSPARSE_TYPES_H
 
 #include "rocsparse-complex-types.h"
 
@@ -34,17 +34,17 @@
 #include <stdint.h>
 
 /// \cond DO_NOT_DOCUMENT
-#ifdef WIN32
-#define ROCSPARSE_KERNEL __global__ static
-#define ROCSPARSE_DEVICE_ILF __device__ __forceinline__
-#else
-#define ROCSPARSE_KERNEL __global__
-#define ROCSPARSE_DEVICE_ILF __device__
-#endif
+#define ROCSPARSE_KERNEL_W(MAX_THREADS_PER_BLOCK, MIN_WARPS_PER_EXECUTION_UNIT) \
+    __launch_bounds__(MAX_THREADS_PER_BLOCK, MIN_WARPS_PER_EXECUTION_UNIT) static __global__
+#define ROCSPARSE_KERNEL(MAX_THREADS_PER_BLOCK) \
+    __launch_bounds__(MAX_THREADS_PER_BLOCK) static __global__
+#define ROCSPARSE_DEVICE_ILF static __device__ __forceinline__
 /// \endcond
 
 /*! \ingroup types_module
- *  \brief Specifies whether int32 or int64 is used.
+ *  \brief Specifies rocSPARSE integer type (defaults to int32_t).
+ *
+ *  \note When rocSPARSE is built with rocsparse_ILP64, \ref rocsparse_int is typedef to int64_t.
  */
 #if defined(rocsparse_ILP64)
 typedef int64_t rocsparse_int;
@@ -115,6 +115,17 @@ typedef struct _rocsparse_mat_info* rocsparse_mat_info;
  */
 typedef struct _rocsparse_spvec_descr* rocsparse_spvec_descr;
 
+/*! \ingroup types_module
+ *  \brief Generic API descriptor of the sparse vector.
+ *
+ *  \details
+ *  The rocSPARSE constant sparse vector descriptor is a structure holding all properties of a sparse vector.
+ *  It must be initialized using \ref rocsparse_create_const_spvec_descr() and the returned
+ *  descriptor must be passed to all subsequent generic API library calls that involve the sparse vector.
+ *  It should be destroyed at the end using rocsparse_destroy_spvec_descr().
+ */
+typedef struct _rocsparse_spvec_descr const* rocsparse_const_spvec_descr;
+
 /*! \ingroup types_module
  *  \brief Generic API descriptor of the sparse matrix.
  *
@@ -128,6 +139,19 @@ typedef struct _rocsparse_spvec_descr* rocsparse_spvec_descr;
  */
 typedef struct _rocsparse_spmat_descr* rocsparse_spmat_descr;
 
+/*! \ingroup types_module
+ *  \brief Generic API descriptor of the sparse matrix.
+ *
+ *  \details
+ *  The rocSPARSE constant sparse matrix descriptor is a structure holding all properties of a sparse matrix.
+ *  It must be initialized using rocsparse_create_const_coo_descr(),
+ *  rocsparse_create_const_csr_descr(), rocsparse_create_const_csc_descr(),
+ *  or rocsparse_create_const_bell_descr() and the returned
+ *  descriptor must be passed to all subsequent generic API library calls that involve the sparse matrix.
+ *  It should be destroyed at the end using rocsparse_destroy_spmat_descr().
+ */
+typedef struct _rocsparse_spmat_descr const* rocsparse_const_spmat_descr;
+
 /*! \ingroup types_module
  *  \brief Generic API descriptor of the dense vector.
  *
@@ -139,6 +163,17 @@ typedef struct _rocsparse_spmat_descr* rocsparse_spmat_descr;
  */
 typedef struct _rocsparse_dnvec_descr* rocsparse_dnvec_descr;
 
+/*! \ingroup types_module
+ *  \brief Generic API descriptor of the dense vector.
+ *
+ *  \details
+ *  The rocSPARSE constant dense vector descriptor is a structure holding all properties of a dense vector.
+ *  It must be initialized using rocsparse_create_const_dnvec_descr() and the returned
+ *  descriptor must be passed to all subsequent generic API library calls that involve the dense vector.
+ *  It should be destroyed at the end using rocsparse_destroy_dnvec_descr().
+ */
+typedef struct _rocsparse_dnvec_descr const* rocsparse_const_dnvec_descr;
+
 /*! \ingroup types_module
  *  \brief Generic API descriptor of the dense matrix.
  *
@@ -150,6 +185,17 @@ typedef struct _rocsparse_dnvec_descr* rocsparse_dnvec_descr;
  */
 typedef struct _rocsparse_dnmat_descr* rocsparse_dnmat_descr;
 
+/*! \ingroup types_module
+ *  \brief Generic API descriptor of the dense matrix.
+ *
+ *  \details
+ *  The rocSPARSE constant dense matrix descriptor is a structure holding all properties of a dense matrix.
+ *  It must be initialized using rocsparse_create_const_dnmat_descr() and the returned
+ *  descriptor must be passed to all subsequent generic API library calls that involve the dense matrix.
+ *  It should be destroyed at the end using rocsparse_destroy_dnmat_descr().
+ */
+typedef struct _rocsparse_dnmat_descr const* rocsparse_const_dnmat_descr;
+
 /*! \ingroup types_module
  *  \brief Coloring info structure to hold data gathered during analysis and later used in
  *  rocSPARSE sparse matrix coloring routines.
@@ -163,6 +209,22 @@ typedef struct _rocsparse_dnmat_descr* rocsparse_dnmat_descr;
  */
 typedef struct _rocsparse_color_info* rocsparse_color_info;
 
+/*! \ingroup types_module
+ * \brief rocsparse_sparse_to_sparse_descr is a structure holding the rocsparse sparse_to_sparse
+ * descr data. It must be initialized using
+ * the rocsparse_create_sparse_to_sparse_descr() routine. It should be destroyed at the
+ * end using rocsparse_destroy_sparse_to_sparse_descr().
+ */
+typedef struct _rocsparse_sparse_to_sparse_descr* rocsparse_sparse_to_sparse_descr;
+
+/*! \ingroup types_module
+ * \brief rocsparse_extract_descr is a structure holding the rocsparse extract
+ * descr data. It must be initialized using
+ * the rocsparse_create_extract_descr() routine. It should be destroyed at the
+ * end using rocsparse_destroy_extract_descr().
+ */
+typedef struct _rocsparse_extract_descr* rocsparse_extract_descr;
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -199,10 +261,25 @@ typedef enum rocsparse_index_base_
  *  \brief Specify the matrix type.
  *
  *  \details
- *  The \ref rocsparse_matrix_type indices the type of a matrix. For a given
+ *  The \ref rocsparse_matrix_type indicates the type of a matrix. For a given
  *  \ref rocsparse_mat_descr, the \ref rocsparse_matrix_type can be set using
  *  rocsparse_set_mat_type(). The current \ref rocsparse_matrix_type of a matrix can be
  *  obtained by rocsparse_get_mat_type().
+ *
+ *  For the matrix types \ref rocsparse_matrix_type_symmetric, \ref rocsparse_matrix_type_hermitian,
+ *  and \ref rocsparse_matrix_type_triangular, only the upper or lower part of the matrix
+ *  (specified by setting the \ref rocsparse_fill_mode) is assumed to be stored. The purpose of this
+ *  is to minimize the amount of memory required to store the matrix.
+ *
+ *  Routines that accept \ref rocsparse_matrix_type_symmetric or \ref rocsparse_matrix_type_hermitian
+ *  will only read from the stored upper or lower part of the matrix but will perform the computation
+ *  as if the full symmetric/hermitian matrix existed. For example, when computing \f$y=A*x\f$ where
+ *  A is symmetric and only the lower part is stored, internally the multiplication will be performed
+ *  in two steps. First the computation \f$y=(L+D)*x\f$ will be performed. Secondly the multiplication
+ *  will be completed by performing \f$y=L^T*x + y\f$. This second step involves a transposed
+ *  multiplication which is slower. For this reason, where space allows, it is faster to store the
+ *  entire symmetric matrix and use \ref rocsparse_matrix_type_general instead of
+ *  \ref rocsparse_matrix_type_symmetric.
  */
 typedef enum rocsparse_matrix_type_
 {
@@ -251,7 +328,7 @@ typedef enum rocsparse_fill_mode_
  *  \details
  *  The \ref rocsparse_storage_mode indicates whether the matrix is stored sorted or not.
  *  For a given \ref rocsparse_mat_descr, the \ref rocsparse_storage_mode can be set
- *  using rocsparse_set_storage_mode(). The current \ref rocsparse_storage_mode of a
+ *  using rocsparse_set_mat_storage_mode(). The current \ref rocsparse_storage_mode of a
  *  matrix can be obtained by rocsparse_get_mat_storage_mode().
  */
 typedef enum rocsparse_storage_mode_
@@ -305,9 +382,10 @@ typedef enum rocsparse_hyb_partition_
  *
  *  \details
  *  The \ref rocsparse_analysis_policy specifies whether gathered analysis data should be
- *  re-used or not. If meta data from a previous e.g. rocsparse_csrilu0_analysis() call
- *  is available, it can be re-used for subsequent calls to e.g.
- *  rocsparse_csrsv_analysis() and greatly improve performance of the analysis function.
+ *  re-used or not. If meta data from a previous e.g. \ref rocsparse_scsrilu0_analysis
+ *  "rocsparse_Xcsrilu0_analysis()" call is available, it can be re-used for subsequent calls to e.g.
+ *  \ref rocsparse_scsrsv_analysis "rocsparse_Xcsrsv_analysis()" and greatly improve performance
+ *  of the analysis function.
  */
 typedef enum rocsparse_analysis_policy_
 {
@@ -351,7 +429,7 @@ typedef enum rocsparse_layer_mode
 {
     rocsparse_layer_mode_none      = 0x0, /**< layer is not active. */
     rocsparse_layer_mode_log_trace = 0x1, /**< layer is in logging mode. */
-    rocsparse_layer_mode_log_bench = 0x2, /**< layer is in benchmarking mode. */
+    rocsparse_layer_mode_log_bench = 0x2, /**< layer is in benchmarking mode (deprecated) */
     rocsparse_layer_mode_log_debug = 0x4 /**< layer is in debug mode. */
 } rocsparse_layer_mode;
 
@@ -377,7 +455,8 @@ typedef enum rocsparse_status_
     rocsparse_status_not_initialized         = 10, /**< descriptor has not been initialized. */
     rocsparse_status_type_mismatch           = 11, /**< index types do not match. */
     rocsparse_status_requires_sorted_storage = 12, /**< sorted storage required. */
-    rocsparse_status_thrown_exception        = 13 /**< exception being thrown. */
+    rocsparse_status_thrown_exception        = 13, /**< exception being thrown. */
+    rocsparse_status_continue                = 14 /**< Nothing preventing function to proceed */
 } rocsparse_status;
 
 /*! \ingroup types_module
@@ -472,6 +551,56 @@ typedef enum rocsparse_spmat_attribute_
     rocsparse_spmat_storage_mode = 3 /**< Matrix storage attribute. */
 } rocsparse_spmat_attribute;
 
+/*! \ingroup types_module
+   *  \brief List of sparse to sparse algorithms.
+   *
+   *  \details
+   *  This is a list of supported \ref rocsparse_sparse_to_sparse_alg types that are used to perform
+   *  sparse to sparse conversion.
+   */
+typedef enum rocsparse_sparse_to_sparse_alg_
+{
+    rocsparse_sparse_to_sparse_alg_default
+    = 0, /**< Default sparse to sparse algorithm for the given format. */
+} rocsparse_sparse_to_sparse_alg;
+
+/*! \ingroup types_module
+   *  \brief List of sparse_to_sparse stages.
+   *
+   *  \details
+   *  This is a list of possible stages during sparse_to_sparse conversion. Typical order is
+   *  rocsparse_sparse_to_sparse_stage_analysis, rocsparse_sparse_to_sparse_stage_compute.
+   */
+typedef enum rocsparse_sparse_to_sparse_stage_
+{
+    rocsparse_sparse_to_sparse_stage_analysis = 0, /**< Data analysis. */
+    rocsparse_sparse_to_sparse_stage_compute  = 1 /**< Performs the actual conversion. */
+} rocsparse_sparse_to_sparse_stage;
+
+/*! \ingroup types_module
+   *  \brief List of extract algorithms.
+   *
+   *  \details
+   *  This is a list of supported \ref rocsparse_extract_alg types that are used to perform
+   *  the submatrix extraction.
+   */
+typedef enum rocsparse_extract_alg_
+{
+    rocsparse_extract_alg_default = 0, /**< Default extract algorithm for the given format. */
+} rocsparse_extract_alg;
+
+/*! \ingroup types_module
+   *  \brief List of extract stages.
+   *
+   *  \details
+   *  The analysis \ref rocsparse_extract_stage_analysis must be done before the first call of the calculation \ref rocsparse_extract_stage_compute.
+   */
+typedef enum rocsparse_extract_stage_
+{
+    rocsparse_extract_stage_analysis = 0, /**< Data analysis. */
+    rocsparse_extract_stage_compute  = 1 /**< Performs the actual extraction. */
+} rocsparse_extract_stage;
+
 /*! \ingroup types_module
  *  \brief List of Iterative ILU0 algorithms.
  *
@@ -528,16 +657,28 @@ typedef enum rocsparse_gtsv_interleaved_alg_
     = 3 /**< Solve interleaved gtsv using QR algorithm (stable). */
 } rocsparse_gtsv_interleaved_alg;
 
+/*! \ingroup types_module
+ *  \brief List of check_matrix stages.
+ *
+ *  \details
+ *  This is a list of possible stages during check_matrix computation. Typical order is
+ *  rocsparse_check_spmat_stage_buffer_size, rocsparse_check_spmat_stage_compute.
+ */
+typedef enum rocsparse_check_spmat_stage_
+{
+    rocsparse_check_spmat_stage_buffer_size = 0, /**< Returns the required buffer size. */
+    rocsparse_check_spmat_stage_compute     = 1, /**< Performs check. */
+} rocsparse_check_spmat_stage;
+
 /*! \ingroup types_module
  *  \brief List of SpMV stages.
  *
  *  \details
  *  This is a list of possible stages during SpMV computation. Typical order is
- *  rocsparse_spmv_buffer_size, rocsparse_spmv_preprocess, rocsparse_spmv_compute.
+ *  rocsparse_spmv_stage_buffer_size, rocsparse_spmv_stage_preprocess, rocsparse_spmv_stage_compute.
  */
 typedef enum rocsparse_spmv_stage_
 {
-    rocsparse_spmv_stage_auto        = 0, /**< Automatic stage detection. */
     rocsparse_spmv_stage_buffer_size = 1, /**< Returns the required buffer size. */
     rocsparse_spmv_stage_preprocess  = 2, /**< Preprocess data. */
     rocsparse_spmv_stage_compute     = 3 /**< Performs the actual SpMV computation. */
@@ -558,7 +699,8 @@ typedef enum rocsparse_spmv_alg_
     rocsparse_spmv_alg_csr_stream   = 3, /**< CSR SpMV algorithm 2 (stream) for CSR matrices. */
     rocsparse_spmv_alg_ell          = 4, /**< ELL SpMV algorithm for ELL matrices. */
     rocsparse_spmv_alg_coo_atomic   = 5, /**< COO SpMV algorithm 2 (atomic) for COO matrices. */
-    rocsparse_spmv_alg_bsr          = 6 /**< BSR SpMV algorithm 1 for BSR matrices. */
+    rocsparse_spmv_alg_bsr          = 6, /**< BSR SpMV algorithm 1 for BSR matrices. */
+    rocsparse_spmv_alg_csr_lrb      = 7 /**< CSR SpMV algorithm 3 (LRB) for CSR matrices. */
 } rocsparse_spmv_alg;
 
 /*! \ingroup types_module
@@ -578,11 +720,10 @@ typedef enum rocsparse_spsv_alg_
  *
  *  \details
  *  This is a list of possible stages during SpSV computation. Typical order is
- *  rocsparse_spsv_buffer_size, rocsparse_spsv_preprocess, rocsparse_spsv_compute.
+ *  rocsparse_spsv_stage_buffer_size, rocsparse_spsv_stage_preprocess, rocsparse_spsv_stage_compute.
  */
 typedef enum rocsparse_spsv_stage_
 {
-    rocsparse_spsv_stage_auto        = 0, /**< Automatic stage detection. */
     rocsparse_spsv_stage_buffer_size = 1, /**< Returns the required buffer size. */
     rocsparse_spsv_stage_preprocess  = 2, /**< Preprocess data. */
     rocsparse_spsv_stage_compute     = 3 /**< Performs the actual SpSV computation. */
@@ -605,11 +746,10 @@ typedef enum rocsparse_spitsv_alg_
  *
  *  \details
  *  This is a list of possible stages during SpITSV computation. Typical order is
- *  buffer_size, preprocess, compute.
+ *  rocsparse_spitsv_stage_buffer_size, rocsparse_spitsv_stage_preprocess, rocsparse_spitsv_stage_compute.
  */
 typedef enum rocsparse_spitsv_stage_
 {
-    rocsparse_spitsv_stage_auto        = 0, /**< Automatic stage detection. */
     rocsparse_spitsv_stage_buffer_size = 1, /**< Returns the required buffer size. */
     rocsparse_spitsv_stage_preprocess  = 2, /**< Preprocess data. */
     rocsparse_spitsv_stage_compute     = 3 /**< Performs the actual SpITSV computation. */
@@ -632,11 +772,10 @@ typedef enum rocsparse_spsm_alg_
  *
  *  \details
  *  This is a list of possible stages during SpSM computation. Typical order is
- *  rocsparse_spsm_buffer_size, rocsparse_spsm_preprocess, rocsparse_spsm_compute.
+ *  rocsparse_spsm_stage_buffer_size, rocsparse_spsm_stage_preprocess, rocsparse_spsm_stage_compute.
  */
 typedef enum rocsparse_spsm_stage_
 {
-    rocsparse_spsm_stage_auto        = 0, /**< Automatic stage detection. */
     rocsparse_spsm_stage_buffer_size = 1, /**< Returns the required buffer size. */
     rocsparse_spsm_stage_preprocess  = 2, /**< Preprocess data. */
     rocsparse_spsm_stage_compute     = 3 /**< Performs the actual SpSM computation. */
@@ -656,10 +795,13 @@ typedef enum rocsparse_spmm_alg_
     rocsparse_spmm_alg_coo_segmented, /**< SpMM algorithm for COO format using segmented scan. */
     rocsparse_spmm_alg_coo_atomic, /**< SpMM algorithm for COO format using atomics. */
     rocsparse_spmm_alg_csr_row_split, /**< SpMM algorithm for CSR format using row split and shfl. */
-    rocsparse_spmm_alg_csr_merge, /**< SpMM algorithm for CSR format using conversion to COO. */
+    rocsparse_spmm_alg_csr_merge, /**< SpMM algorithm for CSR format using nnz split algorithm. Is the same as rocsparse_spmm_alg_csr_nnz_split. */
     rocsparse_spmm_alg_coo_segmented_atomic, /**< SpMM algorithm for COO format using segmented scan and atomics. */
     rocsparse_spmm_alg_bell, /**< SpMM algorithm for Blocked ELL format. */
-    rocsparse_spmm_alg_bsr /**< SpMM algorithm for BSR format. */
+    rocsparse_spmm_alg_bsr, /**< SpMM algorithm for BSR format. */
+    rocsparse_spmm_alg_csr_merge_path, /**< SpMM algorithm for CSR format using merge path algorithm. */
+    rocsparse_spmm_alg_csr_nnz_split
+    = rocsparse_spmm_alg_csr_merge /**< SpMM algorithm for CSR format using nnz split algorithm. */
 } rocsparse_spmm_alg;
 
 /*! \ingroup types_module
@@ -672,6 +814,7 @@ typedef enum rocsparse_spmm_alg_
 typedef enum rocsparse_sddmm_alg_
 {
     rocsparse_sddmm_alg_default = 0, /**< Default sddmm algorithm for the given format. */
+    rocsparse_sddmm_alg_dense   = 1 /**< Sddmm algorithm using dense blas operations. */
 } rocsparse_sddmm_alg;
 
 /*! \ingroup types_module
@@ -705,11 +848,10 @@ typedef enum rocsparse_dense_to_sparse_alg_
  *
  *  \details
  *  This is a list of possible stages during SpMM computation. Typical order is
- *  rocsparse_spmm_buffer_size, rocsparse_spmm_preprocess, rocsparse_spmm_compute.
+ *  rocsparse_spmm_stage_buffer_size, rocsparse_spmm_stage_preprocess, rocsparse_spmm_stage_compute.
  */
 typedef enum rocsparse_spmm_stage_
 {
-    rocsparse_spmm_stage_auto        = 0, /**< Automatic stage detection. */
     rocsparse_spmm_stage_buffer_size = 1, /**< Returns the required buffer size. */
     rocsparse_spmm_stage_preprocess  = 2, /**< Preprocess data. */
     rocsparse_spmm_stage_compute     = 3 /**< Performs the actual SpMM computation. */
@@ -720,11 +862,10 @@ typedef enum rocsparse_spmm_stage_
  *
  *  \details
  *  This is a list of possible stages during SpGEMM computation. Typical order is
- *  rocsparse_spgemm_buffer_size, rocsparse_spgemm_nnz, rocsparse_spgemm_compute.
+ *  rocsparse_spgemm_stage_buffer_size, rocsparse_spgemm_stage_nnz, rocsparse_spgemm_stage_compute.
  */
 typedef enum rocsparse_spgemm_stage_
 {
-    rocsparse_spgemm_stage_auto        = 0, /**< Automatic stage detection. */
     rocsparse_spgemm_stage_buffer_size = 1, /**< Returns the required buffer size. */
     rocsparse_spgemm_stage_nnz         = 2, /**< Computes number of non-zero entries. */
     rocsparse_spgemm_stage_compute     = 3, /**< Performs the actual SpGEMM computation. */
@@ -761,4 +902,4 @@ typedef enum rocsparse_gpsv_interleaved_alg_
 }
 #endif
 
-#endif /* _ROCSPARSE_TYPES_H_ */
+#endif /* ROCSPARSE_TYPES_H */
