@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,82 +22,66 @@
  *
  * ************************************************************************ */
 
+#include "internal/conversion/rocsparse_nnz_compress.h"
+#include "control.h"
 #include "rocsparse_nnz_compress.hpp"
-#include "definitions.h"
 #include "utility.h"
 
 #include "nnz_compress_device.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.h"
 
-template <rocsparse_int BLOCK_SIZE,
-          rocsparse_int SEGMENTS_PER_BLOCK,
-          rocsparse_int SEGMENT_SIZE,
-          rocsparse_int WF_SIZE,
-          typename T>
-__launch_bounds__(BLOCK_SIZE) ROCSPARSE_KERNEL
+namespace rocsparse
+{
+    template <rocsparse_int BLOCK_SIZE,
+              rocsparse_int SEGMENTS_PER_BLOCK,
+              rocsparse_int SEGMENT_SIZE,
+              rocsparse_int WF_SIZE,
+              typename T>
+    ROCSPARSE_KERNEL(BLOCK_SIZE)
     void nnz_compress_kernel(rocsparse_int        m,
                              rocsparse_index_base idx_base_A,
                              const T* __restrict__ csr_val_A,
                              const rocsparse_int* __restrict__ csr_row_ptr_A,
                              rocsparse_int* __restrict__ nnz_per_row,
                              T threshold)
-{
-    nnz_compress_device<BLOCK_SIZE, SEGMENTS_PER_BLOCK, SEGMENT_SIZE, WF_SIZE>(
-        m, idx_base_A, csr_val_A, csr_row_ptr_A, nnz_per_row, threshold);
+    {
+        rocsparse::nnz_compress_device<BLOCK_SIZE, SEGMENTS_PER_BLOCK, SEGMENT_SIZE, WF_SIZE>(
+            m, idx_base_A, csr_val_A, csr_row_ptr_A, nnz_per_row, threshold);
+    }
 }
 
 template <typename T>
-rocsparse_status rocsparse_nnz_compress_template(rocsparse_handle          handle,
-                                                 rocsparse_int             m,
-                                                 const rocsparse_mat_descr descr_A,
-                                                 const T*                  csr_val_A,
-                                                 const rocsparse_int*      csr_row_ptr_A,
-                                                 rocsparse_int*            nnz_per_row,
-                                                 rocsparse_int*            nnz_C,
-                                                 T                         tol)
+rocsparse_status rocsparse::nnz_compress_template(rocsparse_handle          handle, //0
+                                                  rocsparse_int             m, //1
+                                                  const rocsparse_mat_descr descr_A, //2
+                                                  const T*                  csr_val_A, //3
+                                                  const rocsparse_int*      csr_row_ptr_A, //4
+                                                  rocsparse_int*            nnz_per_row, //5
+                                                  rocsparse_int*            nnz_C, //6
+                                                  T                         tol) //7
 {
-    // Check for valid handle and matrix descriptor
-    if(handle == nullptr)
-    {
-        return rocsparse_status_invalid_handle;
-    }
 
     // Logging
-    log_trace(handle,
-              replaceX<T>("rocsparse_Xnnz_compress"),
-              m,
-              descr_A,
-              (const void*&)csr_val_A,
-              (const void*&)csr_row_ptr_A,
-              (const void*&)nnz_per_row,
-              (const void*&)nnz_C,
-              tol);
+    rocsparse::log_trace(handle,
+                         rocsparse::replaceX<T>("rocsparse_Xnnz_compress"),
+                         m,
+                         descr_A,
+                         (const void*&)csr_val_A,
+                         (const void*&)csr_row_ptr_A,
+                         (const void*&)nnz_per_row,
+                         (const void*&)nnz_C,
+                         tol);
 
-    log_bench(
-        handle, "./rocsparse-bench -f nnz_compress -r", replaceX<T>("X"), "--mtx <matrix.mtx>");
+    ROCSPARSE_CHECKARG_HANDLE(0, handle);
+    ROCSPARSE_CHECKARG_SIZE(1, m);
+    ROCSPARSE_CHECKARG_POINTER(2, descr_A);
+    ROCSPARSE_CHECKARG(2,
+                       descr_A,
+                       (descr_A->storage_mode != rocsparse_storage_mode_sorted),
+                       rocsparse_status_requires_sorted_storage);
 
-    // Check matrix descriptor
-    if(descr_A == nullptr)
-    {
-        return rocsparse_status_invalid_pointer;
-    }
-
-    // Check matrix sorting mode
-    if(descr_A->storage_mode != rocsparse_storage_mode_sorted)
-    {
-        return rocsparse_status_not_implemented;
-    }
-
-    // Check sizes
-    if(m < 0)
-    {
-        return rocsparse_status_invalid_size;
-    }
-
-    if(std::real(tol) < std::real(static_cast<T>(0)))
-    {
-        return rocsparse_status_invalid_value;
-    }
+    ROCSPARSE_CHECKARG(
+        7, tol, (std::real(tol) < std::real(static_cast<T>(0))), rocsparse_status_invalid_value);
 
     // Quick return if possible
     if(m == 0)
@@ -118,11 +102,9 @@ rocsparse_status rocsparse_nnz_compress_template(rocsparse_handle          handl
         return rocsparse_status_success;
     }
 
-    // Check pointer arguments
-    if(csr_row_ptr_A == nullptr || nnz_per_row == nullptr || nnz_C == nullptr)
-    {
-        return rocsparse_status_invalid_pointer;
-    }
+    ROCSPARSE_CHECKARG_ARRAY(4, m, csr_row_ptr_A);
+    ROCSPARSE_CHECKARG_POINTER(5, nnz_per_row);
+    ROCSPARSE_CHECKARG_POINTER(6, nnz_C);
 
     // Find number of non-zeros in input CSR matrix on host
     rocsparse_int nnz_A, nnz_A_0;
@@ -135,14 +117,11 @@ rocsparse_status rocsparse_nnz_compress_template(rocsparse_handle          handl
     nnz_A -= nnz_A_0;
     if(nnz_A < 0)
     {
-        return rocsparse_status_invalid_size;
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_size);
     }
 
     // CSR values array can be nullptr if nnz_A is zero
-    if(nnz_A != 0 && csr_val_A == nullptr)
-    {
-        return rocsparse_status_invalid_pointer;
-    }
+    ROCSPARSE_CHECKARG_ARRAY(3, nnz_A, csr_val_A);
 
     // Stream
     hipStream_t stream = handle->stream;
@@ -162,85 +141,90 @@ rocsparse_status rocsparse_nnz_compress_template(rocsparse_handle          handl
             constexpr rocsparse_int segments_per_block = block_size / 2;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 2, 32>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 2, 32>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else if(mean_nnz_per_row < 8)
         {
             constexpr rocsparse_int segments_per_block = block_size / 4;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 4, 32>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 4, 32>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else if(mean_nnz_per_row < 16)
         {
             constexpr rocsparse_int segments_per_block = block_size / 8;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 8, 32>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 8, 32>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else if(mean_nnz_per_row < 32)
         {
             constexpr rocsparse_int segments_per_block = block_size / 16;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 16, 32>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 16, 32>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else
         {
             constexpr rocsparse_int segments_per_block = block_size / 32;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 32, 32>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 32, 32>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
     }
     else if(handle->wavefront_size == 64)
@@ -250,102 +234,108 @@ rocsparse_status rocsparse_nnz_compress_template(rocsparse_handle          handl
             constexpr rocsparse_int segments_per_block = block_size / 2;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 2, 64>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 2, 64>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else if(mean_nnz_per_row < 8)
         {
             constexpr rocsparse_int segments_per_block = block_size / 4;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 4, 64>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 4, 64>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else if(mean_nnz_per_row < 16)
         {
             constexpr rocsparse_int segments_per_block = block_size / 8;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 8, 64>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 8, 64>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else if(mean_nnz_per_row < 32)
         {
             constexpr rocsparse_int segments_per_block = block_size / 16;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 16, 64>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 16, 64>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else if(mean_nnz_per_row < 64)
         {
             constexpr rocsparse_int segments_per_block = block_size / 32;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 32, 64>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 32, 64>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
         else
         {
             constexpr rocsparse_int segments_per_block = block_size / 64;
             rocsparse_int           grid_size = (m + segments_per_block - 1) / segments_per_block;
 
-            hipLaunchKernelGGL((nnz_compress_kernel<block_size, segments_per_block, 64, 64>),
-                               dim3(grid_size),
-                               dim3(block_size),
-                               0,
-                               stream,
-                               m,
-                               descr_A->base,
-                               csr_val_A,
-                               csr_row_ptr_A,
-                               nnz_per_row,
-                               tol);
+            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+                (rocsparse::nnz_compress_kernel<block_size, segments_per_block, 64, 64>),
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream,
+                m,
+                descr_A->base,
+                csr_val_A,
+                csr_row_ptr_A,
+                nnz_per_row,
+                tol);
         }
     }
     else
@@ -365,10 +355,10 @@ rocsparse_status rocsparse_nnz_compress_template(rocsparse_handle          handl
     }
 
     // Perform inclusive scan on csr row pointer array
-    auto   op = rocprim::plus<rocsparse_int>();
     size_t temp_storage_size_bytes;
-    RETURN_IF_HIP_ERROR(
-        rocprim::reduce(nullptr, temp_storage_size_bytes, nnz_per_row, dnnz_C, m, op, stream));
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::find_sum_buffer_size<rocsparse_int, rocsparse_int>(
+            handle, m, &temp_storage_size_bytes)));
 
     bool  temp_alloc       = false;
     void* temp_storage_ptr = nullptr;
@@ -384,8 +374,8 @@ rocsparse_status rocsparse_nnz_compress_template(rocsparse_handle          handl
         temp_alloc = true;
     }
 
-    RETURN_IF_HIP_ERROR(rocprim::reduce(
-        temp_storage_ptr, temp_storage_size_bytes, nnz_per_row, dnnz_C, m, op, stream));
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::find_sum(
+        handle, nnz_per_row, dnnz_C, m, temp_storage_size_bytes, temp_storage_ptr));
 
     if(handle->pointer_mode == rocsparse_pointer_mode_host)
     {
@@ -417,9 +407,15 @@ extern "C" rocsparse_status rocsparse_snnz_compress(rocsparse_handle          ha
                                                     rocsparse_int*            nnz_per_row,
                                                     rocsparse_int*            nnz_C,
                                                     float                     tol)
+try
 {
-    return rocsparse_nnz_compress_template(
-        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol);
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
+        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
+    return rocsparse_status_success;
+}
+catch(...)
+{
+    RETURN_ROCSPARSE_EXCEPTION();
 }
 
 extern "C" rocsparse_status rocsparse_dnnz_compress(rocsparse_handle          handle,
@@ -430,9 +426,15 @@ extern "C" rocsparse_status rocsparse_dnnz_compress(rocsparse_handle          ha
                                                     rocsparse_int*            nnz_per_row,
                                                     rocsparse_int*            nnz_C,
                                                     double                    tol)
+try
 {
-    return rocsparse_nnz_compress_template(
-        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol);
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
+        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
+    return rocsparse_status_success;
+}
+catch(...)
+{
+    RETURN_ROCSPARSE_EXCEPTION();
 }
 
 extern "C" rocsparse_status rocsparse_cnnz_compress(rocsparse_handle               handle,
@@ -443,9 +445,15 @@ extern "C" rocsparse_status rocsparse_cnnz_compress(rocsparse_handle
                                                     rocsparse_int*                 nnz_per_row,
                                                     rocsparse_int*                 nnz_C,
                                                     rocsparse_float_complex        tol)
+try
 {
-    return rocsparse_nnz_compress_template(
-        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol);
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
+        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
+    return rocsparse_status_success;
+}
+catch(...)
+{
+    RETURN_ROCSPARSE_EXCEPTION();
 }
 
 extern "C" rocsparse_status rocsparse_znnz_compress(rocsparse_handle                handle,
@@ -456,7 +464,13 @@ extern "C" rocsparse_status rocsparse_znnz_compress(rocsparse_handle
                                                     rocsparse_int*                  nnz_per_row,
                                                     rocsparse_int*                  nnz_C,
                                                     rocsparse_double_complex        tol)
+try
 {
-    return rocsparse_nnz_compress_template(
-        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol);
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
+        handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
+    return rocsparse_status_success;
+}
+catch(...)
+{
+    RETURN_ROCSPARSE_EXCEPTION();
 }
