@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,13 +22,13 @@
  * ************************************************************************ */
 
 #include "handle.h"
-#include "definitions.h"
+#include "control.h"
 #include "logging.h"
 #include "utility.h"
 
 #include <hip/hip_runtime.h>
 
-ROCSPARSE_KERNEL void init_kernel(){};
+ROCSPARSE_KERNEL(1) void init_kernel(){};
 
 /*******************************************************************************
  * constructor
@@ -78,7 +78,7 @@ _rocsparse_handle::_rocsparse_handle()
     THROW_IF_HIP_ERROR(rocsparse_hipMalloc(&zone, sizeof(rocsparse_double_complex)));
 
     // Execute empty kernel for initialization
-    hipLaunchKernelGGL(init_kernel, dim3(1), dim3(1), 0, stream);
+    THROW_IF_HIPLAUNCHKERNELGGL_ERROR(init_kernel, dim3(1), dim3(1), 0, stream);
 
     // Execute memset for initialization
     THROW_IF_HIP_ERROR(hipMemsetAsync(sone, 0, sizeof(float), stream));
@@ -102,22 +102,42 @@ _rocsparse_handle::_rocsparse_handle()
     // Wait for device transfer to finish
     THROW_IF_HIP_ERROR(hipStreamSynchronize(stream));
 
+    // create blas handle
+    rocsparse::blas_impl blas_impl;
+
+#ifdef ROCSPARSE_WITH_ROCBLAS
+
+    blas_impl = rocsparse::blas_impl_rocblas;
+
+#else
+
+    //
+    // Other implementation available? Otherwise, set it to none.
+    //
+    blas_impl = rocsparse::blas_impl_none;
+#endif
+
+    THROW_IF_ROCSPARSE_ERROR(rocsparse::blas_create_handle(&this->blas_handle, blas_impl));
+    THROW_IF_ROCSPARSE_ERROR(rocsparse::blas_set_stream(this->blas_handle, this->stream));
+    THROW_IF_ROCSPARSE_ERROR(
+        rocsparse::blas_set_pointer_mode(this->blas_handle, this->pointer_mode));
+
     // Open log file
     if(layer_mode & rocsparse_layer_mode_log_trace)
     {
-        open_log_stream(&log_trace_os, &log_trace_ofs, "ROCSPARSE_LOG_TRACE_PATH");
+        rocsparse::open_log_stream(&log_trace_os, &log_trace_ofs, "ROCSPARSE_LOG_TRACE_PATH");
     }
 
     // Open log_bench file
     if(layer_mode & rocsparse_layer_mode_log_bench)
     {
-        open_log_stream(&log_bench_os, &log_bench_ofs, "ROCSPARSE_LOG_BENCH_PATH");
+        rocsparse::open_log_stream(&log_bench_os, &log_bench_ofs, "ROCSPARSE_LOG_BENCH_PATH");
     }
 
     // Open log_debug file
     if(layer_mode & rocsparse_layer_mode_log_debug)
     {
-        open_log_stream(&log_debug_os, &log_debug_ofs, "ROCSPARSE_LOG_DEBUG_PATH");
+        rocsparse::open_log_stream(&log_debug_os, &log_debug_ofs, "ROCSPARSE_LOG_DEBUG_PATH");
     }
 }
 
@@ -132,6 +152,13 @@ _rocsparse_handle::~_rocsparse_handle()
     PRINT_IF_HIP_ERROR(rocsparse_hipFree(cone));
     PRINT_IF_HIP_ERROR(rocsparse_hipFree(zone));
 
+    // destroy blas handle
+    rocsparse_status status = rocsparse::blas_destroy_handle(this->blas_handle);
+    if(status != rocsparse_status_success)
+    {
+        ROCSPARSE_ERROR_MESSAGE(status, "handle error");
+    }
+
     // Close log files
     if(log_trace_ofs.is_open())
     {
@@ -160,6 +187,10 @@ rocsparse_status _rocsparse_handle::set_stream(hipStream_t user_stream)
 {
     // TODO check if stream is valid
     stream = user_stream;
+
+    // blas set stream
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::blas_set_stream(this->blas_handle, user_stream));
+
     return rocsparse_status_success;
 }
 
@@ -172,13 +203,36 @@ rocsparse_status _rocsparse_handle::get_stream(hipStream_t* user_stream) const
     return rocsparse_status_success;
 }
 
+/*******************************************************************************
+ * set pointer mode
+ ******************************************************************************/
+rocsparse_status _rocsparse_handle::set_pointer_mode(rocsparse_pointer_mode user_mode)
+{
+    // TODO check if stream is valid
+    this->pointer_mode = user_mode;
+
+    // blas set stream
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::blas_set_pointer_mode(this->blas_handle, user_mode));
+
+    return rocsparse_status_success;
+}
+
+/*******************************************************************************
+ * get pointer mode
+ ******************************************************************************/
+rocsparse_status _rocsparse_handle::get_pointer_mode(rocsparse_pointer_mode* user_mode) const
+{
+    *user_mode = this->pointer_mode;
+    return rocsparse_status_success;
+}
+
 /********************************************************************************
  * \brief rocsparse_csrmv_info is a structure holding the rocsparse csrmv info
  * data gathered during csrmv_analysis. It must be initialized using the
- * rocsparse_create_csrmv_info() routine. It should be destroyed at the end
- * using rocsparse_destroy_csrmv_info().
+ * create_csrmv_info() routine. It should be destroyed at the end
+ * using destroy_csrmv_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_csrmv_info(rocsparse_csrmv_info* info)
+rocsparse_status rocsparse::create_csrmv_info(rocsparse_csrmv_info* info)
 {
     if(info == nullptr)
     {
@@ -202,8 +256,8 @@ rocsparse_status rocsparse_create_csrmv_info(rocsparse_csrmv_info* info)
 /********************************************************************************
  * \brief Copy csrmv info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_csrmv_info(rocsparse_csrmv_info       dest,
-                                           const rocsparse_csrmv_info src)
+rocsparse_status rocsparse::copy_csrmv_info(rocsparse_csrmv_info       dest,
+                                            const rocsparse_csrmv_info src)
 {
     if(dest == nullptr || src == nullptr || dest == src)
     {
@@ -212,11 +266,19 @@ rocsparse_status rocsparse_copy_csrmv_info(rocsparse_csrmv_info       dest,
 
     // check if destination already contains data. If it does, verify its allocated arrays are the same size as source
     bool previously_created = false;
-    previously_created |= (dest->size != 0);
 
-    previously_created |= (dest->row_blocks != nullptr);
-    previously_created |= (dest->wg_flags != nullptr);
-    previously_created |= (dest->wg_ids != nullptr);
+    previously_created |= (dest->adaptive.size != 0);
+    previously_created |= (dest->adaptive.first_row != 0);
+    previously_created |= (dest->adaptive.last_row != 0);
+    previously_created |= (dest->adaptive.row_blocks != nullptr);
+    previously_created |= (dest->adaptive.wg_flags != nullptr);
+    previously_created |= (dest->adaptive.wg_ids != nullptr);
+
+    previously_created |= (dest->lrb.size != 0);
+    previously_created |= (dest->lrb.wg_flags != nullptr);
+    previously_created |= (dest->lrb.rows_offsets_scratch != nullptr);
+    previously_created |= (dest->lrb.rows_bins != nullptr);
+    previously_created |= (dest->lrb.n_rows_bins != nullptr);
 
     previously_created |= (dest->trans != rocsparse_operation_none);
     previously_created |= (dest->m != 0);
@@ -233,7 +295,10 @@ rocsparse_status rocsparse_copy_csrmv_info(rocsparse_csrmv_info       dest,
     {
         // Sparsity pattern of dest and src must match
         bool invalid = false;
-        invalid |= (dest->size != src->size);
+        invalid |= (dest->adaptive.size != src->adaptive.size);
+        invalid |= (dest->adaptive.first_row != src->adaptive.first_row);
+        invalid |= (dest->adaptive.last_row != src->adaptive.last_row);
+        invalid |= (dest->lrb.size != src->lrb.size);
         invalid |= (dest->trans != src->trans);
         invalid |= (dest->m != src->m);
         invalid |= (dest->n != src->n);
@@ -288,47 +353,102 @@ rocsparse_status rocsparse_copy_csrmv_info(rocsparse_csrmv_info       dest,
     }
     }
 
-    if(src->row_blocks != nullptr)
+    if(src->adaptive.row_blocks != nullptr)
     {
-        if(dest->row_blocks == nullptr)
+        if(dest->adaptive.row_blocks == nullptr)
         {
-            RETURN_IF_HIP_ERROR(rocsparse_hipMalloc((void**)&dest->row_blocks, I_size * src->size));
+            RETURN_IF_HIP_ERROR(rocsparse_hipMalloc((void**)&dest->adaptive.row_blocks,
+                                                    I_size * src->adaptive.size));
         }
-        RETURN_IF_HIP_ERROR(hipMemcpy(
-            dest->row_blocks, src->row_blocks, I_size * src->size, hipMemcpyDeviceToDevice));
-    }
-
-    if(src->wg_flags != nullptr)
-    {
-        if(dest->wg_flags == nullptr)
-        {
-            RETURN_IF_HIP_ERROR(
-                rocsparse_hipMalloc((void**)&dest->wg_flags, sizeof(unsigned int) * src->size));
-        }
-        RETURN_IF_HIP_ERROR(hipMemcpy(dest->wg_flags,
-                                      src->wg_flags,
-                                      sizeof(unsigned int) * src->size,
+        RETURN_IF_HIP_ERROR(hipMemcpy(dest->adaptive.row_blocks,
+                                      src->adaptive.row_blocks,
+                                      I_size * src->adaptive.size,
                                       hipMemcpyDeviceToDevice));
     }
 
-    if(src->wg_ids != nullptr)
+    if(src->adaptive.wg_flags != nullptr)
     {
-        if(dest->wg_ids == nullptr)
+        if(dest->adaptive.wg_flags == nullptr)
         {
-            RETURN_IF_HIP_ERROR(rocsparse_hipMalloc((void**)&dest->wg_ids, J_size * src->size));
+            RETURN_IF_HIP_ERROR(rocsparse_hipMalloc((void**)&dest->adaptive.wg_flags,
+                                                    sizeof(uint32_t) * src->adaptive.size));
         }
-        RETURN_IF_HIP_ERROR(
-            hipMemcpy(dest->wg_ids, src->wg_ids, J_size * src->size, hipMemcpyDeviceToDevice));
+        RETURN_IF_HIP_ERROR(hipMemcpy(dest->adaptive.wg_flags,
+                                      src->adaptive.wg_flags,
+                                      sizeof(uint32_t) * src->adaptive.size,
+                                      hipMemcpyDeviceToDevice));
     }
 
-    dest->size         = src->size;
-    dest->trans        = src->trans;
-    dest->m            = src->m;
-    dest->n            = src->n;
-    dest->nnz          = src->nnz;
-    dest->max_rows     = src->max_rows;
-    dest->index_type_I = src->index_type_I;
-    dest->index_type_J = src->index_type_J;
+    if(src->adaptive.wg_ids != nullptr)
+    {
+        if(dest->adaptive.wg_ids == nullptr)
+        {
+            RETURN_IF_HIP_ERROR(
+                rocsparse_hipMalloc((void**)&dest->adaptive.wg_ids, J_size * src->adaptive.size));
+        }
+        RETURN_IF_HIP_ERROR(hipMemcpy(dest->adaptive.wg_ids,
+                                      src->adaptive.wg_ids,
+                                      J_size * src->adaptive.size,
+                                      hipMemcpyDeviceToDevice));
+    }
+
+    if(src->lrb.wg_flags != nullptr)
+    {
+        if(dest->lrb.wg_flags == nullptr)
+        {
+            RETURN_IF_HIP_ERROR(
+                rocsparse_hipMalloc((void**)&dest->lrb.wg_flags, sizeof(uint32_t) * src->lrb.size));
+        }
+        RETURN_IF_HIP_ERROR(hipMemcpy(dest->lrb.wg_flags,
+                                      src->lrb.wg_flags,
+                                      sizeof(uint32_t) * src->lrb.size,
+                                      hipMemcpyDeviceToDevice));
+    }
+
+    if(src->lrb.rows_offsets_scratch != nullptr)
+    {
+        if(dest->lrb.rows_offsets_scratch == nullptr)
+        {
+            RETURN_IF_HIP_ERROR(
+                rocsparse_hipMalloc((void**)&dest->lrb.rows_offsets_scratch, J_size * src->m));
+        }
+        RETURN_IF_HIP_ERROR(hipMemcpy(dest->lrb.rows_offsets_scratch,
+                                      src->lrb.rows_offsets_scratch,
+                                      J_size * src->m,
+                                      hipMemcpyDeviceToDevice));
+    }
+
+    if(src->lrb.rows_bins != nullptr)
+    {
+        if(dest->lrb.rows_bins == nullptr)
+        {
+            RETURN_IF_HIP_ERROR(rocsparse_hipMalloc((void**)&dest->lrb.rows_bins, J_size * src->m));
+        }
+        RETURN_IF_HIP_ERROR(hipMemcpy(
+            dest->lrb.rows_bins, src->lrb.rows_bins, J_size * src->m, hipMemcpyDeviceToDevice));
+    }
+
+    if(src->lrb.n_rows_bins != nullptr)
+    {
+        if(dest->lrb.n_rows_bins == nullptr)
+        {
+            RETURN_IF_HIP_ERROR(rocsparse_hipMalloc((void**)&dest->lrb.n_rows_bins, J_size * 32));
+        }
+        RETURN_IF_HIP_ERROR(hipMemcpy(
+            dest->lrb.n_rows_bins, src->lrb.n_rows_bins, J_size * 32, hipMemcpyDeviceToDevice));
+    }
+
+    dest->adaptive.size      = src->adaptive.size;
+    dest->adaptive.first_row = src->adaptive.first_row;
+    dest->adaptive.last_row  = src->adaptive.last_row;
+    dest->lrb.size           = src->lrb.size;
+    dest->trans              = src->trans;
+    dest->m                  = src->m;
+    dest->n                  = src->n;
+    dest->nnz                = src->nnz;
+    dest->max_rows           = src->max_rows;
+    dest->index_type_I       = src->index_type_I;
+    dest->index_type_J       = src->index_type_J;
 
     // Not owned by the info struct. Just pointers to externally allocated memory
     dest->descr       = src->descr;
@@ -341,21 +461,34 @@ rocsparse_status rocsparse_copy_csrmv_info(rocsparse_csrmv_info       dest,
 /********************************************************************************
  * \brief Destroy csrmv info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_csrmv_info(rocsparse_csrmv_info info)
+rocsparse_status rocsparse::destroy_csrmv_info(rocsparse_csrmv_info info)
 {
     if(info == nullptr)
     {
         return rocsparse_status_success;
     }
 
-    // Clean up row blocks
-    if(info->size > 0)
+    // Clean up adaptive arrays
+    if(info->adaptive.size > 0)
     {
-        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->row_blocks));
-        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->wg_flags));
-        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->wg_ids));
+        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->adaptive.row_blocks));
+        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->adaptive.wg_flags));
+        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->adaptive.wg_ids));
     }
 
+    if(info->lrb.size > 0)
+    {
+        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->lrb.wg_flags));
+    }
+
+    if(info->m > 0)
+    {
+        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->lrb.rows_offsets_scratch));
+        RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->lrb.rows_bins));
+    }
+
+    RETURN_IF_HIP_ERROR(rocsparse_hipFree(info->lrb.n_rows_bins));
+
     // Destruct
     try
     {
@@ -372,10 +505,10 @@ rocsparse_status rocsparse_destroy_csrmv_info(rocsparse_csrmv_info info)
  * \brief rocsparse_trm_info is a structure holding the rocsparse bsrsv, csrsv,
  * csrsm, csrilu0 and csric0 data gathered during csrsv_analysis,
  * csrilu0_analysis and csric0_analysis. It must be initialized using the
- * rocsparse_create_trm_info() routine. It should be destroyed at the end
- * using rocsparse_destroy_trm_info().
+ * create_trm_info() routine. It should be destroyed at the end
+ * using destroy_trm_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_trm_info(rocsparse_trm_info* info)
+rocsparse_status rocsparse::create_trm_info(rocsparse_trm_info* info)
 {
     if(info == nullptr)
     {
@@ -399,7 +532,7 @@ rocsparse_status rocsparse_create_trm_info(rocsparse_trm_info* info)
 /********************************************************************************
  * \brief Copy trm info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_trm_info(rocsparse_trm_info dest, const rocsparse_trm_info src)
+rocsparse_status rocsparse::copy_trm_info(rocsparse_trm_info dest, const rocsparse_trm_info src)
 {
     if(dest == nullptr || src == nullptr || dest == src)
     {
@@ -550,7 +683,7 @@ rocsparse_status rocsparse_copy_trm_info(rocsparse_trm_info dest, const rocspars
 /********************************************************************************
  * \brief Destroy trm info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_trm_info(rocsparse_trm_info info)
+rocsparse_status rocsparse::destroy_trm_info(rocsparse_trm_info info)
 {
     if(info == nullptr)
     {
@@ -602,10 +735,10 @@ rocsparse_status rocsparse_destroy_trm_info(rocsparse_trm_info info)
 }
 
 /********************************************************************************
- * \brief rocsparse_check_trm_shared checks if the given trm info structure
+ * \brief check_trm_shared checks if the given trm info structure
  * shares its meta data with another trm info structure.
  *******************************************************************************/
-bool rocsparse_check_trm_shared(const rocsparse_mat_info info, rocsparse_trm_info trm)
+bool rocsparse::check_trm_shared(const rocsparse_mat_info info, rocsparse_trm_info trm)
 {
     if(info == nullptr)
     {
@@ -653,10 +786,10 @@ bool rocsparse_check_trm_shared(const rocsparse_mat_info info, rocsparse_trm_inf
 /********************************************************************************
  * \brief rocsparse_csrgemm_info is a structure holding the rocsparse csrgemm
  * info data gathered during csrgemm_buffer_size. It must be initialized using
- * the rocsparse_create_csrgemm_info() routine. It should be destroyed at the
- * end using rocsparse_destroy_csrgemm_info().
+ * the create_csrgemm_info() routine. It should be destroyed at the
+ * end using destroy_csrgemm_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_csrgemm_info(rocsparse_csrgemm_info* info)
+rocsparse_status rocsparse::create_csrgemm_info(rocsparse_csrgemm_info* info)
 {
     if(info == nullptr)
     {
@@ -680,8 +813,8 @@ rocsparse_status rocsparse_create_csrgemm_info(rocsparse_csrgemm_info* info)
 /********************************************************************************
  * \brief Copy csrgemm info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_csrgemm_info(rocsparse_csrgemm_info       dest,
-                                             const rocsparse_csrgemm_info src)
+rocsparse_status rocsparse::copy_csrgemm_info(rocsparse_csrgemm_info       dest,
+                                              const rocsparse_csrgemm_info src)
 {
     if(dest == nullptr || src == nullptr || dest == src)
     {
@@ -697,7 +830,7 @@ rocsparse_status rocsparse_copy_csrgemm_info(rocsparse_csrgemm_info       dest,
 /********************************************************************************
  * \brief Destroy csrgemm info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_csrgemm_info(rocsparse_csrgemm_info info)
+rocsparse_status rocsparse::destroy_csrgemm_info(rocsparse_csrgemm_info info)
 {
     if(info == nullptr)
     {
@@ -719,10 +852,10 @@ rocsparse_status rocsparse_destroy_csrgemm_info(rocsparse_csrgemm_info info)
 /********************************************************************************
  * \brief rocsparse_csritsv_info is a structure holding the rocsparse csritsv
  * info data gathered during csritsv_buffer_size. It must be initialized using
- * the rocsparse_create_csritsv_info() routine. It should be destroyed at the
- * end using rocsparse_destroy_csritsv_info().
+ * the create_csritsv_info() routine. It should be destroyed at the
+ * end using destroy_csritsv_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_csritsv_info(rocsparse_csritsv_info* info)
+rocsparse_status rocsparse::create_csritsv_info(rocsparse_csritsv_info* info)
 {
     if(info == nullptr)
     {
@@ -746,8 +879,8 @@ rocsparse_status rocsparse_create_csritsv_info(rocsparse_csritsv_info* info)
 /********************************************************************************
  * \brief Copy csritsv info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_csritsv_info(rocsparse_csritsv_info       dest,
-                                             const rocsparse_csritsv_info src)
+rocsparse_status rocsparse::copy_csritsv_info(rocsparse_csritsv_info       dest,
+                                              const rocsparse_csritsv_info src)
 {
     if(dest == nullptr || src == nullptr || dest == src)
     {
@@ -763,7 +896,7 @@ rocsparse_status rocsparse_copy_csritsv_info(rocsparse_csritsv_info       dest,
 /********************************************************************************
  * \brief Destroy csritsv info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_csritsv_info(rocsparse_csritsv_info info)
+rocsparse_status rocsparse::destroy_csritsv_info(rocsparse_csritsv_info info)
 {
     if(info == nullptr)
     {
@@ -787,3 +920,107 @@ rocsparse_status rocsparse_destroy_csritsv_info(rocsparse_csritsv_info info)
     }
     return rocsparse_status_success;
 }
+
+// Emulate C++17 std::void_t
+template <typename...>
+using void_t = void;
+
+// By default, use gcnArch converted to a string prepended by gfx
+template <typename PROP, typename = void>
+struct ArchName
+{
+    std::string operator()(const PROP& prop) const;
+};
+
+// If gcnArchName exists as a member, use it instead
+template <typename PROP>
+struct ArchName<PROP, void_t<decltype(PROP::gcnArchName)>>
+{
+    std::string operator()(const PROP& prop) const
+    {
+        // strip out xnack/ecc from name
+        std::string gcnArchName(prop.gcnArchName);
+        std::string gcnArch = gcnArchName.substr(0, gcnArchName.find(":"));
+        return gcnArch;
+    }
+};
+
+// If gcnArchName not present, no xnack mode
+template <typename PROP, typename = void>
+struct XnackMode
+{
+    std::string operator()(const PROP& prop) const
+    {
+        return "";
+    }
+};
+
+// If gcnArchName exists as a member, use it
+template <typename PROP>
+struct XnackMode<PROP, void_t<decltype(PROP::gcnArchName)>>
+{
+    std::string operator()(const PROP& prop) const
+    {
+        // strip out xnack/ecc from name
+        std::string gcnArchName(prop.gcnArchName);
+        auto        loc = gcnArchName.find("xnack");
+        std::string xnackMode;
+        if(loc != std::string::npos)
+        {
+            xnackMode = gcnArchName.substr(loc, 6);
+            // guard against missing +/- at end of xnack mode
+            if(xnackMode.size() < 6)
+                xnackMode = "";
+        }
+        return xnackMode;
+    }
+};
+
+std::string rocsparse::handle_get_arch_name(rocsparse_handle handle)
+{
+    return ArchName<hipDeviceProp_t>{}(handle->properties);
+}
+
+std::string rocsparse::handle_get_xnack_mode(rocsparse_handle handle)
+{
+    return XnackMode<hipDeviceProp_t>{}(handle->properties);
+}
+
+/*******************************************************************************
+ * \brief convert hipError_t to rocsparse_status
+ * TODO - enumerate library calls to hip runtime, enumerate possible errors from
+ * those calls
+ ******************************************************************************/
+rocsparse_status rocsparse::get_rocsparse_status_for_hip_status(hipError_t status)
+{
+    switch(status)
+    {
+    // success
+    case hipSuccess:
+        return rocsparse_status_success;
+
+    // internal hip memory allocation
+    case hipErrorMemoryAllocation:
+    case hipErrorLaunchOutOfResources:
+        return rocsparse_status_memory_error;
+
+    // user-allocated hip memory
+    case hipErrorInvalidDevicePointer: // hip memory
+        return rocsparse_status_invalid_pointer;
+
+    // user-allocated device, stream, event
+    case hipErrorInvalidDevice:
+    case hipErrorInvalidResourceHandle:
+        return rocsparse_status_invalid_handle;
+
+    // library using hip incorrectly
+    case hipErrorInvalidValue:
+        return rocsparse_status_internal_error;
+
+    // hip runtime failing
+    case hipErrorNoDevice: // no hip devices
+    case hipErrorUnknown:
+    default:
+        return rocsparse_status_internal_error;
+    }
+}
