@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,12 +24,12 @@
 
 #pragma once
 
-#include "rocsparse.h"
+#include "rocsparse-auxiliary.h"
+#include "rocsparse-version.h"
 
+#include "rocsparse_blas.h"
 #include <fstream>
 #include <hip/hip_runtime_api.h>
-#include <iostream>
-#include <vector>
 
 /*! \brief typedefs to opaque info structs */
 typedef struct _rocsparse_trm_info*     rocsparse_trm_info;
@@ -55,6 +55,10 @@ struct _rocsparse_handle
     rocsparse_status set_stream(hipStream_t user_stream);
     // get stream
     rocsparse_status get_stream(hipStream_t* user_stream) const;
+    // set pointer mode
+    rocsparse_status set_pointer_mode(rocsparse_pointer_mode user_mode);
+    // get pointer mode
+    rocsparse_status get_pointer_mode(rocsparse_pointer_mode* user_mode) const;
 
     // device id
     int device;
@@ -71,14 +75,16 @@ struct _rocsparse_handle
     // logging mode
     rocsparse_layer_mode layer_mode;
     // device buffer
-    size_t buffer_size;
-    void*  buffer;
+    size_t buffer_size{};
+    void*  buffer{};
     // device one
-    float*  sone;
-    double* done;
+    float*  sone{};
+    double* done{};
     // device complex one
-    rocsparse_float_complex*  cone;
-    rocsparse_double_complex* zone;
+    rocsparse_float_complex*  cone{};
+    rocsparse_double_complex* zone{};
+    // blas handle
+    rocsparse::blas_handle blas_handle;
 
     // logging streams
     std::ofstream log_trace_ofs;
@@ -182,6 +188,12 @@ struct _rocsparse_mat_info
     // zero pivot for csrsv, csrsm, csrilu0, csric0
     void* zero_pivot{};
 
+    // singular pivot for csric0
+    void* singular_pivot{};
+
+    // tolerance used for determining near singularity
+    double singular_tol{};
+
     // numeric boost for ilu0
     int         boost_enable{};
     int         use_double_prec_tol{};
@@ -200,20 +212,38 @@ struct _rocsparse_color_info
 {
 };
 
+struct rocsparse_adaptive_info
+{
+    size_t    size{}; // num row blocks
+    int64_t   first_row{};
+    int64_t   last_row{};
+    void*     row_blocks{};
+    uint32_t* wg_flags{};
+    void*     wg_ids{};
+};
+
+struct rocsparse_lrb_info
+{
+    void* rows_offsets_scratch{}; // size of m
+    void* rows_bins{}; // size of m
+    void* n_rows_bins{}; // size of 32
+
+    size_t    size{};
+    uint32_t* wg_flags{};
+
+    int64_t nRowsBins[32]{}; // host array
+};
+
 /********************************************************************************
  * \brief rocsparse_csrmv_info is a structure holding the rocsparse csrmv info
  * data gathered during csrmv_analysis. It must be initialized using the
- * rocsparse_create_csrmv_info() routine. It should be destroyed at the end
- * rocsparse_destroy_csrmv_info().
+ * create_csrmv_info() routine. It should be destroyed at the end
+ * destroy_csrmv_info().
  *******************************************************************************/
 struct _rocsparse_csrmv_info
 {
-    // num row blocks
-    size_t size{};
-    // row blocks
-    void*         row_blocks{};
-    unsigned int* wg_flags{};
-    void*         wg_ids{};
+    rocsparse_adaptive_info adaptive;
+    rocsparse_lrb_info      lrb;
 
     // some data to verify correct execution
     rocsparse_operation         trans = rocsparse_operation_none;
@@ -229,24 +259,26 @@ struct _rocsparse_csrmv_info
     rocsparse_indextype index_type_J = rocsparse_indextype_u16;
 };
 
-/********************************************************************************
+namespace rocsparse
+{
+    /********************************************************************************
  * \brief rocsparse_csrmv_info is a structure holding the rocsparse csrmv info
  * data gathered during csrmv_analysis. It must be initialized using the
- * rocsparse_create_csrmv_info() routine. It should be destroyed at the end
- * using rocsparse_destroy_csrmv_info().
+ * create_csrmv_info() routine. It should be destroyed at the end
+ * using destroy_csrmv_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_csrmv_info(rocsparse_csrmv_info* info);
+    rocsparse_status create_csrmv_info(rocsparse_csrmv_info* info);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Copy csrmv info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_csrmv_info(rocsparse_csrmv_info       dest,
-                                           const rocsparse_csrmv_info src);
+    rocsparse_status copy_csrmv_info(rocsparse_csrmv_info dest, const rocsparse_csrmv_info src);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Destroy csrmv info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_csrmv_info(rocsparse_csrmv_info info);
+    rocsparse_status destroy_csrmv_info(rocsparse_csrmv_info info);
+}
 
 struct _rocsparse_trm_info
 {
@@ -273,36 +305,39 @@ struct _rocsparse_trm_info
     rocsparse_indextype index_type_J = rocsparse_indextype_u16;
 };
 
-/********************************************************************************
+namespace rocsparse
+{
+    /********************************************************************************
  * \brief rocsparse_trm_info is a structure holding the rocsparse bsrsv, csrsv,
  * csrsm, csrilu0 and csric0 data gathered during csrsv_analysis,
  * csrilu0_analysis and csric0_analysis. It must be initialized using the
- * rocsparse_create_trm_info() routine. It should be destroyed at the end
- * using rocsparse_destroy_trm_info().
+ * create_trm_info() routine. It should be destroyed at the end
+ * using destroy_trm_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_trm_info(rocsparse_trm_info* info);
+    rocsparse_status create_trm_info(rocsparse_trm_info* info);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Copy trm info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_trm_info(rocsparse_trm_info dest, const rocsparse_trm_info src);
+    rocsparse_status copy_trm_info(rocsparse_trm_info dest, const rocsparse_trm_info src);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Destroy trm info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_trm_info(rocsparse_trm_info info);
+    rocsparse_status destroy_trm_info(rocsparse_trm_info info);
 
-/********************************************************************************
- * \brief rocsparse_check_trm_shared checks if the given trm info structure
+    /********************************************************************************
+ * \brief check_trm_shared checks if the given trm info structure
  * shares its meta data with another trm info structure.
  *******************************************************************************/
-bool rocsparse_check_trm_shared(const rocsparse_mat_info info, rocsparse_trm_info trm);
+    bool check_trm_shared(const rocsparse_mat_info info, rocsparse_trm_info trm);
+}
 
 /********************************************************************************
  * \brief rocsparse_csritsv_info is a structure holding the rocsparse csritsv
  * info data gathered during csritsv_buffer_size. It must be initialized using
- * the rocsparse_create_csritsv_info() routine. It should be destroyed at the
- * end using rocsparse_destroy_csritsv_info().
+ * the create_csritsv_info() routine. It should be destroyed at the
+ * end using destroy_csritsv_info().
  *******************************************************************************/
 struct _rocsparse_csritsv_info
 {
@@ -312,57 +347,65 @@ struct _rocsparse_csritsv_info
     void*               ptr_end{};
 };
 
-/********************************************************************************
+namespace rocsparse
+{
+    /********************************************************************************
  * \brief rocsparse_csritsv_info is a structure holding the rocsparse csritsv
  * info data gathered during csritsv_buffer_size. It must be initialized using
- * the rocsparse_create_csritsv_info() routine. It should be destroyed at the
- * end using rocsparse_destroy_csritsv_info().
+ * the create_csritsv_info() routine. It should be destroyed at the
+ * end using destroy_csritsv_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_csritsv_info(rocsparse_csritsv_info* info);
+    rocsparse_status create_csritsv_info(rocsparse_csritsv_info* info);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Copy csritsv info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_csritsv_info(rocsparse_csritsv_info       dest,
-                                             const rocsparse_csritsv_info src);
+    rocsparse_status copy_csritsv_info(rocsparse_csritsv_info       dest,
+                                       const rocsparse_csritsv_info src);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Destroy csritsv info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_csritsv_info(rocsparse_csritsv_info info);
+    rocsparse_status destroy_csritsv_info(rocsparse_csritsv_info info);
+}
 
 /********************************************************************************
  * \brief rocsparse_csrgemm_info is a structure holding the rocsparse csrgemm
  * info data gathered during csrgemm_buffer_size. It must be initialized using
- * the rocsparse_create_csrgemm_info() routine. It should be destroyed at the
- * end using rocsparse_destroy_csrgemm_info().
+ * the create_csrgemm_info() routine. It should be destroyed at the
+ * end using destroy_csrgemm_info().
  *******************************************************************************/
 struct _rocsparse_csrgemm_info
 {
+    size_t buffer_size{};
+    bool   is_initialized{};
     // Perform alpha * A * B
-    bool mul = true;
+    bool mul{true};
     // Perform beta * D
-    bool add = true;
+    bool add{true};
 };
 
-/********************************************************************************
+namespace rocsparse
+{
+    /********************************************************************************
  * \brief rocsparse_csrgemm_info is a structure holding the rocsparse csrgemm
  * info data gathered during csrgemm_buffer_size. It must be initialized using
- * the rocsparse_create_csrgemm_info() routine. It should be destroyed at the
- * end using rocsparse_destroy_csrgemm_info().
+ * the create_csrgemm_info() routine. It should be destroyed at the
+ * end using destroy_csrgemm_info().
  *******************************************************************************/
-rocsparse_status rocsparse_create_csrgemm_info(rocsparse_csrgemm_info* info);
+    rocsparse_status create_csrgemm_info(rocsparse_csrgemm_info* info);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Copy csrgemm info.
  *******************************************************************************/
-rocsparse_status rocsparse_copy_csrgemm_info(rocsparse_csrgemm_info       dest,
-                                             const rocsparse_csrgemm_info src);
+    rocsparse_status copy_csrgemm_info(rocsparse_csrgemm_info       dest,
+                                       const rocsparse_csrgemm_info src);
 
-/********************************************************************************
+    /********************************************************************************
  * \brief Destroy csrgemm info.
  *******************************************************************************/
-rocsparse_status rocsparse_destroy_csrgemm_info(rocsparse_csrgemm_info info);
+    rocsparse_status destroy_csrgemm_info(rocsparse_csrgemm_info info);
+}
 
 /********************************************************************************
  * \brief ELL format indexing
@@ -381,6 +424,9 @@ struct _rocsparse_spvec_descr
     void* idx_data{};
     void* val_data{};
 
+    const void* const_idx_data{};
+    const void* const_val_data{};
+
     rocsparse_indextype idx_type{};
     rocsparse_datatype  data_type{};
 
@@ -390,7 +436,8 @@ struct _rocsparse_spvec_descr
 struct _rocsparse_spmat_descr
 {
     bool init{};
-    bool analysed{};
+
+    mutable bool analysed{};
 
     int64_t rows{};
     int64_t cols{};
@@ -401,6 +448,11 @@ struct _rocsparse_spmat_descr
     void* ind_data{};
     void* val_data{};
 
+    const void* const_row_data{};
+    const void* const_col_data{};
+    const void* const_ind_data{};
+    const void* const_val_data{};
+
     rocsparse_indextype row_type{};
     rocsparse_indextype col_type{};
     rocsparse_datatype  data_type{};
@@ -428,6 +480,7 @@ struct _rocsparse_dnvec_descr
 
     int64_t            size{};
     void*              values{};
+    const void*        const_values{};
     rocsparse_datatype data_type{};
 };
 
@@ -441,9 +494,29 @@ struct _rocsparse_dnmat_descr
 
     void* values{};
 
+    const void* const_values{};
+
     rocsparse_datatype data_type{};
     rocsparse_order    order{};
 
     int64_t batch_count{};
     int64_t batch_stride{};
 };
+
+namespace rocsparse
+{
+    //
+    // Get architecture name.
+    //
+    std::string handle_get_arch_name(rocsparse_handle handle);
+
+    struct rocpsarse_arch_names
+    {
+        static constexpr const char* gfx908 = "gfx908";
+    };
+
+    //
+    // Get xnack mode.
+    //
+    std::string handle_get_xnack_mode(rocsparse_handle handle);
+}
