@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,12 @@
 
 #pragma once
 
-#include "rocsparse.h"
+#include "rocsparse-types.h"
+
+#include "internal/level2/rocsparse_bsrmv.h"
+#include "internal/level2/rocsparse_csrsv.h"
+#include "internal/level2/rocsparse_gebsrmv.h"
+#include "internal/level3/rocsparse_csrmm.h"
 
 //
 // For reusing without recompiling.
@@ -83,6 +88,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle          handle,
                                         const rocsparse_int*      bsr_row_ptr,
                                         const rocsparse_int*      bsr_col_ind,
                                         rocsparse_int             bsr_dim,
+                                        rocsparse_mat_info        info,
                                         const T*                  x,
                                         const T*                  beta,
                                         T*                        y);
@@ -100,6 +106,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle          handle,
                                         const rocsparse_int*      bsr_row_ptr,
                                         const rocsparse_int*      bsr_col_ind,
                                         rocsparse_int             bsr_dim,
+                                        rocsparse_mat_info        info,
                                         const float*              x,
                                         const float*              beta,
                                         float*                    y)
@@ -116,6 +123,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle          handle,
                             bsr_row_ptr,
                             bsr_col_ind,
                             bsr_dim,
+                            info,
                             x,
                             beta,
                             y);
@@ -134,6 +142,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle          handle,
                                         const rocsparse_int*      bsr_row_ptr,
                                         const rocsparse_int*      bsr_col_ind,
                                         rocsparse_int             bsr_dim,
+                                        rocsparse_mat_info        info,
                                         const double*             x,
                                         const double*             beta,
                                         double*                   y)
@@ -150,6 +159,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle          handle,
                             bsr_row_ptr,
                             bsr_col_ind,
                             bsr_dim,
+                            info,
                             x,
                             beta,
                             y);
@@ -168,6 +178,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle               handle,
                                         const rocsparse_int*           bsr_row_ptr,
                                         const rocsparse_int*           bsr_col_ind,
                                         rocsparse_int                  bsr_dim,
+                                        rocsparse_mat_info             info,
                                         const rocsparse_float_complex* x,
                                         const rocsparse_float_complex* beta,
                                         rocsparse_float_complex*       y)
@@ -184,6 +195,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle               handle,
                             bsr_row_ptr,
                             bsr_col_ind,
                             bsr_dim,
+                            info,
                             x,
                             beta,
                             y);
@@ -202,6 +214,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle                handle,
                                         const rocsparse_int*            bsr_row_ptr,
                                         const rocsparse_int*            bsr_col_ind,
                                         rocsparse_int                   bsr_dim,
+                                        rocsparse_mat_info              info,
                                         const rocsparse_double_complex* x,
                                         const rocsparse_double_complex* beta,
                                         rocsparse_double_complex*       y)
@@ -218,6 +231,7 @@ inline rocsparse_status rocsparse_bsrmv(rocsparse_handle                handle,
                             bsr_row_ptr,
                             bsr_col_ind,
                             bsr_dim,
+                            info,
                             x,
                             beta,
                             y);
@@ -401,10 +415,10 @@ inline rocsparse_status rocsparse_csrmm(rocsparse_handle          handle,
                                         const rocsparse_int*      csr_row_ptr,
                                         const rocsparse_int*      csr_col_ind,
                                         const T*                  B,
-                                        rocsparse_int             ldb,
+                                        int64_t                   ldb,
                                         const T*                  beta,
                                         T*                        C,
-                                        rocsparse_int             ldc);
+                                        int64_t                   ldc);
 
 template <>
 inline rocsparse_status rocsparse_csrmm(rocsparse_handle          handle,
@@ -420,10 +434,10 @@ inline rocsparse_status rocsparse_csrmm(rocsparse_handle          handle,
                                         const rocsparse_int*      csr_row_ptr,
                                         const rocsparse_int*      csr_col_ind,
                                         const float*              B,
-                                        rocsparse_int             ldb,
+                                        int64_t                   ldb,
                                         const float*              beta,
                                         float*                    C,
-                                        rocsparse_int             ldc)
+                                        int64_t                   ldc)
 {
     return rocsparse_scsrmm(handle,
                             trans_A,
@@ -458,10 +472,10 @@ inline rocsparse_status rocsparse_csrmm(rocsparse_handle          handle,
                                         const rocsparse_int*      csr_row_ptr,
                                         const rocsparse_int*      csr_col_ind,
                                         const double*             B,
-                                        rocsparse_int             ldb,
+                                        int64_t                   ldb,
                                         const double*             beta,
                                         double*                   C,
-                                        rocsparse_int             ldc)
+                                        int64_t                   ldc)
 {
     return rocsparse_dcsrmm(handle,
                             trans_A,
@@ -496,10 +510,10 @@ inline rocsparse_status rocsparse_csrmm(rocsparse_handle               handle,
                                         const rocsparse_int*           csr_row_ptr,
                                         const rocsparse_int*           csr_col_ind,
                                         const rocsparse_float_complex* B,
-                                        rocsparse_int                  ldb,
+                                        int64_t                        ldb,
                                         const rocsparse_float_complex* beta,
                                         rocsparse_float_complex*       C,
-                                        rocsparse_int                  ldc)
+                                        int64_t                        ldc)
 {
     return rocsparse_ccsrmm(handle,
                             trans_A,
@@ -534,10 +548,10 @@ inline rocsparse_status rocsparse_csrmm(rocsparse_handle                handle,
                                         const rocsparse_int*            csr_row_ptr,
                                         const rocsparse_int*            csr_col_ind,
                                         const rocsparse_double_complex* B,
-                                        rocsparse_int                   ldb,
+                                        int64_t                         ldb,
                                         const rocsparse_double_complex* beta,
                                         rocsparse_double_complex*       C,
-                                        rocsparse_int                   ldc)
+                                        int64_t                         ldc)
 {
     return rocsparse_zcsrmm(handle,
                             trans_A,
