@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -292,10 +292,10 @@ private:
 //
 // This forces instantation (but that's not the purpose of it).
 //
-bool memstat::s_enabled = ROCSPARSE_ENVARIABLES.get(rocsparse_envariables::MEMSTAT);
+bool memstat::s_enabled = ROCSPARSE_ENVARIABLES.get(rocsparse::envariables::MEMSTAT);
 bool memstat::s_force_managed
-    = ROCSPARSE_ENVARIABLES.get(rocsparse_envariables::MEMSTAT_FORCE_MANAGED);
-bool memstat::s_guards_enabled = ROCSPARSE_ENVARIABLES.get(rocsparse_envariables::MEMSTAT_GUARDS);
+    = ROCSPARSE_ENVARIABLES.get(rocsparse::envariables::MEMSTAT_FORCE_MANAGED);
+bool memstat::s_guards_enabled = ROCSPARSE_ENVARIABLES.get(rocsparse::envariables::MEMSTAT_GUARDS);
 
 template <memstat_mode::value_t MODE>
 size_t memstat_allocator<MODE>::compute_nbytes(size_t s)
@@ -600,7 +600,10 @@ hipError_t memstat_allocator<MODE>::check_guards(char* d, size_t size)
             }
 
             // Make sure no corruption has occurred
-            assert(memcmp(host, guard, sizeof(guard)) == 0);
+            if(memcmp(host, guard, sizeof(guard)) != 0)
+            {
+                FORWARD_WITH_MESSAGE_IF_HIP_ERROR(hipErrorUnknown, "Memory corruption detected");
+            }
 
             // Point to guard before allocated memory
             d -= PAD;
@@ -613,7 +616,10 @@ hipError_t memstat_allocator<MODE>::check_guards(char* d, size_t size)
             }
 
             // Make sure no corruption has occurred
-            assert(memcmp(host, guard, sizeof(guard)) == 0);
+            if(memcmp(host, guard, sizeof(guard)) != 0)
+            {
+                FORWARD_WITH_MESSAGE_IF_HIP_ERROR(hipErrorUnknown, "Memory corruption detected");
+            }
         }
         return hipSuccess;
     }
@@ -840,6 +846,8 @@ memstat::memstat()
 
 void memstat::add(void* address, size_t nbytes, memstat_mode::value_t mode, const char* tag)
 {
+    if(address == nullptr)
+        return;
     if(!contains(address))
     {
         double t = get_time_us();
@@ -865,8 +873,8 @@ void memstat::add(void* address, size_t nbytes, memstat_mode::value_t mode, cons
     }
     else
     {
-        std::cerr << " already exist" << std::endl;
-        exit(1);
+        THROW_WITH_MESSAGE_IF_ROCSPARSE_ERROR(rocsparse_status_internal_error,
+                                              "Address already exists in the memstat database.");
     }
 }
 
@@ -941,13 +949,15 @@ void memstat::flush_report(bool finalize)
                           (first) ? std::ios_base::out : std::ios_base::app);
         this->flush_report(out, finalize);
         out.close();
-        this->m_next_flush_report += std::min(m_data.size(), size_t(128));
+        this->m_next_flush_report += rocsparse::min(m_data.size(), size_t(128));
         this->m_data.clear();
     }
 }
 
 void memstat::remove(void* address, const char* tag)
 {
+    if(address == nullptr)
+        return;
     auto it = m_map.find(address);
     if(it != m_map.end())
     {
@@ -967,14 +977,14 @@ void memstat::remove(void* address, const char* tag)
     }
     else
     {
-        std::cerr << "ROCSPARSE MEMSTAT, address not found." << std::endl;
-        exit(1);
+        THROW_WITH_MESSAGE_IF_ROCSPARSE_ERROR(rocsparse_status_internal_error,
+                                              "Cannot remove address from the memstat database");
     }
 }
 
 bool memstat::contains(void* address) const
 {
-    return (m_map.find(address) != m_map.end());
+    return ((address != nullptr) && (m_map.find(address) != m_map.end()));
 }
 
 //
